/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package examples.inventory;

import jsl.modeling.*;

/**
 */
public class TestLKInventoryModel {

	/**
	 * 
	 */
	public TestLKInventoryModel() {
		super();
	}

	public static void main(String[] args) {
		System.out.println("LKInventory Test");
            
		testReplication();
//		testExperiment();
        
		System.out.println("Done!");
        
	}
	
	public static void testReplication(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		LKInventoryModel im = new LKInventoryModel(m);
        
		im.setReorderPoint(20);
		im.setOrderUpToLevel(40);
		m.turnOnLogReport();
       
        // create a replication to run the model
		Replication r = new Replication(m);
		
		// set the running parameters of the replication
		r.setLengthOfReplication(120.0);
//		r.setMaximumExecutionTime(1000);
		r.setLengthOfWarmUp(20.0);
		
		// turn on the desired reporting
		r.turnOnBatchReport();
		//r.turnOnLogReport();
		
		// tell the replication to run
		r.runAll();
 
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		LKInventoryModel im = new LKInventoryModel(m);
        
		im.setReorderPoint(20);
		im.setOrderUpToLevel(40);
		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(120.0);
		e.setLengthOfWarmUp(20.0);
		
		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}
}
