/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package examples.jobshopV1;

import java.util.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.reporting.JSL;
import jsl.modeling.ModelElement;
/**
 *
 */
public class Job {
    
    private static int jobCounter;
    private int myNumber;
    private int myType;
    private List mySequence;
    private Iterator stepIterator;
    private WorkStation myCurrentStation;
    private Variable myCurrentServiceTime;
    private double myCreationTime;
    private ResponseVariable mySystemTime;
    
    /** Creates a new instance of Job */
    public Job(int type, double createTime, List sequence, ResponseVariable systemTime) {
        jobCounter = jobCounter + 1;
        myNumber = jobCounter;
        mySystemTime = systemTime;
        myType = type;
        myCreationTime = createTime;
        mySequence = sequence;
        stepIterator = mySequence.iterator();
    }
    
    public int getNumber(){
        return(myNumber);
    }
    
    public double getCreateTime(){
        return(myCreationTime);
    }
    
    public double getType(){
        return(myType);
    }
    
    public void nextJobStep(){
        if (stepIterator.hasNext()){
            JobStep step = (JobStep)stepIterator.next();
            myCurrentStation = step.getWorkStation();
            myCurrentServiceTime = step.getServiceTime();
            if (myType == 3.0)
                JSL.out.println("Job " + myNumber +  " Going to workstation " + myCurrentStation);
            myCurrentStation.arrive(this);
        }
        else {
            double tnow = ModelElement.getTime();
            mySystemTime.setValue(tnow-myCreationTime);
        }
    }
    
    public double getJobStepServiceTime(){
        return(myCurrentServiceTime.getValue());
    }
}
