/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/


package examples.jobshopV1;

import jsl.modeling.elements.queue.*;
import jsl.modeling.elements.variable.*;
import jsl.modeling.*;

/**
 *
 */
public class WorkStation extends SchedulingElement {
    
    private Queue myQueue;
    private int myNumServers;
    private TimeWeighted myNumBusy;
    
    private EndServiceListener myEndServiceListener;
    
    /** Creates a new instance of WorkStation */
    public WorkStation(ModelElement parent, String name, int numServers) {
        super(parent, name);
        myNumServers = numServers;
        myQueue = new Queue(this, name + "Q");

        myNumBusy = new TimeWeighted(this, 0.0, name + "NB");

        myEndServiceListener = new EndServiceListener();
    }
    
    public void arrive(Job job){
        myQueue.enqueue(job);
        if (myNumBusy.getValue() < myNumServers){
            myNumBusy.increment();
            QObject c = myQueue.removeNext();
            scheduleEndService(c);
        }
    }
    
    private void scheduleEndService(QObject c) {
        Job job = (Job)c.getQueuedObject();
        double t = job.getJobStepServiceTime();
        scheduleEvent(myEndServiceListener, t,
              "Job " + job.getNumber() + " End Service at " + this.getName(), job);
    }
    
    class EndServiceListener implements ActionListenerIfc {
        public void action(JSLEvent event) {
            myNumBusy.decrement();
            
            Job job = (Job)event.getMessage();
            job.nextJobStep();
            
            if (myQueue.size() > 0 ) {
                myNumBusy.increment();
                QObject c = myQueue.removeNext();
                scheduleEndService(c);
            }
            
            
        }
    }
    
}
