/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package examples.jobshopV2;

import jsl.modeling.ActionListenerIfc;
import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.queue.Queue;
import jsl.modeling.elements.resource.Request;
import jsl.modeling.elements.resource.Resource;

public class WorkStation extends SchedulingElement {
    
    private Queue myQueue;
    private Resource myResource;
    private EndServiceListener myEndServiceListener;
    
    public WorkStation(ModelElement parent){
    	this(parent, 1, null);
    }
    
    public WorkStation(ModelElement parent, int numServers){
    	this(parent, numServers, null);
    }
    
     public WorkStation(ModelElement parent, int numServers, String name) {
        super(parent, name);
        myQueue = new Queue(this, getName() + "_Q");
        myResource = new Resource(this, numServers, getName() + "_R");
        myEndServiceListener = new EndServiceListener();
    }
    
    public void arrive(JobGenerator.Job job){
        myQueue.enqueue(job);
        if (myResource.isIdle()){
    		// at least 1 unit is available, check if the arriving customer
    		// is the next in the queue, if so start into service
    		Request nextJob = (Request)myQueue.peekNext();
    		if (job == nextJob){
    			myQueue.removeNext();
    			myResource.allocate(job);
    	        scheduleEndService(job);
    		}
   		}
    }
    
    private void scheduleEndService(JobGenerator.Job job) {
        double t = job.getTimeRequested();
        scheduleEvent(myEndServiceListener, t,
              "Job " + job.getId() + " End Service at " + this.getName(), job);
    }
    
    class EndServiceListener implements ActionListenerIfc {
        public void action(JSLEvent event) {            
        	JobGenerator.Job job = (JobGenerator.Job)event.getMessage();
        	myResource.release(job);
            if (myQueue.size() > 0 ) {// queue has a job
            	//remove the next job
            	JobGenerator.Job nextJob = (JobGenerator.Job)myQueue.removeNext(); 
				myResource.allocate(nextJob);
                scheduleEndService(nextJob);
            }
            job.doNextJobStep();
        }
    } 
}
