/*
* Created on Apr 23, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package examples.kanban;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.EventGeneratorListenerIfc;
import jsl.modeling.elements.queue.QObject;
import jsl.modeling.elements.queue.Queue;
import jsl.modeling.elements.variable.Counter;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.Constant;

/**
 * @author rossetti
 *
 */
public class CustomerGenerator extends EventGenerator implements CustomerIfc {

	protected Queue myCustQ;
	protected SupplierIfc mySupplier;
	protected Counter myThroughPut;
	
	/**
	 * @param parent
	 */
	public CustomerGenerator(ModelElement parent) {
		this(parent, null, Constant.ZERO, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param listener
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener) {
		this(parent, listener, Constant.ZERO, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param name
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, String name) {
		this(parent, listener, Constant.ZERO, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, name);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst) {
		this(parent, listener, timeUntilFirst, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext) {
		this(parent, listener, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param timeUntilFirst
	 */
	public CustomerGenerator(ModelElement parent, RandomIfc timeUntilFirst) {
		this(parent, null, timeUntilFirst, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 */
	public CustomerGenerator(ModelElement parent, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext) {
		this(parent, null, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 * @param name
	 */
	public CustomerGenerator(ModelElement parent, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, String name) {
		this(parent, null, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, name);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 * @param name
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, String name) {
		this(parent, listener, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, name);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 * @param maxNum
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, long maxNum) {
		this(parent, listener, timeUntilFirst, timeUntilNext, maxNum, Double.POSITIVE_INFINITY, null);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 * @param maxNum
	 * @param timeUntilLast
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, long maxNum, double timeUntilLast) {
		this(parent, listener, timeUntilFirst, timeUntilNext, maxNum,
				timeUntilLast, null);
	}

	/**
	 * @param parent
	 * @param listener
	 * @param timeUntilFirst
	 * @param timeUntilNext
	 * @param maxNum
	 * @param timeUntilLast
	 * @param name
	 */
	public CustomerGenerator(ModelElement parent,
			EventGeneratorListenerIfc listener, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, long maxNum, double timeUntilLast,
			String name) {
		super(parent, listener, timeUntilFirst, timeUntilNext, maxNum,
				timeUntilLast, name);
		myCustQ = new Queue(this, getName() + "CustQ");
		myThroughPut = new Counter(this, getName() +"Throughput");
		myThroughPut.setTimedUpdateInterval(1.0);
	}

	public void setSupplier(SupplierIfc s){
		mySupplier = s;
	}
	
	public boolean needsPart() {
		return (!myCustQ.isEmpty());
	}

	public void receivePart(QObject part) {
		myCustQ.removeNext();
		// collect statistics on part
		myThroughPut.increment();
		
	}
	
	protected void generate(JSLEvent e){
		QObject qo = new QObject();
		myCustQ.enqueue(qo);
		mySupplier.sendPart();
	}

}
