/*
* Created on Apr 23, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package examples.kanban;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.Exponential;

/**
 * @author rossetti
 *
 */
public class KBSystem {


	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		runModel(buildModel());
	}

	public static Model buildModel(){
		Model m = Model.createModel();
		
		RandomIfc tba = new Exponential(1.0/2.0);
		CustomerGenerator cg = new CustomerGenerator(m, tba, tba);
		LeadTimeSupplier s = new LeadTimeSupplier(m, "Supplier");
		KanbanStation kb1 = new KanbanStation(m, "KBS1");
		KanbanStation kb2 = new KanbanStation(m, "KBS2");
		KanbanStation kb3 = new KanbanStation(m, "KBS3");
		
		cg.setSupplier(kb3);
		kb3.setCustomer(cg);
		kb3.setSupplier(kb2);
		kb2.setCustomer(kb3);
		kb2.setSupplier(kb1);
		kb1.setCustomer(kb2);
		kb1.setSupplier(s);
		s.setCustomer(kb1);
		
		return(m);
	}
	
	public static void runModel1(Model m){
		Replication r = new Replication(m);
		r.setLengthOfReplication(5000.0);
		r.setLengthOfWarmUp(1000.0);
		r.turnOnBatchReport();
		r.runAll();
	}
	
	public static void runModel(Model m){
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(5000.0);
		e.setLengthOfWarmUp(1000.0);

//		e.turnOnReplicationConsoleOutput();
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
//		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}
}
