/**
 * 
 */
package examples.montecarlo;

import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.statistic.*;

/**
 * @author rossetti
 *
 */
public class LindleyEquation {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

//		replicationDeletionExample();
		
		oneLongRunExample();
		
	}
	
	public static void replicationDeletionExample(){
//		RngIfc rng = new AR1CorrelatedRng(0.8);
//		RandomIfc y = new Lognormal(1.0, 0.5, rng);
		// inter-arrival time distribution
		RandomIfc y = new Exponential(1.0);
		// service time distribution
		DistributionIfc x = new Exponential(0.7);
		int r = 30; // number of replications
		int n = 100000; // number of customers
		int d = 10000; // warm up
		AbstractStatistic avgw = new Statistic("Across rep avg waiting time");
		AbstractStatistic avgpw = new Statistic("Across rep prob of wait");
		AbstractStatistic wbar = new Statistic("Within rep avg waiting time");
		AbstractStatistic pw = new Statistic("Within rep prob of wait");
		for(int i=1;i<=r;i++){
			double w = 0; // initial waiting time
			for(int j=1;j<=n;j++){
				w = Math.max(0.0, w + x.getValue() - y.getValue());
				wbar.collect(w);// collect waiting time
				pw.collect((w>0.0)); // collect P(W>0)
				if (j == d){// clear stats at warmup
					wbar.reset();
					pw.reset();
				}
			}
			//collect across replication statistics
			avgw.collect(wbar.getAverage());
			avgpw.collect(pw.getAverage());
			// clear within replication statistics for next rep
			wbar.reset();
			pw.reset();
		}
		System.out.println("Replication/Deletion Lindley Equation Example");
		System.out.println(avgw);
		System.out.println(avgpw);
	}

	public static void oneLongRunExample(){
		// inter-arrival time distribution
		RandomIfc y = new Exponential(1.0);
		// service time distribution
		DistributionIfc x = new Exponential(0.7);
		int n = 100000; // number of customers
		int d = 10000; // warm up
		AbstractStatistic wbar = new BatchStatistic("Batch waiting time");
		AbstractStatistic wbarSTS = new StandardizedTimeSeriesStatistic("STS waiting time");
		double w = 0; // initial waiting time
		for(int j=1;j<=n;j++){
			w = Math.max(0.0, w + x.getValue() - y.getValue());
			wbar.collect(w);// collect waiting time
			wbarSTS.collect(w);
			if (j == d){// clear stats at warmup
				wbar.reset();
				wbarSTS.reset();
			}
		}
		System.out.println("One long Run Lindley Equation Example");
		System.out.println(wbar);
		System.out.println(wbarSTS);
	}

}
