/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package examples.montecarlo;

import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.statistic.Statistic;

public class TestAV {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		int m = 1000; // pairs
		double[] x = new double[m];
		double[] y = new double[m];
		
		Exponential e = new Exponential();
		// generate m variates
		for(int i= 0; i< m; i++)
			x[i]= e.getValue();
		
		// tell the generator to reset
		e.resetStartSubstream();
		// tell it to start antithetic
		e.setAntithetic(true);
		
		// generate m antithetic variates
		for(int i= 0; i< m; i++)
			y[i]= e.getValue();
		
		// compute the antithetic statistics
		// generate m variates
		Statistic av = new Statistic("AV");
		for(int i= 0; i< m; i++){
			av.collect((x[i]+y[i])/2.0);
		}
		
		// report the AV statistics
		System.out.println(av);

		// do the independent case with the same amount of data
		// compute the antithetic statistics
		// generate m variates
		Statistic crude = new Statistic("Crude");
		for(int i= 0; i< 2*m; i++){
			crude.collect(e.getValue());
		}
		
		// report the AV statistics
		System.out.println(crude);

	}

}
