package examples.practice;

import java.io.IOException;
import jsl.modeling.SchedulingElement;
import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.queue.*;
import jsl.modeling.elements.variable.*;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReplicationReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.Exponential;

public class SimpleQueue extends SchedulingElement {

	public static final int ARRIVAL = 0;
	public static final int END_OF_SERVICE = 1;
    
	Queue myQueue;

	RandomVariable myTBA;
	RandomVariable myServiceTime;

	TimeWeighted myServerState;

	ResponseVariable mySystemTime;

	public SimpleQueue(ModelElement parent){
		super(parent);
		myQueue = new Queue(this, getName() + " Queue");
		myTBA = new RandomVariable(this, new Exponential(1));
		myServiceTime = new RandomVariable(this, new Exponential(0.7));

		myServerState = new TimeWeighted(this,0.0," Server Utilization");

		mySystemTime = new ResponseVariable(this, "System Time");
	}

	protected void initialize(){
		super.initialize();
		scheduleEvent(myTBA.getValue(),ARRIVAL);
	}

	protected void handleEvent(JSLEvent event){
		switch(event.getType()){
		case ARRIVAL:
			arriveToQ(event);
			break;
		case END_OF_SERVICE:
			departQ(event);
			break;
		}
	}

	private void arriveToQ(JSLEvent event) {
		myQueue.enqueue();
		if(myServerState.getValue()==0.0){
			myServerState.setValue(1.0);
			QObject nc = myQueue.removeNext();
			scheduleEvent(myServiceTime.getValue(),END_OF_SERVICE, nc);
		}
		scheduleEvent(myTBA.getValue(), ARRIVAL);	
	}

	private void departQ(JSLEvent event) {
		myServerState.setValue(0.0);
		if(!myQueue.isEmpty()){
			myServerState.setValue(1.0);
			QObject nc = myQueue.removeNext();
			scheduleEvent(myServiceTime.getValue(),END_OF_SERVICE, nc);
		}
		
		QObject customer = (QObject)event.getMessage();
       // mySystemTime.setValue(getTime() - customer.getCreateTime());
        mySystemTime.setValue(customer.getTimeSinceCreation());
	
	}

	public static void main(String[] args) {
		Model m = Model.createModel();
		new SimpleQueue(m);

        try{
        	CSVReport rcsv = new CSVReplicationReport("RepTest");
        	CSVReport bcsv = new CSVExperimentReport("ExpTest");
        	m.addObserver(rcsv);
        	m.addObserver(bcsv);
        }
        catch(IOException e){
			System.out.println("I/O exception opening file Output.txt");
			e.printStackTrace();
        }

		Experiment e = new Experiment(m);
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(1000.0);
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
        //e.turnOnAntitheticExperimentReport();
		e.runAll();
		System.out.println("Done!");


        e.setResetStartStreamOption(true);

        e.runAll();
	}
}
