/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package examples.queueing;

import jsl.modeling.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionFunctionIfc;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.modeling.elements.resource.*;
import jsl.modeling.elements.queue.*;
import jsl.modeling.elements.processview.description.*;
import jsl.modeling.elements.processview.description.commands.*;

public class GGcProcessView extends ModelElement {
    
    private int myNumServers;
    private DistributionFunctionIfc myServiceDistribution;
    private DistributionFunctionIfc myArrivalDistribution;
    private RandomVariable myServiceRV;
    private EntityProcessGenerator myEntityProcessGenerator;
    
    public GGcProcessView(ModelElement parent) {
        this(parent, 1, new Exponential(1.0), new Exponential(0.5));
    }
    
    public GGcProcessView(ModelElement parent, int numServers, DistributionIfc ad, DistributionIfc sd) {
    	super(parent);
		if (numServers < 0)
			throw new IllegalArgumentException("Number of servers < 0");
        
		myNumServers = numServers;

		// create the arrival random variable for use by the entity generator
		if (ad == null)
			throw new IllegalArgumentException("Arrival Time Distribution was null!");

		// create the process description
		ProcessDescription pd = new ProcessDescription(this, "GGC");

		// create the model element to generate entities
		myEntityProcessGenerator = new EntityProcessGenerator(this, pd, ad, ad, "Entity Generator");

		// create the model elements used in the process description

		if (sd == null)
			throw new IllegalArgumentException("Service Time Distribution was null!");
		myServiceRV = new RandomVariable(pd, sd, "Service RV");
		       
		Variable v1 = new Variable(pd, 1.0, "amt R1");

		Resource r1 = new Resource(pd, myNumServers, "R1");

		Queue q1 = new Queue(pd, "R1 Queue");

		// create the commands used in the process description
		
		Seize s1 = new Seize(pd, v1, r1, q1,"seize resource 1");

		Delay d1 = new Delay(pd, myServiceRV, "service delay");
		
		Release rl1 = new Release(pd, r1, q1, "release resource 1");
		
		// add the commands to the process description
		// order matters when adding commands
		
		pd.addProcessCommand(s1);
		pd.addProcessCommand(d1);
		pd.addProcessCommand(rl1);
		     
    }            
}
