package examples.queueing;

import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Exponential;
import jsl.modeling.elements.queue.*;
import jsl.modeling.elements.variable.*;
import jsl.modeling.*;
import jsl.modeling.elements.resource.*;

public class TandemServers1Test extends SchedulingElement{
	private Queue myQ1;
	private Queue myQ2;
	
	private Resource myServer1;
	private Resource myServer2;
	
	private TimeWeighted NS;
	
	private ResponseVariable myTimeInSystem;
	
	private Station1ArrivalListener myStation1ArrivalListener;
	private Station2ArrivalListener myStation2ArrivalListener;
	private EndServiceListener myEndServiceListener;

	private RandomVariable myStation1Arrival;
	private RandomVariable myStation1Service;
	private RandomVariable myStation2Service;
	
	public TandemServers1Test(ModelElement parent){
		super(parent);
		myStation1Arrival = new RandomVariable(this, new Exponential(1.0));
		myStation1Service = new RandomVariable(this, new Exponential(0.7));
		myStation2Service = new RandomVariable(this, new Exponential(0.9));
		
		myQ1 = new Queue(this, "Station1 Queue");
		myQ2 = new Queue(this, "Station2 Queue");
		
		myServer1 = new Resource(this, 1, "Server 1");
		myServer2 = new Resource(this, 1, "Server 2");
		
		NS = new TimeWeighted(this, 0.0, "Number in system");
		
		myTimeInSystem = new ResponseVariable(this, 0.0, "Time in system");
		
		myStation1ArrivalListener = new Station1ArrivalListener();
		myStation2ArrivalListener = new Station2ArrivalListener();
		myEndServiceListener = new EndServiceListener();
	}
	
	public void initialize(){
		super.initialize();
		scheduleEvent(myStation1ArrivalListener, myStation1Arrival.getValue(), "customer arrives at station 1");
	}
	
	class Station1ArrivalListener implements ActionListenerIfc {
		public void action(JSLEvent event){
			NS.increment();
			Request arrivingCustomer = Request.create();
			myQ1.enqueue(arrivingCustomer);
			if(myServer1.isIdle()){
				Request nextCustomer = (Request)myQ1.peekNext();
				if(arrivingCustomer == nextCustomer){
					myQ1.removeNext();
					myServer1.allocate(arrivingCustomer);
					scheduleEvent(myStation2ArrivalListener, myStation1Service.getValue(), "customer arrives at station2", arrivingCustomer);
				}
			}
			scheduleEvent(myStation1ArrivalListener, myStation1Arrival.getValue(), "customer arrives at station 1");	
		}
	}
	
	class Station2ArrivalListener implements ActionListenerIfc {
		public void action(JSLEvent event){
			Request arrivingStation2Customer = (Request)event.getMessage();
			myServer1.release(arrivingStation2Customer);
			if(!myQ1.isEmpty()){
				Request nextCustomer = (Request)myQ1.removeNext();
				myServer1.allocate(nextCustomer);
				scheduleEvent(myStation2ArrivalListener, myStation1Service.getValue(), 
						"customer arrives at station2", nextCustomer);
			}
			arrivingStation2Customer.reinitialize();
			myQ2.enqueue(arrivingStation2Customer);
			if(myServer2.isIdle()){
				Request nextCustomer = (Request)myQ2.peekNext();
				if(arrivingStation2Customer == nextCustomer){
					myQ2.removeNext();
					myServer2.allocate(arrivingStation2Customer);
					scheduleEvent(myEndServiceListener, myStation2Service.getValue(), "Customer leaves station2", 
							arrivingStation2Customer);
				}
			}
		}
	}
	
	class EndServiceListener implements ActionListenerIfc {
		public void action(JSLEvent event){
			NS.decrement();
			Request departingCustomer = (Request)event.getMessage();
			myServer2.release(departingCustomer);
			myTimeInSystem.setValue(departingCustomer.getTimeSinceCreation());
			if(!myQ2.isEmpty()){
				Request nextCustomer = (Request)myQ2.removeNext();
				myServer2.allocate(nextCustomer);
				scheduleEvent(myEndServiceListener, myStation2Service.getValue(), "Customer leaves station2", nextCustomer);
			}
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Model m = Model.createModel();
		new TandemServers1Test(m);
		Experiment e = new Experiment(m);
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(1000);
		e.turnOnExperimentConsoleOutput();
		e.runAll();
	}
}
