package examples.queueing;

import jsl.modeling.SchedulingElement;
import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.queue.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Exponential;

public class TwoQueuesinSeries extends SchedulingElement {

	public static final int ARRIVAL = 0;
	public static final int SERVICE1END = 1;
	public static final int SERVICE2END = 2;
	Queue myQueue1;
	Queue myQueue2;
	RandomVariable myTBA;
	RandomVariable myServiceTime1;
	RandomVariable myServiceTime2;
	TimeWeighted myServer1State;
	TimeWeighted myServer2State;
	ResponseVariable mySystemTime;

	public TwoQueuesinSeries(ModelElement parent){
		super(parent);
		myQueue1 = new Queue(this, getName() + " Queue1");
		myQueue2 = new Queue(this, getName() + " Queue2");
		myTBA = new RandomVariable(this, new Exponential(1));
		myServiceTime1 = new RandomVariable(this, new Exponential(0.7));
		myServiceTime2 = new RandomVariable(this, new Exponential(0.9));
		myServer1State = new TimeWeighted(this,0.0," Server1 Utilization");
		myServer2State = new TimeWeighted(this,0.0," Server2 Utilization");
		mySystemTime = new ResponseVariable(this, "System Time");
	}

	protected void initialize(){
		super.initialize();
		scheduleEvent(myTBA.getValue(),ARRIVAL);
	}

	protected void handleEvent(JSLEvent event){
		switch(event.getType()){
		case ARRIVAL:
			arriveToQ1(event);
			break;
		case SERVICE1END:
			departQ1(event);
			break;
		case SERVICE2END:
			departQ2(event);
		}
	}

	private void arriveToQ1(JSLEvent event) {
		myQueue1.enqueue();
		if(myServer1State.getValue()==0.0){
			myServer1State.setValue(1.0);
			QObject nc = myQueue1.removeNext();
			scheduleEvent(myServiceTime1.getValue(),SERVICE1END, nc);
		}
		scheduleEvent(myTBA.getValue(), ARRIVAL);	
	}

	private void departQ1(JSLEvent event) {
		myServer1State.setValue(0.0);
		if(!myQueue1.isEmpty()){
			myServer1State.setValue(1.0);
			QObject nc = myQueue1.removeNext();
			scheduleEvent(myServiceTime1.getValue(),SERVICE1END, nc);
		}
		
		QObject customer = (QObject)event.getMessage();
		myQueue2.enqueue(customer);
		if(myServer2State.getValue()==0.0){
			myServer2State.setValue(1.0);
			QObject nc = myQueue2.removeNext();
			scheduleEvent(myServiceTime2.getValue(),SERVICE2END, nc);
		}		
	}
	
	private void departQ2(JSLEvent event) {
		QObject dc = (QObject)event.getMessage();
		mySystemTime.setValue(dc.getTimeSinceCreation());
		myServer2State.setValue(0.0);
		if(!myQueue2.isEmpty()){
			myServer2State.setValue(1.0);
			QObject nc = myQueue2.removeNext();
			scheduleEvent(myServiceTime2.getValue(),SERVICE2END, nc);
		}
	}

	public static void main(String[] args) {
		Model m = Model.createModel();
		new TwoQueuesinSeries(m);
		Experiment e = new Experiment(m);
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(1000.0);
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
		e.runAll();
		System.out.println("Done!");
	}
}
