/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package examples.queueing.testing;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;

import jsl.modeling.*;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.reporting.JSL;
import jsl.utilities.statistic.AbstractStatistic;
import jsl.utilities.statistic.Statistic;
import examples.queueing.*;

import jsl.utilities.misc.*;

/**
 */
public class TestAntitheticGGCWSModel {

	/**
	 * 
	 */
	public TestAntitheticGGCWSModel() {
		super();
	}

	public static void main(String[] args) {
		System.out.println("GGcWSModel Antithetic Test");
        
		testExperiment();
        
		System.out.println("Done!");
        
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		GGcWSModel ggc = new GGcWSModel(m);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(20);
		e.setLengthOfReplication(20000.0);
		e.setLengthOfWarmUp(5000.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnAntitheticExperimentReport();
		//e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.setSaveReplicationStatisticOption(true);
		e.setSaveExperimentStatisticsOption(true);
		// tell the experiment to run
		e.runAll();
		
/*		Iterator<Replication> i = e.getReplications();
		
		while(i.hasNext()){
			Replication r = i.next();
			List<AbstractStatistic> stats = r.getReplicationStatistics();
			for(AbstractStatistic stat:stats)
				System.out.println(stat);
		}
*/		
		List<AbstractStatistic> list = e.getAcrossReplicationStatistics();
		
		for(AbstractStatistic s: list){
			System.out.println(s);
		}
	}
}
