/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.modeling.conditions;

/**
 * This interface provides a general function that can
 * be used to check the truth of a condition.  The primary use of this interface
 * is in defining conditions to end a replication or to end the simulation.
 *
 */
public interface ConditionIfc {
    
    /** Sets the name of this condition
     * @param str The name as a string.
     */
    public void setName(String str);
    
    /** Gets this condition's name.
     * @return The name of the model element.
     */
    public String getName();
    
    /** Determines the current value for the condition
     *  This method should be overwritten to provide the logic necessary
     *  to properly determine whether the condition is true or false
     * @return True if the condition is true, false otherwise
     */
    public void evaluate();
    
    /** Checks whether the condition has been set.
     * @return True if the condition has been set, false otherwise
     */
    public boolean isSet();
    
    /** Clears the condition, i.e. ensures that isSet() returns false
     */
    public void clear();
    
    /** Sets the condition, i.e. ensures that isSet() returns true
     */
    public void set();
    
}
