/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.modeling.elements;

import java.util.ArrayList;
import java.util.List;

import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;

/** This class provide a basic implementation for the StartActionIfc
 * 
 *
 */
abstract public class AbstractAction extends SchedulingElement implements StartActionIfc {

	protected List<ActionCompletedListenerIfc> myActionListeners;

	/**
	 * @param parent
	 */
	public AbstractAction(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public AbstractAction(ModelElement parent, String name) {
		super(parent, name);
		myActionListeners = new ArrayList<ActionCompletedListenerIfc>();
	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.StartActionIfc#startAction(java.lang.Object, jsl.modeling.elements.ActionCompletedListenerIfc)
	 */
	public final void startAction(Object obj, ActionCompletedListenerIfc listener) {
		addActionCompletedListener(listener);
		startAction(obj);
	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.StartActionIfc#addActionCompletedListener(jsl.modeling.elements.ActionCompletedListenerIfc)
	 */
	public final void addActionCompletedListener(ActionCompletedListenerIfc listener) {
		if (listener == null)
		    	throw new IllegalArgumentException("Attempted to add a null listener");
			
		if(myActionListeners.contains(listener))
			return;
		
		myActionListeners.add(listener);

	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.StartActionIfc#deleteActionCompletedListener(jsl.modeling.elements.ActionCompletedListenerIfc)
	 */
	public final void deleteActionCompletedListener(ActionCompletedListenerIfc listener) {
		if (listener == null)
			throw new IllegalArgumentException("Attempted to delete a null listener");
		
		myActionListeners.remove(listener);

	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.StartActionIfc#notifyActionListeners(java.lang.Object)
	 */
	public void notifyActionListeners(Object obj) {
		
		for(ActionCompletedListenerIfc listener:myActionListeners){
			listener.actionCompleted(this, obj);
		}

	}

}
