/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.modeling.elements.component;

import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.utilities.random.distributions.Exponential;

public class SimpleComponentTest1  {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		System.out.println("Simple Component Test 1");
        
		Model m = Model.createModel();
		
		Component c1 = new Component(m, new Exponential(10.0), new Exponential(8.0), new Exponential(5.0));
		c1.setAutomaticRestartAfterOperatingFlag(true);
		c1.setAutomaticRestartAfterRepairFlag(true);
		c1.setAutomaticStartRepairAfterFailureFlag(true);
		c1.setOperateAtInitializationFlag(true);
				
		m.turnOnLogReport();
        
        // create a replication to run the model
		Replication r = new Replication(m);

		// set the running parameters of the replication
		r.setLengthOfReplication(100.0);
		
		// turn on the desired reporting
		
		r.turnOffBatchReport();
		r.setDefaultBatchingOption(false);
		// tell the replication to run
		r.runAll();
		
		System.out.println("Done!");	

	}

}
