/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.modeling.elements.operation;

import java.util.LinkedList;
import java.util.List;

import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;

public abstract class Operation extends SchedulingElement implements OperationIfc {

	protected List<OperationListenerIfc> myCompletedListeners;
	protected List<OperationListenerIfc> myStartedListeners;
	protected long myStartCount;

	/**
	 * @param parent
	 */
	public Operation(ModelElement parent) {
		this(parent, null);
	}
	
	/**
	 * @param parent
	 * @param name
	 */
	public Operation(ModelElement parent, String name) {
		super(parent, name);
		myCompletedListeners = new LinkedList<OperationListenerIfc>() ;
		myStartedListeners = new LinkedList<OperationListenerIfc>() ;
		myStartCount = 0;
	}

	public final void addStartedListener(OperationListenerIfc listener){
		if (listener == null)
	    		throw new IllegalArgumentException("Attempted to add a null listener");
		if (myStartedListeners.contains(listener))
    			throw new IllegalArgumentException("Supplied listener was already added.");
		myStartedListeners.add(listener);
	}

	public final void addCompletionListener(OperationListenerIfc listener){
		if (listener == null)
	    		throw new IllegalArgumentException("Attempted to add a null listener");
		if (myCompletedListeners.contains(listener))
			throw new IllegalArgumentException("Supplied listener was already added.");
		myCompletedListeners.add(listener);
	}

	protected final void removeStartedListener(OperationListenerIfc listener){
		if (listener == null)
	    		throw new IllegalArgumentException("Attempted to remove a null listener");
		myStartedListeners.remove(listener);
	}

	protected final void removeCompletionListener(OperationListenerIfc listener){
		if (listener == null)
	    		throw new IllegalArgumentException("Attempted to remove a null listener");
		myCompletedListeners.remove(listener);
	}
	
	public final void notifyOperationStartedListeners(Task task){
		for(OperationListenerIfc a: myStartedListeners){
			a.update(this, task);
		}
	}

	public final void notifyOperationCompletedListeners(Task task){
		for(OperationListenerIfc a: myCompletedListeners){
			a.update(this, task);
		}
	}

}
