/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.modeling.elements.operation;

import jsl.modeling.JSLEvent;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.EventGeneratorListenerIfc;

public class TaskGenerationListener implements EventGeneratorListenerIfc {

	/** This operation will be started after the task is created
	 * 
	 */
	private OperationIfc myFirstOperation;
		
	/** Creates a listener to listen for a event generation to
	 *  create tasks
	 * 
	 * @param firstOperation The operation that should be started
	 *  after the task is created
	 */
	public TaskGenerationListener(OperationIfc firstOperation) {
		super();
		setFirstOperation(firstOperation);
	}

	/** Can be overwritten by subclasses to generate different kinds
	 *  of tasks.  The default new Task() is used to create tasks
	 *  and then the supplied operation is started on that task
	 *  This method is called by an EventGenerator when an event occurs
	 */
	public void generate(EventGenerator generator, JSLEvent event) {
		
		Task task = new Task();
		myFirstOperation.start(task);
		
	}

	/** The operation that will be called when the task is created
	 * 
	 * @return
	 */
	public final OperationIfc getFirstOperation() {
		return myFirstOperation;
	}

	/** Can be used the set the operation that will be called after
	 *  the task is created.  
	 * 
	 * @param firstOperation  Must not be null
	 */
	public final void setFirstOperation(OperationIfc firstOperation) {
		if (firstOperation == null)
			throw new IllegalArgumentException("The supplied operation was null.");
		myFirstOperation = firstOperation;
	}

}
