/**
 * 
 */
package jsl.modeling.elements.variable;

import jsl.modeling.ModelElement;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 * @author rossetti
 *
 */
public class ResponseVariableAverageObserver extends Aggregate {

	/** This is used to remember the aggregate value
	 *  when any of its aggregatable's change
	 * 
	 */
	protected ResponseVariable myAggResponse;

	/**
	 * @param parent
	 */
	public ResponseVariableAverageObserver(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public ResponseVariableAverageObserver(ModelElement parent, String name) {
		super(parent, name);
		myAggResponse = new ResponseVariable(this, "Delegate Response Variable-" + getName());
		// the aggregate variable does not get initialized by the model
		// it must be initialized after all its observed variables are initialized
		myAggResponse.setInitializationOption(false);
		// the aggregate variable does not get warmed up by the model
		// it must be warmed up after all its observed variables are warmed up
		myAggResponse.setWarmUpOption(false);
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.variable.ResponseVariable#getDefaultReportingOption()
	 */
	public boolean getDefaultReportingOption() {
		return myAggResponse.getDefaultReportingOption();
	}

	/**
	 * @param flag
	 * @see jsl.modeling.elements.variable.ResponseVariable#setDefaultReportingOption(boolean)
	 */
	public void setDefaultReportingOption(boolean flag) {
		myAggResponse.setDefaultReportingOption(flag);
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.variable.PreviousValueIfc#getPreviousValue()
	 */
	public double getPreviousValue() {
		return myAggResponse.getPreviousValue();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.variable.ResponseVariable#getValue()
	 */
	public double getValue() {
		return myAggResponse.getValue();
	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.variable2.Aggregate#valueChanged(jsl.modeling.elements.variable2.Aggregatable)
	 */
	@Override
	protected void valueChangedBeforeReplication(Aggregatable variable) {
		// no need to do anything since it is before the replication	

	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.variable2.Aggregate#variableAdded(jsl.modeling.elements.variable2.Aggregatable)
	 */
	@Override
	protected void variableAddedBeforeReplication(Aggregatable variable) {
		// A new variable has been added for aggregating
		// no need to do anything since it is the value of the variable
		// when it changes that is important.  The average
		// across all presented doesn't change when a new variable is added
	}

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.variable2.Aggregate#variableRemoved(jsl.modeling.elements.variable2.Aggregatable)
	 */
	@Override
	protected void variableRemovedBeforeReplication(Aggregatable variable) {
		// A new variable has been removed from aggregating
		// no need to do anything since it is the value of the variable
		// when it changes that is important.  The average
		// across all presented doesn't change when a new variable is added
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.variable.ResponseVariable#getAcrossReplicationAverage()
	 */
	public double getAcrossReplicationAverage() {
		return myAggResponse.getAcrossReplicationAverage();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.variable.ResponseVariable#getAcrossReplicationStatistic()
	 */
	public StatisticAccessorIfc getAcrossReplicationStatistic() {
		return myAggResponse.getAcrossReplicationStatistic();
	}

	/**
	 * @param name
	 * @see jsl.modeling.elements.variable.ResponseVariable#setAcrossReplicationStatisticName(java.lang.String)
	 */
	public void setAcrossReplicationStatisticName(String name) {
		myAggResponse.setAcrossReplicationStatisticName(name);
	}

	@Override
	protected void valueChangedDuringReplication(Aggregatable variable) {
		myAggResponse.setValue(variable.getValue());		
		notifyAggregatesOfValueChange();
	}

	@Override
	protected void variableAddedDuringReplication(Aggregatable variable) {
		// A new variable has been added for aggregating
		// no need to do anything since it is the value of the variable
		// when it changes that is important.  The average
		// across all presented doesn't change when a new variable is added		
	}

	@Override
	protected void variableRemovedDuringReplication(Aggregatable variable) {
		// A new variable has been removed from aggregating
		// no need to do anything since it is the value of the variable
		// when it changes that is important.  The average
		// across all presented doesn't change when a new variable is added		
	}
	
	@Override
	protected void initializeAggregate() {
		myAggResponse.initialize_();
		myAggResponse.initialize();
	}

	@Override
	protected void warmUpAggregate() {
		myAggResponse.warmUp_();
	}
	
	protected void removedFromModel(){
		super.removedFromModel();
		myAggResponse = null;
	}
}
