/*
* Created on Sep 17, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.modeling.elements.variable.nhpp;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.EventGeneratorIfc;
import jsl.modeling.elements.EventGeneratorListenerIfc;
import jsl.utilities.random.RandomIfc;

/**
 * @author rossetti
 *
 */
public class NHPPEventGenerator extends ModelElement implements EventGeneratorIfc {

	protected EventGenerator myEventGenerator;
	
	protected NHPPTimeBtwEventRV myTBARV;
	
	/**
	 * @param parent
	 */
	public NHPPEventGenerator(ModelElement parent, InvertibleCumulativeRateFunctionIfc rateFunction, 
			EventGeneratorListenerIfc listener) {
		this(parent, rateFunction, listener, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public NHPPEventGenerator(ModelElement parent, InvertibleCumulativeRateFunctionIfc rateFunction, 
			EventGeneratorListenerIfc listener, String name) {
		super(parent, name);
		myTBARV = new NHPPTimeBtwEventRV(this, rateFunction);
		myEventGenerator = new EventGenerator(this, listener, myTBARV, myTBARV);
	}
	
	
	//TODO Vijith added this
	/**
	 * @param parent
	 * @param name
	 */
	public NHPPEventGenerator(ModelElement parent, InvertibleCumulativeRateFunctionIfc rateFunction, 
			EventGeneratorListenerIfc listener, double lastrate, String name) {
		super(parent, name);
		myTBARV = new NHPPTimeBtwEventRV(this, rateFunction, lastrate);
		myEventGenerator = new EventGenerator(this, listener, myTBARV, myTBARV);
	}
	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getEndingTime()
	 */
	public double getEndingTime() {
		return myEventGenerator.getEndingTime();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getGenerationEndingTimeForReplications()
	 */
	public double getGenerationEndingTimeForReplications() {
		return myEventGenerator.getGenerationEndingTimeForReplications();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getMaximumNumberOfEvents()
	 */
	public long getMaximumNumberOfEvents() {
		return myEventGenerator.getMaximumNumberOfEvents();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getMaximumNumberOfEventsForReplications()
	 */
	public long getMaximumNumberOfEventsForReplications() {
		return myEventGenerator.getMaximumNumberOfEventsForReplications();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getNumberOfEventsGenerated()
	 */
	public long getNumberOfEventsGenerated() {
		return myEventGenerator.getNumberOfEventsGenerated();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getTimeBetweenEventsForReplications()
	 */
	public RandomIfc getTimeBetweenEventsForReplications() {
		return myEventGenerator.getTimeBetweenEventsForReplications();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#getTimeUntilFirstEventForReplications()
	 */
	public RandomIfc getTimeUntilFirstEventForReplications() {
		return myEventGenerator.getTimeUntilFirstEventForReplications();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#isGeneratorDone()
	 */
	public boolean isGeneratorDone() {
		return myEventGenerator.isGeneratorDone();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGeneratorIfc#isSuspended()
	 */
	public boolean isSuspended() {
		return myEventGenerator.isSuspended();
	}

	/**
	 * 
	 * @see jsl.modeling.elements.EventGeneratorIfc#resume()
	 */
	public void resume() {
		myEventGenerator.resume();
	}

	/**
	 * @param endingTime
	 * @see jsl.modeling.elements.EventGeneratorIfc#setEndingTime(double)
	 */
	public void setEndingTime(double endingTime) {
		myEventGenerator.setEndingTime(endingTime);
	}

	/**
	 * @param endingTime
	 * @see jsl.modeling.elements.EventGeneratorIfc#setGenerationEndingTimeForReplications(double)
	 */
	public void setGenerationEndingTimeForReplications(double endingTime) {
		myEventGenerator.setGenerationEndingTimeForReplications(endingTime);
	}

	/**
	 * @param maxNum
	 * @see jsl.modeling.elements.EventGeneratorIfc#setMaximumNumberOfEvents(long)
	 */
	public void setMaximumNumberOfEvents(long maxNum) {
		myEventGenerator.setMaximumNumberOfEvents(maxNum);
	}

	/**
	 * @param maxNumEvents
	 * @see jsl.modeling.elements.EventGeneratorIfc#setMaximumNumberOfEventsForReplications(long)
	 */
	public void setMaximumNumberOfEventsForReplications(long maxNumEvents) {
		myEventGenerator.setMaximumNumberOfEventsForReplications(maxNumEvents);
	}

	/**
	 * @param timeBtwEvents
	 * @param maxNumEvents
	 * @see jsl.modeling.elements.EventGeneratorIfc#setTimeBetweenEvents(jsl.utilities.random.RandomIfc, long)
	 */
	public void setTimeBetweenEvents(RandomIfc timeBtwEvents, long maxNumEvents) {
		myEventGenerator.setTimeBetweenEvents(timeBtwEvents, maxNumEvents);
	}

	/**
	 * @param timeUntilNext
	 * @see jsl.modeling.elements.EventGeneratorIfc#setTimeBetweenEvents(jsl.utilities.random.RandomIfc)
	 */
	public void setTimeBetweenEvents(RandomIfc timeUntilNext) {
		myEventGenerator.setTimeBetweenEvents(timeUntilNext);
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGenerator#getTimeBetweenEvents()
	 */
	public final RandomIfc getTimeBetweenEvents() {
		return myEventGenerator.getTimeBetweenEvents();
	}

	/**
	 * @param timeBtwEvents
	 * @param maxNumEvents
	 * @see jsl.modeling.elements.EventGeneratorIfc#setTimeBetweenEventsForReplications(jsl.utilities.random.RandomIfc, long)
	 */
	public void setTimeBetweenEventsForReplications(RandomIfc timeBtwEvents, long maxNumEvents) {
		myEventGenerator.setTimeBetweenEventsForReplications(timeBtwEvents, maxNumEvents);
	}

	/**
	 * @param timeBtwEvents
	 * @see jsl.modeling.elements.EventGeneratorIfc#setTimeBetweenEventsForReplications(jsl.utilities.random.RandomIfc)
	 */
	public void setTimeBetweenEventsForReplications(RandomIfc timeBtwEvents) {
		myEventGenerator.setTimeBetweenEventsForReplications(timeBtwEvents);
	}

	/**
	 * @param timeUntilFirst
	 * @see jsl.modeling.elements.EventGeneratorIfc#setTimeUntilFirstEventForReplications(jsl.utilities.random.RandomIfc)
	 */
	public void setTimeUntilFirstEventForReplications(RandomIfc timeUntilFirst) {
		myEventGenerator.setTimeUntilFirstEventForReplications(timeUntilFirst);
	}

	/**
	 * 
	 * @see jsl.modeling.elements.EventGeneratorIfc#suspend()
	 */
	public void suspend() {
		myEventGenerator.suspend();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.EventGenerator#getStartOnInitializeFlag()
	 */
	public final boolean getStartOnInitializeFlag() {
		return myEventGenerator.getStartOnInitializeFlag();
	}

	/**
	 * @param flag
	 * @see jsl.modeling.elements.EventGenerator#setStartOnInitializeFlag(boolean)
	 */
	public final void setStartOnInitializeFlag(boolean flag) {
		myEventGenerator.setStartOnInitializeFlag(flag);
	}

	/**
	 * 
	 * @see jsl.modeling.elements.EventGenerator#turnOnGenerator()
	 */
	public final void turnOnGenerator() {
		myEventGenerator.turnOnGenerator();
	}

	/**
	 * @param t
	 * @see jsl.modeling.elements.EventGenerator#turnOnGenerator(double)
	 */
	public final void turnOnGenerator(double t) {
		myEventGenerator.turnOnGenerator(t);
	}

	/**
	 * @param r
	 * @see jsl.modeling.elements.EventGenerator#turnOnGenerator(jsl.utilities.random.RandomIfc)
	 */
	public void turnOnGenerator(RandomIfc r) {
		myEventGenerator.turnOnGenerator(r);
	}

	/**
	 * 
	 * @see jsl.modeling.elements.EventGeneratorIfc#turnOffGenerator()
	 */
	public void turnOffGenerator() {
		myEventGenerator.turnOffGenerator();
	}

}
