package jsl.modeling.elements.variable.nhpp;

public interface RateSegmentIfc {
	
	/** Returns true if the supplied time is within this
	 *  rate segments time interval
	 * 
	 * @param time
	 * @return
	 */
	public boolean contains(double time);

	/** Returns the rate for the interval
	 * 
	 * @return
	 */
	public double getRate(double time);
	
	/** The rate at the time that the time interval begins
	 * 
	 * @return
	 */
	public double getRateAtLowerTimeLimit();
	
	/** The rate at the time that the time interval ends
	 * 
	 * @return
	 */
	public double getRateAtUpperTimeLimit();

	/** The lower time limit
	 * 
	 * @return
	 */
	public double getLowerTimeLimit();

	/** The upper time limit
	 * 
	 * @return
	 */
	public double getUpperTimeLimit();

	/** The width of the interval
	 * 
	 * @return
	 */
	public double getTimeWidth();

	/** The lower limit on the cumulative rate axis
	 * 
	 * @return
	 */
	public double getCumulativeRateLowerLimit();

	/** The upper limit on the cumulative rate axis
	 * 
	 * @return
	 */
	public double getCumulativeRateUpperLimit();

	/** The cumulative rate interval width
	 * 
	 * @return
	 */
	public double getCumulativeRateIntervalWidth();

	/** Returns the value of the cumulative rate function for the interval
	 * given a value of time within that interval 
	 * 
	 * @param time
	 * @return
	 */
	public double getCumulativeRate(double time);

	/** Returns the inverse of the cumulative rate function given the interval
	 *  and a cumulative rate value within that interval.  Returns a time
	 * 
	 * @param cumRate
	 * @return
	 */
	public double getInverseCumulativeRate(double cumRate);

}