/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.modeling.utilities;

import java.util.TimerTask;

import jsl.modeling.Experiment;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.ExperimentRunner;

/**
 */
public class SimulationShowTimeTask extends TimerTask {

	protected ExperimentRunner mySimulation;
	
	public SimulationShowTimeTask(ExperimentRunner simulation){
		if (simulation == null)
			throw new IllegalArgumentException("The simulation was null");
		mySimulation = simulation;
	}
	
	/* (non-Javadoc)
	 * @see java.util.TimerTask#run()
	 */
	public void run() {
		Experiment e = mySimulation.getExperiment();
		
//		Model m = e.getModel();
		Replication r = e.getCurrentReplication();
		String s = mySimulation.getName() + " Elapsed Time " + mySimulation.getElapsedExecutionTime()/1000 + "\n"
				   + e.getName() + " Elapsed Time " + e.getElapsedExecutionTime()/1000 + "\n"
                   + " Replication " + r.getReplicationNumber() 
				   + " Elapsed Time: " + r.getElapsedExecutionTime()/1000 
		           + " seconds,  current simulated time = " + ModelElement.getTime();
		System.out.println(s);
	}

}
