/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers;

import java.util.Observable;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.spatial.AbstractMover;


import jsl.utilities.reporting.JSL;

/**
 *
 */
public class Mover2DObserver extends ModelElementObserver {

	/**
	 * 
	 */
	public Mover2DObserver() {
		this(null);
	}

	/**
	 * @param name
	 */
	public Mover2DObserver(String name) {
		super(name);
	}

	public void update(Observable observable, Object arg) {
		
		super.update(observable, arg);
				
		ModelElement m = (ModelElement) observable;
		 
		int state = m.getObserverState();
		
		if (state == AbstractMover.TRIP_STARTED) {
			if (JSL.LOGGER.isLoggable(java.util.logging.Level.FINEST))
				JSL.LOGGER.finest("Calling observer's tripStarted() method");
			tripStarted(m, arg);
		} else if (state == AbstractMover.TRIP_ENDED) {
			if (JSL.LOGGER.isLoggable(java.util.logging.Level.FINEST))
				JSL.LOGGER.finest("Calling observer's tripEnded() method");
			tripEnded(m, arg);
		} else if (state == AbstractMover.MOVE_STARTED) {
			if (JSL.LOGGER.isLoggable(java.util.logging.Level.FINEST))
				JSL.LOGGER.finest("Calling observer's moveStarted() method");
			moveStarted(m, arg);
		} else if (state == AbstractMover.MOVE_ENDED){
			if (JSL.LOGGER.isLoggable(java.util.logging.Level.FINEST))
				JSL.LOGGER.finest("Calling observer's moveEnded() method");
			moveEnded(m, arg);
		}
	}

	/**
	 * @param sm
	 * @param arg
	 */
	protected void moveEnded(ModelElement element, Object arg) {
	}

	/**
	 * @param sm
	 * @param arg
	 */
	protected void moveStarted(ModelElement element, Object arg) {
	}

	/**
	 * @param sm
	 * @param arg
	 */
	protected void tripEnded(ModelElement element, Object arg) {
	}

	/**
	 * @param sm
	 * @param arg
	 */
	protected void tripStarted(ModelElement element, Object arg) {	
	}

}
