/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.observers.database.access;

import java.sql.*;
import jsl.observers.ModelElementObserver;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 *
 */
public class AccessDBReport extends ModelElementObserver {
    
    protected Connection myCon;
    protected Statement myInsertRowStmt;
    protected ResultSet myResultSet;
    protected String myUrl = "jdbc:odbc:JSLDB";
    protected String myUserId = "jsl";
    protected String myPassword = "jsl";
    protected double[] stats;
    
    /** Creates a new instance of AccessDBReport **/
    public AccessDBReport(String name){
        super(name);
        stats = new double[21];
        
        // check for and load driver
        try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch(java.lang.ClassNotFoundException e) {
            System.err.print("ClassNotFoundException: ");
            System.err.println(e.getMessage());
        }
        
        // make the connection, statement, and resultset
        try {
            myCon = DriverManager.getConnection(myUrl, myUserId, myPassword);
            myInsertRowStmt = myCon.createStatement();
            
        } catch(SQLException ex) {
            System.out.println("\n--- SQLException caught ---\n");
            while (ex != null) {
                System.out.println("Message:   "
                + ex.getMessage());
                System.out.println("SQLState:  "
                + ex.getSQLState());
                System.out.println("ErrorCode: "
                + ex.getErrorCode());
                ex = ex.getNextException();
                System.out.println("");
            }
        }
    }
    
    protected void close(){
        try {
            //myResultSet.close();
            myInsertRowStmt.close();
            myCon.close();
        } catch(SQLException ex) {
            System.out.println("\n--- SQLException caught ---\n");
            while (ex != null) {
                System.out.println("Message:   "
                + ex.getMessage());
                System.out.println("SQLState:  "
                + ex.getSQLState());
                System.out.println("ErrorCode: "
                + ex.getErrorCode());
                ex = ex.getNextException();
                System.out.println("");
            }
        }
    }
    
    protected void makeCSVStatistic(StatisticAccessorIfc stat, StringBuilder sb){
    	stat.getStatistics(stats);
    	for(int i=0; i< stats.length; i++){
    		if (Double.isNaN(stats[i]) || Double.isInfinite(stats[i]))
    			sb.append("");
    		else
    			sb.append(stats[i]);
    		if (i < stats.length - 1)
    			sb.append(",");
    	}
    }
    
}
