/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers.scheduler;

import java.lang.String;
import java.util.Observer;
import java.util.Observable;
import java.io.IOException;

import jsl.modeling.Scheduler;
import jsl.modeling.JSLEvent;

import jsl.utilities.reporting.TextReport;
import java.text.DecimalFormat;

public class SchedulerTraceReport extends TextReport implements Observer{
    DecimalFormat df = new DecimalFormat("0.###");
    
    public SchedulerTraceReport(String name) throws IOException{
        super(name);
        addHeaderLines();
    }
    
    
    public void update(Observable subject, Object arg){
        Scheduler scheduler = (Scheduler)subject;
        
        if (scheduler.getObserverState() == Scheduler.INITIALIZED){
            addLines("Initialized scheduler.");
        }
        
        if (scheduler.getObserverState() == Scheduler.STOPPED){
        	addLines("Scheduler is done.");       	
        }
        
        if (scheduler.getObserverState() == Scheduler.EVENT_EXECUTED) {
	        JSLEvent event = (JSLEvent)arg;
	        
	        myLine.append(df.format(event.getTime()));
	        myLine.append("\t");
	        myLine.append(event.getId());
	        myLine.append("\t");
	        myLine.append(event.getName());
	        myLine.append("\t");
	        myLine.append(event.getPriority());
	        myLine.append("\t");
	        myLine.append(event.getType());
	        myLine.append("\t");
	//        myLine.append(event.getDispatcher());
	//        myLine.append("\t");
	//        myLine.append(event.getListener());
	//        myLine.append("\t");

	        addLines(myLine);
	        
	        myLine.delete(0,myLine.length());
        }
        
    }

    
}
