/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers.textfile;

import java.io.IOException;
import java.util.List;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.modeling.elements.variable.TimeWeighted;
import jsl.observers.variable.StatisticalObserver;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 *
 */
public class CSVBatchReport extends CSVReplicationReport {

	/**
	 * @param name
	 * @throws IOException
	 */
	public CSVBatchReport(String name) throws IOException {
		this(null, name);
	}

	/**
	 * @param directory
	 * @param name
	 * @throws IOException
	 */
	public CSVBatchReport(String directory, String name) throws IOException {
		super(directory, name);
	}

    protected void afterReplication(ModelElement m, Object arg){
    	
    	Model model = m.getModel();
        Replication r = model.getReplication();
		Experiment e = r.getExperiment();
        double nreps = r.getReplicationNumber();
        
        List<ResponseVariable> rvs = model.getResponseVariables();
        
        for(ResponseVariable rv: rvs){
        	if (rv.getDefaultReportingOption()){
                String type;
                if (rv instanceof TimeWeighted)
                    type = "Time-Persistent";
                else
                    type = "Observational";
                
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossBatchStatistic();
    			writeRow(stat, e, (long)rv.getId(), nreps, type);        		
        	}
       	
        }   	
 
    }
}
