/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers.textfile;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import jsl.modeling.ModelElement;
import jsl.observers.ModelElementObserver;

/**
 *
 */
public abstract class CSVReport extends ModelElementObserver {

    protected File myFile;
    protected PrintWriter myWriter;
    protected StringBuffer myLine; 
    protected int myLineWidth = 300;
    protected char quoteChar = '"';
    protected boolean myHeaderFlag;
	/**
	 * 
	 */
	public CSVReport(String name) throws IOException {
		this(null, name);
	}

	/**
	 * @param name
	 */
	public CSVReport(String directory, String name) throws IOException {
		super(name);
         if (directory == null)
        	directory = "jslOutput";
        
        File d = new File(directory);
        d.mkdir();
        
        myFile = new File(d, makeFileName(name));
        
        myWriter = new PrintWriter(new FileWriter(myFile),true);
        
        myLine = new StringBuffer(myLineWidth);
	}
	
    public void close(){
        myWriter.close();
    }
        
    private String makeFileName(String name){
        //construct filename to ensure .txt
        String s;
        int dot = name.lastIndexOf(".");
        
        if ( dot == -1 ) // no period found
            s = name + ".csv";
        else // period found
            s = name.substring(dot) + "csv";
        
        return(s);
    }

	protected void writeHeader() {
		if (myHeaderFlag == true)
			return;
		myHeaderFlag = true;
		StringBuilder sb = new StringBuilder();
		sb.append("\"ResponseID\",");
		sb.append("ExpNum,");
		sb.append("ExpName,");
		sb.append("RepNum,");
		sb.append("Name,");
		sb.append("StatType,");
		sb.append("Count,");
		sb.append("Average,");
		sb.append("Standard Deviation,");
		sb.append("Standard Error,");
		sb.append("Half-width,");     
		sb.append("Confidence Coefficient,");  
		sb.append("Minimum,");
		sb.append("Maximum,");
		sb.append("Sum,");
		sb.append("Variance,");
		sb.append("Weighted Average,");
		sb.append("Weighted Sum,");
		sb.append("Sum of Weights,");
		sb.append("Weighted Sum of Squares,");
		sb.append("Deviation Sum of Squares,");
		sb.append("Last value collected,");
		sb.append("Last weighted collected,");
		sb.append("Kurtosis,");
		sb.append("Skewness,");
		sb.append("Lag 1 Covariance,");
		sb.append("Lag 1 Correlation,");
		sb.append("Von Neumann Lag 1 Test Statistic,");
		sb.append("Number of missing observations");
		myWriter.println(sb.toString());
	}

	protected void beforeExperiment(ModelElement m, Object arg) {
	    // write header
	    writeHeader();
	}
	
	protected void finalize(){
		close();
	}
}
