/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers.textfile;

import java.util.Collection;
import java.util.List;
import java.util.Observer;
import java.io.IOException;

import jsl.modeling.Model;
import jsl.modeling.elements.variable.*;
import jsl.utilities.statistic.StatisticAccessorIfc;
import jsl.utilities.reporting.TextReport;
import jsl.observers.variable.CounterObserver;
import jsl.observers.variable.StatisticalObserver;

public class MSummaryReport extends TextReport implements Observer {
    
    //  ===========================================
    //      CLASS AND OBJECT ATTRIBUTES
    //  ===========================================
    
    protected Collection<ResponseVariable> myResponseVariables;
    protected Model myModel;
    
    //  ===========================================
    //        CONSTRUCTORS
    //  ===========================================

    public MSummaryReport(String name) throws IOException{
        this(null, name);
    }
    
    public MSummaryReport(String directory, String name) throws IOException{
        super(directory, name);
    }
    
    public void update(java.util.Observable observable, java.lang.Object obj) {

        myModel = (Model)observable;
        
        if (myModel.checkForAfterExperiment()){

			skipLine();
			skipLine();
			addLines("----------------------------------------------------------");
			addLines("Across Replication statistics");
			addLines("----------------------------------------------------------");
			skipLine();
			
			List<ResponseVariable> rvs = myModel.getResponseVariables();
			for(ResponseVariable rv: rvs){
				if (rv.getDefaultReportingOption()){
					StatisticalObserver so = rv.getStatisticalObserver();
					StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
					addLines(stat);					
				}
			}	

			skipLine();
			skipLine();
			addLines("----------------------------------------------------------");
			addLines("Counter statistics:");
			addLines("----------------------------------------------------------");
			skipLine();
			
			List<Counter> counters = myModel.getCounters();
			
			for(Counter c: counters){
				if (c.getDefaultReportingOption()){
					CounterObserver co = c.getCounterObserver();
					StatisticAccessorIfc stat = co.getAcrossReplicationStatistic();
					addLines(stat);
					stat = co.getAcrossRepTimedUpdateStatistic();
					if (stat != null)
						addLines(stat);
				}
			}
        }
    }
    
}
