/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.observers.textfile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Observer;

import jsl.observers.ModelElementObserver;
import jsl.utilities.reporting.TextReport;

abstract public class StatisticalReport extends TextReport implements Observer {

	protected List<ModelElementObserver> myModelElementObservers;
	
	// ===========================================
	// CONSTRUCTORS
	// ===========================================

	public StatisticalReport(String name) throws IOException {
		this(null, name);
	}

	public StatisticalReport(String directory, String name) throws IOException {
		super(directory, name);
		myModelElementObservers = new ArrayList<ModelElementObserver>();
	}

	/* (non-Javadoc)
	 * @see java.util.List#add(E)
	 */
	public boolean addModelElementObserver(ModelElementObserver arg0) {
		return myModelElementObservers.add(arg0);
	}

	/* (non-Javadoc)
	 * @see java.util.List#contains(java.lang.Object)
	 */
	public boolean containsModelElementObserver(Object arg0) {
		return myModelElementObservers.contains(arg0);
	}

	/* (non-Javadoc)
	 * @see java.util.List#remove(java.lang.Object)
	 */
	public boolean removeModelElementObserver(Object arg0) {
		return myModelElementObservers.remove(arg0);
	}

	protected void writeOutModelElementObservers(){

		skipLine();
		skipLine();
		addLines("----------------------------------------------------------");
		addLines("User Defined ModelElementObservers:");
		addLines("----------------------------------------------------------");
		skipLine();

		for(ModelElementObserver so: myModelElementObservers){
			addLines(so.toString());
		}
	}
}
