/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.observers.variable;

import jsl.utilities.statistic.*;
import jsl.modeling.*;
import jsl.modeling.elements.variable.ResponseVariable;

/**
 *
 */
public class TimeWeightedStatisticalObserver extends StatisticalObserver {
    
    protected Statistic myWithinBatchStat;
        
    /** Creates a new instance of ResponseVariableObserver */
    public TimeWeightedStatisticalObserver() {
        this(null);
    }
    
    /** Creates a new instance of ResponseVariableObserver */
    public TimeWeightedStatisticalObserver(String name) {
        super(name);
    }
    
    public void resetStatistics(){
        super.resetStatistics();
        if (myWithinBatchStat != null)
            myWithinBatchStat.reset();
    }
    
    protected void initialize(ModelElement m, Object arg){
         
        if (myResponseVariable.isBatchingOn()){ // if batching option is on, create the statistic for collecting on the batch means
            // TimeWeighted uses a statistic to collect between batching events
            if (myWithinBatchStat == null)
            		myWithinBatchStat = new Statistic("WithinBatch Stat " + getName());
        }
        super.initialize(m, arg);
  }
    
    protected void update(ModelElement m, Object arg){
		myResponseVariable = (ResponseVariable) m;
        
        double weight = myResponseVariable.getWeight();
        double prev = myResponseVariable.getPreviousValue();
        myWithinRepStat.collect(prev, weight);
        if (myWithinBatchStat != null)
            myWithinBatchStat.collect(prev, weight);
        
    }
    
    protected void batch(ModelElement m, Object arg){
        // record the previous batches average
        if(myBatchStatistic != null)
            myBatchStatistic.collect(myWithinBatchStat.getWeightedAverage());
        // reset the within batch collector
        if (myWithinBatchStat != null)
            myWithinBatchStat.reset();
    }
    
	protected void removedFromModel(ModelElement m, Object arg) {
		super.removedFromModel(m, arg);
		myWithinBatchStat = null;
	}

}
