/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.observers.variable;

import java.io.IOException;
import java.util.Observer;

import jsl.modeling.elements.variable.Variable;
import jsl.utilities.reporting.TextReport;
import jsl.modeling.*;

/**
 *
 */
public class VariableTraceTextReport extends TextReport implements Observer {
    
	protected long myCount = 0;
	protected long myRepCount = 0;
	protected double myRepNum = 0;
	
    /** Creates new ResponseVariableTrace */
    public VariableTraceTextReport() throws IOException {
        super(null);
    }
    
    public VariableTraceTextReport(String name) throws IOException{
        super(name);
    }
    
    public void update(java.util.Observable observable, java.lang.Object obj) {
        Variable v = (Variable)observable;
        Model m = v.getModel();

        if (v.checkForUpdate()){
        	myCount++;
            myLine.append(myCount);
            myLine.append("\t");       	
            myLine.append(v.getTimeOfChange());
            myLine.append("\t");
            myLine.append(v.getValue());
            myLine.append("\t");
            myLine.append(v.getPreviousTimeOfChange());
            myLine.append("\t");
            myLine.append(v.getPreviousValue());
            myLine.append("\t");
            myLine.append(v.getWeight());
            myLine.append("\t");
            Replication r = m.getReplication();
            if (r != null){
            	if (myRepNum != r.getReplicationNumber()){
            		myRepCount = 0;
            	}
            	myRepCount++;
            	myRepNum = r.getReplicationNumber();
            	myLine.append(myRepNum);
            	myLine.append("\t");
            	myLine.append(myRepCount);
            	myLine.append("\t");
            	Experiment e = r.getExperiment();
            	if (e != null){
                	myLine.append(e.getName());
                	myLine.append("\t");           		
            	}
            }
            
            addLines(myLine);
            
            myLine.delete(0,myLine.length());
        }
    }
}
