/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.utilities.random.rng;

import jslx.utilities.random.ar.AR1Normal;
import jsl.utilities.random.distributions.Normal;
import jsl.utilities.random.rng.RngIfc;
import jsl.utilities.random.rng.RngStream;

/**
 *
 */
public class AR1CorrelatedRng implements RngIfc {

	private AR1Normal myAR1;

	/**
	 * 
	 */
	public AR1CorrelatedRng() {
		this(0.0, new RngStream());
	}

	/**
	 * 
	 */
	public AR1CorrelatedRng(double correlation) {
		this(correlation, new RngStream());
	}
	
	/**
	 * 
	 */
	public AR1CorrelatedRng(double correlation, RngIfc rng) {
		myAR1 = new AR1Normal(0.0, 1.0, correlation, rng);
	}
	
	public double randU01() {
		// generate the correlated normal
		double z = myAR1.getValue();
		// invert to get the correlated uniforms
		double u = Normal.stdNormalCDF(z);
		return u;
	}
	
   /** Returns a (pseudo)random number from the discrete uniform distribution
    * over the integers {i, i + 1, . . . , j }, using this stream. Calls randU01 once.
    * @param i start of range
    * @param j end of range
    * @return The integer pseudo random number
    */   
   public final int randInt (int i, int j)  {
      return (i + (int)(randU01() * (j - i + 1)));
   } 
	   
	public final void resetNextSubstream() {
		myAR1.resetNextSubstream();
	}
	
	public final void resetStartStream() {
		myAR1.resetStartStream();
	}
	
	public final void resetStartSubstream() {
		myAR1.resetStartSubstream();
	}
	
	public final void setAntithetic(boolean flag) {
		myAR1.setAntithetic(flag);
	}
	
	public final void setLag1Correlation(double phi) {
		myAR1.setLag1Correlation(phi);
	}
	
	public final double getLag1Correlation() {
		return myAR1.getLag1Correlation();
	}
}
