/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 *
 * Copyright (c) Manuel D. Rossetti (rossetti@uark.edu)
 *
 * Contact:
 *	Manuel D. Rossetti, Ph.D., P.E.
 *	Department of Industrial Engineering
 *	University of Arkansas
 *	4207 Bell Engineering Center
 *	Fayetteville, AR 72701
 *	Phone: (479) 575-6756
 *	Email: rossetti@uark.edu
 *	Web: www.uark.edu/~rossetti
 *
 * This file is part of the JSL (a Java Simulation Library). The JSL is a framework
 * of Java classes that permit the easy development and execution of discrete event
 * simulation programs.
 *
 * The JSL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The JSL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the JSL (see file COPYING in the distribution);
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA, or see www.fsf.org
 *
 */
package jsl.utilities.random.rng;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/** A wrapper for holding a list of streams so that
 *  all streams can be managed together
 *
 *  The methods of the RandomStreamIfc are applied
 *  to all contained random number streams
 *
 * @author rossetti
 */
public class RNGStreamManager implements RandomStreamManagerIfc {

    /** Holds the streams
     */
    protected List<RngIfc> myStreams;

    public RNGStreamManager() {
        myStreams = new ArrayList<RngIfc>();
    }

    public void resetAllStartStreams() {
        for (RngIfc r : myStreams)
            r.resetStartStream();
    }

    public void resetAllStartSubstreams() {
        for (RngIfc r : myStreams)
            r.resetStartSubstream();
    }

    public void resetAllNextSubstreams() {
        for (RngIfc r : myStreams)
            r.resetNextSubstream();
    }

    public void setAllAntithetic(boolean flag) {
        for (RngIfc r : myStreams)
            r.setAntithetic(flag);
    }

    public int size() {
        return myStreams.size();
    }

    public RngIfc set(int index, RngIfc element) {
        return myStreams.set(index, element);
    }

    public RngIfc remove(int index) {
        return myStreams.remove(index);
    }

    public boolean remove(RngIfc o) {
        return myStreams.remove(o);
    }

    public Iterator<RngIfc> iterator() {
        return myStreams.iterator();
    }

    public boolean isEmpty() {
        return myStreams.isEmpty();
    }

    public int indexOf(Object o) {
        return myStreams.indexOf(o);
    }

    public RngIfc get(int index) {
        return myStreams.get(index);
    }

    public boolean contains(RngIfc o) {
        return myStreams.contains(o);
    }

    public void clear() {
        myStreams.clear();
    }

    public void add(int index, RngIfc element) {
        myStreams.add(index, element);
    }

    public boolean add(RngIfc e) {
        return myStreams.add(e);
    }
}
