/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.utilities.reporting;

import java.io.*;
import java.util.logging.*;

public class JSL {
 
	/** Used to assign unique enum constants 
	 */
	private static int myEnumCounter_;

	/** for logging
	 */
	public static Logger LOGGER = Logger.getLogger(JSL.class.getName());

	public static Logger OUTPUT = Logger.getLogger("jsl.output");

    /** Can be used like System.out, but instead writes to a file
     *  jslOutput.txt found in the jslOutput directory
     *
     */
	public static LogPrintWriter out;

    /** Used by the loggers
     *
     */
	public static final int LOG_ROTATION_COUNT = 3;

	private static FileHandler myFileHandler;

    private static String workingDirectory;
    
    static {
 
        try{
    		File d1 = new File("jslOutput");
    		d1.mkdir();
    		out = new LogPrintWriter(new FileWriter(new File(d1,"jslOutput.txt")),true);
        }
        catch(IOException e){
			LOGGER.severe("I/O exception opening file jslOutput/jslOutput.txt");
			e.printStackTrace();
        }
        
    	File d = new File("jslMessages");
    	d.mkdir();
    	
        try{
            workingDirectory = System.getProperty("user.dir");
            //System.out.println(workingDirectory);
        }
        catch (SecurityException noAccess){
        		LOGGER.severe("Access to system information denied");
            noAccess.printStackTrace();
        }
                        
		try {
			File f = new File(d,"JSLLog.txt");
			myFileHandler = new FileHandler(f.getAbsolutePath());

//			myFileHandler.setFormatter(new SimpleFormatter());
			myFileHandler.setFormatter(new LogFormatter());
			// Send logger output to our FileHandler.
			LOGGER.addHandler(myFileHandler);
			// Request that nothing gets logged by default.
			LOGGER.setLevel(Level.OFF);
		} catch (IOException e) {
			LOGGER.severe("Error setting the FileHandler for IterativeProcess.");
			e.printStackTrace();
		}

		try {
			File f = new File(d,"outputLog");
			myFileHandler = new FileHandler(f.getAbsolutePath()+".log",0,LOG_ROTATION_COUNT);

//			myFileHandler.setFormatter(new SimpleFormatter());
			myFileHandler.setFormatter(new LogFormatter());
			// Send logger output to our FileHandler.
			OUTPUT.addHandler(myFileHandler);
			// Request that nothing gets logged by default.
			OUTPUT.setLevel(Level.OFF);
		} catch (IOException e) {
			LOGGER.severe("Error setting the FileHandler for IterativeProcess.");
			e.printStackTrace();
		}

    }
       
	/**
	 * @return Returns the workingDirectory.
	 */
	public static final String getWorkingDirectory() {
		return workingDirectory;
	}
	
	/** Should be used by subclasses to get the next observer constant
	 *  so that unique constants can be used to represent observer states
	 * 
	 * @return
	 */
	public final static int getNextEnumConstant(){
		return(++myEnumCounter_);
	}
}
