/*
* Created on Apr 18, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.utilities.reporting;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.Locale;

/**
 * @author rossetti
 *
 */
public class LogPrintWriter extends PrintWriter {

	public boolean OUTPUT_ON = true;
		
	/**
	 * @param out
	 */
	public LogPrintWriter(Writer out) {
		super(out);
	}

	/**
	 * @param out
	 */
	public LogPrintWriter(OutputStream out) {
		super(out);
	}

	/**
	 * @param fileName
	 * @throws FileNotFoundException
	 */
	public LogPrintWriter(String fileName) throws FileNotFoundException {
		super(fileName);
	}

	/**
	 * @param file
	 * @throws FileNotFoundException
	 */
	public LogPrintWriter(File file) throws FileNotFoundException {
		super(file);
	}

	/**
	 * @param out
	 * @param autoFlush
	 */
	public LogPrintWriter(Writer out, boolean autoFlush) {
		super(out, autoFlush);
	}

	/**
	 * @param out
	 * @param autoFlush
	 */
	public LogPrintWriter(OutputStream out, boolean autoFlush) {
		super(out, autoFlush);
	}

	/**
	 * @param fileName
	 * @param csn
	 * @throws FileNotFoundException
	 * @throws UnsupportedEncodingException
	 */
	public LogPrintWriter(String fileName, String csn)
			throws FileNotFoundException, UnsupportedEncodingException {
		super(fileName, csn);
	}

	/**
	 * @param file
	 * @param csn
	 * @throws FileNotFoundException
	 * @throws UnsupportedEncodingException
	 */
	public LogPrintWriter(File file, String csn) throws FileNotFoundException,
			UnsupportedEncodingException {
		super(file, csn);
	}

	public void println(String x){
		if (OUTPUT_ON)
			super.println(x);
	}

	public PrintWriter append(char c) {
		if (OUTPUT_ON)
			return (super.append(c));
		else
			return (this);
	}

	public PrintWriter append(CharSequence csq, int start, int end) {
		if (OUTPUT_ON)
			return(super.append(csq, start, end));
		else
			return (this);
	}

	public PrintWriter append(CharSequence csq) {
		if (OUTPUT_ON)
			return(super.append(csq));
		else
			return(this);
	}

	public void print(boolean b) {
		if (OUTPUT_ON)
			super.print(b);
	}

	public void print(char c) {
		if (OUTPUT_ON)
			super.print(c);
	}

	public void print(char[] s) {
		if (OUTPUT_ON)
			super.print(s);
	}

	public void print(double d) {
		if (OUTPUT_ON)
			super.print(d);
	}

	public void print(float f) {
		if (OUTPUT_ON)
			super.print(f);
	}

	public void print(int i) {
		if (OUTPUT_ON)
			super.print(i);
	}

	public void print(long l) {
		if (OUTPUT_ON)
			super.print(l);
	}

	public void print(Object obj) {
		if (OUTPUT_ON)
			super.print(obj);
	}

	public void print(String s) {
		if (OUTPUT_ON)
			super.print(s);
	}

	public PrintWriter printf(Locale l, String format, Object... args) {
		if (OUTPUT_ON)
			return super.printf(l, format, args);
		else
			return(this);
	}

	public PrintWriter printf(String format, Object... args) {
		if (OUTPUT_ON)
			return super.printf(format, args);
		else
			return this;
	}

	public void println() {
		if (OUTPUT_ON)
			super.println();
	}

	public void println(boolean x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(char x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(char[] x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(double x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(float x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(int x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(long x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void println(Object x) {
		if (OUTPUT_ON)
			super.println(x);
	}

	public void write(char[] buf, int off, int len) {
		if (OUTPUT_ON)
			super.write(buf, off, len);
	}

	public void write(char[] buf) {
		if (OUTPUT_ON)
			super.write(buf);
	}

	public void write(int c) {
		if (OUTPUT_ON)
			super.write(c);
	}

	public void write(String s, int off, int len) {
		if (OUTPUT_ON)
			super.write(s, off, len);
	}

	public void write(String s) {
		if (OUTPUT_ON)
			super.write(s);
	}
}
