/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jsl.utilities.reporting;

import java.io.*;
import java.util.Date;

public class TextReport extends Report{
    
    private int myLineWidth = 120;
    private File myFile;
    protected PrintWriter myPrintWriter;
    protected StringBuilder myLine;
    
    public TextReport(String name) throws IOException {
    	this(null, name);
    }
    
    public TextReport(String directory, String name) throws IOException {
        super(name);
        if (directory == null)
        	directory = "jslOutput";
        
        File d = new File(directory);
        d.mkdir();
        
        myFile = new File(d, makeFileName(name));
        
        myPrintWriter = new PrintWriter(new FileWriter(myFile),true);
        
        myLine = new StringBuilder(myLineWidth);
    }
    
    public void setLineWidth(int lineWidth){
        myLineWidth = lineWidth;
        myLine.setLength(myLineWidth);
    }
    
    public int getLineWidth(){
        return(myLineWidth);
    }
    
    public void close(){
        myPrintWriter.close();
    }
    
    public void addHeaderLines(){
        myPrintWriter.println(getName());
        myPrintWriter.println(new Date());
        myPrintWriter.println();
    }
    
    public void addLines(Object o){
        myPrintWriter.println(o.toString());
    }
    
    public void skipLine(){
        myPrintWriter.println();
    }
    
    private String makeFileName(String name){
        //construct filename to ensure .txt
        String s;
        int dot = name.lastIndexOf(".");
        
        if ( dot == -1 ) // no period found
            s = name + ".txt";
        else // period found
            s = name.substring(dot) + "txt";
        
        return(s);
    }
    
}
