/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jsl.utilities.statistic;

/**
 * @author rossetti
 *
 */
public class WeightedAverage extends AbstractCollector {

	/** Holds the number of observations observed
	 */
	protected double num = 0.0;
	
	/** Holds the weighted sum of the data.
	 */
	protected double wsum = 0.0;
	
	/** Holds the sum of the weights observed.
	 */
	protected double sumw = 0.0;

	/**
	 * 
	 */
	public WeightedAverage() {
		this(null);
	}

	/**
	 * @param name
	 */
	public WeightedAverage(String name) {
		super(name);
	}

	/* (non-Javadoc)
	 * @see jsl.utilities.statistic.AbstractCollector#collect(double, double)
	 */
	public void collect(double x, double weight) {
		// update moments
		num = num + 1;
		sumw = sumw + weight;
		wsum = wsum + x*weight;
	}

	/* (non-Javadoc)
	 * @see jsl.utilities.statistic.AbstractCollector#reset()
	 */
	public void reset() {
		num = 0.0;
		wsum = 0.0;
		sumw = 0.0;
	}
	
	/** Gets the weighted average of the collected observations.
	 * @return A double representing the weighted average or Double.NaN if no observations.
	 */
	public final double getWeightedAverage(){
		if (sumw <= 0.0)
			return Double.NaN;
		return(wsum/sumw);
	}

	/** Gets the count of the number of the observations.
	 * @return A double representing the count
	 */
	public final double getCount(){
		return (num);
	}
		
	/** Gets the weighted sum of observations observed.
	 * @return A double representing the weighted sum
	 */
	public final double getWeightedSum(){
		return (wsum);
	}

	/** Gets the sum of the observed weights.
	 * @return A double representing the sum of the weights
	 */
	public final double getSumOfWeights(){
		return(sumw);
	}

}
