/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.component.Component;

public class ComponentBasedAircraft extends Aircraft {
	
	public ComponentBasedAircraft(ModelElement parent, AirBase airfield) {
		this(parent, airfield, null);
	}

	public ComponentBasedAircraft(ModelElement parent, AirBase airfield, String name) {
		super(parent, airfield, name);
	}

	protected void beginSortie(){		
		if (!isFailed()){
			System.out.println("t> " + getTime() + " " + getName() + " Sortie started.");
			double time = mySortieTimeRV.getValue();
			scheduleEvent(mySortieCompletionAction, time);	
			for(Component c: myComponents){
				if (c.isAvailable()){
					c.startOperation(time);
				}				
			}
		}
	}

	protected void setRepairedIndicator(){
		for(Component c: myComponents){
			if (c.isInRepair()){
				c.endRepair();
			}				
		}
		myStatus.setValue(1.0);
		
	}

	protected void sortieCompletionAction(JSLEvent event){
		System.out.println("t> " + getTime() + " " + getName() + " Sortie completed.");

		// the plane has completed its sortie/mission
		// indicate its failure status
		boolean failed = false;
		for(Component c: myComponents){
			if (c.isFailed()){
				failed = true;
				c.startRepair();
			}				
		}
		
		if (failed)
			myStatus.setValue(0);
		else
			myStatus.setValue(1);
		
		// ask the airfield to service the aircraft			
		myAirbase.serviceAircraft(this);
		
	}

}
