/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.spatial.RectangularGridSpatialModel2D;
import jsl.modeling.elements.spatial.ResourceLocation;
import jsl.modeling.elements.spatial.SpatialModel;
import jsl.modeling.elements.spatial.SpatialResource;
import jslx.adc.adcsystem.ADCSystem;

public class ReaderDemo extends ModelElement {
protected SpatialModel myAirField;
	
	protected AirBase myAirBase;
	
	protected Aircraft myPlane1;

	
	
	public ReaderDemo(ModelElement parent) {
		this(parent, null);
	}

	public ReaderDemo(ModelElement parent, String name) {
		super(parent, name);
		
		// create the specfications of the spatial representation for the airfield
		myAirField = new RectangularGridSpatialModel2D();
		
		// create the spatial elements used in the spatial model
		// and set their spatial attributes
		
		// create the airbase
		myAirBase= new AirBase(this, myAirField);
				
		// create the planes and attach them to the airbase
		myPlane1 = new Aircraft(this, myAirBase, "Plane 1");
		new Aircraft(this, myAirBase, "Plane 2");
		
		// create the repair stations and add them to the airbase
		SpatialResource rs1 = myAirBase.addRepairStation("Repair Station 1", 100,100);
		SpatialResource rs2 = myAirBase.addRepairStation("Repair Station 2", 100,200);

		

		// create the resources and add them to the airbase 
		
		Worker w = myAirBase.addWorker("Worker 1", 100,50);
		Worker w2 = myAirBase.addWorker("Worker 2",200,50);
		ResourceLocation r1 = myAirBase.addEquipmentLocation(300,100);
		ResourceLocation r2 = myAirBase.addEquipmentLocation(300,200);
		SpatialResource e = myAirBase.addEquipment("Power Unit 1", r1);
		SpatialResource e2 = myAirBase.addEquipment("Power Unit 2", r2);
		
		
		// create the ADC system
		ADCSystem n = myAirBase.addNetwork(myAirField);
		n.addReader("Repair Station 1",rs1);
		n.addReader("Repair Station 2",rs2);
		n.addReader("Worker Rest Location 1", w.getResourceLocation());
		n.addReader("Worker Rest Location 2", w2.getResourceLocation());
		n.addReader("Equipment Location 1", r1);
		n.addReader("Equipment Location 1", r2);
		n.addRFIDTag(w);
		n.addRFIDTag(w2);
		n.addRFIDTag(e);
		n.addRFIDTag(e2);


	}
}
