/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.operation.Operation;
import jsl.modeling.elements.operation.Task;
import jsl.modeling.elements.resource.Request;
import jsl.modeling.elements.resource.Resource;
import jsl.modeling.elements.resource.ResourceAllocationListenerIfc;
import jsl.modeling.elements.resource.ResourceProvider;

public class RepairOperation extends Operation {

	private WorkerAllocationListener myWorkerAllocationListener = new WorkerAllocationListener();
	
	public RepairOperation(ModelElement parent) {
		this(parent, null);
	}

	public RepairOperation(ModelElement parent, String name) {
		super(parent, name);
	}

	public void start(Task task) {
		
		RepairTask rtask = (RepairTask)task;
		
		AirBase base = rtask.getAirBase();
		
		ResourceProvider rp = base.getWorkerProvider();

		rtask.setRepairOperation(this);
		
		System.out.println("t> " + getTime() + " Seizing a worker");
		rp.seize(1, myWorkerAllocationListener, rtask);

	}

	// ===========================================
	// PRIVATE CLASSES
	// ===========================================

	private class WorkerAllocationListener implements ResourceAllocationListenerIfc {

		public void allocated(Request request, Resource resource) {
			// a worker has been allocated to a repair task	
			System.out.println("t> " + getTime() + " Worker was allocated to repair operation task");
			// get the repair task
			RepairTask rtask = (RepairTask)request.getEntity();
			
			Worker worker = (Worker)resource;
			
			// remember the seized worker
			rtask.setWorker(worker);
			
			// make the worker do something
			// the worker must eventually tell the repair operation to notify its listeners
			System.out.println("t> " + getTime() + " Telling worker to perform task");
			worker.performTask(rtask);
		}	
	}

}
