/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc;

import jsl.modeling.elements.operation.Task;
import jsl.modeling.elements.spatial.SpatialResource;


public class RepairTask extends Task {

	private SpatialResource myRepairStation;
	private RepairOperation myRepairOperation;
	private Aircraft myAircraft;
	private AirBase myAirBase;
	private Worker myWorker;
	
	// can add worker and equipment references here
	
	public RepairTask() {
		this(null);
	}

	public RepairTask(String name) {
		super(name);
	}

	public final Aircraft getAircraft() {
		return myAircraft;
	}

	public final void setAircraft(Aircraft aircraft) {
		myAircraft = aircraft;
	}

	public final SpatialResource getRepairStation() {
		return myRepairStation;
	}

	public final void setRepairStation(SpatialResource repairStation) {
		myRepairStation = repairStation;
	}

	public final AirBase getAirBase() {
		return myAirBase;
	}

	public final void setAirBase(AirBase airBase) {
		myAirBase = airBase;
	}
	
	/**
	 * @return Returns the myWorker.
	 */
	public final Worker getWorker() {
		return myWorker;
	}
	/**
	 * @param myWorker The myWorker to set.
	 */
	public final void setWorker(Worker worker) {
		myWorker = worker;
	}
	
	/**
	 * @return Returns the myRepairOperation.
	 */
	public final RepairOperation getRepairOperation() {
		return myRepairOperation;
	}
	/**
	 * @param myRepairOperation The myRepairOperation to set.
	 */
	public final void setRepairOperation(RepairOperation repairOperation) {
		myRepairOperation = repairOperation;
	}

}
