/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc.adcsystem;
/**
 *
 * This instance of a Range function depicts if a reader is placed against a metal wall.  An orientation is 
 * required in order to use this function.  The possible wall orientations are North, South, East, and West.  The 
 * isTagInRange function works as the circular range function, except the tag cannot be read if it is on the other
 * side of the wall.
 * 
 */
public class WalledRangeFunction implements RangeFunctionIfc{
	
	private RFIDReader myReader;
	
	private String myWallOrientation;
	
	public static final double DEFAULT_RANGE = 5.0;
	
	private double myDefaultRange = DEFAULT_RANGE;
	
	protected void setOrientation(String s){
		if (s == "North"){
			myWallOrientation = "North";
		}else if(s == "South"){
			myWallOrientation = "South";
		}else if(s == "West"){
			myWallOrientation = "West";
		}else if(s == "East"){
			myWallOrientation = "East";
		}else{
			throw new IllegalArgumentException("You must enter either North, South, East, or West!");
		}
	}
	public boolean isTagInRange(RFIDTag tag){
		double xDistance = Math.abs(myReader.getX1() - tag.getX1());
		double yDistance = Math.abs(myReader.getX2() - tag.getX2());
		if(Math.sqrt((xDistance * xDistance) + (yDistance * yDistance))<= myDefaultRange){
			if (myWallOrientation == "North" && tag.getX2()>myReader.getX2()){
					return(false);
			}else if (myWallOrientation == "South" && tag.getX2()<myReader.getX2()){
					return(false);
			}else if (myWallOrientation == "East" && tag.getX1()>myReader.getX1()){
					return(false);
			}else if (myWallOrientation == "West" && tag.getX1()<myReader.getX1()){
					return(false);
			}else
				return(true);
		}	
		else
			return(false);
	}
}