/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.adc.hmsystem;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.component.Component;
import jsl.modeling.elements.variable.RandomVariable;
import jsl.utilities.random.distributions.Bernoulli;

public class BernoulliDetection extends ModelElement implements DetectionModelIfc {

	protected RandomVariable myIndicator;
	
	public BernoulliDetection(ModelElement parent, double prob) {
		this(parent, null, prob);
	}

	public BernoulliDetection(ModelElement parent, String name, double prob) {
		super(parent, name);
		
		myIndicator = new RandomVariable(this, new Bernoulli(prob));
	}

	public int componentFailure(HMSensor sensor, Component c) {
		int state = HMSensor.FAILED;
		if (myIndicator.getValue() == 0.0){
			state = HMSensor.OPERATING;
			System.out.println("---------------");
			System.out.println("Detection model: sensor did not detect failure.");
			System.out.println("---------------");
		}
		return state;
	}

}
