package jslx.forecasting;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * An abstraction of a metaforecaster
 * @author vvarghe
 *
 */
public abstract class AbstractMetaForecaster implements MetaForecasterIfc{

	public List <ForecastTechnique> myForecastTechniques;
	public ForecastTechnique myBaseForecastTechnique;
	public Set <ForecastTechnique> myRankedForecastTechniques;
	protected long myId;
    protected String myName;
    private static long myIdCounter_;
    
	public AbstractMetaForecaster(){
		this(null);
	}
	
	public AbstractMetaForecaster(String name){
		myForecastTechniques = new ArrayList<ForecastTechnique>();
		myRankedForecastTechniques = new LinkedHashSet<ForecastTechnique>();
		setId();
        setName(name);
	}
	
	protected void setId(){
		myIdCounter_ = myIdCounter_ + 1;
		myId = myIdCounter_;
	}
	
	public final void setName(String str) {
		if (str == null){
			String s = this.getClass().getName();
			int k = s.lastIndexOf(".");
			if (k != -1)
				s = s.substring(k+1);
			myName = s;
		}
		else
			myName = str;
	}
	
	public String getName(){
		return myName;
	}
	
	public long getId(){		
		return myId;
	}
	
	public void addForecastTechnique(ForecastTechnique ft){
		if(myForecastTechniques.contains(ft))
			new IllegalArgumentException("The forecasting technique already added");
		
		myForecastTechniques.add(ft);
		
	}
	public void setBaseForecastTechnique(ForecastTechnique bft){
		if(!myForecastTechniques.contains(bft))
			new IllegalArgumentException("The forecasting technique not added");
		
		myBaseForecastTechnique = bft;		
	}
	
	public ForecastTechnique getBaseForecastTechnique(){
		return myBaseForecastTechnique;		
	}
	
	public List<ForecastTechnique> getForecastTechniques(){
		return myForecastTechniques;		
	}
		
	public abstract ForecastTechnique getReccommendedForecastTechnique(IterativeDataSource ds);
	
	public abstract ForecastTechnique getReccommendedForecastTechnique(double [] demandattributevector);
	
	public abstract Set<ForecastTechnique> getRankedForecastTechniques();
	
}
