/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package jslx.forecasting;

import java.util.Observable;
import java.util.Observer;

import jsl.utilities.IdentityIfc;
import jsl.utilities.statistic.Statistic;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 * Implements the statistics estimator for an intermittent demand scenario
 * @author vvarghe
 *
 */
public class IntermittentDemandStatistic implements StatisticAccessorIfc, IdentityIfc, Observer {


	protected long myId;
    protected String myName;
    private static long myIdCounter_;

    private double myDemand;
    private Statistic myNZDemandStat;
    private Statistic myDemandStat;
    private Statistic myIntervalBetweenNonZeroDemandsStat;
    private Statistic myIntervalBetweenZeroDemandsStat;

    /*
     * This can be true only after the first the demand state is observed: zero or non-zero
     *
     */
    private boolean TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED;
    private int myPreviousEvent;
    private int numSuccess;
    private int numFailure;
    private int numSuccessAfterSuccess;
    private int numSuccessAfterFailure;

    private int myTimeSpanSinceLastDemand;
    private int myTimeSpanSinceLastZeroDemand;
    private int myTimeIndexofLastDemand;
    private int myTimeIndexofLastZeroDemand;

    private boolean INTERVAL_BTWN_NONZERODEMANDS_INITIALISED;
    private boolean INTERVAL_BTWN_ZERODEMANDS_INITIALISED;
    private int numTransBefore_IntvlBtwnNonZeroDemandsStatInitialized;
    private int numTransBefore_IntvlBtwnZeroDemandsStatInitialized;

    private int myCurrentTimeIndex = 0;

    public IntermittentDemandStatistic() {
    	this(null);
    }

    public IntermittentDemandStatistic(String name) {
		setId();
        setName(name);

        myNZDemandStat = new Statistic("Non-zero demand series");
        myDemandStat = new Statistic("Demand series");
        myIntervalBetweenNonZeroDemandsStat = new Statistic("Interval between non zero demands");
        myIntervalBetweenZeroDemandsStat = new Statistic("Interval between zero demands");
        //myNZDemandStat.setSaveDataOption(true);
        //myDemandStat.setSaveDataOption(true);
        //myIntervalBetweenNonZeroDemandsStat.setSaveDataOption(true);
        //myIntervalBetweenZeroDemandsStat.setSaveDataOption(true);

        TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED = false;
        numSuccess = 0;
        numFailure = 0;
        numSuccessAfterSuccess = 0;
        numSuccessAfterFailure = 0;

        INTERVAL_BTWN_NONZERODEMANDS_INITIALISED = false;
        INTERVAL_BTWN_ZERODEMANDS_INITIALISED = false;
    }

    public void reset(){
        myNZDemandStat.reset();
        myDemandStat.reset();
        myIntervalBetweenNonZeroDemandsStat.reset();
        myIntervalBetweenZeroDemandsStat.reset();

        myCurrentTimeIndex = 0;

        TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED = false;
        numSuccess = 0;
        numFailure = 0;
        numSuccessAfterSuccess = 0;
        numSuccessAfterFailure = 0;

        INTERVAL_BTWN_NONZERODEMANDS_INITIALISED = false;
        INTERVAL_BTWN_ZERODEMANDS_INITIALISED = false;

        myPreviousEvent = 0;

        myTimeSpanSinceLastDemand = 0;
        myTimeSpanSinceLastZeroDemand = 0;
        myTimeIndexofLastDemand = 0;
        myTimeIndexofLastZeroDemand = 0;

        numTransBefore_IntvlBtwnNonZeroDemandsStatInitialized = 0;
        numTransBefore_IntvlBtwnZeroDemandsStatInitialized = 0;

    }

    public void setSaveDataOption(boolean flag) {
        myNZDemandStat.setSaveDataOption(flag);
        myDemandStat.setSaveDataOption(flag);
        myIntervalBetweenNonZeroDemandsStat.setSaveDataOption(flag);
        myIntervalBetweenZeroDemandsStat.setSaveDataOption(flag);
    }

	protected void setId(){
		myIdCounter_ = myIdCounter_ + 1;
		myId = myIdCounter_;
	}
	public final void setName(String str) {
		if (str == null){
			String s = this.getClass().getName();
			int k = s.lastIndexOf(".");
			if (k != -1)
				s = s.substring(k+1);
			myName = s;
		}
		else
			myName = str;
	}
	public String getName(){
		return myName;
	}
	public long getId(){
		return myId;
	}

	public void update(Observable o, Object arg) {
        DataSource ds = (DataSource) o;
        double forecastValue = ds.getValue();
        if (!Double.isNaN(forecastValue)) {
            collect(forecastValue);
        }
    }

	public void collect(double [] data) {

		for(double d: data){
			this.collect(d);
		}

	}

    public void collect(double data) {
        myDemand = data;
        if (myDemand > 0) {
            numSuccess++;
            if (TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED == true) {
                if (myPreviousEvent != 0) {
                    numSuccessAfterSuccess++;
                } else {
                    numSuccessAfterFailure++;
                }
            }
            myPreviousEvent = 1;
            //myNZDemandStat.collect(myDemand);
            //myDemandStat.collect(myDemand);
            myNZDemandStat.collect(myDemand, 1.0);
            myDemandStat.collect(myDemand, 1.0);
            TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED = true;
            setIntervalBetweenNonZeroDemands(myCurrentTimeIndex);
            setIntervalBetweenZeroDemands(myCurrentTimeIndex);
        } else {
            myPreviousEvent = 0;
            numFailure++;
            myDemand = 0;
            //myDemandStat.collect(myDemand);
            myDemandStat.collect(myDemand, 1.0);
            TRANSITION_PROB_ESTIMATION_CAN_BE_INITIALIZED = true;
            setIntervalBetweenNonZeroDemands(myCurrentTimeIndex);
            setIntervalBetweenZeroDemands(myCurrentTimeIndex);
        }
        myCurrentTimeIndex++;
    }



    public double[] getNonZeroDemandsData(){
    	return this.myNZDemandStat.getSavedData();
    }

    public double[] getDemandsData(){
    	return this.myDemandStat.getSavedData();
    }

    public double[] getIntervalBetweenNonZeroDemandsData(){
    	return this.myIntervalBetweenNonZeroDemandsStat.getSavedData();
    }

    public double[] getIntervalBetweenZeroDemandsData(){
    	return this.myIntervalBetweenZeroDemandsStat.getSavedData();
    }

    public int getNumTransactions(){
    	return (int)myNZDemandStat.getCount();
    }

    private void setIntervalBetweenNonZeroDemands(int myCurrentTimeIndex) {
        if (INTERVAL_BTWN_NONZERODEMANDS_INITIALISED == false) {
            if (myDemand != 0) {
                numTransBefore_IntvlBtwnNonZeroDemandsStatInitialized++;
                if (numTransBefore_IntvlBtwnNonZeroDemandsStatInitialized == 2) {
                    INTERVAL_BTWN_NONZERODEMANDS_INITIALISED = true;
                    myTimeSpanSinceLastDemand = myCurrentTimeIndex - myTimeIndexofLastDemand;
                    //myIntervalBetweenNonZeroDemandsStat.collect(myTimeSpanSinceLastDemand);
                    myIntervalBetweenNonZeroDemandsStat.collect(myTimeSpanSinceLastDemand, 1.0);
                }
                myTimeIndexofLastDemand = myCurrentTimeIndex;
            }
        } else {
            if (myDemand != 0) {
                numTransBefore_IntvlBtwnNonZeroDemandsStatInitialized++;
                myTimeSpanSinceLastDemand = myCurrentTimeIndex - myTimeIndexofLastDemand;
                //myIntervalBetweenNonZeroDemandsStat.collect(myTimeSpanSinceLastDemand);
                myIntervalBetweenNonZeroDemandsStat.collect(myTimeSpanSinceLastDemand, 1.0);
                myTimeIndexofLastDemand = myCurrentTimeIndex;
            }
        }
    }

    private void setIntervalBetweenZeroDemands(int myCurrentTimeIndex) {
        if (INTERVAL_BTWN_ZERODEMANDS_INITIALISED == false) {
            if (myDemand == 0) {
                numTransBefore_IntvlBtwnZeroDemandsStatInitialized++;
                if (numTransBefore_IntvlBtwnZeroDemandsStatInitialized == 2) {
                    INTERVAL_BTWN_ZERODEMANDS_INITIALISED = true;
                    myTimeSpanSinceLastZeroDemand = myCurrentTimeIndex - myTimeIndexofLastZeroDemand;
                    //myIntervalBetweenZeroDemandsStat.collect(myTimeSpanSinceLastZeroDemand);
                    myIntervalBetweenZeroDemandsStat.collect(myTimeSpanSinceLastZeroDemand, 1.0);
                }
                myTimeIndexofLastZeroDemand = myCurrentTimeIndex;
            }
        } else {
            if (myDemand == 0) {
                numTransBefore_IntvlBtwnZeroDemandsStatInitialized++;
                myTimeSpanSinceLastZeroDemand = myCurrentTimeIndex - myTimeIndexofLastZeroDemand;
                //myIntervalBetweenZeroDemandsStat.collect(myTimeSpanSinceLastZeroDemand);
                myIntervalBetweenZeroDemandsStat.collect(myTimeSpanSinceLastZeroDemand, 1.0);
                myTimeIndexofLastZeroDemand = myCurrentTimeIndex;
            }
        }
    }

    public Statistic getIntervalBetweenNonZeroDemandStat() {
        return myIntervalBetweenNonZeroDemandsStat;
    }
    public Statistic getIntervalBetweenZeroDemandsStat() {
		return myIntervalBetweenZeroDemandsStat;
	}

	public Statistic getNonZeroDemandStat() {
        return myNZDemandStat;
    }
    public Statistic getDemandStat() {
        return myDemandStat;
    }
    public double getProbSuccessAfterSuccess(){
        return (double) (numSuccessAfterSuccess) / (double) numSuccess;
    }
    public double getProbSuccessAfterFailure(){
        return (double) numSuccessAfterFailure / (double) numFailure;
    }
    public double getProbSuccess(){
        return (double) numSuccess / (double) (numSuccess+numFailure);
    }
    public double getProbFailure(){
        return (double) numFailure / (double) (numSuccess+numFailure);
    }
	public double getAverage() {
		return myDemandStat.getAverage();
	}
	public double getConfidenceCoefficient() {
		return myDemandStat.getConfidenceCoefficient();
	}
	public double getCount() {
		return myDemandStat.getCount();
	}
	public double getDeviationSumOfSquares() {
		return myDemandStat.getDeviationSumOfSquares();
	}
	public double getHalfWidth() {
		return myDemandStat.getHalfWidth();
	}
	public double getKurtosis() {
		return myDemandStat.getKurtosis();
	}
	public double getLag1Correlation() {
		return myDemandStat.getLag1Correlation();
	}
	public double getLag1Covariance() {
		return myDemandStat.getLag1Covariance();
	}
	public double getLastValue() {
		return myDemandStat.getLastValue();
	}
	public double getLastWeight() {
		return myDemandStat.getLastWeight();
	}
	public double getMax() {
		return myDemandStat.getMax();
	}
	public double getMin() {
		return myDemandStat.getMin();
	}
	public double getNumberMissing() {
		return myDemandStat.getNumberMissing();
	}
	public double getSkewness() {
		return myDemandStat.getSkewness();
	}
	public double getStandardDeviation() {
		return myDemandStat.getStandardDeviation();
	}
	public double getStandardError() {
		return myDemandStat.getStandardError();
	}

	public void getStatistics(double[] statistics) {
		if (statistics.length != 87)
    		throw new IllegalArgumentException("The supplied array was not of size 87");

        statistics[0] = getCount();
        statistics[1] = getAverage();
        statistics[2] = getVariance();
        statistics[3] = getStandardDeviation();
        statistics[4] = getStandardError();
        statistics[5] = getMin();
        statistics[6] = getMax();
        statistics[7] = getSum();
        statistics[8] = getWeightedSum();
        statistics[9] = getWeightedSumOfSquares();
        statistics[10] = getSumOfWeights();
        statistics[11] = getWeightedAverage();
        statistics[12] = getDeviationSumOfSquares();
        statistics[13] = getLastValue();
        statistics[14] = getLastWeight();
        statistics[15] = getKurtosis();
        statistics[16] = getSkewness();
        statistics[17] = getLag1Correlation();
        statistics[18] = getLag1Covariance();
        statistics[19] = getVonNeumannLag1TestStatistic();
        statistics[20] = getNumberMissing();

        statistics[21] = myNZDemandStat.getCount();
        statistics[22] = myNZDemandStat.getAverage();
        statistics[23] = myNZDemandStat.getVariance();
        statistics[24] = myNZDemandStat.getStandardDeviation();
        statistics[25] = myNZDemandStat.getStandardError();
        statistics[26] = myNZDemandStat.getMin();
        statistics[27] = myNZDemandStat.getMax();
        statistics[28] = myNZDemandStat.getSum();
        statistics[29] = myNZDemandStat.getWeightedSum();
        statistics[30] = myNZDemandStat.getWeightedSumOfSquares();
        statistics[31] = myNZDemandStat.getSumOfWeights();
        statistics[32] = myNZDemandStat.getWeightedAverage();
        statistics[33] = myNZDemandStat.getDeviationSumOfSquares();
        statistics[34] = myNZDemandStat.getLastValue();
        statistics[35] = myNZDemandStat.getLastWeight();
        statistics[36] = myNZDemandStat.getKurtosis();
        statistics[37] = myNZDemandStat.getSkewness();
        statistics[38] = myNZDemandStat.getLag1Correlation();
        statistics[39] = myNZDemandStat.getLag1Covariance();
        statistics[40] = myNZDemandStat.getVonNeumannLag1TestStatistic();
        statistics[41] = myNZDemandStat.getNumberMissing();

        statistics[42] = myIntervalBetweenNonZeroDemandsStat.getCount();
        statistics[43] = myIntervalBetweenNonZeroDemandsStat.getAverage();
        statistics[44] = myIntervalBetweenNonZeroDemandsStat.getVariance();
        statistics[45] = myIntervalBetweenNonZeroDemandsStat.getStandardDeviation();
        statistics[46] = myIntervalBetweenNonZeroDemandsStat.getStandardError();
        statistics[47] = myIntervalBetweenNonZeroDemandsStat.getMin();
        statistics[48] = myIntervalBetweenNonZeroDemandsStat.getMax();
        statistics[49] = myIntervalBetweenNonZeroDemandsStat.getSum();
        statistics[50] = myIntervalBetweenNonZeroDemandsStat.getWeightedSum();
        statistics[51] = myIntervalBetweenNonZeroDemandsStat.getWeightedSumOfSquares();
        statistics[52] = myIntervalBetweenNonZeroDemandsStat.getSumOfWeights();
        statistics[53] = myIntervalBetweenNonZeroDemandsStat.getWeightedAverage();
        statistics[54] = myIntervalBetweenNonZeroDemandsStat.getDeviationSumOfSquares();
        statistics[55] = myIntervalBetweenNonZeroDemandsStat.getLastValue();
        statistics[56] = myIntervalBetweenNonZeroDemandsStat.getLastWeight();
        statistics[57] = myIntervalBetweenNonZeroDemandsStat.getKurtosis();
        statistics[58] = myIntervalBetweenNonZeroDemandsStat.getSkewness();
        statistics[59] = myIntervalBetweenNonZeroDemandsStat.getLag1Correlation();
        statistics[60] = myIntervalBetweenNonZeroDemandsStat.getLag1Covariance();
        statistics[61] = myIntervalBetweenNonZeroDemandsStat.getVonNeumannLag1TestStatistic();
        statistics[62] = myIntervalBetweenNonZeroDemandsStat.getNumberMissing();

        statistics[63] = myIntervalBetweenZeroDemandsStat.getCount();
        statistics[64] = myIntervalBetweenZeroDemandsStat.getAverage();
        statistics[65] = myIntervalBetweenZeroDemandsStat.getVariance();
        statistics[66] = myIntervalBetweenZeroDemandsStat.getStandardDeviation();
        statistics[67] = myIntervalBetweenZeroDemandsStat.getStandardError();
        statistics[68] = myIntervalBetweenZeroDemandsStat.getMin();
        statistics[69] = myIntervalBetweenZeroDemandsStat.getMax();
        statistics[70] = myIntervalBetweenZeroDemandsStat.getSum();
        statistics[71] = myIntervalBetweenZeroDemandsStat.getWeightedSum();
        statistics[72] = myIntervalBetweenZeroDemandsStat.getWeightedSumOfSquares();
        statistics[73] = myIntervalBetweenZeroDemandsStat.getSumOfWeights();
        statistics[74] = myIntervalBetweenZeroDemandsStat.getWeightedAverage();
        statistics[75] = myIntervalBetweenZeroDemandsStat.getDeviationSumOfSquares();
        statistics[76] = myIntervalBetweenZeroDemandsStat.getLastValue();
        statistics[77] = myIntervalBetweenZeroDemandsStat.getLastWeight();
        statistics[78] = myIntervalBetweenZeroDemandsStat.getKurtosis();
        statistics[79] = myIntervalBetweenZeroDemandsStat.getSkewness();
        statistics[80] = myIntervalBetweenZeroDemandsStat.getLag1Correlation();
        statistics[81] = myIntervalBetweenZeroDemandsStat.getLag1Covariance();
        statistics[82] = myIntervalBetweenZeroDemandsStat.getVonNeumannLag1TestStatistic();
        statistics[83] = myIntervalBetweenZeroDemandsStat.getNumberMissing();

        statistics[84] = this.getProbSuccessAfterSuccess();
        statistics[85] = this.getProbSuccessAfterFailure();
        statistics[86] = this.getProbFailure();

	}
	public double[] getStatistics() {
		double[] x = new double[87];
    	getStatistics(x);
    	return(x);
	}

	public double getSum() {
		return myDemandStat.getSum();
	}
	public double getSumOfWeights() {
		return myDemandStat.getSumOfWeights();
	}
	public double getVariance() {
		return myDemandStat.getVariance();
	}
	public double getVonNeumannLag1TestStatistic() {
		return myDemandStat.getVonNeumannLag1TestStatistic();
	}
	public double getWeightedAverage() {
		return myDemandStat.getWeightedAverage();
	}
	public double getWeightedSum() {
		return myDemandStat.getWeightedSum();
	}
	public double getWeightedSumOfSquares() {
		return myDemandStat.getWeightedSumOfSquares();
	}


}
