package jslx.forecasting;

/**
 * An abstraction of the intermittent forecasters
 * @author vvarghe
 *
 */
public abstract class IntermittentForecastTechnique extends ForecastTechnique {

	/*
	 * IntervalBtwnNZDemands
	 * 
	 */
	protected double myPt;
	
	/*
	 * NZDemandSize
	 * 
	 */
	protected double myZt;
	
	/*
	 * TimeSpanSinceLastDemand
	 * 
	 */
	protected double myQ;
	
	/** 
	 * the data until required non zero demands are collected
	 *  
	 * the array keep on increasing
	 * 
	 */
	protected double[] myData;

	/*
	 * The required number of non zero demands in order to initialize
	 * 
	 */
	protected int myReqdNumNZDemandsBeforeInitialization = 3;
	
	/** This monitors the current number of non zero demands before
	 * initializing the forecast
	 * 
	 */
	protected int myNumNZDemandsBeforeInitialization;
	
	/** 
	 * The smoothing parameters
	 * 
	 */
	protected double [] myAlpha;
	
	public IntermittentForecastTechnique(double alpha, int reqdnzdem, String name) {
		super(name);
		myNumNZDemandsBeforeInitialization = 0;
		myReqdNumNZDemandsBeforeInitialization = reqdnzdem;
		myPt = Double.NaN;		
		myZt = Double.NaN;		
		myQ = Double.NaN;		
		myData = new double[1];
		double [] a = {alpha, alpha};
		setSmoothingParameter(a);
	}
	
	public IntermittentForecastTechnique(double [] alpha, int reqdnzdem, String name) {
		super(name);
		myNumNZDemandsBeforeInitialization = 0;
		myReqdNumNZDemandsBeforeInitialization = reqdnzdem;
		myPt = Double.NaN;		
		myZt = Double.NaN;		
		myQ = Double.NaN;		
		myData = new double[1];
		setSmoothingParameter(alpha);	
	}
	
	public double [] getSmoothingParameter(){
		double [] alpha = myAlpha;
		return alpha;
	}
	
	public abstract void setSmoothingParameter(double [] alpha);
	
	@Override
	protected void checkForSufficientHistory() {
		if (!isInitialized()){
			if(myXt!=0){
				myNumNZDemandsBeforeInitialization++;
				initializeEstimatesOfConstituentSeries();				
	        }
		}
	}
	
	protected void initializeEstimatesOfConstituentSeries(){
       boolean NONZEROTRANSACTIONSINITIALIZED = false;
		if(myNumNZDemandsBeforeInitialization==myReqdNumNZDemandsBeforeInitialization){
			
			//TODO			
			initializeHistory(myData, null, null);
        }
	}
    //TODO check this
    public double [] estimateInitialForecastParameter(double [] history){
    	boolean NONZEROTRANSACTIONSINITIALIZED = false;
       	int numNZdemandsbeforeinit = 0;
		int currentIntervalBT=0;
	    double q =0;
	    double pt = 0;
	    double zt = 0;
        for(int i = 0; i < (history.length); i++ ){
        	if(history[i]!= 0){
        		numNZdemandsbeforeinit++;
        	}
        	
        	if(NONZEROTRANSACTIONSINITIALIZED==false){
                if(history[i]==0){
                	NONZEROTRANSACTIONSINITIALIZED=false;
                }else{
                	//myCurrentNumNonZeroDemands++;
                	zt=history[i];
                	pt=0;
                    NONZEROTRANSACTIONSINITIALIZED=true;                   
                }
            }else{
                if(history[i]==0){                	
                	currentIntervalBT++;
                	q++;
                }else{
                	//myCurrentNumNonZeroDemands++;
                	zt=zt+history[i];
                    currentIntervalBT++;                    
                    pt=pt+currentIntervalBT;
                    currentIntervalBT=0;
                    q=1;
                }
            }       	
        }	
        pt=pt/(numNZdemandsbeforeinit-1);        
        zt=zt/numNZdemandsbeforeinit;
        //System.out.println("Zt = "+zt+", myPt = "+pt);	        
        double [] init_stat_para = {q,pt,zt};	
        
		return init_stat_para;
		
	}
	
	protected abstract void estimateAverageDemand();

	@Override
	public double getForecast(int n) {
		return myNextPeriodForecast;
	}

	@Override
	public double[] getForecasts(int n) {
		if (n < 1)	
			throw new IllegalArgumentException("The supplied n was < 1");
		
		double[] f = new double[n+1];
		f[0] = Double.NaN;
		for (int i=1;i<=n;i++)
			f[i] = myNextPeriodForecast;
		return f;
	}

	@Override
	public double[] getHistory() {
		if (!isInitialized()){
			return myData;
		}else{
			double [] tempData = new double [1];
			tempData[0] = myXt;					
			myData = tempData;
			return myData;
		}	
	}

	@Override
	protected void updateHistory(int i, double x) {
		if(!isInitialized()){
			double [] tempData = new double [i];
			System.arraycopy(myData, 0, tempData, 0, myData.length);
			//index begins at 0
			tempData[i-1] = x;					
			myData = tempData;
		}
	}
	

}
