/**
 * 
 */
package jslx.forecasting;

import jsl.utilities.reporting.JSL;

/**
 * @author rossetti
 *
 */
public abstract class IterativeDataSource extends DataSource {

	//  ===========================================
	//      CLASS ATTRIBUTES
	//  ===========================================

	/** An "enum" to indicate that this iterative data source has just been reset to observers
	 */
	public static final int RESET = JSL.getNextEnumConstant();

	/** An "enum" to indicate that this iterative data source has just reached it's end via iteration
	 */
	public static final int END_ITERATION = JSL.getNextEnumConstant();

	/**
	 * 
	 */
	public IterativeDataSource() {
		this(null);
	}

	/**
	 * @param name
	 */
	public IterativeDataSource(String name) {
		super(name);

	}
	
	/** Indicates to clients whether there is another value after
	 *  the current value for the data source
	 * 
	 * @return
	 */
	abstract public boolean hasNext();
	
	/** Advances to the next value and returns it.  Implementers must
	 *  use setValue(double value) to properly set the current value.
	 *  getValue() should then return this current value until advanced again
	 *  
	 *  Should throw a NoSuchElementException if there is no next value
	 * 
	 * @return
	 */
	abstract public double next();
	
	
	/** This method should reset the IterativeDataSource so that it
	 *  is positioned just prior to the 1st value to be returned
	 *  via the use of the next() method
	 *  
	 *  Implementers should ensure that observers are notified of the reset
	 *
	 */
	abstract public void reset();
	
	/** This method causes the IterativeDataSource to iterate through
	 *  all its values.  
	 *  
	 *  Implementers should ensure that observers are notified that
	 *  the iterative data source has reached its end
	 * 
	 * Vijith: We have to reset before initializing the observers so reset method may be commented
	 * or use a while loop to iterate through the data source
	 */
	public void iterateAll() {
		reset();
		while(hasNext())
			next();
		notifyObservers(END_ITERATION);
	}
}
