package jslx.forecasting;

import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.GetValueIfc;
import jsl.utilities.statistic.Statistic;
import jslx.forecasting.demandgeneration.TwoSMarkovChainGeneration;

/**
 * Implements underlying bootstrap process for the MCARTA
 * @author vvarghe
 *
 */
public class MCARTAGenerator implements GetValueIfc {

    private DistributionIfc myRandomAmount;
    private TwoSMarkovChainGeneration myRandomOccur;
    private Statistic myForecastStat;
    private double myForecast;
    private double myBSSize;
    
    private int myStateOfXt;
    
    private boolean medianEstimatesOn;
    
    public MCARTAGenerator(DistributionIfc RA, TwoSMarkovChainGeneration RO, double size) {
    	myRandomAmount = RA;
    	myRandomOccur = RO;
    	myForecastStat = new Statistic("Forecast");
    	myBSSize = size;
        myStateOfXt = 1;
    }
    
    public void setRandomAmount(DistributionIfc myRandomAmount) {
        this.myRandomAmount = myRandomAmount;
    }

    public void setStateOfXt(int myStateOfXt) {
        this.myStateOfXt = myStateOfXt;
        myRandomOccur.setStateOfXt(myStateOfXt);
    }
    
    public void setMedianEstimates(boolean updatable){
    	medianEstimatesOn = updatable;
    }
    
    public double getValue(){
    	int size = (int)myBSSize;
    	myForecastStat = new Statistic();
    	myForecastStat.setSaveDataOption(true);    	
    	
    	for(int w=0; w<size; w++){
            double x=myRandomOccur.getValue();
            if(x==1){
            	double y=myRandomAmount.getValue();
            	myForecast = y;
            	myForecastStat.collect(myForecast);
            }else{
            	myForecast = 0;
                myForecastStat.collect(myForecast);
            }
            myRandomOccur.setStateOfXt(myStateOfXt);
        }
    	
    	double estimate;
    	
    	if(medianEstimatesOn){
    		estimate = Statistic.getMedian(myForecastStat.getSavedData());    		
    	}else{
    		estimate = myForecastStat.getAverage();
    	}
    	
        return estimate;
    }
    
    
    
    public double[] getValue(int n){
        double[] f = new double[n+1];
        Statistic [] s = new Statistic[n];
        for(int i=0; i<n; i++){
            s[i] = new Statistic("Lap "+i);
            s[i].setSaveDataOption(true);
        } 
        
        for(int w=0; w<myBSSize; w++){        
            for(int v=0; v<n; v++){
                double x=myRandomOccur.getValue();
                if(x==1){
                    double y = myRandomAmount.getValue();
                    s[v].collect(y);
                }else{
                    double y = 0;
                    s[v].collect(y);
                }
            }            
            myRandomOccur.setStateOfXt(myStateOfXt);            
        }
        
        
        for(int i=1; i<=n; i++){
        	
        	double estimate;
        	
        	if(medianEstimatesOn){
        		estimate = Statistic.getMedian(s[i].getSavedData());    		
        	}else{
        		estimate = s[i].getAverage();
        	}
        	
            f[i] = estimate;
        }        
        
        return f;
    }
    
    public void setBSSize(double myBSSize) {
            this.myBSSize = myBSSize;
    }

}
