package jslx.forecasting;

/** Implements the random walk or Naive1 forecasting technique where the next forecast is equal to the current
 * observation.
 */
public class NaiveForecastTechnique extends ForecastTechnique {
	
	/** Creates a no named forecaster for NaiveTechnique with no parameters
	*/
	public NaiveForecastTechnique() {
		this(null);
	}
	
	/** Creates a named forecaster for NaiveTechnique with no parameters
	 * @param name the name of the technique
	*/
	public NaiveForecastTechnique(String name) {
		super(name);
	}
	
	
	protected void checkForSufficientHistory() {
		if (!isInitialized()){
			// not already initialized
			// need to check if enough data is available for initialization
			if (myTimeIndex == 1){
				// not initialized, and this is the first data point
				// make forecast for next period the same as first data point
				myNextPeriodForecast = myXt;
				// initialize it, so when 2nd point is collected
				// it is ready
				
				//TODO vijith added
				double [] hist = {myXt}; 
				double [] para = this.estimateInitialForecastParameter(hist);
				setInitialForecastParameter(para);
				
				setInitFlag(true);
			}
		}
	}

	protected void clearHistory() {
		//nothing to clear
	}
	
	public double getForecast(int n) {
		return myNextPeriodForecast;
	}

	public double[] getForecasts(int n) {
		if (n < 1)	
			throw new IllegalArgumentException("The supplied n was < 1");
		double[] f = new double[n+1];
		f[0] = Double.NaN;
		for (int i=1;i<=n;i++)
			f[i] = myNextPeriodForecast;
		return f;
	}

	public double[] getHistory() {
		//nothing to return
		return null;
	}

	public void initializeHistory(double[] data, double[] forecasts,
			double[] parameters) {
		if (data == null)
			throw new IllegalArgumentException("The supplied data array was null");
				
		//TODO Vijith added this
		double [] para = this.estimateInitialForecastParameter(data);		
		setInitialForecastParameter(para);
		
		myNextPeriodForecast = data[data.length-1];
		// initialize it				
		
		setInitFlag(true);
	}
	

    
    public double [] estimateInitialForecastParameter(double [] history){
    	double x = history[history.length - 1];

        double [] init_stat_para = {x};
        
		return init_stat_para;
		
	}
	
	/** This method simply sets the next forecast to the value of the current observation.
	 *  
	 * @param i, the index of the currently observed data point
	 * @param x, the observed data for the current period
	 */
	protected void updateForecast(int i, double x) {
		myNextPeriodForecast=x;
	}

	/**Nothing to update for this technique*/
	protected void updateHistory(int i, double x) {
		// Nothing to update
	}
	
	/**No parameters to return for this technique*/
	public double[] getParameters() {
		//nothing to return
		return null;
	}
	
	/**No parameters to set for this technique*/
	public void setParameters(double[] parameters) {
		//nothing to set
	}
	
	/**
	 * Creates a shallow copy of a Naive forecast technique object.
	 */
	public ForecastTechnique copyInstance(){
        return new NaiveForecastTechnique(this.getName());
    }

	/*@Override
	public ForecastTechnique copyInstance(double[] history) {
		NaiveForecastTechnique a = new NaiveForecastTechnique(this.getName());
		
		double[] forecasts = new double[1];
		double[] parameters = new double[1];
		
		a.initializeHistory(history, forecasts, parameters);
		
		return a;
	}*/
}
