/**
 * 
 */
package jslx.forecasting.demandgeneration;

import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Geometric;
import jsl.utilities.random.GetValueIfc;
import jsl.utilities.random.distributions.NegativeBinomial;
import jsl.utilities.random.ParametersIfc;
import jsl.utilities.random.distributions.Poisson;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.ShiftedDistribution;
import jsl.utilities.random.distributions.Binomial;
import jsl.utilities.random.distributions.ShiftedGeometric;
import jsl.utilities.random.rng.RandomStreamIfc;
import jslx.forecasting.IntermittentDemandStatistic;
import jslx.forecasting.IterativeDataSource;
  
/**
 * @author vvarghe
 *
 */
public class BatchOnOffDemandGenerator extends IterativeDataSource implements
		ParametersIfc, GetValueIfc, RandomStreamIfc {

	private RandomIfc myDistNumBusySlots;//slots between zero demands
	private RandomIfc myDistNumIdleSlots;//slots between non zero demands
	private RandomIfc myDistBatchNZDemands;//non-zero demand at each active slot
	
	private int myCurrentState;//0 if idle, 1 if active
	
	private int myCurrentTimeIndex;
	private int myNextStateOccurTimeIndex;
	private double myDemand;
	
	private IntermittentDemandStatistic myIntermittentDemandStatistic;
	
	public static BatchOnOffDemandGenerator getDefaultBatchOnOffDemandGenerator(double meannumbusyslots, double meannumidleslots, double meanbatchsize, String name){
		ShiftedGeometric busyslots = new ShiftedGeometric(1.0/meannumbusyslots);
		
		ShiftedGeometric idleslots = new ShiftedGeometric(1.0/meannumidleslots);
		
		ShiftedGeometric batch = new ShiftedGeometric(1.0/meanbatchsize);
		
		return new BatchOnOffDemandGenerator(busyslots, idleslots, batch, "Default "+name);
	}

	
	public static BatchOnOffDemandGenerator getDefaultBatchOnOffDemandGenerator(double meannumbusyslots, double varnumbusyslots, 
			double meannumidleslots, double varnumidleslots, double meanbatchsize, double varbatchsize, String name){
		/*Geometric busyslots = new Geometric(1.0/meannumbusyslots);
		busyslots.setLowerRangeLimit(1);
		
		Geometric idleslots = new Geometric(1.0/meannumidleslots);
		idleslots.setLowerRangeLimit(1);
		
		Geometric batch = new Geometric(1.0/meanbatchsize);
		batch.setLowerRangeLimit(1);*/
		
		DistributionIfc busyslots = getDistribution(meannumbusyslots, varnumbusyslots);
		
		DistributionIfc idleslots = getDistribution(meannumidleslots, varnumidleslots);
		
		DistributionIfc batch = getDistribution(meanbatchsize, varbatchsize);
		
		return new BatchOnOffDemandGenerator(busyslots, idleslots, batch, "Default "+name);
	}
	
	public static BatchOnOffDemandGenerator getDefaultBatchOnOffDemandGenerator(double meannumbusyslots, 
			double meannumidleslots, double meanbatchsize, double varbatchsize, String name){
		ShiftedGeometric busyslots = new ShiftedGeometric(1.0/meannumbusyslots);
		
		ShiftedGeometric idleslots = new ShiftedGeometric(1.0/meannumidleslots);
		
		DistributionIfc batch = getDistribution(meanbatchsize, varbatchsize);
		
		return new BatchOnOffDemandGenerator(busyslots, idleslots, batch, name);
	}
	
	public static DistributionIfc getDistribution(double mu, double sigma2){		
		DistributionIfc dist;		
		if (mu < 1.0)
			throw new IllegalArgumentException("mu must be >= 1.0");
		if (sigma2 <=0)
			throw new IllegalArgumentException("sigma2 must be > 0");
		
		if(mu == 1.0){
			dist = getConstant(mu - 1.0);
		}else{			
			double c = sigma2/(mu-1.0);		
			if(c > 0.9 && c < 1.1){
				dist = getPoissonDistribution((mu-1.0));
			} else if (mu < sigma2){
				dist = getNegativeBinomialDistribution(mu - 1.0, sigma2);			
			} else {
				dist = getPoissonDistribution((mu-1.0));
			}
			
		}		
		return new ShiftedDistribution(dist,1.0);
	}
	
	public static Poisson getPoissonDistribution(double mu){		
		return new Poisson(mu);		
	}
	
	public static Constant getConstant(double mu){		
		return new Constant(mu);		
	}
	
	public static NegativeBinomial getNegativeBinomialDistribution(double mu, double sigma2){
		double p = mu/sigma2;
		double r = (p/(1.0-p))*mu;
		return new NegativeBinomial(p,r);		
	}
	
	public static Binomial getBinomialDistribution(double mu, double sigma2){
		double p = 1 - (sigma2/mu);
		int n = (int)(Math.floor(mu/p)+1.0);
		return new Binomial(p, n);		
	}
	
	/**
	 * 
	 */
	public BatchOnOffDemandGenerator() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * 
	 * @param myDistNumBusySlots
	 * @param myDistNumIdleSlots
	 * @param myDistBatchNZDemands
	 * @param name
	 */
	public BatchOnOffDemandGenerator(RandomIfc busyslots, RandomIfc idleslots, RandomIfc batch, String name) {
		super(name);
		this.myDistNumBusySlots = busyslots;
		this.myDistNumIdleSlots = idleslots;
		this.myDistBatchNZDemands = batch;
		
		myIntermittentDemandStatistic = new IntermittentDemandStatistic();
		
		//assumed the first state is busy periods
		myCurrentState = 1;
		myCurrentTimeIndex = 0;
		myNextStateOccurTimeIndex = myCurrentTimeIndex + 1 + (int)(myDistNumBusySlots.getValue());
	}
	
	public double getValue(){
		myCurrentTimeIndex ++;
		if(myCurrentTimeIndex == myNextStateOccurTimeIndex){
			if(this.myCurrentState == 1){
				int num = (int)(myDistNumIdleSlots.getValue());
				myCurrentState = 0;
				myNextStateOccurTimeIndex = myCurrentTimeIndex + num;
			}else{
				int num = (int)(myDistNumBusySlots.getValue());
				myCurrentState = 1;
				myNextStateOccurTimeIndex = myCurrentTimeIndex + num;
			}
		}else{
			//state remain the same
			//next state occur time remain the same
			
		}
		myDemand = getDemand();
    	setValue(myDemand);
    	myIntermittentDemandStatistic.collect(myDemand);
        return myDemand;
    } 
	
	public double [] getValue(int i){
		double [] values = new double [i];
		for(int j = 0; j < i - 1; j++){
			values [j] = getValue();			
		}
		return values;
	}
	
	private double getDemand(){
		if(this.myCurrentState == 1){
			return myDistBatchNZDemands.getValue();
		}else{
			return 0.0;
		}
	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.ParametersIfc#getParameters()
	 */
	public double[] getParameters() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.ParametersIfc#setParameters(double[])
	 */
	public void setParameters(double[] parameters) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.RandomStreamIfc#resetNextSubstream()
	 */
	public void resetNextSubstream() {
		myDistBatchNZDemands.resetNextSubstream();
		myDistNumBusySlots.resetNextSubstream();
		myDistNumIdleSlots.resetNextSubstream();

	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.RandomStreamIfc#resetStartStream()
	 */
	public void resetStartStream() {
		myDistBatchNZDemands.resetStartStream();
		myDistNumBusySlots.resetStartStream();
		myDistNumIdleSlots.resetStartStream();

	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.RandomStreamIfc#resetStartSubstream()
	 */
	public void resetStartSubstream() {
		myDistBatchNZDemands.resetStartSubstream();
		myDistNumBusySlots.resetStartSubstream();
		myDistNumIdleSlots.resetStartSubstream();

	}

	/* (non-Javadoc)
	 * @see jsl.utilities.random.RandomStreamIfc#setAntithetic(boolean)
	 */
	public void setAntithetic(boolean flag) {
		myDistBatchNZDemands.setAntithetic(flag);
		myDistNumBusySlots.setAntithetic(flag);
		myDistNumIdleSlots.setAntithetic(flag);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		double meannumbusyslots = 1.4;
		double meannumidleslots = 2.6;
		double meanbatchsize = 2.0;
		String name = "BatchOnOffDG";
		
		BatchOnOffDemandGenerator dg = BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(meannumbusyslots, meannumidleslots, meanbatchsize, name);
		
		for(int i = 1; i <= 100; i++){
			
			System.out.println(i+": "+dg.getValue());
			
		}

	}


	@Override
	public boolean hasNext() {
		return true;
	}


	@Override
	public double next() {
		return getValue();
	}


	@Override
	public void reset() {
		myCurrentState = 1;
		myCurrentTimeIndex = 0;
		myIntermittentDemandStatistic = new IntermittentDemandStatistic();
		myNextStateOccurTimeIndex = myCurrentTimeIndex + 1 + (int)(myDistNumBusySlots.getValue());
	}

	public IntermittentDemandStatistic getIntermittentDemandStatistic() {
		return myIntermittentDemandStatistic;
	}
	
}
