package jslx.forecasting.demandgeneration;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.EventGeneratorListenerIfc;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.utilities.random.distributions.Binomial;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.NegativeBinomial;
import jsl.utilities.random.distributions.Poisson;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.ShiftedDistribution;
import jsl.utilities.random.rng.AR1CorrelatedRng;
import jslx.forecasting.IntermittentDemandStatistic;

public class CPP extends SchedulingElement {

	public RandomIfc distIntervalBetweenTransactions;
	public RandomIfc distAmount; 
	public EventGenerator myDemandGenerator;
	public IntermittentDemandStatistic myStat;
	public ResponseVariable nonbucketeddemand;
	public ResponseVariable nonbucketedvariance;
	public ResponseVariable nonbucketedinterval;
	public ResponseVariable mean;
	public ResponseVariable variance;
	public ResponseVariable lag1corrcoeff;
	public ResponseVariable nzmean;
	public ResponseVariable nzvariance;
	public ResponseVariable nzlag1corrcoeff;
	public ResponseVariable ibtmean;
	public ResponseVariable ibtvariance;
	public ResponseVariable ibtlag1corrcoeff;
	public ResponseVariable p0;
	
	public double sumDemandAcrossBuckets;
	public double sumDemandCurrentBucket;
	public double myCurrentDemand;
	public double myCurrentTime;
	public double myInterval;
	public Constant myIntervalTime;
	
	public CPP(ModelElement parent, RandomIfc distinterval, RandomIfc distAmount, double interval) {
		this(parent, distinterval, distAmount, interval, "CPP");
		// TODO Auto-generated constructor stub
	}

	public CPP(ModelElement parent, RandomIfc distinterval, RandomIfc distamount, double interval, String name) {
		super(parent, name);
		myDemandGenerator = new EventGenerator(this, new DemandArrivalListener(), distinterval, distinterval, "Demand Generator");
		
		distAmount = distamount;
		myStat = new IntermittentDemandStatistic();			
		
		nonbucketeddemand = new ResponseVariable(this, "nonbucketeddemand");		
		nonbucketedvariance = new ResponseVariable(this, "nonbucketedvariance");		
		nonbucketedinterval = new ResponseVariable(this, "nonbucketedinterval");		
		mean = new ResponseVariable(this, "Mean");
		variance = new ResponseVariable(this, "Variance");
		lag1corrcoeff = new ResponseVariable(this, "Lag1CorrCoeff");
		nzmean = new ResponseVariable(this, "NZMean");
		nzvariance = new ResponseVariable(this, "NZVariance");
		nzlag1corrcoeff = new ResponseVariable(this, "NZLag1CorrCoeff");
		ibtmean = new ResponseVariable(this, "IBTMean");
		ibtvariance = new ResponseVariable(this, "IBTVariance");
		ibtlag1corrcoeff = new ResponseVariable(this, "IBTLag1CorrCoeff");
		p0 = new ResponseVariable(this, "ProbZero");
		
		myCurrentTime = interval;
		sumDemandAcrossBuckets = 0.0;
		sumDemandCurrentBucket = 0.0;
		myIntervalTime = new Constant((double)myCurrentTime);
		
	}
	
	protected void initialize(){
		scheduleEvent(myIntervalTime, "Demand");		
	}
	
	protected void replicationEnded() {
		nonbucketedvariance.setValue(nonbucketeddemand.getWithinReplicationStatistic().getVariance());
		nonbucketedinterval.setValue(this.myDemandGenerator.getTimedUpdateInterval());
		mean.setValue(myStat.getDemandStat().getAverage());
		variance.setValue(myStat.getDemandStat().getVariance());
		lag1corrcoeff.setValue(myStat.getDemandStat().getLag1Correlation());
		nzmean.setValue(myStat.getNonZeroDemandStat().getAverage());
		nzvariance.setValue(myStat.getNonZeroDemandStat().getVariance());
		nzlag1corrcoeff.setValue(myStat.getNonZeroDemandStat().getLag1Correlation());
		ibtmean.setValue(myStat.getIntervalBetweenNonZeroDemandStat().getAverage());
		ibtvariance.setValue(myStat.getIntervalBetweenNonZeroDemandStat().getVariance());
		ibtlag1corrcoeff.setValue(myStat.getIntervalBetweenNonZeroDemandStat().getLag1Correlation());
		p0.setValue(myStat.getProbFailure());
	}
	
	protected void scheduleDemandProcessing(JSLEvent event){
		double x = sumDemandCurrentBucket;
		myStat.collect(x);
		sumDemandCurrentBucket = 0.0;
		rescheduleEvent(event,myIntervalTime,"Demand");
	}
	
	protected void handleEvent(JSLEvent event){
		scheduleDemandProcessing(event);
	}
	
	public class DemandArrivalListener implements EventGeneratorListenerIfc{
		public void generate(EventGenerator generator, JSLEvent event) {
			double x = distAmount.getValue();
			nonbucketeddemand.setValue(x);
			myCurrentDemand = x;
			sumDemandAcrossBuckets = sumDemandAcrossBuckets + myCurrentDemand;
			sumDemandCurrentBucket = sumDemandCurrentBucket + myCurrentDemand;		
		}		
	}
	
	public static void main(String[] args) {

		Model m = Model.createModel();	
		
		//double lambda = 0.653926467;
		//double lambda = 0.653926467;
		double lambda = 2.0;
		
		AR1CorrelatedRng rng1 = new AR1CorrelatedRng(-0.8);
		DistributionIfc n = new Exponential(1.0 / lambda, rng1);
		
		//double nzmean = 3.670137423;
		//double nzmean = 3.670137423;
		double nzmean = 3.0;
		
		//double nzvar = 0.660120374;
		//double nzvar = 69.47519706;
		double nzvar = 2.0;
		
		
		double shift = 1.0;
		
		/*double myUL_Desired = Double.POSITIVE_INFINITY;
		
		double myLL_Input = 0.0;
		
		double myUL_Input = Double.POSITIVE_INFINITY;
		
		double delta = 1 - (new Poisson(nzmean)).cdf(1.0);
		
		double nzmean_ = nzmean * delta;
		
		double nzvar_ = ((nzvar + (nzmean*nzmean)) * delta) - (nzvar * nzvar);
		
		System.out.println("delta: "+delta);
		System.out.println("nzmean_: "+nzmean_);
		System.out.println("nzvar_: "+nzvar_);*/
			
		DistributionIfc q;
		
		if(nzvar/(nzmean-shift) < 0.9){
			double p = 1 - (nzvar/(nzmean-shift));
			int n0 = (int)(Math.floor((nzmean-shift)/p)+1.0);			
			q = new Binomial(p, n0);			
		}else if (nzvar/(nzmean-shift) <= 1.1){
			q = new Poisson(nzmean-shift);			
		}else {
			double p = (nzmean-shift)/nzvar;			
			int r = (int)(Math.floor((nzmean-shift)*p*(1/(1-p))) + 1.0);			
			q = new NegativeBinomial(p, r);			
		}
		
		ShiftedDistribution a = new ShiftedDistribution(q, shift); // A'~r.v.
		
		double interval = 1.0;		
		
		CPP c = new CPP(m, n, a, interval);
		
		System.out.println(n);
		System.out.println(a);		
		//double p0 = Math.exp(-1 * lambda * interval *(1 - a.pdf(0.0)));	
		double p0 = Math.exp(-1 * lambda * interval );
		double p1 = 1 - p0;
		double bucketednzmean = lambda * interval * nzmean * (1/p1);
		
		double sec_mmnt_nzmean = nzvar + (nzmean * nzmean);
		
		
		
		double bucketednzvar = ((lambda * interval * sec_mmnt_nzmean) + 
				(lambda * interval * lambda * interval * nzmean * nzmean * (1 - (1/p1))))/p1;
		
		System.out.println("Prob of zero: "+p0);
		System.out.println("NZ Mean: "+bucketednzmean);
		System.out.println("NZ Variance: "+bucketednzvar);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m, "CPP Experiments");
		
		// set the parameters of the experiment
		e.setNumberOfReplications(100);
		e.setLengthOfReplication(1000);

		// turn on the desired reporting
		//e.turnOnReplicationConsoleOutput();
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
		
		// tell the experiment to run
		e.runAll();
		System.out.println("Done!");
	}

}
