package jslx.forecasting.demandgeneration;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import jsl.utilities.statistic.Statistic;
import jslx.forecasting.IntermittentDemandStatistic;

public class DemandGeneratorExperiment {
	
	private double [][] designpoints;
	
	public void createDesignPoints(){
		double [] arrayP0 = {0.10, 0.30, 0.52, 0.75};
	    double [] arrayPhiT = {- 0.80, - 0.20, 0.0, 0.2, 0.80};
	    double [] arrayMu = {5.0, 50.0, 250.0};
	    double [] cvs = {0.25, 4.0};	
	    double interval = 1.0;
	    designpoints = new double [1][5];
	    
	    for(int i = 0; i < arrayP0.length; i++ ){
	    	double valueP0 = arrayP0[i];	    	
	    	double lambda =  -1 * (1/interval) * Math.log(valueP0);
	    	double valueP1 = 1 - valueP0;
	    	for(int j = 0; j < arrayPhiT.length; j++ ){
	        	double valuePhiT = arrayPhiT[j];	    		
	    		for(int k = 0; k < arrayMu.length; k++ ){
	            	double valueMu = arrayMu[k];	    			
	    			for(int l = 0; l < cvs.length; l++ ){
	    	        	double valuecvs = cvs[l];
	    	        	double valueVar = valueMu * valueMu * valuecvs;
	    	        	
	    	        	double lhs = valuecvs + 1;
	    	        	double rhs = valueP1 * ((1 + (lambda*interval))/(lambda*interval));
	    	        	
	    	        	double flag;
	    	        	if(lhs > rhs){
	    	        		flag = 1.0;	    	        		
	    	        	}else{
	    	        		flag = 0.0;    	        		
	    	        	}
	    	        	
	    	        	System.out.println(valueMu+", "+valueVar+", "+valuePhiT+", "+valueP0+", "+flag);	    	        	
	    	        	int oldLength = designpoints.length;	    	        	
	    	        	designpoints [oldLength - 1] [0] = valueMu;
	    	        	designpoints [oldLength - 1] [1] = valueVar;
	    	        	designpoints [oldLength - 1] [2] = valuePhiT;
	    	        	designpoints [oldLength - 1] [3] = valueP0;	
	    	        	designpoints [oldLength - 1] [4] = flag;	
	    				int newLength = oldLength + 1;			
	    				double [][] tempdesignpoints = new double [newLength][5];			
	    				System.arraycopy(designpoints, 0, tempdesignpoints, 0, designpoints.length);			
	    				designpoints = tempdesignpoints;    	        	
	    	        }	            	
	            }	    	
	    	}	    	
	    }	
	}
	
	public void runExperiment(){

		try{
			BufferedWriter out = new BufferedWriter(new FileWriter("DemandGenerator.csv"));
			StringBuilder sb = new StringBuilder();
			sb.append("IP--UserSpecified,"+","+","+","+","+"IP--Computed,"+","+","+","+","
					+"OP,"+","+","+","+","+","+","+","+","+","+","+","+","+",");			
            sb.append("\n");
			sb.append("IP:Buck_Size"+",");
			sb.append("IP:P_Z_Dem_Buck"+",");
			sb.append("IP:L1CorrCoeff_Ivl_NonBuck"+",");
			sb.append("IP:Mean_NZ_Dem_Buck"+",");
			sb.append("IP:Var_NZ_Dem_Buck"+",");
			sb.append("IP:CVS_NZ_Dem_Buck"+",");
			sb.append("IP->:Lambda_E[N(t)]"+",");
			sb.append("IP->:Mean_NZ_Dem_NonBuck_E[A]"+",");
			sb.append("IP->:Var_NZ_Dem_NonBuck_Var[A]"+",");
			sb.append("IP:DesignLevel"+",");			
			sb.append("OP->:P_Z_Dem_Buck"+",");
			sb.append("OP:Mean_Dem_Buck"+",");
			sb.append("OP:Var_Dem_Buck"+",");
			sb.append("OP:L1CorrCoeff_Dem_Buck"+",");
			sb.append("OP:Mean_NZ_Dem_Buck"+",");
			sb.append("OP:Var_NZ_Dem_Buck"+",");
			sb.append("OP:CVS_NZ_Dem_Buck"+",");
			sb.append("OP:L1CorrCoeff_NZ_Dem_Buck"+",");
			sb.append("OP:Mean_Ivl_Buck"+",");
			sb.append("OP:Var_Ivl_Buck"+",");
			sb.append("OP:L1CorrCoeff_Ivl_Buck"+",");
			sb.append("OP:Mean_Dem_NonBuck"+",");
			sb.append("OP:Var_Dem_NonBuck"+",");
			sb.append("OP:L1CorrCoeff_Dem_NonBuck"+",");
			sb.append("OP:Mean_Ivl_NonBuck"+",");
			sb.append("OP:Var_Ivl_NonBuck"+",");
			sb.append("OP:L1CorrCoeff_Ivl_NonBuck"+",");
			sb.append("OP:DesignLevel"+",");			
            sb.append("\n");
            out.write(sb.toString());
            out.close();			
		}catch(IOException io){
			
		}
		
		for(int a = 0; a < designpoints.length; a++ ){
			
			if(designpoints[a][4] > 0.0){
				double t = 1.0;
	            double p0 = designpoints[a][3];
	            double phiT = designpoints[a][2];
	            double nz_mu_bucketed = designpoints[a][0];
	            double nz_var_bucketed = designpoints[a][1];
		    	
	            System.out.println("Experiment begins: "+nz_mu_bucketed+", "+nz_var_bucketed+", "+phiT+", "+p0);          
	            
		    	Statistic bucketedmean = new Statistic("bucketedMean");
	            Statistic bucketedvariance = new Statistic("bucketedVariance");
	            Statistic bucketedlag1corrcoeff = new Statistic("bucketedLag1CorrCoeff");	            
	            Statistic bucketednzmean = new Statistic("bucketedNZMean");
	            Statistic bucketednzvariance = new Statistic("bucketedNZVariance");
	            Statistic bucketednzlag1corrcoeff = new Statistic("bucketedNZLag1CorrCoeff");	            
	            Statistic bucketedibtmean = new Statistic("bucketedIBTMean");
	            Statistic bucketedibtvariance = new Statistic("bucketedIBTVariance");
	            Statistic bucketedibtlag1corrcoeff = new Statistic("bucketedIBTLag1CorrCoeff");	            
	            Statistic bucketedp0 = new Statistic("bucketedProbZero");	            
	            Statistic nonbucketednzmean = new Statistic("nonbucketedNZMean");
	            Statistic nonbucketednzvariance = new Statistic("nonbucketedNZVariance");
	            Statistic nonbucketednzlag1corrcoeff = new Statistic("nonbucketedNZlag1corrcoeff");	            
	            Statistic nonbucketedmeaninterval = new Statistic("nonbucketedmeaninterval");
	            Statistic nonbucketedvarianceinterval = new Statistic("nonbucketedvarianceinterval");
	            Statistic nonbucketedlag1corrcoeffinterval = new Statistic("nonbucketedLag1CorrCoeffinterval");            
	            
	                        

	            double [] cpp = DemandGenerator.getCPP(t, p0, phiT, nz_mu_bucketed, nz_var_bucketed);             
	            /*System.out.println("p0: "+p0);
	            System.out.println("phiT: "+phiT);
	            System.out.println("nz_mu_bucketed: "+nz_mu_bucketed); 
	            System.out.println("nz_var_bucketed: "+nz_var_bucketed);       
	            System.out.println("Lambda: "+cpp[0]);
	            System.out.println("NZMean: "+cpp[1]);
	            System.out.println("NZVar: "+cpp[2]);*/     
	            
	            for(int j = 1; j <= 50; j++){
	            	DemandGenerator dg = DemandGenerator.getAxsaterBasedAmountDistCPP(t, p0, phiT, nz_mu_bucketed, nz_var_bucketed);
	                IntermittentDemandStatistic stat = new IntermittentDemandStatistic();

                    for(int k = 0; k < 1000; k++){
	                        double x = dg.getValue();
	                        stat.collect(x);
	                }
	                
	                double mean_ = stat.getDemandStat().getAverage();
	                double variance_ = stat.getDemandStat().getVariance();
	                double lag1corrcoeff_ = stat.getDemandStat().getLag1Correlation();                
	                double nzmean_ = stat.getNonZeroDemandStat().getAverage();
	                double nzvariance_ = stat.getNonZeroDemandStat().getVariance();
	                double nzlag1corrcoeff_ = stat.getNonZeroDemandStat().getLag1Correlation();                
	                double ibtmean_ = stat.getIntervalBetweenNonZeroDemandStat().getAverage();
	                double ibtvariance_ = stat.getIntervalBetweenNonZeroDemandStat().getVariance();
	                double ibtlag1corrcoeff_ = stat.getIntervalBetweenNonZeroDemandStat().getLag1Correlation();                
	                double p0_ = stat.getProbFailure();                
	                double nonbucketednzmean_ = dg.getNonBucketedNZDemandMean();
	                double nonbucketednzvariance_ = dg.getNonBucketedNZDemandVariance();
	                double nonbucketednzlag1corrcoeff_ = dg.getNonBucketedNZDemandLag1Correlation();                
	                double nonbucketedmeaninterval_ = dg.getNonBucketedDemandIntervalMean();
	                double nonbucketedvarianceinterval_ = dg.getNonBucketedDemandIntervalVariance();            
	                double nonbucketedlag1corrcoeffinterval_ = dg.getNonBucketedDemandIntervalLag1Correlation();
	                
	                bucketedmean.collect(mean_);
	                bucketedvariance.collect(variance_);
	                bucketedlag1corrcoeff.collect(lag1corrcoeff_);                
	                bucketednzmean.collect(nzmean_);
	                bucketednzvariance.collect(nzvariance_);
	                bucketednzlag1corrcoeff.collect(nzlag1corrcoeff_);                
	                bucketedibtmean.collect(ibtmean_);
	                bucketedibtvariance.collect(ibtvariance_);
	                bucketedibtlag1corrcoeff.collect(ibtlag1corrcoeff_);                
	                bucketedp0.collect(p0_);                
	                nonbucketednzmean.collect(nonbucketednzmean_);
	                nonbucketednzvariance.collect(nonbucketednzvariance_);
	                nonbucketednzlag1corrcoeff.collect(nonbucketednzlag1corrcoeff_);                
	                nonbucketedmeaninterval.collect(nonbucketedmeaninterval_);
	                nonbucketedvarianceinterval.collect(nonbucketedvarianceinterval_);
	                nonbucketedlag1corrcoeffinterval.collect(nonbucketedlag1corrcoeffinterval_);
	                
	            }            
	    		try{
	    			BufferedWriter out = new BufferedWriter(new FileWriter("DemandGenerator.csv", true));
	    			StringBuilder sb = new StringBuilder();
	    			sb.append(t+",");
	    			sb.append(p0+",");
	    			sb.append(phiT+",");
	    			sb.append(nz_mu_bucketed+",");
	    			sb.append(nz_var_bucketed+",");
	    			sb.append((nz_var_bucketed/(nz_mu_bucketed*nz_mu_bucketed))+",");
	    			sb.append(cpp[0]+",");
	    			sb.append(cpp[1]+",");
	    			sb.append(cpp[2]+",");
	    			String s1 = "";
	    			if(p0 > 0.7){
	    				s1 = s1 + "H_INT-";
	    			}else{
	    				s1 = s1 + "L_INT-";
	    			}
	    			if((nz_var_bucketed/nz_mu_bucketed) > 0.5){
	    				s1 = s1 + "H_LMP-";
	    			}else{
	    				s1 = s1 + "L_LMP-";
	    			}
	    			if(phiT > 0.5){
	    				s1 = s1 + "H_BST-";
	    			}else{
	    				s1 = s1 + "L_BST-";
	    			}
	    			sb.append(s1+",");			
	    			sb.append(bucketedp0.getAverage()+",");
	    			sb.append(bucketedmean.getAverage()+",");
	    			sb.append(bucketedvariance.getAverage()+",");
	    			sb.append(bucketedlag1corrcoeff.getAverage()+",");
	    			sb.append(bucketednzmean.getAverage()+",");
	    			sb.append(bucketednzvariance.getAverage()+",");
	    			sb.append((bucketednzmean.getAverage()/bucketednzvariance.getAverage())+",");
	    			sb.append(bucketednzlag1corrcoeff.getAverage()+",");
	    			sb.append(bucketedibtmean.getAverage()+",");
	    			sb.append(bucketedibtvariance.getAverage()+",");
	    			sb.append(bucketedibtlag1corrcoeff.getAverage()+",");
	    			sb.append(nonbucketednzmean.getAverage()+",");
	    			sb.append(nonbucketednzvariance.getAverage()+",");
	    			sb.append(nonbucketednzlag1corrcoeff.getAverage()+",");
	    			sb.append(nonbucketedmeaninterval.getAverage()+",");
	    			sb.append(nonbucketedvarianceinterval.getAverage()+",");
	    			sb.append(nonbucketedlag1corrcoeffinterval.getAverage()+",");
	    			String s2 = "";
	    			if(bucketedp0.getAverage() > 0.7){
	    				s2 = s2 + "H_INT-";
	    			}else{
	    				s2 = s2 + "L_INT-";
	    			}
	    			if((bucketednzvariance.getAverage()/(bucketednzmean.getAverage()*bucketednzmean.getAverage())) > 0.5){
	    				s2 = s2 + "H_LMP-";
	    			}else{
	    				s2 = s2 + "L_LMP-";
	    			}
	    			if(nonbucketedlag1corrcoeffinterval.getAverage() > 0.5){
	    				s2 = s2 + "H_BST-";
	    			}else{
	    				s2 = s2 + "L_BST-";
	    			}
	    			sb.append(s2+",");			
	                sb.append("\n");
	                out.write(sb.toString());
	                out.close();			
	    		}catch(IOException io){    
	    			
	    		} 
	    		
	    		System.out.println("Experiment ends: "+nz_mu_bucketed+", "+nz_var_bucketed+", "+phiT+", "+p0);
		        			
			}else{
				double t = 1.0;
	            double p0 = designpoints[a][3];
	            double phiT = designpoints[a][2];
	            double nz_mu_bucketed = designpoints[a][0];
	            double nz_var_bucketed = designpoints[a][1];
	            
	            System.out.println("Experiment skipped: "+nz_mu_bucketed+", "+nz_var_bucketed+", "+phiT+", "+p0);
	            
	            try{
	            	BufferedWriter out = new BufferedWriter(new FileWriter("DemandGenerator.csv", true));
	    			StringBuilder sb = new StringBuilder();
	    			sb.append(t+",");
	    			sb.append(p0+",");
	    			sb.append(phiT+",");
	    			sb.append(nz_mu_bucketed+",");
	    			sb.append(nz_var_bucketed+",");
	    			sb.append((nz_var_bucketed/(nz_mu_bucketed*nz_mu_bucketed))+",");
	    			sb.append("NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			String s1 = ",";
	    			if(p0 > 0.7){
	    				s1 = s1 + "H_INT-";
	    			}else{
	    				s1 = s1 + "L_INT-";
	    			}
	    			if((nz_var_bucketed/(nz_mu_bucketed*nz_mu_bucketed)) > 0.5){
	    				s1 = s1 + "H_LMP-";
	    			}else{
	    				s1 = s1 + "L_LMP-";
	    			}
	    			if(phiT > 0.5){
	    				s1 = s1 + "H_BST-";
	    			}else{
	    				s1 = s1 + "L_BST-";
	    			}
	    			sb.append(s1+",");	
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");
	    			sb.append(", NA");		
	                sb.append("\n");
	                out.write(sb.toString());
	                out.close();	            	
	            }catch(IOException io){	            	
	            }            
				
			}	
	    	    	
	    }
		
	}
	
	
	public static void main(String[] args) { 
		DemandGeneratorExperiment exp = new DemandGeneratorExperiment();
		exp.createDesignPoints();
		exp.runExperiment();
		
		
	}
	
	

}
