package jslx.forecasting.demandgeneration;

import jsl.utilities.random.GetValueIfc;
import jsl.utilities.random.ParametersIfc;
import jsl.utilities.random.distributions.Bernoulli;
import jsl.utilities.random.rng.RandomStreamIfc;

public class TwoSMarkovChainGeneration implements ParametersIfc, GetValueIfc, RandomStreamIfc{
    private int myStateOfXt;
    private int FIRSTEVENT;
    private double myProbSuccessAfterSuccess;
    private double myProbSuccessAfterFailure;
    private double myProbSuccess;
    private double myProbFailure;
    private Bernoulli myBn1;
    private Bernoulli myBn0;	    
    
    public TwoSMarkovChainGeneration(){
    	this(1,0.5,0.5);
    }
    
    public TwoSMarkovChainGeneration(int firstEvent,double probSuccessAfterSuccess,double probSuccessAfterFailure){
        myBn1 = new Bernoulli(probSuccessAfterSuccess);
        myBn0 = new Bernoulli(probSuccessAfterFailure);
        FIRSTEVENT = firstEvent;
        myStateOfXt = FIRSTEVENT;
        double [] prob = {probSuccessAfterSuccess, probSuccessAfterFailure}; 
        setParameters(prob);
    }

    public void setStateOfXt(int myStateOfXt) {
        this.myStateOfXt = myStateOfXt;
    }
    
    public double getValue(){
        if(myStateOfXt ==1){
            myStateOfXt = (int)myBn1.getValue();
            return myStateOfXt;
        }else{
            myStateOfXt = (int)myBn0.getValue();
            return myStateOfXt;
        }
    }
    
    public double getProbFailure() {
        return myProbFailure;
    }
    public double getProbSuccess() {
        return myProbSuccess;
    }
    public double getProbSuccessAfterFailure() {
        return myProbSuccessAfterFailure;
    }
    public double getProbSuccessAfterSuccess() {
        return myProbSuccessAfterSuccess;
    }
    public double[] getParameters() {
        double [] prob = {myProbSuccessAfterSuccess, myProbSuccessAfterFailure};
        return prob;
    }
    public void setParameters(double[] parameters) {
        myProbSuccessAfterSuccess=parameters[0];
        myProbSuccessAfterFailure=parameters[1];
        myBn1.setProbabilityOfSuccess(myProbSuccessAfterSuccess);
        myBn0.setProbabilityOfSuccess(myProbSuccessAfterFailure); 
        myProbFailure = 1-(myProbSuccessAfterFailure/(1-myProbSuccessAfterSuccess+myProbSuccessAfterFailure));
        myProbSuccess = 1-myProbFailure;
    }
    public void resetNextSubstream() {
        myBn1.resetNextSubstream();
        myBn0.resetNextSubstream();		
    }
    public void resetStartStream() {
        myBn1.resetStartStream();
        myBn0.resetStartStream();	
    }
    public void resetStartSubstream() {
        myBn1.resetStartSubstream();
        myBn0.resetStartSubstream();	
    }
    public void setAntithetic(boolean flag) {
        myBn1.setAntithetic(flag);
        myBn0.setAntithetic(flag);	
    }	    
}
