/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jslx.forecasting.testing;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import jslx.forecasting.AverageDemand;
import jslx.forecasting.Croston;
import jslx.forecasting.CumulativeAverage;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.IterativeDataSource;
import jslx.forecasting.MCARTA;
import jslx.forecasting.MovingAverage;
import jslx.forecasting.NaiveForecastTechnique;
import jslx.forecasting.SimpleExponentialSmoothing;
import jslx.forecasting.SyntetosBoylan;
import jslx.forecasting.demandgeneration.BatchOnOffDemandGenerator;
import jslx.forecasting.metrics.ForecastMetric;
import jslx.forecasting.metrics.RelativeForecastMetric;

/**
 *
 * @author vvarghe
 */
public class ForecastExperiment {
    
    
    /*
     * List of forecast techniques
     * 
     */
    public List <ForecastTechnique> myForecastTechniques;
    
    
    /*
     * List of data sources. It can be a demand generator or any other iterative
     * data sources.
     * 
     */
    public List <BatchOnOffDemandGenerator> myDataSources;
    private BatchOnOffDemandGenerator myCurrentDataSource;
    private double [] myCurrentDataSourceHistory;
    
    /*
     * Array that stores the factorial levels of the factor: steps ahead forecast
     * 
     */
    public int [] myLevelsStepsAheadForecast;
    
    /*
     * Array that stores the factorial levels of the factor: demand history size to 
     * initialize the forecast
     * 
     */
    public int [] myLevelsDemandHistorySize;
    public int myCurrentDemandHistorySize;
    
    public int myNumReplications;
    public int myRunLength;
    public int myCurrentReplication;
    
    public ForecastExperiment(){
    	this(10, 200);
    	
    }
    
    public ForecastExperiment(int rep, int length){
    	myLevelsDemandHistorySize = new int [1];
    	myNumReplications = rep;
    	myRunLength = length;
    	myDataSources = new ArrayList <BatchOnOffDemandGenerator>();
    	myForecastTechniques = new ArrayList<ForecastTechnique>();
    	myLevelsDemandHistorySize [0] = 100;
    	myCurrentDemandHistorySize = myLevelsDemandHistorySize [0];
    	myCurrentReplication = 0;
    	System.out.println("Number of replications of the experiment: "+ myNumReplications);
    	System.out.println("Run length of the experimen: "+ myRunLength);
    	System.out.println();
		
    	try {
	        BufferedWriter out = new BufferedWriter(new FileWriter("ForecastExperiment.csv"));
	        out.write("DataSource,Busy,Idle,CV,HistorySize,Replication," +
	        		"ForecastTechnique,MAD,MSE,MAPE,Bias,Variance,MeanUAPE," +
	        		"MeanRAE,RelMeanAE,MeanASE," +
	        		"Mean,Var,Phi,NZMean,NZVar,NZPhi," +
	        		"IdlPrdMean,IdlPrdVar,IdlPrdPhi," +
	        		"BsyPrdMean,BsyPrdVar,BsyPrdPhi,p01,p11,p0,CompTime");
	        out.close();
    	} 
			catch (IOException e1){	
    	}
    	
    }
    
    /*
     * The method adds the data sources of the experiment
     * 
     */
    public void addDataSource(BatchOnOffDemandGenerator ds){
        this.myDataSources.add(ds);
    	System.out.println("Added the Demand source "+ds.getName());
    	System.out.println();
        
    }
    
    /*
     * The method adds the forecast techniques of the experiment
     * Should we add different forecast technique for levels of steps ahead forecast
     * How to iterate through it?
     * 
     */
    public void addForecastTechnique(ForecastTechnique ft){
        myForecastTechniques.add(ft);
    	System.out.println("Added the Forecast technique "+ft.getName());
    	System.out.println();
    }
    
    /*
     * The method initialize the experiment
     * 
     */
    public void initializeExperiment(){    
        /*
         * I am not sure what has to be added here
         * 
         */
    	System.out.println("Initialize the experiment");
    	System.out.println();   	
    }
    
    /*
     * The method runs the experiment
     * 
     */
    public void runExperiment(){
    	System.out.println("Experiment run begins");
    	System.out.println();
        iterateThroughDataSource();
    }
    
    /*
     * The method iterate through each data sources
     * 
     */
    public void iterateThroughDataSource(){
        for(BatchOnOffDemandGenerator ds: this.myDataSources){
            myCurrentDataSource = ds;
            for(int i = 0; i < myLevelsDemandHistorySize.length; i++){
            	myCurrentDemandHistorySize = myLevelsDemandHistorySize[i];
            	System.out.println("Select the demand source "+myCurrentDataSource.getName()); 
            	
                runReplication();
            }
        }
    }
    
    /*
     * The method iterate through each replication: (1) init. the replication
     * (2) run each rep (3) write the errors (4) and reset each replication
     * 
     */
    public void runReplication(){
        for(int i = 1; i <= this.myNumReplications; i++){
        	myCurrentReplication = i;
        	System.out.println("Initialize the replication "+i);
            initializeEachReplication();
        	System.out.println("Running the replication "+i);
            runEachReplication();
            for(ForecastTechnique ft: myForecastTechniques){
                writeError(ft);
            }
        	System.out.println("Reset the replication "+i);
            resetEachReplication();
        }
    }
    
    /*
     * The method gets the demand history according to the demand history length
     * and then initialize the forecast techniques. How to iterate through the levels
     * of demand history length
     * 
     */
    public void initializeEachReplication(){
        for(ForecastTechnique ft: myForecastTechniques){ 
        	double [] history = myCurrentDataSource.getValue(myCurrentDemandHistorySize);
        	myCurrentDataSourceHistory = history; 
        	double [] forecasts = new double[1];
        	//TODO
        	ft.reset();
        	ft.removeAllMetrics();
        	ft.attachMetric(ForecastMetric.createMeanUAPE());
        	//ft.attachMetric(ForecastMetric.createMedianUAPE()); 
        	ft.attachMetric(ForecastMetric.createMeanRAE());
        	//ft.attachMetric(ForecastMetric.createMedianRelAE());
        	//ft.attachMetric(ForecastMetric.createGeoMeanRAE());
        	ft.attachMetric(ForecastMetric.createRelMeanAE());
        	ft.attachMetric(ForecastMetric.createMeanASE());
        	
        	ft.initializeHistory(history, forecasts, ft.estimateInitialForecastParameter(history));
        	System.out.println("Current data source is observed by the Forecast technique "+ft.getName());
        }
    }
    
    /*
     * The method runs each replication, across the run length and triggers each 
     * forecast technique
     * 
     */
    public void runEachReplication(){
        for(int i = 1; i <= this.myRunLength; i++){
            double d = this.myCurrentDataSource.getValue();
        	//System.out.println("Time index "+i+": "+d); 
            for(ForecastTechnique ft: myForecastTechniques){
            	ft.collect(d);
            }         
        }
    }
    
    public void resetEachReplication(){
    	//TODO change the random stream?
        myCurrentDataSource.reset();
        
    }
    
    /*
     * The method observes MAD, MSE and MAPE
     * Write into a file
     * 
     */
    public void writeError(ForecastTechnique ft){
    	
    	ft.getAttachedMetrics().get(0).computeMetricValue();
    	ft.getAttachedMetrics().get(1).computeMetricValue();
    	ft.getAttachedMetrics().get(2).computeMetricValue();
    	ft.getAttachedMetrics().get(3).computeMetricValue();   
    	//ft.getAttachedMetrics().get(4).computeMetricValue(); 
    	//ft.getAttachedMetrics().get(5).computeMetricValue();
    	//ft.getAttachedMetrics().get(6).computeMetricValue();
    	
    	System.out.print(myCurrentDataSource.getName()+"\t");    	
    	System.out.print(myCurrentDemandHistorySize+"\t");    	
    	System.out.print(myCurrentReplication+"\t");   	
    	System.out.print(ft.getName()+"\t"+"\t"+"\t"+"\t");   	
    	System.out.print(ft.getMAD()+"\t");   	
    	System.out.print(ft.getMSE()+"\t");   	
    	System.out.print(ft.getMAPE()+"\t"); 	
    	System.out.print(ft.getErrorStatistics().getAverage()+"\t");	
    	System.out.print(ft.getErrorStatistics().getVariance()+"\t"); 
    	System.out.print(ft.getAttachedMetrics().get(0).getMetricValue()+"\t");    	
    	System.out.print(ft.getAttachedMetrics().get(1).getMetricValue()+"\t");  	
    	System.out.print(ft.getAttachedMetrics().get(2).getMetricValue()+"\t");    	
    	System.out.print(ft.getAttachedMetrics().get(3).getMetricValue()+"\t"); 	
    	//System.out.print(ft.getAttachedMetrics().get(4).getMetricValue()+"\t");    	
    	//System.out.print(ft.getAttachedMetrics().get(5).getMetricValue()+"\t"); 	
    	//System.out.print(ft.getAttachedMetrics().get(6).getMetricValue()+"\t");   
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getAverage()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getVariance()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getLag1Correlation()+"\t"); 
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getAverage()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getVariance()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getLag1Correlation()+"\t");
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getAverage()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getVariance()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getLag1Correlation()+"\t");
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getAverage()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getVariance()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getLag1Correlation()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getProbSuccessAfterFailure()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getProbSuccessAfterSuccess()+"\t");  
    	System.out.print(myCurrentDataSource.getIntermittentDemandStatistic().getProbFailure()+"\t");  
    	System.out.print(ft.getTimeElapsed()+"\t"); 
    	
    	System.out.println();
    	
    	try {
	        BufferedWriter out = new BufferedWriter(new FileWriter("ForecastExperiment.csv",true));
	        out.newLine();
	        out.write(myCurrentDataSource.getName()
	        		+","+myCurrentDataSource.getName().charAt(2)
	        		+","+myCurrentDataSource.getName().charAt(3)
	        		+","+myCurrentDataSource.getName().charAt(4)
	        		+","+myCurrentDemandHistorySize
	        		+","+myCurrentReplication
	        		+","+ft.getName()
	        		+","+ft.getMAD()
	        		+","+ft.getMSE()
	        		+","+ft.getMAPE()
	        		+","+ft.getErrorStatistics().getAverage()
	        		+","+ft.getErrorStatistics().getVariance()
    				+","+ft.getAttachedMetrics().get(0).getMetricValue()
    				+","+ft.getAttachedMetrics().get(1).getMetricValue()
    				+","+ft.getAttachedMetrics().get(2).getMetricValue()
    				+","+ft.getAttachedMetrics().get(3).getMetricValue()
    				//+","+ft.getAttachedMetrics().get(4).getMetricValue()
    				//+","+ft.getAttachedMetrics().get(5).getMetricValue() 
    				//+","+ft.getAttachedMetrics().get(6).getMetricValue()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getAverage()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getVariance()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getLag1Correlation() 
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getAverage()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getVariance()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getNonZeroDemandStat().getLag1Correlation()
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getAverage()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getVariance()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenNonZeroDemandStat().getLag1Correlation()
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getAverage()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getVariance()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getIntervalBetweenZeroDemandsStat().getLag1Correlation() 
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getProbSuccessAfterFailure()
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getProbSuccessAfterSuccess()  
    		    	+","+myCurrentDataSource.getIntermittentDemandStatistic().getProbFailure()
    		    	+","+ft.getTimeElapsed()); 
	        out.close();
    	} 
			catch (IOException e1){	
    	}
    
    }
    
    public static void main(String args []){
    	
    	ForecastExperiment fe = new ForecastExperiment(30, 200); 
    	
    	/*double [] demandscenario1111 = {1.11, 1.11, 1.8, 0.2025}; 
    	double [] demandscenario1211 = {1.11, 2.0, 1.8, 0.2025}; 
    	double [] demandscenario1311 = {1.11, 4.0, 1.8, 0.2025}; 
    	double [] demandscenario1411 = {1.11, 10.0, 1.8, 0.2025}; 
    	
    	double [] demandscenario2111 = {2.0, 1.11, 1.8, 0.2025}; 
    	double [] demandscenario2211 = {2.0, 2.0, 1.8, 0.2025}; 
    	double [] demandscenario2311 = {2.0, 4.0, 1.8, 0.2025}; 
    	double [] demandscenario2411 = {2.0, 10.0, 1.8, 0.2025}; 
    	
    	double [] demandscenario3111 = {4.0, 1.11, 1.8, 0.2025}; 
    	double [] demandscenario3211 = {4.0, 2.0, 1.8, 0.2025}; 
    	double [] demandscenario3311 = {4.0, 4.0, 1.8, 0.2025}; 
    	double [] demandscenario3411 = {4.0, 10.0, 1.8, 0.2025}; 
    	
    	double [] demandscenario4111 = {10.0, 1.11, 1.8, 0.2025}; 
    	double [] demandscenario4211 = {10.0, 2.0, 1.8, 0.2025}; 
    	double [] demandscenario4311 = {10.0, 4.0, 1.8, 0.2025}; 
    	double [] demandscenario4411 = {10.0, 10.0, 1.8, 0.2025}; 
    	
    	double [] demandscenario1122 = {1.11, 1.11, 1.8, 1.8225}; 
    	double [] demandscenario1222 = {1.11, 2.0, 1.8, 1.8225}; 
    	double [] demandscenario1322 = {1.11, 4.0, 1.8, 1.8225}; 
    	double [] demandscenario1422 = {1.11, 10.0, 1.8, 1.8225}; 
    	
    	double [] demandscenario2122 = {2.0, 1.11, 1.8, 1.8225}; 
    	double [] demandscenario2222 = {2.0, 2.0, 1.8, 1.8225}; 
    	double [] demandscenario2322 = {2.0, 4.0, 1.8, 1.8225}; 
    	double [] demandscenario2422 = {2.0, 10.0, 1.8, 1.8225}; 
    	
    	double [] demandscenario3122 = {4.0, 1.11, 1.8, 1.8225}; 
    	double [] demandscenario3222 = {4.0, 2.0, 1.8, 1.8225}; 
    	double [] demandscenario3322 = {4.0, 4.0, 1.8, 1.8225}; 
    	double [] demandscenario3422 = {4.0, 10.0, 1.8, 1.8225}; 
    	
    	double [] demandscenario4122 = {10.0, 1.11, 1.8, 1.8225}; 
    	double [] demandscenario4222 = {10.0, 2.0, 1.8, 1.8225}; 
    	double [] demandscenario4322 = {10.0, 4.0, 1.8, 1.8225}; 
    	double [] demandscenario4422 = {10.0, 10.0, 1.8, 1.8225};*/ 
    	
    	double [] demandscenario1133 = {1.11, 1.11, 1.8, 5.0625}; 
    	double [] demandscenario1233 = {1.11, 2.0, 1.8, 5.0625}; 
    	double [] demandscenario1333 = {1.11, 4.0, 1.8, 5.0625}; 
    	double [] demandscenario1433 = {1.11, 10.0, 1.8, 5.0625}; 
    	
    	double [] demandscenario2133 = {2.0, 1.11, 1.8, 5.0625}; 
    	double [] demandscenario2233 = {2.0, 2.0, 1.8, 5.0625}; 
    	double [] demandscenario2333 = {2.0, 4.0, 1.8, 5.0625}; 
    	double [] demandscenario2433 = {2.0, 10.0, 1.8, 5.0625}; 
    	
    	double [] demandscenario3133 = {4.0, 1.11, 1.8, 5.0625}; 
    	double [] demandscenario3233 = {4.0, 2.0, 1.8, 5.0625}; 
    	double [] demandscenario3333 = {4.0, 4.0, 1.8, 5.0625}; 
    	double [] demandscenario3433 = {4.0, 10.0, 1.8, 5.0625}; 
    	
    	double [] demandscenario4133 = {10.0, 1.11, 1.8, 5.0625}; 
    	double [] demandscenario4233 = {10.0, 2.0, 1.8, 5.0625}; 
    	double [] demandscenario4333 = {10.0, 4.0, 1.8, 5.0625}; 
    	double [] demandscenario4433 = {10.0, 10.0, 1.8, 5.0625}; 
    	
    	
    	/*BatchOnOffDemandGenerator dg1111=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1111[0], demandscenario1111[1], demandscenario1111[2], demandscenario1111[3], "DG1111");
    	BatchOnOffDemandGenerator dg1211=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1211[0], demandscenario1211[1], demandscenario1211[2], demandscenario1211[3], "DG1211");
    	BatchOnOffDemandGenerator dg1311=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1311[0], demandscenario1311[1], demandscenario1311[2], demandscenario1311[3], "DG1311");
    	BatchOnOffDemandGenerator dg1411=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1411[0], demandscenario1411[1], demandscenario1411[2], demandscenario1411[3], "DG1411");
    	BatchOnOffDemandGenerator dg2111=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2111[0], demandscenario2111[1], demandscenario2111[2], demandscenario2111[3], "DG2111");
    	BatchOnOffDemandGenerator dg2211=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2211[0], demandscenario2211[1], demandscenario2211[2], demandscenario2211[3], "DG2211");
    	BatchOnOffDemandGenerator dg2311=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2311[0], demandscenario2311[1], demandscenario2311[2], demandscenario2311[3], "DG2311");
    	BatchOnOffDemandGenerator dg2411=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2411[0], demandscenario2411[1], demandscenario2411[2], demandscenario2411[3], "DG2411");
    	BatchOnOffDemandGenerator dg3111=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3111[0], demandscenario3111[1], demandscenario3111[2], demandscenario3111[3], "DG3111");
    	BatchOnOffDemandGenerator dg3211=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3211[0], demandscenario3211[1], demandscenario3211[2], demandscenario3211[3], "DG3211");
    	BatchOnOffDemandGenerator dg3311=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3311[0], demandscenario3311[1], demandscenario3311[2], demandscenario3311[3], "DG3311");
    	BatchOnOffDemandGenerator dg3411=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3411[0], demandscenario3411[1], demandscenario3411[2], demandscenario3411[3], "DG3411");
    	BatchOnOffDemandGenerator dg4111=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4111[0], demandscenario4111[1], demandscenario4111[2], demandscenario4111[3], "DG4111");
    	BatchOnOffDemandGenerator dg4211=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4211[0], demandscenario4211[1], demandscenario4211[2], demandscenario4211[3], "DG4211");
    	BatchOnOffDemandGenerator dg4311=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4311[0], demandscenario4311[1], demandscenario4311[2], demandscenario4311[3], "DG4311");
    	BatchOnOffDemandGenerator dg4411=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4411[0], demandscenario4411[1], demandscenario4411[2], demandscenario4411[3], "DG4411");
    	
    	BatchOnOffDemandGenerator dg1122=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1122[0], demandscenario1122[1], demandscenario1122[2], demandscenario1122[3], "DG1122");
    	BatchOnOffDemandGenerator dg1222=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1222[0], demandscenario1222[1], demandscenario1222[2], demandscenario1222[3], "DG1222");
    	BatchOnOffDemandGenerator dg1322=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1322[0], demandscenario1322[1], demandscenario1322[2], demandscenario1322[3], "DG1322");
    	BatchOnOffDemandGenerator dg1422=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1422[0], demandscenario1422[1], demandscenario1422[2], demandscenario1422[3], "DG1422");
    	BatchOnOffDemandGenerator dg2122=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2122[0], demandscenario2122[1], demandscenario2122[2], demandscenario2122[3], "DG2122");
    	BatchOnOffDemandGenerator dg2222=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2222[0], demandscenario2222[1], demandscenario2222[2], demandscenario2222[3], "DG2222");
    	BatchOnOffDemandGenerator dg2322=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2322[0], demandscenario2322[1], demandscenario2322[2], demandscenario2322[3], "DG2322");
    	BatchOnOffDemandGenerator dg2422=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2422[0], demandscenario2422[1], demandscenario2422[2], demandscenario2422[3], "DG2422");
    	BatchOnOffDemandGenerator dg3122=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3122[0], demandscenario3122[1], demandscenario3122[2], demandscenario3122[3], "DG3122");
    	BatchOnOffDemandGenerator dg3222=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3222[0], demandscenario3222[1], demandscenario3222[2], demandscenario3222[3], "DG3222");
    	BatchOnOffDemandGenerator dg3322=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3322[0], demandscenario3322[1], demandscenario3322[2], demandscenario3322[3], "DG3322");
    	BatchOnOffDemandGenerator dg3422=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3422[0], demandscenario3422[1], demandscenario3422[2], demandscenario3422[3], "DG3422");
    	BatchOnOffDemandGenerator dg4122=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4122[0], demandscenario4122[1], demandscenario4122[2], demandscenario4122[3], "DG4122");
    	BatchOnOffDemandGenerator dg4222=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4222[0], demandscenario4222[1], demandscenario4222[2], demandscenario4222[3], "DG4222");
    	BatchOnOffDemandGenerator dg4322=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4322[0], demandscenario4322[1], demandscenario4322[2], demandscenario4322[3], "DG4322");
    	BatchOnOffDemandGenerator dg4422=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4422[0], demandscenario4422[1], demandscenario4422[2], demandscenario4422[3], "DG4422");
*/
    	BatchOnOffDemandGenerator dg1133=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1133[0], demandscenario1133[1], demandscenario1133[2], demandscenario1133[3], "DG1133");
    	BatchOnOffDemandGenerator dg1233=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1233[0], demandscenario1233[1], demandscenario1233[2], demandscenario1233[3], "DG1233");
    	BatchOnOffDemandGenerator dg1333=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1333[0], demandscenario1333[1], demandscenario1333[2], demandscenario1333[3], "DG1333");
    	BatchOnOffDemandGenerator dg1433=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario1433[0], demandscenario1433[1], demandscenario1433[2], demandscenario1433[3], "DG1433");
    	BatchOnOffDemandGenerator dg2133=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2133[0], demandscenario2133[1], demandscenario2133[2], demandscenario2133[3], "DG2133");
    	BatchOnOffDemandGenerator dg2233=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2233[0], demandscenario2233[1], demandscenario2233[2], demandscenario2233[3], "DG2233");
    	BatchOnOffDemandGenerator dg2333=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2333[0], demandscenario2333[1], demandscenario2333[2], demandscenario2333[3], "DG2333");
    	BatchOnOffDemandGenerator dg2433=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario2433[0], demandscenario2433[1], demandscenario2433[2], demandscenario2433[3], "DG2433");
    	BatchOnOffDemandGenerator dg3133=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3133[0], demandscenario3133[1], demandscenario3133[2], demandscenario3133[3], "DG3133");
    	BatchOnOffDemandGenerator dg3233=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3233[0], demandscenario3233[1], demandscenario3233[2], demandscenario3233[3], "DG3233");
    	BatchOnOffDemandGenerator dg3333=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3333[0], demandscenario3333[1], demandscenario3333[2], demandscenario3333[3], "DG3333");
    	BatchOnOffDemandGenerator dg3433=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario3433[0], demandscenario3433[1], demandscenario3433[2], demandscenario3433[3], "DG3433");
    	BatchOnOffDemandGenerator dg4133=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4133[0], demandscenario4133[1], demandscenario4133[2], demandscenario4133[3], "DG4133");
    	BatchOnOffDemandGenerator dg4233=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4233[0], demandscenario4233[1], demandscenario4233[2], demandscenario4233[3], "DG4233");
    	BatchOnOffDemandGenerator dg4333=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4333[0], demandscenario4333[1], demandscenario4333[2], demandscenario4333[3], "DG4333");
    	BatchOnOffDemandGenerator dg4433=  BatchOnOffDemandGenerator.getDefaultBatchOnOffDemandGenerator(demandscenario4433[0], demandscenario4433[1], demandscenario4433[2], demandscenario4433[3], "DG4433");

    	/*fe.addDataSource(dg1111);
    	fe.addDataSource(dg1211);
    	fe.addDataSource(dg1311);
    	fe.addDataSource(dg1411);
    	fe.addDataSource(dg2111);
    	fe.addDataSource(dg2211);
    	fe.addDataSource(dg2311);
    	fe.addDataSource(dg2411);
    	fe.addDataSource(dg3111);
    	fe.addDataSource(dg3211);
    	fe.addDataSource(dg3311);
    	fe.addDataSource(dg3411);
    	fe.addDataSource(dg4111);
    	fe.addDataSource(dg4211);
    	fe.addDataSource(dg4311);
    	fe.addDataSource(dg4411);
    	
    	fe.addDataSource(dg1122);
    	fe.addDataSource(dg1222);
    	fe.addDataSource(dg1322);
    	fe.addDataSource(dg1422);
    	fe.addDataSource(dg2122);
    	fe.addDataSource(dg2222);
    	fe.addDataSource(dg2322);
    	fe.addDataSource(dg2422);
    	fe.addDataSource(dg3122);
    	fe.addDataSource(dg3222);
    	fe.addDataSource(dg3322);
    	fe.addDataSource(dg3422);
    	fe.addDataSource(dg4122);
    	fe.addDataSource(dg4222);
    	fe.addDataSource(dg4322);
    	fe.addDataSource(dg4422);*/
    	
    	fe.addDataSource(dg1133);
    	fe.addDataSource(dg1233);
    	fe.addDataSource(dg1333);
    	fe.addDataSource(dg1433);
    	fe.addDataSource(dg2133);
    	fe.addDataSource(dg2233);
    	fe.addDataSource(dg2333);
    	fe.addDataSource(dg2433);
    	fe.addDataSource(dg3133);
    	fe.addDataSource(dg3233);
    	fe.addDataSource(dg3333);
    	fe.addDataSource(dg3433);
    	fe.addDataSource(dg4133);
    	fe.addDataSource(dg4233);
    	fe.addDataSource(dg4333);
    	fe.addDataSource(dg4433);
    	
    	/*dg1111.resetNextSubstream();
    	dg1211.resetNextSubstream();
    	dg1311.resetNextSubstream();
    	dg1411.resetNextSubstream();
    	dg2111.resetNextSubstream();
    	dg2211.resetNextSubstream();
    	dg2311.resetNextSubstream();
    	dg2411.resetNextSubstream();
    	dg3111.resetNextSubstream();
    	dg3211.resetNextSubstream();
    	dg3311.resetNextSubstream();
    	dg3411.resetNextSubstream();
    	dg4111.resetNextSubstream();
    	dg4211.resetNextSubstream();
    	dg4311.resetNextSubstream();
    	dg4411.resetNextSubstream();
    	
    	dg1122.resetNextSubstream();
    	dg1222.resetNextSubstream();
    	dg1322.resetNextSubstream();
    	dg1422.resetNextSubstream();
    	dg2122.resetNextSubstream();
    	dg2222.resetNextSubstream();
    	dg2322.resetNextSubstream();
    	dg2422.resetNextSubstream();
    	dg3122.resetNextSubstream();
    	dg3222.resetNextSubstream();
    	dg3322.resetNextSubstream();
    	dg3422.resetNextSubstream();
    	dg4122.resetNextSubstream();
    	dg4222.resetNextSubstream();
    	dg4322.resetNextSubstream();
    	dg4422.resetNextSubstream()*/
    	
    	dg1133.resetNextSubstream();
    	dg1233.resetNextSubstream();
    	dg1333.resetNextSubstream();
    	dg1433.resetNextSubstream();
    	dg2133.resetNextSubstream();
    	dg2233.resetNextSubstream();
    	dg2333.resetNextSubstream();
    	dg2433.resetNextSubstream();
    	dg3133.resetNextSubstream();
    	dg3233.resetNextSubstream();
    	dg3333.resetNextSubstream();
    	dg3433.resetNextSubstream();
    	dg4133.resetNextSubstream();
    	dg4233.resetNextSubstream();
    	dg4333.resetNextSubstream();
    	dg4433.resetNextSubstream();
    	
    	/*dg1111.resetNextSubstream();
    	dg1211.resetNextSubstream();
    	dg1311.resetNextSubstream();
    	dg1411.resetNextSubstream();
    	dg2111.resetNextSubstream();
    	dg2211.resetNextSubstream();
    	dg2311.resetNextSubstream();
    	dg2411.resetNextSubstream();
    	dg3111.resetNextSubstream();
    	dg3211.resetNextSubstream();
    	dg3311.resetNextSubstream();
    	dg3411.resetNextSubstream();
    	dg4111.resetNextSubstream();
    	dg4211.resetNextSubstream();
    	dg4311.resetNextSubstream();
    	dg4411.resetNextSubstream();
    	
    	dg1122.resetNextSubstream();
    	dg1222.resetNextSubstream();
    	dg1322.resetNextSubstream();
    	dg1422.resetNextSubstream();
    	dg2122.resetNextSubstream();
    	dg2222.resetNextSubstream();
    	dg2322.resetNextSubstream();
    	dg2422.resetNextSubstream();
    	dg3122.resetNextSubstream();
    	dg3222.resetNextSubstream();
    	dg3322.resetNextSubstream();
    	dg3422.resetNextSubstream();
    	dg4122.resetNextSubstream();
    	dg4222.resetNextSubstream();
    	dg4322.resetNextSubstream();
    	dg4422.resetNextSubstream()*/
    	
    	dg1133.resetNextSubstream();
    	dg1233.resetNextSubstream();
    	dg1333.resetNextSubstream();
    	dg1433.resetNextSubstream();
    	dg2133.resetNextSubstream();
    	dg2233.resetNextSubstream();
    	dg2333.resetNextSubstream();
    	dg2433.resetNextSubstream();
    	dg3133.resetNextSubstream();
    	dg3233.resetNextSubstream();
    	dg3333.resetNextSubstream();
    	dg3433.resetNextSubstream();
    	dg4133.resetNextSubstream();
    	dg4233.resetNextSubstream();
    	dg4333.resetNextSubstream();
    	dg4433.resetNextSubstream();
    	
    	double [] controlparameters1 = {1000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta1 = new MCARTA(controlparameters1,"MCARTA1");
    	
    	double [] controlparameters2 = {10000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta2 = new MCARTA(controlparameters2,"MCARTA2");
    	
    	double [] controlparameters3 = {1000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta3 = new MCARTA(controlparameters3,"MCARTA3");
    	
    	double [] controlparameters4 = {10000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta4 = new MCARTA(controlparameters4,"MCARTA4");
    	
    	double [] controlparameters5 = {1000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta5 = new MCARTA(controlparameters5,"MCARTA5");
    	
    	double [] controlparameters6 = {10000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta6 = new MCARTA(controlparameters6,"MCARTA6");
    	
    	double [] controlparameters7 = {1000, 3, 1, 1};
    	MCARTA mcarta7 = new MCARTA(controlparameters7,"MCARTA7");
    	
    	double [] controlparameters8 = {10000, 3, 1, 1};
    	MCARTA mcarta8 = new MCARTA(controlparameters8,"MCARTA8");
    	
    	double [] controlparameters9 = {1000, 3, 10, 10};
    	MCARTA mcarta9 = new MCARTA(controlparameters9,"MCARTA9");
    	
    	double [] controlparameters10 = {10000, 3, 10, 10};
    	MCARTA mcarta10 = new MCARTA(controlparameters10,"MCARTA10");
    	
    	double [] controlparameters11 = {1000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta11 = new MCARTA(controlparameters11,"MCARTA11");
    	mcarta11.setMedianEstimates(true);
    	
    	double [] controlparameters12 = {10000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta12 = new MCARTA(controlparameters12,"MCARTA12");
    	mcarta12.setMedianEstimates(true);
    	
    	double [] controlparameters13 = {1000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta13 = new MCARTA(controlparameters13,"MCARTA13");
    	mcarta13.setMedianEstimates(true);
    	
    	double [] controlparameters14 = {10000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta14 = new MCARTA(controlparameters14,"MCARTA14");
    	mcarta14.setMedianEstimates(true);
    	
    	double [] controlparameters15 = {1000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta15 = new MCARTA(controlparameters15,"MCARTA15");
    	mcarta15.setMedianEstimates(true);
    	
    	double [] controlparameters16 = {10000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta16 = new MCARTA(controlparameters16,"MCARTA16");
    	mcarta16.setMedianEstimates(true);
    	
    	double [] controlparameters17 = {1000, 3, 1, 1};
    	MCARTA mcarta17 = new MCARTA(controlparameters17,"MCARTA17");
    	mcarta17.setMedianEstimates(true);
    	
    	double [] controlparameters18 = {10000, 3, 1, 1};
    	MCARTA mcarta18 = new MCARTA(controlparameters18,"MCARTA18");
    	mcarta18.setMedianEstimates(true);
    	
    	double [] controlparameters19 = {1000, 3, 10, 10};
    	MCARTA mcarta19 = new MCARTA(controlparameters19,"MCARTA19");
    	mcarta19.setMedianEstimates(true);
    	
    	double [] controlparameters20 = {10000, 3, 10, 10};
    	MCARTA mcarta20 = new MCARTA(controlparameters20,"MCARTA20");
    	mcarta20.setMedianEstimates(true);
    	
    	double [] controlparameters21 = {1000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta21 = new MCARTA(controlparameters21,"MCARTA21");
    	mcarta21.setParemeterEstimationScope(50);
    	
    	double [] controlparameters22 = {10000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta22 = new MCARTA(controlparameters22,"MCARTA22");
    	mcarta22.setParemeterEstimationScope(50);
    	
    	double [] controlparameters23 = {1000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta23 = new MCARTA(controlparameters23,"MCARTA23");
    	mcarta23.setParemeterEstimationScope(50);
    	
    	double [] controlparameters24 = {10000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta24 = new MCARTA(controlparameters24,"MCARTA24");
    	mcarta24.setParemeterEstimationScope(50);
    	
    	double [] controlparameters25 = {1000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta25 = new MCARTA(controlparameters25,"MCARTA25");
    	mcarta25.setParemeterEstimationScope(50);
    	
    	double [] controlparameters26 = {10000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta26 = new MCARTA(controlparameters26,"MCARTA26");
    	mcarta26.setParemeterEstimationScope(50);
    	
    	double [] controlparameters27 = {1000, 3, 1, 1};
    	MCARTA mcarta27 = new MCARTA(controlparameters27,"MCARTA27");
    	mcarta27.setParemeterEstimationScope(50);
    	
    	double [] controlparameters28 = {10000, 3, 1, 1};
    	MCARTA mcarta28 = new MCARTA(controlparameters28,"MCARTA28");
    	mcarta28.setParemeterEstimationScope(50);
    	
    	double [] controlparameters29 = {1000, 3, 10, 10};
    	MCARTA mcarta29 = new MCARTA(controlparameters29,"MCARTA29");
    	mcarta29.setParemeterEstimationScope(50);
    	
    	double [] controlparameters30 = {10000, 3, 10, 10};
    	MCARTA mcarta30 = new MCARTA(controlparameters30,"MCARTA30");
    	mcarta30.setParemeterEstimationScope(50);
    	
    	double [] controlparameters31 = {1000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta31 = new MCARTA(controlparameters31,"MCARTA31");
    	mcarta31.setMedianEstimates(true);
    	mcarta31.setParemeterEstimationScope(50);
    	
    	double [] controlparameters32 = {10000, 3, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    	MCARTA mcarta32 = new MCARTA(controlparameters32,"MCARTA32");
    	mcarta32.setMedianEstimates(true);
    	mcarta32.setParemeterEstimationScope(50);
    	
    	double [] controlparameters33 = {1000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta33 = new MCARTA(controlparameters33,"MCARTA33");
    	mcarta33.setMedianEstimates(true);
    	mcarta33.setParemeterEstimationScope(50);
    	
    	double [] controlparameters34 = {10000, 3, 1, Double.POSITIVE_INFINITY};
    	MCARTA mcarta34 = new MCARTA(controlparameters34,"MCARTA34");
    	mcarta34.setMedianEstimates(true);
    	mcarta34.setParemeterEstimationScope(50);
    	
    	double [] controlparameters35 = {1000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta35 = new MCARTA(controlparameters35,"MCARTA35");
    	mcarta35.setMedianEstimates(true);
    	mcarta35.setParemeterEstimationScope(50);
    	
    	double [] controlparameters36 = {10000, 3, 10, Double.POSITIVE_INFINITY};
    	MCARTA mcarta36 = new MCARTA(controlparameters36,"MCARTA36");
    	mcarta36.setMedianEstimates(true);
    	
    	double [] controlparameters37 = {1000, 3, 1, 1};
    	MCARTA mcarta37 = new MCARTA(controlparameters37,"MCARTA37");
    	mcarta37.setMedianEstimates(true);
    	mcarta37.setParemeterEstimationScope(50);
    	
    	double [] controlparameters38 = {10000, 3, 1, 1};
    	MCARTA mcarta38 = new MCARTA(controlparameters38,"MCARTA38");
    	mcarta38.setMedianEstimates(true);
    	mcarta38.setParemeterEstimationScope(50);
    	
    	double [] controlparameters39 = {1000, 3, 10, 10};
    	MCARTA mcarta39 = new MCARTA(controlparameters39,"MCARTA39");
    	mcarta39.setMedianEstimates(true);
    	mcarta39.setParemeterEstimationScope(50);
    	
    	double [] controlparameters40 = {10000, 3, 10, 10};
    	MCARTA mcarta40 = new MCARTA(controlparameters40,"MCARTA40");
    	mcarta40.setMedianEstimates(true);
    	mcarta40.setParemeterEstimationScope(50);
    	    	
//    	fe.addForecastTechnique(mcarta1);
//    	fe.addForecastTechnique(mcarta2);
    	fe.addForecastTechnique(mcarta3);
//    	fe.addForecastTechnique(mcarta4);
//    	fe.addForecastTechnique(mcarta5);
//    	fe.addForecastTechnique(mcarta6);
//    	fe.addForecastTechnique(mcarta7);
//    	fe.addForecastTechnique(mcarta8);
//    	fe.addForecastTechnique(mcarta9);
//    	fe.addForecastTechnique(mcarta10);
//    	fe.addForecastTechnique(mcarta11);
//    	fe.addForecastTechnique(mcarta12);
//    	fe.addForecastTechnique(mcarta13);
//    	fe.addForecastTechnique(mcarta14);
//    	fe.addForecastTechnique(mcarta15);
//    	fe.addForecastTechnique(mcarta16);
//    	fe.addForecastTechnique(mcarta17);
//    	fe.addForecastTechnique(mcarta18);
//    	fe.addForecastTechnique(mcarta19);
//    	fe.addForecastTechnique(mcarta20);
//
//    	fe.addForecastTechnique(mcarta21);
//    	fe.addForecastTechnique(mcarta22);
//    	fe.addForecastTechnique(mcarta23);
//    	fe.addForecastTechnique(mcarta24);
//    	fe.addForecastTechnique(mcarta25);
//    	fe.addForecastTechnique(mcarta26);
//    	fe.addForecastTechnique(mcarta27);
//    	fe.addForecastTechnique(mcarta28);
//    	fe.addForecastTechnique(mcarta29);
//    	fe.addForecastTechnique(mcarta30);
//    	fe.addForecastTechnique(mcarta31);
//    	fe.addForecastTechnique(mcarta32);
//    	fe.addForecastTechnique(mcarta33);
//    	fe.addForecastTechnique(mcarta34);
//    	fe.addForecastTechnique(mcarta35);
//    	fe.addForecastTechnique(mcarta36);
//    	fe.addForecastTechnique(mcarta37);
//    	fe.addForecastTechnique(mcarta38);
//    	fe.addForecastTechnique(mcarta39);
//    	fe.addForecastTechnique(mcarta40);
    	
    	Croston c1 = new Croston(0.1, "Croston-0.1");
    	Croston c2 = new Croston(0.2, "Croston-0.2");
    	
    	double [] optpara11 = {0.1, 0.1};
    	SyntetosBoylan sb11 = new SyntetosBoylan(optpara11, "Syntetos11");
    	double [] optpara12 = {0.1, 0.15};
    	SyntetosBoylan sb12 = new SyntetosBoylan(optpara12, "Syntetos12");
    	double [] optpara13 = {0.1, 0.2};
    	SyntetosBoylan sb13 = new SyntetosBoylan(optpara13, "Syntetos13");
    	double [] optpara14 = {0.1, 0.25};
    	SyntetosBoylan sb14 = new SyntetosBoylan(optpara14, "Syntetos14");
    	double [] optpara21 = {0.15, 0.1};
    	SyntetosBoylan sb21 = new SyntetosBoylan(optpara21, "Syntetos21");
    	double [] optpara22 = {0.15, 0.15};
    	SyntetosBoylan sb22 = new SyntetosBoylan(optpara22, "Syntetos22");
    	double [] optpara23 = {0.15, 0.2};
    	SyntetosBoylan sb23 = new SyntetosBoylan(optpara23, "Syntetos23");
    	double [] optpara24 = {0.15, 0.25};
    	SyntetosBoylan sb24 = new SyntetosBoylan(optpara24, "Syntetos24");
    	double [] optpara31 = {0.2, 0.1};
    	SyntetosBoylan sb31 = new SyntetosBoylan(optpara31, "Syntetos31");
    	double [] optpara32 = {0.2, 0.15};
    	SyntetosBoylan sb32 = new SyntetosBoylan(optpara32, "Syntetos32");
    	double [] optpara33 = {0.2, 0.2};
    	SyntetosBoylan sb33 = new SyntetosBoylan(optpara33, "Syntetos33");
    	double [] optpara34 = {0.2, 0.25};
    	SyntetosBoylan sb34 = new SyntetosBoylan(optpara34, "Syntetos34");
    	double [] optpara41 = {0.25, 0.1};
    	SyntetosBoylan sb41 = new SyntetosBoylan(optpara41, "Syntetos41");
    	double [] optpara42 = {0.25, 0.15};
    	SyntetosBoylan sb42 = new SyntetosBoylan(optpara42, "Syntetos42");
    	double [] optpara43 = {0.25, 0.2};
    	SyntetosBoylan sb43 = new SyntetosBoylan(optpara43, "Syntetos43");
    	double [] optpara44 = {0.25, 0.25};
    	SyntetosBoylan sb44 = new SyntetosBoylan(optpara44, "Syntetos44");
    	
    	SimpleExponentialSmoothing ses1 = new SimpleExponentialSmoothing(0.1, "SES-0.1");
    	SimpleExponentialSmoothing ses2 = new SimpleExponentialSmoothing(0.2, "SES-0.2");
    	
    	MovingAverage ma1 = new MovingAverage(19, "MA-19");
    	MovingAverage ma2 = new MovingAverage(9, "MA-9");
    	
    	CumulativeAverage ca = new CumulativeAverage("CA");
    	NaiveForecastTechnique nft = new NaiveForecastTechnique("NF");
    	    	
    	fe.addForecastTechnique(c1);
    	fe.addForecastTechnique(c2);
    	fe.addForecastTechnique(sb11);
    	fe.addForecastTechnique(sb12);
    	fe.addForecastTechnique(sb13);
    	fe.addForecastTechnique(sb14);
    	fe.addForecastTechnique(sb21);
    	fe.addForecastTechnique(sb22);
    	fe.addForecastTechnique(sb23);
    	fe.addForecastTechnique(sb24);
    	fe.addForecastTechnique(sb31);
    	fe.addForecastTechnique(sb32);
    	fe.addForecastTechnique(sb33);
    	fe.addForecastTechnique(sb34);
    	fe.addForecastTechnique(sb41);
    	fe.addForecastTechnique(sb42);
    	fe.addForecastTechnique(sb43);
    	fe.addForecastTechnique(sb44);
    	fe.addForecastTechnique(ses1);
    	fe.addForecastTechnique(ses2);
    	fe.addForecastTechnique(ma1);
    	fe.addForecastTechnique(ma2);
    	fe.addForecastTechnique(ca);
    	fe.addForecastTechnique(nft);
    	
    	
    	fe.initializeExperiment();
    	fe.runExperiment();
    	
    	
    	
    	
    }

}
