/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jslx.inventory.intermittent;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Geometric;
import jslx.utilities.random.sp.BatchOnOffProcess;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.SyntetosBoylan;

import java.io.*;
import jsl.utilities.random.distributions.ShiftedGeometric;

/**
 *
 * @author vvarghe
 */
public class ExperimentIERC2009 {
    
    public static Model createForecastBasedSLSIRQInventoryModel(int reorderpoint, 
            int orderquantity, int initiallevel, RandomIfc leadtime, double idleperiodmean,
            double busyperiodmean, double nonzeromean, double aggregationlevel,
            ForecastTechnique ft){
        
        Model m = Model.createModel();
        ForecastBasedSLSIRQInventoryModel i = new ForecastBasedSLSIRQInventoryModel(m);
        
        RandomIfc tbd = Constant.ONE;

        BatchOnOffProcess amt = new BatchOnOffProcess();

        if(idleperiodmean<=1.0)
            throw new IllegalArgumentException("Idle period mean may not be less that or equal to 1.0");

        if(busyperiodmean<=1.0)
            throw new IllegalArgumentException("Busy period mean may not be less that or equal to 1.0");

        if(nonzeromean<=1.0)
            throw new IllegalArgumentException("Non zero mean may not be less that or equal to 1.0");

        amt.setNumberOfBusySlotsDistribution(new ShiftedGeometric(1/idleperiodmean));
        amt.setNumberOfIdleSlotsDistribution(new ShiftedGeometric(1/busyperiodmean));
        amt.setBusyAmountDistribution(new ShiftedGeometric(1/nonzeromean));

        i.setTimeUntilFirstEventForReplications(tbd);
        i.setTimeBetweenEventsForReplications(tbd);
        i.setAmountDistribution(amt);

        i.setLeadTime(leadtime);

        i.setInitialPolicyParameters(reorderpoint, orderquantity);
        i.setInitialOnHand(initiallevel);
        
        return m;
    
    }
    
    public ExperimentIERC2009(String name){
    
    }
    
    
    
    /**
     * @param args
     */
    public static void main(String[] args)throws IOException {
        System.out.println("ForecastBasedSLSIRQInventoryModel Test");
        
        int count = 1;
        
        int oq = 1;
        while (oq<=4){
                    
            int rp =-1;      
            while (rp<=4){
        
                double lt = 0.5;    
                while (lt<=1){
        
                    double idlep = 2.0;    
                    while (idlep<=4){
        
                        double busyp = 2.0;    
                        while (busyp<=4){
                            
                            double nonzerop = 2.0;    
                            while (nonzerop<=4){
                        
                            
        int initiallevel = 0;
        
        int orderquantity = oq;
        int reorderpoint =rp;      
        RandomIfc leadtime = new Constant(lt);
        double idleperiodprobparameter = idlep;
        double busyperiodprobparameter = busyp;
        double nonzeromeanprobparameter = nonzerop;
        
        double aggregationlevel = 1.0;
        ForecastTechnique ft = new SyntetosBoylan(0.1);
        
        Model m = ExperimentIERC2009.createForecastBasedSLSIRQInventoryModel(reorderpoint,
            orderquantity,  initiallevel,  leadtime, idleperiodprobparameter, 
             busyperiodprobparameter,  nonzeromeanprobparameter,  aggregationlevel,  ft);

        Experiment e = new Experiment(m);
		
        // set the parameters of the experiment
        e.setNumberOfReplications(1);
        e.setLengthOfReplication(2*336000.0);
        e.setLengthOfWarmUp(360.0);
        e.turnOnBatchReport();
        //e.turnOnExperimentReport();
        //e.turnOnReplicationReport();
        //e.turnOnExperimentConsoleOutput();

      
        
        System.out.println("Q: "+orderquantity);
        System.out.println("r: "+reorderpoint);
        System.out.println("LT: "+lt);
        System.out.println("Mu(I): "+idleperiodprobparameter);
        System.out.println("Mu(B): "+busyperiodprobparameter);
        System.out.println("Mu(NZ): "+nonzeromeanprobparameter);
        
       
      e.runAll();
     

       
        System.out.println("Done!");      
        
        
        nonzerop = nonzerop + 2; 
        }
                            
        busyp = busyp + 2;
        }
        
        idlep = idlep+2;    
        }
        
        lt = lt+0.5;
        }
        
        rp = rp + 5;    
        }
    
        oq = oq + 3; 
        }
        
        
        
        }

}
