/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 *
 * Copyright (c) Manuel D. Rossetti (rossetti@uark.edu)
 *
 * Contact:
 *	Manuel D. Rossetti, Ph.D., P.E.
 *	Department of Industrial Engineering
 *	University of Arkansas
 *	4207 Bell Engineering Center
 *	Fayetteville, AR 72701
 *	Phone: (479) 575-6756
 *	Email: rossetti@uark.edu
 *	Web: www.uark.edu/~rossetti
 *
 * This file is part of the JSL (a Java Simulation Library). The JSL is a framework
 * of Java classes that permit the easy development and execution of discrete event
 * simulation programs.
 *
 * The JSL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The JSL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the JSL (see file COPYING in the distribution);
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA, or see www.fsf.org
 *
 */
package jslx.inventory.intermittent;

import jsl.modeling.Experiment;
import jsl.modeling.ExperimentProvider;
import jsl.modeling.ExperimentRunner;
import jsl.modeling.Model;
import jsl.utilities.Controls;

/** The purpose of this class is to run
 *  experiments for the lumpy intermittent
 *  r, Q inventory model
 *
 * @author rossetti
 */
public class LIExperiments {

    protected int myNumExp;
    protected int myCountExp = 0;
    protected ExperimentRunner myRunner;
    protected LIExperimentProvider myProvider;
    protected Experiment myExperiment;
    protected LIRQInventoryModel myIM;
    protected Model myModel;

    public LIExperiments() {
        this(1);
    }

    public LIExperiments(int n) {
        if (n <= 0) {
            throw new IllegalArgumentException("The number of experiments to run must be >0");
        }
        myNumExp = n;
        myModel = Model.createModel("Lumpy Intermittent Demand Simulation");
        myProvider = new LIExperimentProvider(myModel);
        myIM = new LIRQInventoryModel(myModel);
        myExperiment = new Experiment(myModel);
        myRunner = new ExperimentRunner(myProvider);

    }

    public static void main(String args[]) {

        LIExperiments e = new LIExperiments(10);
        e.setLengthOfReplication(1000);
        e.setLengthOfWarmUp(100);
        e.setNumberOfReplications(30);
        e.runAll();
    }

    public final void setNumberOfReplications(int numReps) {
        myExperiment.setNumberOfReplications(numReps);
    }

    public final void setLengthOfWarmUp(double lengthOfWarmUp) {
        myExperiment.setLengthOfWarmUp(lengthOfWarmUp);
    }

    public final void setLengthOfReplication(double lengthOfReplication) {
        myExperiment.setLengthOfReplication(lengthOfReplication);
    }

    public final void runAll() {
        myRunner.runAll();
    }

    protected class LIExperimentProvider extends ExperimentProvider {

        protected Controls myControls;

        private LIExperimentProvider(Model model) {
            super(model);
            myControls = myIM.makeControls();
        }

        @Override
        public boolean hasNext() {
            return (myCountExp <= myNumExp);
        }

        @Override
        public Experiment next() {
            throw new UnsupportedOperationException("Not supported yet.");
            // get/generate the next test case
            // set the controls
            // apply the controls to the model
            // increment the experiment counter
            // return the experiment
        }
    }
}
