package jslx.inventory.rqSim;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;


import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.observers.database.access.AccessDBExperimentReport;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.observers.variable.StatisticalObserver;
import jsl.utilities.statistic.Statistic;
import jslx.inventory.rqmodel.DownloadData;
import jslx.inventory.rqmodel.RQItem;
import jslx.inventory.rqmodel.RQRetailerItem;

public class DS_Test {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

		System.out.println("DS Simulation");

		testExperiments();

		System.out.println("Done!");
	}

	public static void testExperiments(){
		System.out.println("Start");
		System.out.println("Connecting to Database");
		DownloadData d = new DownloadData("SZ_SYS_DATA", "Shyam", "Shyam", "Table1");
		System.out.println("Connected!");

		int s, N;		
		double Dr;
		int Lr,Qr,rr,Lw,Qw,Qw_,rw,rw_;




		try{
			System.out.println("Creating DS_Sim_results.csv");
			BufferedWriter out2 = new BufferedWriter(new FileWriter ("SZ_Sim_Results.csv"));
			System.out.println("Writing column headers");
			out2.write("SystemNo,N,Dr,Lr,Qr,rr,Lw,Qw,rw,WBO,WIL,WFR,RBO,RIL,RFR");

			System.out.println("Begin Simulations");

			ArrayList<ArrayList<Double>> systemBO = new ArrayList<ArrayList<Double>>();
			ArrayList<ArrayList<Double>> systemOH = new ArrayList<ArrayList<Double>>();
			ArrayList<ArrayList<Double>> systemFR = new ArrayList<ArrayList<Double>>();
			systemBO.add(0, null);
			systemOH.add(0, null);
			systemFR.add(0, null);
			
			CSVReport ecsv = new CSVExperimentReport("SZ_Sim_ExperimentReport");

			
			try{
				while(d.myResultSet.next()){
					try{
						ArrayList<Double> BO = new ArrayList<Double>();
						ArrayList<Double> OH = new ArrayList<Double>();
						ArrayList<Double> FR = new ArrayList<Double>();


						s = d.myResultSet.getInt("SystemNo");
						
						systemBO.add(s, BO);
						systemOH.add(s, OH);
						systemFR.add(s, FR);
						
						N = d.myResultSet.getInt("N");
						Dr = d.myResultSet.getDouble("Dr");
						Lr = d.myResultSet.getInt("Lr");
						Qr = d.myResultSet.getInt("Qr");
						rr = d.myResultSet.getInt("rr");
						Lw = d.myResultSet.getInt("Lw");
						Qw_ = d.myResultSet.getInt("Qw");
						Qw = Qw_*Qr;
						rw_ = d.myResultSet.getInt("rw");
						rw = rw_*Qr;

						Model m = Model.createModel();

						RQInventoryModelDS DS = new RQInventoryModelDS(m, "System"+s, N, Dr, Lr, Qr, rr, Lw, Qw, rw);
						
						m.addObserver(ecsv);

						System.out.println("Simulating System: "+s);

						Experiment e = new Experiment(m);
						e.setName("System"+s);
						e.setNumberOfReplications(10);
						e.setLengthOfReplication(3650);
						e.setLengthOfWarmUp(1825);

						//e.turnOnExperimentReport("AccrossRep_Sytem"+i);
						//e.turnOnReplicationReport("WithinRep_System"+i);
						e.runAll();



						String myWarehouseBackLogQueueString = DS.GetWarehouseBackLogString();
						String myWarehouseInventoryString = DS.getWarehouseInventoryString();
						
						//System.out.println(myWarehouseInventoryString);

						String myWarehouseBOString = myWarehouseBackLogQueueString+"-DemandBackLogQ NumberInQ";
						String myWarehouseOHString = myWarehouseInventoryString+"-OnHand";
						String myWarehouseFRString = myWarehouseInventoryString+"-FirstFillRate";

						ResponseVariable Warehouse_BO = (ResponseVariable)m.getVariable(myWarehouseBOString);
						ResponseVariable Warehouse_OH = (ResponseVariable)m.getVariable(myWarehouseOHString);
						ResponseVariable Warehouse_FR = (ResponseVariable)m.getVariable(myWarehouseFRString);
						
						StatisticalObserver so1 = Warehouse_BO.getStatisticalObserver();
						Statistic WBO = (Statistic)so1.getAcrossReplicationStatistic();
						Statistic Stat1 = Statistic.newInstance(WBO);
						Double WEBO = Stat1.getAverage();
						systemBO.get(s).add(0, WEBO);
						
						//BO.toString();
						//System.out.println(BO);
						
						StatisticalObserver so2 = Warehouse_OH.getStatisticalObserver();
						Statistic WOH = (Statistic)so2.getAcrossReplicationStatistic();
						Statistic Stat2 = Statistic.newInstance(WOH);
						Double WEOH = Stat2.getAverage();
						systemOH.get(s).add(0, WEOH);
						
						StatisticalObserver so5 = Warehouse_FR.getStatisticalObserver();
						Statistic WFR = (Statistic)so5.getAcrossReplicationStatistic();
						Statistic Stat5 = Statistic.newInstance(WFR);
						Double WareFR = Stat5.getAverage();
						systemFR.get(s).add(0, WareFR);
						
						Double RBO_sum = 0.0;
						Double ROH_sum = 0.0;
						Double RFR_sum = 0.0;


						for(int j = 1; j<=N; j++){
							String myRetailerBackLogQueueString = DS.GetRetailerBackLogString(j);
							String myRetailerInventoryString = DS.GetRetailerInventoryString(j);
							
							String myRetailerBOString = myRetailerBackLogQueueString+"-DemandBackLogQ NumberInQ";
							String myRetailerOHString = myRetailerInventoryString+"-OnHand";
							String myRetailerFRString = myRetailerInventoryString+"-FirstFillRate";
							
							ResponseVariable Retailer_BO = (ResponseVariable)m.getResponseVariable(myRetailerBOString);//TODO make this easier
							ResponseVariable Retailer_OH = (ResponseVariable)m.getResponseVariable(myRetailerOHString);
							ResponseVariable Retailer_FR = (ResponseVariable)m.getResponseVariable(myRetailerFRString);
							StatisticalObserver so3 = Retailer_BO.getStatisticalObserver();
							StatisticalObserver so4 = Retailer_OH.getStatisticalObserver();
							StatisticalObserver so6 = Retailer_FR.getStatisticalObserver();
							Statistic RBO = (Statistic)so3.getAcrossReplicationStatistic();
							Statistic ROH = (Statistic)so4.getAcrossReplicationStatistic();
							Statistic RFR = (Statistic)so6.getAcrossReplicationStatistic();
							Statistic Stat3 = Statistic.newInstance(RBO);
							Statistic Stat4 = Statistic.newInstance(ROH);
							Statistic Stat6 = Statistic.newInstance(RFR);
							
							Double Ret_BO = Stat3.getAverage();
							Double Ret_OH = Stat4.getAverage();
							Double Ret_FR = Stat6.getAverage();
							
							systemBO.get(s).add(j, Ret_BO);
							systemOH.get(s).add(j, Ret_OH);
							systemFR.get(s).add(j, Ret_FR);
							
							RBO_sum = RBO_sum+Ret_BO;
							ROH_sum = ROH_sum+Ret_OH;
							RFR_sum = RFR_sum+Ret_FR;
						}
						Double REBO = RBO_sum/N;
						Double REOH = ROH_sum/N;
						Double RetFR = RFR_sum/N;
						
						out2.newLine();
						out2.write(s+","+N+","+Dr+","+Lr+","+Qr+","+rr+","+Lw+","+Qw+","+rw+","+WEBO+","+WEOH+","+WareFR+","+REBO+","+REOH+","+RetFR);
					}
					catch(SQLException e){
						System.err.println(e.getMessage());
					}	
				}
			}
			catch(SQLException e){
				System.err.println(e.getMessage());
			}
			out2.close();
		}
		catch(IOException e){
			System.err.println(e.getMessage());
		}

		//RBO.toString();
		//System.out.println(RBO);

		System.out.println("Successfuly completed!");
	}
}
