package jslx.inventory.rqSim;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.supplychain.inventorylayer.*;
import jslx.supplychain.transportlayer.TimeBasedDemandCarrier;

public class RQInventoryModelDS extends ModelElement{

	public RQInventoryModelDS(ModelElement parent, String name, int NoRet, double DemandRate, double RetLT, int RetQ, int Retr, double WareLT, int WareQ, int Warer){
		super(parent, name);

		int N = NoRet;
		double Dr = DemandRate;
		double Lr = RetLT;
		int Qr = RetQ;
		int rr = Retr;
		double Lw = WareLT;
		int Qw = WareQ;
		int rw = Warer;

		//Define item type
		ItemType type1 = new ItemType(this, "Type-1");

		//define external supplier
		LeadTimeDemandFiller myLeadTimeDemandFiller = new LeadTimeDemandFiller(this);

		//lead time from supplier to warehouse
		DistributionIfc LT_W = new Constant(Lw);
		myLeadTimeDemandFiller.addLeadTime(type1, LT_W);

		//define warehouse
		InventoryHoldingPoint warehouse = new InventoryHoldingPoint(this, "warehouse");

		//define and add shipper for warehouse
		TimeBasedDemandCarrier myWarehouseShipper = new TimeBasedDemandCarrier(this);
		warehouse.setDemandCarrier(myWarehouseShipper);
		warehouse.setDemandFiller(myLeadTimeDemandFiller);

		//Create inventory policy at warehouse level
		
		InventoryPolicyAbstract InventoryPolicy_W = new InventoryPolicyReorderPointReorderQuantity(this, rw, Qw);
		Inventory Inventory_W = new Inventory(this, type1, InventoryPolicy_W, (rw+Qw),"W_Type1");
		BackLogQueue BOQW = new BackLogQueue(Inventory_W, "WBQ");
		
		//add the inventory to warehouse
		warehouse.addInventory(Inventory_W);
		
		for(int i = 1; i<=N; i++){
			DistributionIfc LT_R = new Constant(Lr);
			//Create the retailers
			InventoryHoldingPoint retailer = new InventoryHoldingPoint(this, "R"+i);
			myWarehouseShipper.setTransportTime(retailer, LT_R);
			retailer.setDemandFiller(warehouse);

			//create inventory policy at retailers

			InventoryPolicyAbstract InventoryPolicy_R = new InventoryPolicyReorderPointReorderQuantity(this, rr, Qr);
			Inventory Inventory_R = new Inventory(this, type1, InventoryPolicy_R, (rr+Qr), "R_"+i+"_Type1");
			BackLogQueue BOQR = new BackLogQueue(Inventory_R, "BOQR"+i);
			
			retailer.addInventory(Inventory_R);
			DistributionIfc DemandR = new Exponential(1/Dr);
			DemandGenerator DemandG = new DemandGenerator(this, type1, DemandR, DemandR);
			DemandG.setDemandFiller(retailer);
		}
	}
	
	public String GetWarehouseBackLogString(){
		return("WBQ");
	}
	
	public String getWarehouseInventoryString(){
		return("W_Type1");
	}
	
	public String GetRetailerBackLogString(int r){
		return("BOQR"+r);
	}
	
	public String GetRetailerInventoryString(int r){
		return("R_"+r+"_Type1");
	}
}
