/*
* Created on Jun 23, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.rqmodel;

import jsl.utilities.random.distributions.Poisson;

/**
 * @author rossetti
 *
 */
public class PoissonrQModel extends RQItemAbstract {

	/**
	 * 
	 */
	public PoissonrQModel(double demandRate, double leadTime){
		setDemandRate(demandRate);
		setLeadTime(leadTime);
		setLeadTimeDemandDistribution();
	}

	protected void setLeadTimeDemandDistribution(){
		double rate = getDemandRate();
		double lt = getLeadTime();
		if (myLTDCDF == null)
			myLTDCDF = new Poisson(rate*lt);
		else {
			Poisson p = (Poisson)myLTDCDF;
			p.setMean(rate*lt);
		}	
	}
	
	/** The demand rate for the item at the base
	 * 
	 * @param rate Must be > 0
	 */
	protected void setDemandRate(double rate) {
		if (rate < 0)
			throw new IllegalArgumentException("The rate must be >= 0");
		myDemandRate = rate;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("rQ model Test");
		

//		PoissonrQModel b = new PoissonrQModel(3.6, 0.5);
		PoissonrQModel b = new PoissonrQModel(14./365., 45.0);
//		b.setPolicy(1, 4);
		b.setPolicy(3, 4);
		b.setPeriodLength(1.0);
		System.out.println(b);
		System.out.println("SO \t OH \t BO \t fr \t cw");
		//for (int s=2;s<=20;s=s+2{
			double so = b.getExpectedStockOutsPerPeriod();
			double oh = b.getExpectedOnHandInventory();
			double bo = b.getExpectedBackOrders();
			double Fr = b.getFillRate();
			double CW = b.getExpectedCustomerWaitTime();
			
			System.out.println(so + "\t" + oh + "\t" + bo + "\t" + Fr + "\t" + CW);
			System.out.println(b.getProbabilityOfStockOut());
	}
}
