package jslx.inventory.rqmodel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import jsl.utilities.random.distributions.MTP;

public class RQItem extends RQItemAbstract implements RQItemIfc{
		/**holds the retailer items for this item
		 * 
		 */
		protected ArrayList<RQRetailerItem> myRetailerItems;
		
		protected double Qr;
		
		protected double mean;
		
		protected double var;
		
		private double T;

		private double alpha;

		private double beta;

		private double A;

		private double p;

		private double v;
		
		/** Creates an RQItem
		 * 
		 * @param itemNumber
		 * @param reorderPoint
		 * @param reorderQuantity
		 * @param periodlength
		 * @param leadTime
		 */
		public RQItem(int itemNumber, double reorderPoint, double reorderQuantity, double periodlength, double leadTime){
			myRetailerItems = new ArrayList<RQRetailerItem>();
			setItemNumber(itemNumber);
			setPolicy(reorderPoint, reorderQuantity);
			setPeriodLength(periodlength);
			setLeadTime(leadTime);
		}
		
		/**creates and returns a (independent) deep copy of the supplied item
		 * 
		 * @param original
		 * @return
		 */
		public static RQItem newInstance(RQItem original){
			RQItem clone = new RQItem(original.myItemNumber, original.myReorderPt, original.myReorderQty, original.myPeriodLength, original.myLeadTime);
			for(RQRetailerItem bi: original.myRetailerItems){
				clone.addRetailerItem(bi.myRetailerNumber, bi.myDemandRate, bi.myReorderPt, bi.myReorderQty,bi.myLeadTime, bi.myPeriodLength);
			}
			return(clone);
		}
		
		/**Adds a retailer with a default retailer number
		 * 
		 * @param retailerDemandRate
		 * @param reorderPoint
		 * @param reorderQuantity
		 * @param leadTime
		 * @param periodLength
		 * @return
		 */
		public RQRetailerItemIfc addRetailerItem(double retailerDemandRate, double reorderPoint, double reorderQuantity, double leadTime, double periodLength){
			return(addRetailerItem(myRetailerItems.size()+1, retailerDemandRate, reorderPoint, reorderQuantity, leadTime, periodLength));
		}
		
		/**adds a retailer item
		 * 
		 * @param retailerNumber
		 * @param retailerDemandRate
		 * @param reorderPoint
		 * @param reorderQuantity
		 * @param leadTime
		 * @param periodLength
		 * @return
		 */
		public RQRetailerItemIfc addRetailerItem(int retailerNumber, double retailerDemandRate, double reorderPoint, double reorderQuantity, double leadTime, double periodLength){
			if(myRetailerItems.isEmpty())
				myLTDCDF = new MTP();
			RQRetailerItem bi = new RQRetailerItem(this, retailerNumber, retailerDemandRate, leadTime, periodLength, reorderPoint, reorderQuantity);
			myRetailerItems.add(bi);
			updateWarehouseDemandQuantity();
			updateWarehouseDemandRate();
			return(bi);
		}
		
		public double getTotalEBackOrders(){
			return getExpectedBackOrders()+getTotalRetailerEBackOrders();
		}
		
		public double getTotalEOnHandUnitsOfQr(){
			return getExpectedOnHandInventory()+getTotalRetailerEOnHand();
		}
		
		public double getTotalEOnHand(){
			return getExpectedOnHandInventory()*Qr+getTotalRetailerEOnHand();
		}
		
		public int getNumRetailers(){
			return myRetailerItems.size();
		}
		
		public List<RQRetailerItem> getRetailerItems(){
			return Collections.unmodifiableList(myRetailerItems);
		}
		
		public double getTotalRetailerEBackOrders(){
			double t = 0.0;
			for(RQRetailerItem bi:myRetailerItems)
				t = t+bi.getExpectedBackOrders();
			return t;
		}
		
		public double getTotalRetailerEOnHand(){
			double t = 0.0;
			for(RQRetailerItem bi:myRetailerItems)
				t = t+bi.getExpectedOnHandInventory();
			return t;
		}
		
		/**returns the reorder points as an array with s[0] being the warehouse and s[j] being the retailers
		 * 
		 * @return
		 */
		public double[] getReorderPoints(){
			double[] s = new double[getNumRetailers()+1];
			s[0] = getReorderPoint();
			int j = 1;
			for(RQRetailerItem bi:myRetailerItems){
				s[j] = bi.getReorderPoint();
				j++;
			}
			return s;
		}
		
	/**	returns the reorder quantities as an array with s[0] being the warehouse and s[j] being the retailers
	 * 
	 * @return
	 */
		public double[] getReorderQuantities(){
			double[] s = new double[getNumRetailers()+1];
			s[0] = getReorderQty();
			int j = 1;
			for(RQRetailerItem bi:myRetailerItems){
				s[j] = bi.getReorderQty();
				j++;
			}
			return s;
		}
		
		public String toString(){
			StringBuilder sb = new StringBuilder();
			sb.append("******************************************\n");
			sb.append("------------------------------------------------------\n");
			sb.append("Warehouse results\n");
			sb.append(super.toString());
			sb.append("------------------------------------------------------\n");
			sb.append("Aggregate Results\n");
			sb.append("------------------------------------------------------\n");
			sb.append("Total Expected BackOrders " + getTotalEBackOrders() + "\n");
			sb.append("Total Expected Onhand: " + getTotalEOnHand() + "\n");
			sb.append("Number of Retailers: " + getNumRetailers() + "\n");
			sb.append("Total Expected Retailer Backorders: " + getTotalRetailerEBackOrders() + "\n");
			sb.append("Total Expected Retailer Onhand: " + getTotalRetailerEOnHand() + "\n");
			sb.append("\nIndividual Retailer results\n");
			for(RQRetailerItem bi:myRetailerItems){
				sb.append("------------------------------------------------------\n");
				sb.append("Retailer number: " + bi.getRetailerNumber() + "\n");
				sb.append(bi.toString());
			}
			sb.append("******************************************\n");
			return(sb.toString());
		}
		
		protected void updateWarehouseDemandQuantity(){
			for(RQRetailerItem bi: myRetailerItems){
				Qr = (double)bi.getReorderQty();
				break;
			}
		}
		
		protected void updateWarehouseDemandRate(){
			myDemandRate = 0.0;
			for(RQRetailerItem b: myRetailerItems)
				myDemandRate = myDemandRate+b.getDemandRate();
			setLeadTimeDemandDistribution();
			updateRetailerItemLeadTimeDemandDistributions();
		}
		
		protected void setLeadTimeDemandDistribution(){
			double rate = getDemandRate();
			double lt = getLeadTime();
			mean = (lt*rate)/Qr;
			myMLTD = mean;
			T = 0.0;
			double Dr = myRetailerItems.get(0).getDemandRate();
			for(int k = 1; k<=(Qr-1); k++){
				alpha = 1 - Math.cos(2*Math.PI*k/Qr);
				beta = Math.sin(2*Math.PI*k/Qr);
				T = T + (1-Math.exp(-alpha*Dr*lt)*Math.cos(beta*Dr*lt))/alpha;
			}
			var = (1/myRetailerItems.get(0).getReorderQty())*(mean+(myRetailerItems.size()/myRetailerItems.get(0).getReorderQty())*T);
			A = (int)(mean - var);
			p = Math.sqrt(mean-var-A);
			v = mean-A-p;
			MTP n = (MTP)myLTDCDF;
			n.setParameters(v, A, 1-p);
		}
		
		protected void updateRetailerItemLeadTimeDemandDistributions(){
			for(RQRetailerItem b:myRetailerItems)
				b.setLeadTimeDemandDistribution();
		}
	
}
