package jslx.inventory.rqmodel;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

public class RQModel implements RQIfc {
	
	protected ArrayList<RQItem>myItems;
	
	public RQModel(){
		myItems = new ArrayList<RQItem>();
	}
	
	/** Creates and returns a new (independent) deep copy of the supplied model
	 * 
	 * @param original
	 * @return
	 */
	public static RQModel newInstance(RQModel original){
		RQModel clone = new RQModel();
		for(RQItem item:original.myItems)
			clone.addRQItem(RQItem.newInstance(item));
		return clone;
	}
	
	
	/** Creates and adds an item with the supplied parameters to the model
	 * 
	 * @param itemNumber
	 * @param reorderPoint
	 * @param reorderQuantity
	 * @param leadTime
	 * @return
	 */
	public RQItem addRQItem(int itemNumber, int reorderPoint, int reorderQuantity, double periodlength, double leadTime){
		RQItem item = new RQItem(itemNumber, reorderPoint, reorderQuantity, periodlength, leadTime);
		addRQItem(item);
		return item;
	}
	
	/** Adds the supplied item to the model.
	 * 
	 * @param item Must not be Null and must not already be in the model
	 */
	public void addRQItem(RQItem item){
		if(item == null)
			throw new IllegalArgumentException("The item was Null");
		if(myItems.contains(item))
			throw new IllegalArgumentException("The item is already in the model");
		myItems.add(item);
	}
	
	/**Returns an unmodifiable list of items
	 * 
	 */
	public List<RQItem> getItems() {
		return(Collections.unmodifiableList(myItems));
	}

	public int getNumItems() {
		return myItems.size();
	}

	public double getTotalEBackOrders() {
		return getTotalRetailerEBackOrders()+getTotalWarehouseEBackOrders();
	}

	public double getTotalEOnHand() {
		return getTotalRetailerEOnHand()+getTotalWarehouseEOnHand();
	}

	public double getTotalRetailerEBackOrders() {
		double t = 0.0;
		for(RQItemIfc i:myItems){
			t = t + i.getTotalRetailerEBackOrders();
		}
		return t;
	}

	public double getTotalRetailerEOnHand() {
		double t = 0.0;
		for(RQItemIfc i:myItems){
			t = t + i.getTotalRetailerEOnHand();
		}
		return t;
	}

	public double getTotalWarehouseEBackOrders() {
		double t = 0.0;
		for(RQItemIfc i:myItems){
			t = t + i.getTotalEBackOrders();
		}
		return t;
	}

	public double getTotalWarehouseEOnHand() {
		double t = 0.0;
		for(RQItemIfc i:myItems){
			t = t + i.getTotalEOnHand();
		}
		return t;
	}
	
	public String toString(){
		StringBuilder sb = new StringBuilder();
		sb.append("(r, Q) model results: \n");
		sb.append("==================================================== \n");
		sb.append("Across item Aggregate Results\n");
		sb.append("Total Expected BackOrders " + getTotalEBackOrders() + "\n");
		sb.append("Total Expected Onhand: " + getTotalEOnHand() + "\n");
		sb.append("Number of items: " + getNumItems() + "\n");
		sb.append("Warehouse level Aggregate Results across items\n");
		sb.append("Total Expected Warehouse Backorders: " + getTotalWarehouseEBackOrders() + "\n");
		sb.append("Total Expected Warehouse Onhand: " + getTotalWarehouseEOnHand() + "\n");
		sb.append("Retailer level Aggregate Results across items\n");
		sb.append("Total Expected Retailer Backorders: " + getTotalRetailerEBackOrders() + "\n");
		sb.append("Total Expected Retailer Onhand: " + getTotalRetailerEOnHand() + "\n");
		sb.append("\nIndividual item results\n");
		for(RQItem i:myItems){
			sb.append("Item number: " + i.getItemNumber() + "\n");
			sb.append(i.toString());
		}
		sb.append("==================================================== \n");
		return(sb.toString());
	}
	
	

}
