package jslx.inventory.rqmodel;

import jsl.utilities.random.distributions.Distribution;
import jsl.utilities.random.distributions.LossFunctionDistributionIfc;
import jsl.utilities.random.distributions.NegativeBinomial;
import jsl.utilities.random.distributions.Normal;
import jsl.utilities.random.distributions.Poisson;

public class RQRetailerItem extends RQItemAbstract implements RQRetailerItemIfc{

	private double shift;

	private double prob;

	private double rate;

	private double Qw;

	private double Rw;

	private double x;

	private double Velt;

	private double Elt;

	private double ELTDR;

	private double VLTDR;

	private double EBackorderBm1W;

	private double VMR;

	protected double[] myParameters = new double[2];

	protected Poisson p = new Poisson();

	protected LossFunctionDistributionIfc myNegBin = new NegativeBinomial();

	protected LossFunctionDistributionIfc myNormal = new Normal();

	/**Warehouse item associated with the base
	 * 
	 */
	protected RQItem myWarehouseItem;

	/**Retailer number
	 * 
	 */
	protected int myRetailerNumber;

	/**Time to ship to the retailer
	 * 
	 */
	protected double myLeadTime;

	/**Lead time demand
	 * 
	 */
	protected double myExpectedLeadTimeDemand;

	protected LossFunctionDistributionIfc myPoisson = new Poisson();

	protected RQRetailerItem(RQItem WarehouseItem, int RetailerNum, double DemandRate, double LeadTime, double periodlength, double ReorderPoint, double ReorderQuantity){
		myWarehouseItem = WarehouseItem;
		myRetailerNumber = RetailerNum;
		setItemNumber(WarehouseItem.getItemNumber());
		setDemandRate_(DemandRate);
		setLeadTime_(LeadTime);
		setPolicy_(ReorderPoint, ReorderQuantity);
		setPeriodLength(periodlength);
	}

	public int getRetailerNumber(){
		return myRetailerNumber;
	}

	public double getLeadTime(){
		return myLeadTime;
	}

	public double getExpectedLeadTimeDemand(){
		return myExpectedLeadTimeDemand;
	}

	public void setLeadTime(double time){
		setLeadTime_(time);
		setLeadTimeDemandDistribution();
	}

	public void setLeadTime_(double time){
		if(time<0)
			throw new IllegalArgumentException("Lead time must be >=0");
		myLeadTime = time;
	}

	public void setDemandRate(double rate){
		setDemandRate_(rate);
		myWarehouseItem.updateWarehouseDemandRate();
	}

	public void setDemandRate_(double rate){
		if (rate<0)
			throw new IllegalArgumentException("the rate must be > 0");
		myDemandRate = rate;
	}

	public void setPolicy(double Q, double r){
		setPolicy_(Q, r);
		myWarehouseItem.updateWarehouseDemandQuantity();
	}

	public void setPolicy_(double r, double Q){
		if(Q<=0)
			throw new IllegalArgumentException("Reorder quantity must be > 0");
		myReorderQty = Q;
		if(r<-myReorderQty)
			throw new IllegalArgumentException("Reorder point must be >= -Reorder quantity");
		myReorderPt = r;
	}

	protected void setLeadTimeDemandDistribution(){
		shift = (int)(myWarehouseItem.getMeanLeadTimeDemand() - myWarehouseItem.getVarianceOfLeadTimeDemand());
		prob = Math.sqrt(myWarehouseItem.getMeanLeadTimeDemand()-myWarehouseItem.getVarianceOfLeadTimeDemand()-shift);
		if(prob > 1.0)
			throw new IllegalArgumentException("RetailerLTD prob is > 1.0");
		rate = myWarehouseItem.getMeanLeadTimeDemand()-shift-prob;
		Qw = myWarehouseItem.getReorderQty();
		Rw = myWarehouseItem.getReorderPoint();
		if(Rw>=0)
			x = 0;
		else
			x = (Rw*(Rw+1))/((myWarehouseItem.getDemandRate()/getReorderQty())*(myWarehouseItem.getDemandRate()/getReorderQty())*Qw);
		Elt = getLeadTime()+(myReorderQty/myWarehouseItem.getDemandRate())*myWarehouseItem.getExpectedBackOrders();
		EBackorderBm1W = (1-prob)/Qw*(emmaY2(Rw-shift, rate)-emmaY2(Rw+Qw-shift, rate))+prob/Qw*(emmaY2(Rw-shift-1, rate)-emmaY2(Rw+Qw-shift-1, rate));
		Velt = ((EBackorderBm1W-(myWarehouseItem.getExpectedBackOrders()*myWarehouseItem.getExpectedBackOrders()))/((myWarehouseItem.getDemandRate()/getReorderQty())*(myWarehouseItem.getDemandRate()/getReorderQty())))+x;
		if(Velt < 0.0)
			Velt = 0.0;
		ELTDR = getDemandRate()*Elt;
		myMLTD = ELTDR;
		VLTDR = ELTDR+(getDemandRate()*getDemandRate()*Velt);
		VMR = VLTDR/ELTDR;
		if(VMR > 1.0){
			double N = 0.0;
			double p = 0.0;
			N = (ELTDR*ELTDR)/(VLTDR-ELTDR);
			p = ELTDR/VLTDR;
			myLTDCDF = myNegBin;
			myParameters[0] = p;
			myParameters[1] = N;
		}
		else if(VMR < 1.0){
			myLTDCDF = myNormal;
			myParameters[0] = ELTDR;
			myParameters[1] = VLTDR;
		}
		else{

			myLTDCDF = myPoisson;
			myParameters[0] = ELTDR;
		}
		Distribution cdf = (Distribution)myLTDCDF;
		cdf.setParameters(myParameters);
	}

	protected double emmaY2(double y, double v){
		double f = 0.0;
		p.setMean(v);
		f = ((double)1/3)*(v*v*v*p.complementaryCDF(y-3)-3*y*v*v*p.complementaryCDF(y-2)+3*v*y*(y+1)*p.complementaryCDF(y-1)-y*(y+1)*(y+2)*p.complementaryCDF(y));
		return f;
	}	
}
