/*
* Created on Jun 21, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.varimetric;

import jsl.utilities.random.distributions.LossFunctionDistributionIfc;

/**
 * @author rossetti
 *
 */
public interface BaseStockItemIfc {

	/** An integer representing the item's number
	 * 
	 * @return
	 */
	int getItemNumber();
	
	/** Gets the unit cost of the item
	 * 
	 * @return
	 */
	double getUnitCost();
		
	/** Gets the stocking level of the item
	 * 
	 * @return
	 */
	int getStockLevel();
		
	/** Gets the demand rate for the item
	 * 
	 * @return
	 */
	double getDemandRate();
	
	/** Gets the lead time for the item.  The time
	 *  that it takes for the item to be resupplied
	 * 
	 * @return
	 */
	double getLeadTime();
	
	/** Gets the lead time demand distribution
	 * 
	 * @return
	 */
	LossFunctionDistributionIfc getLeadTimeDemandDistribution();

	/** Gets the expected number of back orders for the item
	 * 
	 * @return
	 */
	double getEBackOrders();
	
	/** Gets the variance of the number of back orders for the item
	 * 
	 * @return
	 */
	double getVBackOrders();
	
	/** Gets the expected on hand inventory for the item
	 * 
	 * @return
	 */
	double getEOnHand();
	
	/** Gets the probability of stockout base on {I(t)<=0}
	 * 
	 * @return
	 */
	double getPStockOut();
	
	/** Gets the stocking cost for the item, e.g. unit cost*stocking level
	 * @return
	 */
	double getStockingCost();
	
	/** Gets the expected backorder waiting time
	 * @return
	 */
	double getEBOWaitTime();
	
	/** Gets the variance of the backorder waiting time
	 * @return
	 */
	double getVBOWaitTime();
	
}
