/*
* Created on Jun 21, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.varimetric;

/**
 * @author rossetti
 *
 */
public interface VMBaseItemIfc extends BaseStockItemIfc {
	
	/** An integer representing this item's base
	 * 
	 * @return
	 */
	int getBaseNumber();
	
	/** The demand that goes out from the base to the depot for this item
	 *  This is (1.0-getRepairProb())*getDemandRate()
	 * @return
	 */
	double getReplenishmentDemandRate();

	/** Gets the repair time for this item at the base
	 * 
	 * @return
	 */
	double getRepairTime();
	
	/** Gets the probability that this item will be repaired
	 *  at the base
	 * 
	 * @return
	 */
	double getRepairProb();
	
	/** Gets the order and ship time for the item to this base
	 * 
	 * @return
	 */
	double getShipTime();
	
	/** Gets the expected number items in base repair
	 * 
	 * @return
	 */
	double getENumInBaseRepair();
	
	/** The expected number of items in order and shipping
	 * 
	 * @return
	 */
	double getENumInTransit();

	/** Gets the expected number of items at the depot
	 * @return
	 */
	public double getENumAtDepot();

	/** Gets the expected number of items in resupply
	 * 
	 * @return
	 */
	double getENumInResupply();

	/** Gets the variance of the number of items in resupply
	 * 
	 * @return
	 */
	double getVNumInResupply();

	/** Changes the repair probability at the base.  This method
	 *  will cause the depot demand rate to be updated and subsequently
	 *  all the lead time demand distributions (depot item and all base items)
	 * 
	 * @param prob
	 */
	public void setRepairProb(double prob);

	/** Sets the repair time for the item.  Updates the lead time
	 *  demand distribution for the item
	 * 
	 * @param time
	 */
	public void setRepairTime(double time);

	/** Sets the ship time for the item.  Updates the lead time
	 *  demand distribution for the item
	 * 
	 * @param time
	 */
	public void setShipTime(double time);

	/** Changes the demand rate at the base.  This method
	 *  will cause the depot demand rate to be updated and subsequently
	 *  all the lead time demand distributions (depot item and all base items)
	 * 
	 * @param rate
	 */
	public void setDemandRate(double rate);
	
}
