/*
* Created on Jun 30, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.varimetric;

import java.util.List;

import jsl.utilities.random.distributions.LossFunctionDistributionIfc;
import jsl.utilities.statistic.Statistic;

/**
 * @author rossetti
 *
 */

public class VMFMarginalAnalysisAlgorithm  {
	
	protected VariMetricModel myModel;
	protected List<VMItem> items;
	protected MAFItemData [] vmmaitemdata;
	
	protected int numItems;
	protected double []marginalBenefits;
	protected double myBudget;
	protected double myBudgetTolerance;
	protected double myTotalCost;
	protected double myTotalEBO;
	protected boolean noConvexPoint=false;
	protected boolean budgetExceeded=false;
	protected int itemIndexToAllocate;
	
	public VMFMarginalAnalysisAlgorithm(VariMetricModel model, double budget, double tolerance){
		this(model, budget, tolerance,null,null,null,null);
	}

	/*
	public VMFMarginalAnalysisAlgorithm(VariMetricModel model, double budget, double tolerance){
		
		items = myModel.getItems();
		int n=items.size();
		int [] DLL=new int [n];
		int [] DUL=new int [n];
		int [] Incr=new int [n];
		int [] SM=new int [n];
		int i=0;
		for(VMItem vi: items){
			LossFunctionDistributionIfc cdf = vi.getLeadTimeDemandDistribution();
			double mu = cdf.getMean();
			double s = cdf.getStandardDeviation();
			int fmu = (int)Math.floor(mu);
			int c2s = (int)Math.ceil(2.0*s);
			int sbl = (int)Math.floor(myBudget/vi.getUnitCost());
			int maxS;
			int minS;
			if (sbl <= fmu){
				maxS = sbl;
				minS = Math.max(0, maxS - c2s);
			} else {
				maxS = Math.min(sbl, fmu + c2s);
				minS = Math.max(0, fmu - c2s);
			}
			DLL[i]=minS;
			DUL[i]=maxS;
			SM[i]=maxS;
			Incr[i]=1;
			i++;
		}

	}
	*/
	public VMFMarginalAnalysisAlgorithm(VariMetricModel model, double budget, double tolerance, int [] DLowerLimits, int []DUpperLimits, int [] Increments, int []Smax){
		
		
		myModel=model;
		items = myModel.getItems();
		
		myBudget=budget;
		myBudgetTolerance=tolerance;
		numItems=items.size();
		
		if(DLowerLimits==null&& DUpperLimits==null&& Increments==null &&Smax==null){
			DLowerLimits=new int [numItems];
			DUpperLimits=new int [numItems];
			Increments=new int [numItems];
			Smax=new int [numItems];
			int i=0;
			for(VMItem vi: items){
				LossFunctionDistributionIfc cdf = vi.getLeadTimeDemandDistribution();
				double mu1 = cdf.getMean();
				double s1 = cdf.getStandardDeviation();
			//	double s2 = cdf.getVariance();
				int fmu1 = (int)Math.floor(mu1);
				int c2s1 = (int)Math.ceil(100*s1);
				//int c2s2 = (int)Math.ceil(3*s2);
				int sbl1 = (int)Math.floor(myBudget/vi.getUnitCost());
				int maxS1;
				int minS1;
				if (sbl1 <= fmu1){
					maxS1 = sbl1;
					minS1 = Math.max(0, maxS1 - c2s1);
				} else {
					maxS1 = Math.min(sbl1, fmu1 + c2s1);
					minS1 = Math.max(0, fmu1 - c2s1);
				}
				DLowerLimits[i]=minS1;
				DUpperLimits[i]=maxS1;
				Smax[i]=maxS1;
				Increments[i]=1;
				i++;
			}

		}
		
		//System.out.println("--- "+numItems);
		
		vmmaitemdata = new MAFItemData[numItems];
		marginalBenefits = new double[numItems];
		
		// need to implment check for the inputs of Upper Limits Lower Limits and 
		// provide the default reasonable values
		
		int i=0;
		for(VMItem vi: items){
	//		System.out.println("i AM HERE");
			int d[];
			d=createDepotLevels(DLowerLimits[i],DUpperLimits[i],Increments[i]);
			MAFItemData vmmaidata = new MAFItemData(d, 0.01, Smax[i],vi);
			/*
			vmmaidata.myItemNo=vi.myItemNumber;
			vmmaidata.myCost=vi.myUnitCost;
			System.out.println("NOW HERE");
			vmmaidata.computeMarginalBOArray(vi);
			*/
			vmmaitemdata[i]=vmmaidata;
			i++;
			
		}

	}
	
	public  int[] createDepotLevels(int upperLimit){
		return(createDepotLevels(0, upperLimit, 1));
	}
	
	public int[] createDepotLevels(int lowerLimit, int upperLimit){
		return(createDepotLevels(lowerLimit, upperLimit, 1));
	}
	
	public void optimize(){
		for(int i=0;i<numItems;i++){
			myTotalCost=myTotalCost+vmmaitemdata[i].myCost*vmmaitemdata[i].myScValues[0];
			myTotalEBO=myTotalEBO+vmmaitemdata[i].myAlphaHatcValues[0];
		}
		if(myTotalCost>=myBudget+myBudgetTolerance){
			budgetExceeded=true;
		}
		else{
			for(int i=0;i<numItems;i++){
				marginalBenefits[i]=vmmaitemdata[i].computeNextMarginalBenefit();
			}
			
		}
		
		while(!noConvexPoint&&!budgetExceeded){
			
			// note that different then Langrangian optimizations in terms of stoping criteria
			// algorithm stops at the fisrt convex point allocation
			// AFTER budget is violated.
			//I.e. the budget normally will be exceeded to stop
			// But it stops at the fist point it is violated
			//This issue can be chaged to the first point before budget violation
			itemIndexToAllocate=Statistic.getIndexOfMax(marginalBenefits);
		//	System.out.println("-------------------------");
		//	System.out.println("*** "+itemIndexToAllocate+" ***");
		//	for(int i=0;i<=1;i++){
		//	System.out.print("=="+marginalBenefits[i]);
		//	}
		//	System.out.println();
			
		//	for(int i=0;i<=1;i++){
		//		System.out.print("..."+vmmaitemdata[i].myLastConvexIndex);
		//	}
			if(marginalBenefits[itemIndexToAllocate]>(Double.MIN_VALUE)){
				updatePerformance();
			}

			else{
				noConvexPoint=true;
			}
		// System.out.println("("+myTotalCost+")");
		//	for(int i=0;i<=1;i++){
		//		System.out.print("..."+vmmaitemdata[i].myLastConvexIndex);
		//	}
		//	System.out.println();
	
		}
		setOptimalSolution();
		
	}
	
	protected void setOptimalSolution() {
		// TODO Auto-generated method stub
		int j = 0;
		int counter =0;
		int s =0;
		for(VMItem vi: items){
			s=vmmaitemdata[j].myScValues[vmmaitemdata[j].myLastConvexIndex];
			counter=vmmaitemdata[j].myDStar[s-vmmaitemdata[j].myLL];
			vi.setStockLevel(counter);
			vi.setBaseStockLevelsToZero();
			for(int k=counter+1;k<=s;k++){
				//done=false;
				VMBaseItem bi = vi.getBaseWithLargestProbOfStockout();
				bi.setStockLevel(bi.getStockLevel()+1);
			}
			j++;
		}
	}

	protected void updatePerformance() {
		// TODO Auto-generated method stub
		myTotalCost=myTotalCost+vmmaitemdata[itemIndexToAllocate].myCost*
		(vmmaitemdata[itemIndexToAllocate].myScValues[vmmaitemdata[itemIndexToAllocate].myLastConvexIndex+1]-
		 vmmaitemdata[itemIndexToAllocate].myScValues[vmmaitemdata[itemIndexToAllocate].myLastConvexIndex]);
		
		myTotalEBO=myTotalEBO-
		(vmmaitemdata[itemIndexToAllocate].myAlphaHatcValues[vmmaitemdata[itemIndexToAllocate].myLastConvexIndex]-
		vmmaitemdata[itemIndexToAllocate].myAlphaHatcValues[vmmaitemdata[itemIndexToAllocate].myLastConvexIndex+1]);
	//cHANGED..........
	//	vmmaitemdata[itemIndexToAllocate].myLastConvexIndex=vmmaitemdata[itemIndexToAllocate].myLastConvexIndex+1;
		
		if(myTotalCost>myBudget+myBudgetTolerance){
			budgetExceeded=true;
		}
		else{
			vmmaitemdata[itemIndexToAllocate].myLastConvexIndex=vmmaitemdata[itemIndexToAllocate].myLastConvexIndex+1;
			marginalBenefits[itemIndexToAllocate]=vmmaitemdata[itemIndexToAllocate].computeNextMarginalBenefit();
		}
	}

	public int[] createDepotLevels(int lowerLimit, int upperLimit, int increment){			
		if (lowerLimit < 0)
			throw new IllegalArgumentException("The minimum depot stock level must be >=0");
		if (upperLimit <= lowerLimit)
			throw new IllegalArgumentException("The maximum depot stock level must be > the lower limit");
		if (increment <= 0)
			throw new IllegalArgumentException("The search increment must be > 0");
		if (increment > (upperLimit - lowerLimit))
			throw new IllegalArgumentException("The step size must be <= (upper limit - lower limt");
		// determine the number of elements in the interval
		// the same reasoning can be used to eliminate mySS array in MAItemClass
		int n = (int)Math.floor((upperLimit - lowerLimit)/increment) + 1;
		int[] d = new int[n];
		int i = 0;
		for (int s=lowerLimit;s<=upperLimit; s=s+increment){
			d[i] = s;
			i++;
		}
		
		return(d);
	}
	
	
	
	
		
}
