/*
* Created on Jun 26, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.varimetric;

/**
 * @author rossetti
 *
 */
public class VariMetricModelTest2 {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
//		test();

//		testAlgorithm1();
		
		testAlgorithm2();
	}
	
	public static void test(){
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.001;
		double maxTheta = 2.0;
		int numPoints = 100;
		
		System.out.println("Evaluating the Cost constraint");
		VMLagrangeEnumerationAlgorithm algo = new VMLagrangeEnumerationAlgorithm(vm, budget, numPoints, minTheta, maxTheta);

		algo.evaluateCostConstraint(budget, minTheta, maxTheta, numPoints);
				
		System.out.println("----------------------------------------");
		
	}
	
	public static void testAlgorithm1(){
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.1;
		double maxTheta = 1.0;
		int maxIterations = 10;
		double budgetTolerance = 0.1;
		VMLagrangeIterativeAlgorithm algo = new VMLagrangeIterativeAlgorithm(vm, budget, maxIterations, budgetTolerance, minTheta, maxTheta);

		System.out.println("Testing Algorithm 1");	
		System.out.println("Max theta = " + algo.getRecommendedMaxAllowedValueOfLagrangeMultiplier());

		boolean satisfied = algo.optimize();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(vm);		
		System.out.println("----------------------------------------");
	}
	
	public static void testAlgorithm2(){
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.1;
		double maxTheta = 1.0;
		int numPoints = 10;
		VMLagrangeEnumerationAlgorithm algo = new VMLagrangeEnumerationAlgorithm(vm, budget, numPoints, minTheta, maxTheta);

		boolean satisfied = algo.optimize();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(vm);	
		System.out.println("----------------------------------------");
	}

}
