/*
* Created on Jun 26, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.inventory.varimetric;
import jsl.utilities.random.distributions.DUniform;
import jsl.utilities.random.distributions.Uniform;

/**
 * @author rossetti
 *
 */
public class VariMetricModelTestIterativeAlgorithm {

static DUniform baseRates = new DUniform(5,20);
static Uniform repairTime = new Uniform(0.005,0.03);
static Uniform repairProb = new Uniform(0.1,0.3);
static Uniform shipTime = new Uniform(0.01,0.07);
static int numBases=5;
static int numItems=5;
static double depotRepairTimes []=new double [numItems+1];
static double costOfstocks[] = new double[numItems+1];
static double baseRepTimes[][] =new double [numItems+1][numBases+1];
static double baseRepProb[][] =new double [numItems+1][numBases+1];
static double shipTimesToBase[][] = new double [numItems+1][numBases+1];
static double myBaseRates[][] = new double [numItems+1][numBases+1];
static VMItem myVMItems[]= new VMItem[numItems+1];
static VariMetricModel myVM =new VariMetricModel();
static double budget =1500.0;
static double minTheta = 0.0001;
static double maxTheta = 0.99;
static int numPoints = 100;
static int maxIterations = 100;
static double budgetTolerance = 0.1;
//static int [] myDLowerLimits = new  int[numItems];
//static int []myDUpperLimits=new  int[numItems] ; 

//static int [] myIncrements=new  int[numItems];
//static int []mySmax=new  int[numItems];

static int [] myDLowerLimits;
static int []myDUpperLimits; 
static int [] myIncrements;
static int []mySmax;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
//		test();
//		testAlgorithm1();
		
	//testAlgorithm2();
	//  testAlgorithm3();
  	//    testAlgorithm4();
 	 testAlgorithm5();
	
	}
		
	public static void test(){
		double SolutionTime;
		
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.001;
		double maxTheta = 2.0;
		int numPoints = 100;
		
		System.out.println("Evaluating the Cost constraint");
		VMLagrangeEnumerationAlgorithm algo = new VMLagrangeEnumerationAlgorithm(vm, budget, numPoints, minTheta, maxTheta);
		double StartTime=System.currentTimeMillis();
		algo.evaluateCostConstraint(budget, minTheta, maxTheta, numPoints);
		double EndTime=System.currentTimeMillis();
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
	}
	
	public static void testAlgorithm1(){
		double SolutionTime;
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.1;
		double maxTheta = 1.0;
		int maxIterations = 100;
		double budgetTolerance = 0.1;
		VMLagrangeIterativeAlgorithm algo = new VMLagrangeIterativeAlgorithm(vm, budget, maxIterations, budgetTolerance, minTheta, maxTheta);

		System.out.println("Testing Algorithm 1");	
		
		System.out.println("Max theta = " + algo.getRecommendedMaxAllowedValueOfLagrangeMultiplier());
		double StartTime=System.currentTimeMillis();
		boolean satisfied = algo.optimize();
		double EndTime=System.currentTimeMillis();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(vm);		
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
	}
	
	public static void testAlgorithm2(){
		double SolutionTime;
		VMItem di = new VMItem(1, 1, 1.0, 0.02531);
		
		di.addBaseItem(20, 0.02, 0.25, 0.05, 1, 1.0);
		di.addBaseItem(10, 0.015, 0.15, 0.07, 2, 1.0);
		di.addBaseItem(10, 0.01, 0.2, 0.04, 0, 1.0);
		di.addBaseItem(5, 0.005, 0.25, 0.02, 3, 1.0);
		di.addBaseItem(5, 0.01, 0.2, 0.01, 1, 1.0);
		
		// create the varimetric model
		
		VariMetricModel vm = new VariMetricModel();
		
		// add the items
		vm.addVMItem(di);
		
		// tell the model to optimize
		double budget = 1.0;
		double minTheta = 0.1;
		double maxTheta = 1.0;
		int numPoints = 10;
		VMLagrangeEnumerationAlgorithm algo = new VMLagrangeEnumerationAlgorithm(vm, budget, numPoints, minTheta, maxTheta);
		double StartTime=System.currentTimeMillis();
		boolean satisfied = algo.optimize();
		double EndTime=System.currentTimeMillis();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(vm);	
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
	}

	public static void testAlgorithm3(){
		double SolutionTime;
		
		prepareExperimentalSetup();
		//System.out.println(myVM);
		double StartTime=System.currentTimeMillis();
		VMLagrangeEnumerationAlgorithm algo = new VMLagrangeEnumerationAlgorithm(myVM, budget, numPoints, minTheta, maxTheta);
		
		boolean satisfied = algo.optimize();
		double EndTime=System.currentTimeMillis();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(myVM);	
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
	}
	
	public static void prepareExperimentalSetup() {
		// TODO Auto-generated method stub
		costOfstocks[1]=5.0;
		costOfstocks[2]=3.0;
		costOfstocks[3]=8.0;
		costOfstocks[4]=6.0;
		costOfstocks[5]=4.0;
		
		
		for (int i=1;i<=numItems;i++){
			depotRepairTimes[i]=repairTime.getValue();
			myVMItems[i]=new VMItem(i,0,costOfstocks[i],depotRepairTimes[i]);
			for(int j=1;j<=numBases;j++){
				myBaseRates[i][j]=baseRates.getValue();
				baseRepTimes[i][j]=repairTime.getValue();
				baseRepProb[i][j]=repairProb.getValue();
				shipTimesToBase[i][j]=shipTime.getValue();
				myVMItems[i].addBaseItem(myBaseRates[i][j],baseRepTimes[i][j],baseRepProb[i][j],shipTimesToBase[i][j],0,costOfstocks[i]);
			}
			myVM.addVMItem(myVMItems[i]);
		}
		
		/*
		for(int i=0;i<numItems;i++){
			myDLowerLimits[i]=0;
			myDUpperLimits[i]=60;
			 myIncrements[i]=1;
			 mySmax[i]=60;
		}
		*/
		
		
		
	}

	public static void testAlgorithm4(){
		double SolutionTime;
		prepareExperimentalSetup();
	//	System.out.println(myVM);
		double StartTime=System.currentTimeMillis();
		VMLagrangeIterativeAlgorithm algo = new VMLagrangeIterativeAlgorithm(myVM, budget, maxIterations, budgetTolerance, minTheta, maxTheta);
		
		boolean satisfied = algo.optimize();
		double EndTime=System.currentTimeMillis();
		System.out.println("Budget satisfied = " + satisfied);
		System.out.println("final theta evaluated = " + algo.getFinalLagrangeMultiplierValue());
		System.out.println(myVM);	
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
		
		
	
		
	}
	
	public static void testAlgorithm5(){

		double SolutionTime;
		
		prepareExperimentalSetup();
		double StartTime=System.currentTimeMillis();
		VMFMarginalAnalysisAlgorithm algo =new VMFMarginalAnalysisAlgorithm(myVM,budget,0);
		algo.optimize();
		double EndTime=System.currentTimeMillis();
		


		System.out.println(myVM);		
		System.out.println("----------------------------------------");
		SolutionTime=EndTime-StartTime;
		System.out.println("Solution Time = "+SolutionTime+" miliseconds");
}
}

