/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.scinventory;

import jsl.modeling.SchedulingElement;
import jsl.modeling.ModelElement;

/**
 *
 */
public abstract class InventoryProviderAbstract extends SchedulingElement implements
		InventoryProviderIfc {

	//  ===========================================
	//      CLASS VARIABLES
	//  ===========================================
	

	//  ===========================================
	//      OBJECT VARIABLES
	//  ===========================================


	/** Creates an InventoryProviderAbstract with the supplied parent
	 * @param parent
	 */
	public InventoryProviderAbstract(ModelElement parent) {
		this(parent, null);
	}

	/** Creates an InventoryProviderAbstract with the supplied parent and name
	 * @param parent
	 * @param name
	 */
	public InventoryProviderAbstract(ModelElement parent, String name) {
		super(parent, name);
		
	}

	//  ===========================================
	//      PUBLIC METHODS
	//  ===========================================

	
	//  ===========================================
	//      ABSTRACT METHODS
	//  ===========================================
		
	//  ===========================================
	//      PROTECTED METHODS
	//  ===========================================
	
	/** Allows subclasses to set the inventory provider
	 *  on a demand
	 */
	protected final void setInventoryProvider(Demand d){
		if (d == null)
			throw new IllegalArgumentException("Invalid argument. Demand demand must be non-null.");
		d.setInventoryProvider(this);
	}

	/** Allows subclasses to set the inventory customer
	 *  on a demand
	 */
	protected final void setInventoryCustomer(InventoryCustomerIfc customer, Demand d){
		if (d == null)
			throw new IllegalArgumentException("Invalid argument. Demand demand must be non-null.");
		if (customer == null)
			throw new IllegalArgumentException("Invalid argument. InventoryCustomerIfc customer must be non-null.");
		d.setInventoryCustomer(customer);
	}

	//  ===========================================
	//      PROTECTED STATIC METHODS
	//  ===========================================

}
