/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.scinventory;

import jsl.modeling.ModelElement;

/**
 */
public class RQInventoryPolicy extends InventoryPolicyAbstract {

	private int myReorderPoint;
	
	private int myReorderQty;
	
	/**
	 * @param parent
	 */
	public RQInventoryPolicy(ModelElement parent) {
		this(parent, 0, 1, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public RQInventoryPolicy(ModelElement parent, int reorderPoint, int reorderQty) {
		this(parent, reorderPoint, reorderQty, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public RQInventoryPolicy(ModelElement parent, int reorderPoint, int reorderQty, String name) {
		super(parent, name);
		setPolicyParameters(reorderPoint, reorderQty);
	}

	/* (non-Javadoc)
	 * @see jsl.examples.inventory.InventoryPolicyAbstract#checkInventory(jsl.examples.inventory.Inventory)
	 */
	public void checkInventory() {
		int ip = getInventoryPosition();
		if (ip <= myReorderPoint){
			requestReplenishment(myReorderQty);
		}	
	}

	/**
	 * @return Returns the reorderPoint.
	 */
	public final int getReorderPoint() {
		return myReorderPoint;
	}

	/**
	 * @return Returns the myReorderQty.
	 */
	public final int getReorderQty() {
		return myReorderQty;
	}
	
	/**
	 * @param myReorderPoint The myReorderPoint to set.
	 */
	public final void setPolicyParameters(int reorderPoint, int reorderQty) {
		if (reorderQty < 1)
			throw new IllegalArgumentException("The reorder quantity must be >=1");

		if (reorderPoint < -reorderQty)
			throw new IllegalArgumentException("The reorder quantity must be >= -reorderQty");

		myReorderPoint = reorderPoint;
		myReorderQty = reorderQty;
	}

}
