/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.scinventory;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.EventGeneratorListenerIfc;
import jsl.utilities.random.distributions.Constant;

/**
 *
 */
public class SSPeriodicInventoryPolicy extends InventoryPolicyAbstract {

	private int myReorderPoint;
	
	private int myOrderUpToPoint;
	
	private int myReorderPeriod;
	
	private EventGenerator myInventoryCheckGenerator;
	
	/**
	 * @param parent
	 */
	public SSPeriodicInventoryPolicy(ModelElement parent) {
		this(parent, 0, 1, 1, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public SSPeriodicInventoryPolicy(ModelElement parent, int reorderPoint, int orderUpToPoint, int reorderPeriod) {
		this(parent, reorderPoint, orderUpToPoint, reorderPeriod, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public SSPeriodicInventoryPolicy(ModelElement parent, int reorderPoint, int orderUpToPoint, int reorderPeriod, String name) {
		super(parent, name);
		setPolicyParameters(reorderPoint, orderUpToPoint, reorderPeriod);
		myInventoryCheckGenerator =
			new EventGenerator(this, new InventoryCheck(), new Constant(0.0), new Constant(myReorderPeriod), "Inventory Check");

	}

	/* (non-Javadoc)
	 * @see jsl.examples.inventory.InventoryPolicyAbstract#checkInventory(jsl.examples.inventory.Inventory)
	 */
	public void checkInventory() {
		
	}	
	
	/**
	 * @return Returns the reorderPoint.
	 */
	public final int getReorderPoint() {
		return myReorderPoint;
	}

	/**
	 * @return Returns the myOrderUpToPoint.
	 */
	public final int getOrderUpToPoint() {
		return myOrderUpToPoint;
	}
	
	/**
	 * @param myReorderPoint The myReorderPoint to set.
	 */
	public final void setPolicyParameters(int reorderPoint, int orderUpToPoint, int reorderPeriod) {
		if (orderUpToPoint < 1)
			throw new IllegalArgumentException("The order up to point must be >=1");

		if (reorderPoint < -orderUpToPoint)
			throw new IllegalArgumentException("The order up to point must be >= -reorderQty");
		
		if (reorderPeriod <= 0)
			throw new IllegalArgumentException("The reorder period must be greater than 0");

		myReorderPoint = reorderPoint;
		myOrderUpToPoint = orderUpToPoint;
		myReorderPeriod = reorderPeriod;
	}


private class InventoryCheck implements EventGeneratorListenerIfc {
	public void generate(EventGenerator generator, JSLEvent event) {
		
		if (getInventoryPosition()<= myReorderPoint){
			int orderSize = (int)(myOrderUpToPoint - getInventoryPosition());
			requestReplenishment(orderSize);
	}
	}
}
}
	


