/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.scinventory;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.reporting.JSL;
import jslx.scinventory.*;

/**
 */
public class TestInventoryElements {

	public static void main(String[] args) {

		System.out.println("Test InventoryElements");
        
		JSL.LOGGER.setLevel(Level.INFO);
   
//		testReplication();
		testExperiment();
        
		System.out.println("Done!");		
	}
	
	private static Model buildModel(){
		
		// create the containing model
		Model m = Model.createModel();
		
		// create the distributions
		DistributionIfc tbd = new Exponential(1.0/3.6);
		DistributionIfc lt = new Constant(0.5);
		int rpt = -1;
		int qty = 2;
		int level = 0;

		// define the type
		ItemType type = new ItemType(m, "Type-A");
		
		// define the providers
		LeadTimeReplenishmentProvider supplier = new LeadTimeReplenishmentProvider(m);
		supplier.addLeadTime(type,lt);
		
		DefaultBackLogPolicy bpolicy = new DefaultBackLogPolicy(m);
		RQInventoryPolicy ipolicy = new RQInventoryPolicy(m, rpt, qty);
		Inventory inventory= new Inventory(m, type, supplier, ipolicy, supplier, level, bpolicy, false);
		
		// define the demand generator
		DemandGenerator dg = new DemandGenerator(m, tbd, tbd);		
		dg.setDemandRequirements(type, true, false, inventory);
		
		return(m);
	}
	
	public static void testExperiment(){
		
		Model m = buildModel();
		
		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(30);
		e.setLengthOfReplication(3360.0);
		e.setLengthOfWarmUp(360.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}

}
