/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.scinventory.testing;

import java.util.logging.Level;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestSSInventoryModel {

	public static void main(String[] args) {

		System.out.println("SSInventoryModel Test");
        
		JSL.LOGGER.setLevel(Level.INFO);
   
//		testReplication();
		testExperiment();
        
		System.out.println("Done!");		
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		DistributionIfc tbd = new Exponential(1.0/3.6);
		DistributionIfc lt = new Constant(0.5);
		int rpt = -1;
		int upto = 1;
		int level = 0;
		SSInventoryModel ss = new SSInventoryModel(m, tbd, lt, rpt, upto, level);
		
		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(30);
		e.setLengthOfReplication(3360.0);
		e.setLengthOfWarmUp(360.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}

}
