/*
 * Created on Jun 7, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.afosr.examples;

import jsl.utilities.random.distributions.DistributionIfc;
import jslx.supplychain.afosr.forecasting.DemandAndForecastArrivalListenerForNetwork;
import jslx.supplychain.inventorylayer.Inventory;

/**
 * @author vvarghe
 *
 * This class implements an algorithm that optimizes an (s, S)
 * inventory policy (Hoff and Spearman, Factory Physics).
 * It has reference to a simulation model that can be optimized.
 * 
 * 
 */
public class DefaultSSNetworkInventoryUpdater{

	private double myInitialEstimate;
	private DistributionIfc myLeadTime;	
	private UpdatableSSInventoryNetwork myUpdatable;
	private Inventory myInventory;
	private double myUpdatePeriod;
	private DemandAndForecastArrivalListenerForNetwork myListener;
	
	public DefaultSSNetworkInventoryUpdater(Inventory inv, DemandAndForecastArrivalListenerForNetwork listener, 
			double fcst, DistributionIfc leadtime, double updt){
		myInitialEstimate = fcst;
		myInventory = inv;
		myLeadTime = leadtime;
		myUpdatePeriod = updt;
		myListener = listener;
	}
	
	public double[] initparam(){
		double fcst = myInitialEstimate;
		if(fcst > 0.0){
			double setupcost = 1.0;
			double holdingcost = 1.0;
			double Q = Math.sqrt((2*setupcost*fcst)/holdingcost);
			Q = (int)(Math.ceil(Q));
			Q = Math.max(1.0, Q);				
			double L_bar = myLeadTime.getMean();
			double L_var = myLeadTime.getVariance();
			double L_fcst = L_bar*fcst;
			double mse = fcst;
			double small_s = L_fcst + 0.999*Math.sqrt((L_bar*mse)+(fcst*fcst*L_var));
			small_s = (Math.ceil(small_s));
			small_s = Math.max(0.0, small_s);
			double big_s = Q + small_s;
			big_s = (Math.ceil(big_s));
			double [] par = {small_s,big_s};
			//System.out.println(small_s+","+big_s+" small_s,big_s");
			return par;						
		}else{
			double [] par = {0,1};
			return par;
		}
	}
	
	public double[] optimize() {
		double fcst = myListener.getForecastManager().getLastForecast()/myUpdatePeriod;
		if(fcst > 0.0){
			double setupcost = 1.0;
			double holdingcost = 1.0;
			double Q = Math.sqrt((2*setupcost*fcst)/holdingcost);
			Q = (int)(Math.ceil(Q));
			Q = Math.max(1.0, Q);				
			double L_bar = myLeadTime.getMean();
			double L_var = myLeadTime.getVariance();
			double L_fcst = L_bar*fcst;
			double mse = myListener.getForecastManager().getMSE()/(myUpdatePeriod*myUpdatePeriod);				
			double small_s = L_fcst + 0.95*Math.sqrt((L_bar*mse)+(fcst*fcst*L_var));
			small_s = (Math.ceil(small_s));
			small_s = Math.max(0.0, small_s);				
			double big_s = Q + small_s;
			big_s = (Math.ceil(big_s));				
			double [] par = {small_s,big_s};
			//System.out.println(small_s+","+big_s+" small_s,big_s");
			return par;						
		}else{
			double [] par = {0,1};
			return par;
		}	
	}
	
}
