/*
 * Created on Jun 7, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.afosr.examples;

import jslx.supplychain.afosr.forecasting.UpdatableIfc;

/**
 * @author vvarghe
 *
 * This class implements an algorithm that optimizes an (s, S)
 * inventory policy (Hoff and Spearman, Factory Physics).
 * It has reference to a simulation model that can be optimized.
 * 
 * 
 */
public class DefaultsSInventoryUpdater{

	private ForecastBasedUpdatingsSInventoryModel myUpdatable;
	private double initialestimate;
	private int agg;
	
	public DefaultsSInventoryUpdater(ForecastBasedUpdatingsSInventoryModel updt, int agglevel){
		myUpdatable = updt;	
		agg = agglevel;
	}
	
	public void initparam(){
		double fcst = myUpdatable.myInitialDemandRateEstimate;
		initialestimate = fcst;

		if(fcst > 0.0){
			double setupcost = 1.0;
			double holdingcost = 1.0;
			double Q = Math.sqrt((2*setupcost*fcst)/holdingcost);
			Q = (int)(Math.ceil(Q));
			Q = Math.max(1.0, Q);				
			double L_bar = myUpdatable.getLeadTime().getMean();
			double L_var = myUpdatable.getLeadTime().getVariance();
			double L_fcst = L_bar*fcst;
			double mse = myUpdatable.myInitialDemandRateEstimate;				
			double small_s = L_fcst + 0.999*Math.sqrt((L_bar*mse)+(fcst*fcst*L_var));
			small_s = (Math.ceil(small_s));
			small_s = Math.max(0.0, small_s);				
			double big_s = Q + small_s;
			big_s = (Math.ceil(big_s));				
			double [] par = {small_s,big_s};
			//System.out.println(small_s+","+big_s);
			myUpdatable.receiveInitialParameters(par);						
		}else{
			double [] par = {0,1};
			myUpdatable.receiveInitialParameters(par);
		}
	}
	
	public void optimize(Object obj) {		
		if((ForecastBasedUpdatingsSInventoryModel)obj == myUpdatable){			
			double fcst = myUpdatable.getDemandArrivalListener().getDemandForecaster().getLastForecast()/agg;
			double dem = myUpdatable.getDemandArrivalListener().getDemandForecaster().getDemand();
			double time = myUpdatable.getDemandArrivalListener().getDemandForecaster().getCurrentTimeIndex();
			double rep = myUpdatable.getDemandArrivalListener().getCurrentReplication().getReplicationNumber();
			if(fcst > 0.0){
				double setupcost = 1.0;
				double holdingcost = 1.0;
				double Q = Math.sqrt((2*setupcost*fcst)/holdingcost);
				Q = (int)(Math.ceil(Q));
				Q = Math.max(1.0, Q);
				//System.out.println("q "+Q);
				double L_bar = myUpdatable.getLeadTime().getMean();
				double L_var = myUpdatable.getLeadTime().getVariance();
				double L_fcst = L_bar*fcst;
				double mse = myUpdatable.getDemandArrivalListener().getDemandForecaster().getMSE()/(agg*agg);				
				double small_s = L_fcst + 0.95*Math.sqrt((L_bar*mse)+(fcst*fcst*L_var));
				small_s = (Math.ceil(small_s));
				small_s = Math.max(0.0, small_s);				
				double big_s = Q + small_s;
				big_s = (Math.ceil(big_s));				
				double [] par = {small_s,big_s};
				//System.out.println(small_s+","+big_s);
				myUpdatable.receiveOptimalParameters(par);						
			}else{
				double [] par = {0,1};
				myUpdatable.receiveOptimalParameters(par);
			}
		}	
	}
	
	public UpdatableIfc getUpdatableInventoryModel() {		
		return myUpdatable;
	}
	
}
