/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.afosr.examples;

import java.io.IOException;


import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.elements.variable.nhpp.NHPPTimeBtwEventRV;
import jsl.modeling.elements.variable.nhpp.PiecewiseLinearRateFunction;
import jsl.modeling.elements.variable.nhpp.PiecewiseRateFunction;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.SimpleExponentialSmoothing;
import jslx.supplychain.inventorylayer.ItemType;

/**
 */
public class TestForecastBasedUpdatingSSInventoryModel {

	public static void main(String[] args) {
		System.out.println("ForecastBasedUpdatingsSInventoryAbstractModel Test");       
		testExperiment();       
		System.out.println("Done!");		
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(1.0, 3650.0, 100.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV = new NHPPTimeBtwEventRV(m, f);
		double initforecastest = 1.0;
		
		DistributionIfc lt = new Exponential(1.0);
		ItemType myItemType = new ItemType(m, "Type-A");
		int level = 0;
		int s = 1;
		int S = 3;
		//check that the aggregation level is 1
		int myAggregationLevel = 1;
		ForecastTechnique forecaster = new SimpleExponentialSmoothing(0.25, "SES");
		ForecastBasedUpdatingsSInventoryModel myModel = new ForecastBasedUpdatingsSInventoryModel (m, myItemType, 
				myNHPPTimeBtwEventRV, myNHPPTimeBtwEventRV, lt, level, s, S, myAggregationLevel, initforecastest,
				"ForecastBasedUpdatingsSInventoryModel", forecaster, true);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m,"4-Test-ForecastBasedUpdatingsSInventoryModel");
		
		try{
        	CSVReport ecsv = new CSVExperimentReport("ForecastBasedUpdatingsSInventoryModel Test");
        	m.addObserver(ecsv);
        }
        catch(IOException exception){
			System.out.println("I/O exception opening file Output.txt");
			exception.printStackTrace();
        }
        
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(3650.0);
		
		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput(true);
		
		// tell the experiment to run
		e.runAll();
	}

}
