/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.afosr.examples;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.supplychain.TimeBasedMultiEchelonIHPNetworkWithTypeLocationIndependentCarrier;
import jslx.supplychain.inventorylayer.InventoryHoldingPoint;
import jslx.supplychain.inventorylayer.ItemType;

public class TestMultiEchelonIHPNetworkWithTypeLocationIndependentCarrier {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("6MultiEchelonIHPNetworkWithTypeLocationIndependentCarrier Test");
        
		testExperiment();
        
		System.out.println("Done!");	
	}

	public static Model buildModel(){
		Model m = Model.createModel();
		
		//Create the network
		TimeBasedMultiEchelonIHPNetworkWithTypeLocationIndependentCarrier n = new TimeBasedMultiEchelonIHPNetworkWithTypeLocationIndependentCarrier(m, "ME-Inventory-Network-TypeLocationIndependentCarrier");
		
		//Define item types
		DistributionIfc d = new Constant(1.0);
		ItemType type1 = n.addItemType("Type-1", d);
		ItemType type2 = n.addItemType("Type-2", d);
		ItemType type3 = n.addItemType("Type-3", d);
				
		//Define Warehouse
		InventoryHoldingPoint warehouse = n.addInventoryHoldingPoint("Warehouse");
		
		//addInventory to warehouse
		warehouse.addReorderPointReorderQuantityInventory(type1,4,1,20);
		warehouse.addReorderPointReorderQuantityInventory(type2,4,1,20);
		warehouse.addReorderPointReorderQuantityInventory(type3,4,1,20);
				
		// Create the retailers which use the warehouse as its filler
		InventoryHoldingPoint retailer1 = n.addInventoryHoldingPoint("R1");
		InventoryHoldingPoint retailer2 = n.addInventoryHoldingPoint("R2");
		InventoryHoldingPoint retailer3 = n.addInventoryHoldingPoint("R3");
		InventoryHoldingPoint retailer4 = n.addInventoryHoldingPoint("R4");
		
		//Create the inventory at each retailer
		retailer1.addReorderPointReorderQuantityInventory(type1,2,1,10);
		retailer1.addReorderPointReorderQuantityInventory(type2,2,1,10);
		
		retailer2.addReorderPointReorderQuantityInventory(type1,2,1,10);
		retailer2.addReorderPointReorderQuantityInventory(type2,2,1,10);
		retailer2.addReorderPointReorderQuantityInventory(type3,2,1,10);
		
		retailer3.addReorderPointReorderQuantityInventory(type1,2,1,10);
		retailer3.addReorderPointReorderQuantityInventory(type2,2,1,10);
		retailer3.addReorderPointReorderQuantityInventory(type3,2,1,10);
		
		retailer4.addReorderPointReorderQuantityInventory(type1,2,1,10);
		retailer4.addReorderPointReorderQuantityInventory(type2,2,1,10);
		
		// create the structure of the network
		// attach the warehouse to the external supplier
		n.attachIHPToExternalSupplier(warehouse);
		Constant d0 = new Constant(0.0);
		n.setTransportDelayToExternalSupplier(warehouse, type1, d0);
		n.setTransportDelayToExternalSupplier(warehouse, type2, d0);
		n.setTransportDelayToExternalSupplier(warehouse, type3, d0);
		
		// attach the retailers to the warehouse, reuse the constant leadtime of 1.0
		n.attachIHPToSupplier(warehouse, retailer1);
		n.setTransportDelayToSupplier(warehouse, retailer1, type1, d0);
		n.setTransportDelayToSupplier(warehouse, retailer1, type2, d0);
		
		n.attachIHPToSupplier(warehouse, retailer2);
		n.setTransportDelayToSupplier(warehouse, retailer2, type1, d0);
		n.setTransportDelayToSupplier(warehouse, retailer2, type2, d0);
		n.setTransportDelayToSupplier(warehouse, retailer2, type3, d0);
		
		n.attachIHPToSupplier(warehouse, retailer3);
		n.setTransportDelayToSupplier(warehouse, retailer3, type1, d0);
		n.setTransportDelayToSupplier(warehouse, retailer3, type2, d0);
		n.setTransportDelayToSupplier(warehouse, retailer3, type3, d0);
		
		n.attachIHPToSupplier(warehouse, retailer4);
		n.setTransportDelayToSupplier(warehouse, retailer4, type1, d0);
		n.setTransportDelayToSupplier(warehouse, retailer4, type2, d0);	
		
		
		// create the demand generators for the retailers						
		DistributionIfc dA = new Exponential(1.0);				
		Constant dDG = new Constant(0.0);
		
		n.attachDemandGeneratorToIHP(retailer1, type1, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer1, type2, dA, dDG);
		
		n.attachDemandGeneratorToIHP(retailer2, type1, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer2, type2, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer2, type3, dA, dDG);

		n.attachDemandGeneratorToIHP(retailer3, type1, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer3, type2, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer3, type3, dA, dDG);

		n.attachDemandGeneratorToIHP(retailer4, type1, dA, dDG);
		n.attachDemandGeneratorToIHP(retailer4, type2, dA, dDG);
		
		return(m);
	}
	
	public static void testExperiment(){
		
		Model m = buildModel();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m, "6-MultiEchelon-IHP-Network-With-TypeLocationIndependentCarrier");
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(3650.0);
		e.setLengthOfWarmUp(100.0);
		
		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput(true);
		
		// tell the experiment to run
		e.runAll();
	}
}
