/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.afosr.examples;

import java.io.IOException;


import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.elements.variable.nhpp.NHPPTimeBtwEventRV;
import jsl.modeling.elements.variable.nhpp.PiecewiseLinearRateFunction;
import jsl.modeling.elements.variable.nhpp.PiecewiseRateFunction;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.Bernoulli;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.SimpleExponentialSmoothing;
import jslx.supplychain.afosr.forecasting.ForecastManager;
import jslx.supplychain.inventorylayer.InventoryHoldingPoint;
import jslx.supplychain.inventorylayer.InventoryRepairingHoldingPoint;
import jslx.supplychain.inventorylayer.ItemType;

public class TestUpdatableSSInventoryNetwork {

	/**
	 * @param args
	 */
	
	public static void main(String[] args) {
		System.out.println("ForecastBasedUpdatableInventorySystem Test");        
		testExperiment();        
		System.out.println("Done!");	
	}
	
	public static Model test1(){
		Model m = Model.createModel();
		
						
		return(m);
	}
	
	public static Model test2(){
		
		Model m = Model.createModel();
		
//		Case 1 appropriate	30
//		Case 2 appropriate	120
//		Case -- appropriate	no
//		Case 4 Same	30 (this is 3 in the journal article)
//		Case 5 Same	120 (this is 4 in the journal article)
//		case 6 Same	no (this is 5 in the journal article)

		
		double updatePeriod = 30.0;		
		UpdatableSSInventoryNetwork n = new UpdatableSSInventoryNetwork(m, updatePeriod, "ForecastBasedUpdatableInventorySystem");		
		
		DistributionIfc productionDelayT1 = new Constant(0.0);
		ItemType type1 = n.addItemType("Type-1", productionDelayT1);
		
		int reorderPtDepT1 = 1;
		int ordertoLevelDepT1 = 2;
		int initialLevelDepT1 = 2;//this will be reset to S during the initialization
		DistributionIfc travelTimetoDepotT1 = new Constant(0.0);
		ForecastTechnique forecasterDepT1 = new SimpleExponentialSmoothing(0.15, "Depot"+type1.getName());
		boolean updatableDepT1 = false;
		double initialForecastEstimateDepT1 = 154.0*updatePeriod;		
		n.addDepotInventory(type1, reorderPtDepT1, ordertoLevelDepT1, initialLevelDepT1, travelTimetoDepotT1, forecasterDepT1, 
				initialForecastEstimateDepT1, updatableDepT1);
		
		// Create the retailers which use the warehouse as its filler
		InventoryHoldingPoint base1 = n.addBase("Base1");
		InventoryHoldingPoint base2 = n.addBase("Base2");
		InventoryHoldingPoint base3 = n.addBase("Base3");
		InventoryHoldingPoint base4 = n.addBase("Base4");
		
		PiecewiseRateFunction f1 = new PiecewiseLinearRateFunction(101.0, 3650.0, 1.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV1 = new NHPPTimeBtwEventRV(m, f1);
		
		PiecewiseRateFunction f2 = new PiecewiseLinearRateFunction(1.0, 3650.0, 101.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV2 = new NHPPTimeBtwEventRV(m, f2);
		
		PiecewiseRateFunction f3 = new PiecewiseLinearRateFunction(1.0, 1750.0, 101.0);
		f3.addRateSegment(1750.0,1.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV3 = new NHPPTimeBtwEventRV(m, f3);
		
		PiecewiseRateFunction f4 = new PiecewiseLinearRateFunction(51.0, 3650.0, 51.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV4 = new NHPPTimeBtwEventRV(m, f4);
		
		int reorderPtBas1T1 = 1;
		int ordertoLevelBas1T1 = 2;
		int initialLevelBas1T1 = 2;//this will be reset to S during the initialization
		//ForecasterAbstract forecasterBas1T1 = new HoltsWintersNonSeasonal(0.15, 0.15, base1.getName()+type1.getName());
		ForecastTechnique forecasterBas1T1 = new SimpleExponentialSmoothing(0.15, base1.getName()+type1.getName());		
		boolean updatableBas1T1 = false;
		//double initialForecastEstimateBas1T1 = ((0.5*3650*100)+3650)*1*(updatePeriod/3650);
		double initialForecastEstimateBas1T1 = 101.0*updatePeriod;
		DistributionIfc travelTimefromDepottoBase1T1 = new Exponential(50);
		n.addBaseInventory(base1, type1, reorderPtBas1T1, ordertoLevelBas1T1, initialLevelBas1T1, travelTimefromDepottoBase1T1, 
				forecasterBas1T1, initialForecastEstimateBas1T1, updatableBas1T1, myNHPPTimeBtwEventRV1);
		
		int reorderPtBas2T1 = 1;
		int ordertoLevelBas2T1 = 2;
		int initialLevelBas2T1 = 2;//this will be reset to S during the initialization
		//ForecasterAbstract forecasterBas2T1 = new HoltsWintersNonSeasonal(0.15, 0.15, base2.getName()+type1.getName());
		ForecastTechnique forecasterBas2T1 = new SimpleExponentialSmoothing(0.15, base2.getName()+type1.getName());
		boolean updatableBas2T1 = false;
		//double initialForecastEstimateBas2T1 = ((0.5*3650*100)+3650)*1*(updatePeriod/3650);
		double initialForecastEstimateBas2T1 = 1.0*updatePeriod;
		DistributionIfc travelTimefromDepottoBase2T1 = new Exponential(50);			
		n.addBaseInventory(base2, type1, reorderPtBas2T1, ordertoLevelBas2T1, initialLevelBas2T1, travelTimefromDepottoBase2T1, 
				forecasterBas2T1, initialForecastEstimateBas2T1, updatableBas2T1, myNHPPTimeBtwEventRV2);
		
		int reorderPtBas3T1 = 1;
		int ordertoLevelBas3T1 = 2;
		int initialLevelBas3T1 = 2;//this will be reset to S during the initialization
		ForecastTechnique forecasterBas3T1 = new SimpleExponentialSmoothing(0.15, base3.getName()+type1.getName());
		boolean updatableBas3T1 = false;
		//double initialForecastEstimateBas3T1 = ((0.5*3650*100)+3650)*1*(updatePeriod/3650);
		double initialForecastEstimateBas3T1 = 1.0*updatePeriod;
		DistributionIfc travelTimefromDepottoBase3T1 = new Exponential(50);			
		n.addBaseInventory(base3, type1, reorderPtBas3T1, ordertoLevelBas3T1, initialLevelBas3T1, travelTimefromDepottoBase3T1, 
				forecasterBas3T1, initialForecastEstimateBas3T1, updatableBas3T1, myNHPPTimeBtwEventRV3);
		
		int reorderPtBas4T1 = 1;
		int ordertoLevelBas4T1 = 2;
		int initialLevelBas4T1 = 2;//this will be reset to S during the initialization
		ForecastTechnique forecasterBas4T1 = new SimpleExponentialSmoothing(0.15, base4.getName()+type1.getName());
		boolean updatableBas4T1 = false;
		//double initialForecastEstimateBas4T1 = ((0.5*3650*100)+3650)*1*(updatePeriod/3650);		
		double initialForecastEstimateBas4T1 = 51.0*updatePeriod;
		DistributionIfc travelTimefromDepottoBase4T1 = new Exponential(50);			
		n.addBaseInventory(base4, type1, reorderPtBas4T1, ordertoLevelBas4T1, initialLevelBas4T1, travelTimefromDepottoBase4T1, 
				forecasterBas4T1, initialForecastEstimateBas4T1, updatableBas4T1, myNHPPTimeBtwEventRV4);
		
		return(m);
	}
	
	public static Model buildModel(){
//		Model m = test1();
		
		Model m = test2();
		
		return(m);
	}
	
	public static void testExperiment(){
		
		Model m = buildModel();	
		
		// create the experiment to run the model
		Experiment e = new Experiment(m,"UpdatableSSInventorySystem-TestCase6c");
		
		try{
        	CSVReport ecsv = new CSVExperimentReport("UpdatableSSInventoryNetwork-TestCase6c");
        	m.addObserver(ecsv);
        }
        catch(IOException exception){
			System.out.println("I/O exception opening file Output.txt");
			exception.printStackTrace();
        }
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(3650.0);
		//e.setLengthOfWarmUp(365.0);
		
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput(true);
		
		
		
		// tell the experiment to run
		e.runAll();
	}
}
