/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.afosr.examples;

import java.io.IOException;


import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.elements.variable.nhpp.NHPPTimeBtwEventRV;
import jsl.modeling.elements.variable.nhpp.PiecewiseLinearRateFunction;
import jsl.modeling.elements.variable.nhpp.PiecewiseRateFunction;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.Bernoulli;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.Exponential;
import jslx.forecasting.SimpleExponentialSmoothing;
import jslx.supplychain.afosr.forecasting.ForecastManager;
import jslx.supplychain.inventorylayer.InventoryRepairingHoldingPoint;
import jslx.supplychain.inventorylayer.ItemType;

public class TestVarimetricNetworkLGIterative {

	/**
	 * @param args
	 */
	
	public static void main(String[] args) {
		System.out.println("VarimetricNetworkLGIterative Test");        
		testExperiment();        
		System.out.println("Done!");	
	}
	
	public static Model test1(){
		System.out.println("This test is used to verify the forecast evaluation when forecast are added to Varimetric model");		
		
		Model m = Model.createModel();
		
		//Case 1
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(60.0, 5.0, 10.0);
		
		//Case 2
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 1.0, 60.0);
//		f.addRateSegment(4.0,10.0);
		
		//Case 3
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 2.0, 60.0);
//		f.addRateSegment(3.0,10.0);
		
		//Case 4
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 3.0, 60.0);
//		f.addRateSegment(2.0,10.0);
		
		//Case 5
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 4.0, 60.0);
//		f.addRateSegment(1.0,10.0);
		
		//Case 6
//		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 5.0, 60.0);		
		
		/*NHPPTimeBtwEventRV myNHPPTimeBtwEventRV = new NHPPTimeBtwEventRV(m, f);
		
		double budget = 100000.0;
		double minTheta = 0.0000001;
		double maxTheta = 1.0;
		int maxIterations = 1000;
		double budgetTolerance = 0.1;
		
		VarimetricNetworkLGIterative n = new VarimetricNetworkLGIterative(m, 0.25, "VariMetric-Network-LGIterative", budget,
				minTheta, maxTheta, maxIterations, budgetTolerance);
		
		ItemType type1 = n.addItemType("Type-1");
		n.addDepotInventory(type1, new Exponential(0.5), new Constant(0.0),1000.0);
		
		InventoryRepairingHoldingPoint base1 = n.addBase("Base1");
		
		
		System.out.println(base1.getName());
		System.out.println(type1);
		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV,10.0,0.25,1000.0,
				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME-"+base1.getName()+"-"+type1));
		
		*/
		//Case 7
		/*PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 5.0, 60.0);		
		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV = new NHPPTimeBtwEventRV(m, f);
		
		double budget = 100000.0;
		double minTheta = 0.0000001;
		double maxTheta = 1.0;
		int maxIterations = 1000;
		double budgetTolerance = 0.1;
		
		VarimetricNetworkLGIterative n = new VarimetricNetworkLGIterative(m, 0.25, "VariMetric-Network-LGIterative", budget,
				minTheta, maxTheta, maxIterations, budgetTolerance);
		
		ItemType type1 = n.addItemType("Type-1");
		n.addDepotInventory(type1, new Exponential(0.5), new Constant(0.0),1000.0);
		
		InventoryRepairingHoldingPoint base1 = n.addBase("Base1");
		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV,10.0,1.0,1000.0,
				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME-"+base1.getName()+"-"+type1));
		*/
		
		
		
		//Case 8
		PiecewiseRateFunction f = new PiecewiseLinearRateFunction(10.0, 5.0, 60.0);		
		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV = new NHPPTimeBtwEventRV(m, f);
		
		double budget = 100000.0;
		double minTheta = 0.0000001;
		double maxTheta = 1.0;
		int maxIterations = 1000;
		double budgetTolerance = 10.0;
		
		VarimetricNetworkLGIterative n = new VarimetricNetworkLGIterative(m, 0.25, "VariMetric-Network-LGIterative", budget,
				minTheta, maxTheta, maxIterations, budgetTolerance);
		
		ItemType type1 = n.addItemType("Type-1");
		n.addDepotInventory(type1, new Exponential(0.5), new Constant(0.0),1000.0);
		
		InventoryRepairingHoldingPoint base1 = n.addBase("Base1");
		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV,10.0,0.25,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type1), "-SES-ME-"+base1.getName()+"-"+type1), true);
	
				
		return(m);
	}
	
	public static Model test2(){
		System.out.println("This test is used to verify the varimetric network of sherbrooke example");
		
		Model m = Model.createModel();
		
		//Create the varimetric network
		double budget = 600000.0;
		double minTheta = 0.0000001;
		double maxTheta = 1.0;
		int maxIterations = 1000;
		double budgetTolerance = 0.1;
		
		VarimetricNetworkLGIterative n = new VarimetricNetworkLGIterative(m, 0.25, "VariMetric-Network-LGIterative", budget,
				minTheta, maxTheta, maxIterations, budgetTolerance);
		
//		Define item types
		ItemType type1 = n.addItemType("Type-1");	
		
		//addInventory to warehouse
		n.addDepotInventory(type1, new Exponential(0.5), new Constant(0.0),1000.0);
		
		// Create the retailers which use the warehouse as its filler
		InventoryRepairingHoldingPoint base1 = n.addBase("Base1");
		InventoryRepairingHoldingPoint base2 = n.addBase("Base2");
		InventoryRepairingHoldingPoint base3 = n.addBase("Base3");
		InventoryRepairingHoldingPoint base4 = n.addBase("Base4");
		InventoryRepairingHoldingPoint base5 = n.addBase("Base5");
		InventoryRepairingHoldingPoint base6 = n.addBase("Base6");
		
		PiecewiseRateFunction f1 = new PiecewiseLinearRateFunction(6.0, 5.0, 1.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV1 = new NHPPTimeBtwEventRV(m, f1);
		
		PiecewiseRateFunction f2 = new PiecewiseLinearRateFunction(10.0, 1.0, 60.0);
		f2.addRateSegment(4.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV2 = new NHPPTimeBtwEventRV(m, f2);
		
		PiecewiseRateFunction f3 = new PiecewiseLinearRateFunction(10.0, 2.0, 60.0);
		f3.addRateSegment(3.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV3 = new NHPPTimeBtwEventRV(m, f3);
		
		PiecewiseRateFunction f4 = new PiecewiseLinearRateFunction(10.0, 3.0, 60.0);
		f4.addRateSegment(2.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV4 = new NHPPTimeBtwEventRV(m, f4);
		
		PiecewiseRateFunction f5 = new PiecewiseLinearRateFunction(10.0, 3.0, 60.0);
		f5.addRateSegment(2.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV5 = new NHPPTimeBtwEventRV(m, f5);
		
		PiecewiseRateFunction f6 = new PiecewiseLinearRateFunction(60.0, 5.0, 10.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV6 = new NHPPTimeBtwEventRV(m, f6);
				
//		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV1,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1));
//		n.addBaseInventory(base2,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV2,10.0,0.25,1000.0,
//				new ForecasterModelElement(base2, new SimpleExponentialSmoothing(0.15, base2.getName()+"-"+type1), "-SES-ME"+base2.getName()+"-"+type1));
//		n.addBaseInventory(base3,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV3,10.0,0.25,1000.0,
//				new ForecasterModelElement(base3, new SimpleExponentialSmoothing(0.15, base3.getName()+"-"+type1), "-SES-ME"+base3.getName()+"-"+type1));
//		n.addBaseInventory(base4,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV4,10.0,0.25,1000.0,
//				new ForecasterModelElement(base4, new SimpleExponentialSmoothing(0.15, base4.getName()+"-"+type1), "-SES-ME"+base4.getName()+"-"+type1));
//		n.addBaseInventory(base5,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV5,10.0,0.25,1000.0,
//				new ForecasterModelElement(base5, new SimpleExponentialSmoothing(0.15, base5.getName()+"-"+type1), "-SES-ME"+base5.getName()+"-"+type1));
//		n.addBaseInventory(base6,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV6,10.0,0.25,1000.0,
//				new ForecasterModelElement(base6, new SimpleExponentialSmoothing(0.15, base6.getName()+"-"+type1), "-SES-ME"+base6.getName()+"-"+type1));
		
//		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV1,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1));
//		n.addBaseInventory(base2,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV2,10.0,0.25,1000.0,
//				new ForecasterModelElement(base2, new SimpleExponentialSmoothing(0.9, base2.getName()+"-"+type1), "-SES-ME"+base2.getName()+"-"+type1));
//		n.addBaseInventory(base3,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV3,10.0,0.25,1000.0,
//				new ForecasterModelElement(base3, new SimpleExponentialSmoothing(0.9, base3.getName()+"-"+type1), "-SES-ME"+base3.getName()+"-"+type1));
//		n.addBaseInventory(base4,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV4,10.0,0.25,1000.0,
//				new ForecasterModelElement(base4, new SimpleExponentialSmoothing(0.9, base4.getName()+"-"+type1), "-SES-ME"+base4.getName()+"-"+type1));
//		n.addBaseInventory(base5,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV5,10.0,0.25,1000.0,
//				new ForecasterModelElement(base5, new SimpleExponentialSmoothing(0.9, base5.getName()+"-"+type1), "-SES-ME"+base5.getName()+"-"+type1));
//		n.addBaseInventory(base6,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV6,10.0,0.25,1000.0,
//				new ForecasterModelElement(base6, new SimpleExponentialSmoothing(0.9, base6.getName()+"-"+type1), "-SES-ME"+base6.getName()+"-"+type1));
		
		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV1,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1), true);
		n.addBaseInventory(base2,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV2,10.0,1.0,1000.0,
				new ForecastManager(base2, new SimpleExponentialSmoothing(0.15, base2.getName()+"-"+type1), "-SES-ME"+base2.getName()+"-"+type1), true);
		n.addBaseInventory(base3,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV3,10.0,1.0,1000.0,
				new ForecastManager(base3, new SimpleExponentialSmoothing(0.15, base3.getName()+"-"+type1), "-SES-ME"+base3.getName()+"-"+type1), true);
		n.addBaseInventory(base4,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV4,10.0,1.0,1000.0,
				new ForecastManager(base4, new SimpleExponentialSmoothing(0.15, base4.getName()+"-"+type1), "-SES-ME"+base4.getName()+"-"+type1), true);
		n.addBaseInventory(base5,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV5,10.0,1.0,1000.0,
				new ForecastManager(base5, new SimpleExponentialSmoothing(0.15, base5.getName()+"-"+type1), "-SES-ME"+base5.getName()+"-"+type1), true);
		n.addBaseInventory(base6,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV6,10.0,0.25,1000.0,
				new ForecastManager(base6, new SimpleExponentialSmoothing(0.15, base6.getName()+"-"+type1), "-SES-ME"+base6.getName()+"-"+type1), true);
		
		
		return(m);
	}

	public static Model test3(){
		System.out.println("This test is used to verify the varimetric network of sherbrooke example");
		
		Model m = Model.createModel();
		
		//Create the varimetric network
		double budget = 600000.0;
		double minTheta = 0.0000001;
		double maxTheta = 1.0;
		int maxIterations = 1000;
		double budgetTolerance = 0.1;
		
		VarimetricNetworkLGIterative n = new VarimetricNetworkLGIterative(m, 0.25, "VariMetric-Network-LGIterative", budget,
				minTheta, maxTheta, maxIterations, budgetTolerance);
		
		//Define item types
		ItemType type1 = n.addItemType("Type-1");
		ItemType type2 = n.addItemType("Type-2");
		ItemType type3 = n.addItemType("Type-3");
		ItemType type4 = n.addItemType("Type-4");
		ItemType type5 = n.addItemType("Type-5");
		ItemType type6 = n.addItemType("Type-6");
		
		
		//addInventory to warehouse
		n.addDepotInventory(type1, new Exponential(0.5), new Constant(0.0),1.0);
		n.addDepotInventory(type2, new Exponential(0.5), new Constant(0.0),1.0);
		n.addDepotInventory(type3, new Exponential(0.5), new Constant(0.0),1.0);
		n.addDepotInventory(type4, new Exponential(0.5), new Constant(0.0),1.0);
		n.addDepotInventory(type5, new Exponential(0.5), new Constant(0.0),1.0);
		n.addDepotInventory(type6, new Exponential(0.5), new Constant(0.0),1.0);
		
		// Create the retailers which use the warehouse as its filler
		InventoryRepairingHoldingPoint base1 = n.addBase("Base1");
		
		PiecewiseRateFunction f1 = new PiecewiseLinearRateFunction(6.0, 5.0, 1.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV1 = new NHPPTimeBtwEventRV(m, f1);
		
		PiecewiseRateFunction f2 = new PiecewiseLinearRateFunction(10.0, 1.0, 60.0);
		f2.addRateSegment(4.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV2 = new NHPPTimeBtwEventRV(m, f2);
		
		PiecewiseRateFunction f3 = new PiecewiseLinearRateFunction(10.0, 2.0, 60.0);
		f3.addRateSegment(3.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV3 = new NHPPTimeBtwEventRV(m, f3);
		
		PiecewiseRateFunction f4 = new PiecewiseLinearRateFunction(10.0, 3.0, 60.0);
		f4.addRateSegment(2.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV4 = new NHPPTimeBtwEventRV(m, f4);
		
		PiecewiseRateFunction f5 = new PiecewiseLinearRateFunction(10.0, 3.0, 60.0);
		f5.addRateSegment(2.0,10.0);		
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV5 = new NHPPTimeBtwEventRV(m, f5);
		
		PiecewiseRateFunction f6 = new PiecewiseLinearRateFunction(60.0, 5.0, 10.0);
		NHPPTimeBtwEventRV myNHPPTimeBtwEventRV6 = new NHPPTimeBtwEventRV(m, f6);
				
//		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV1,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1));
//		n.addBaseInventory(base1,type2,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV2,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type2), "-SES-ME"+base1.getName()+"-"+type2));
//		n.addBaseInventory(base1,type3,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV3,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type3), "-SES-ME"+base1.getName()+"-"+type3));
//		n.addBaseInventory(base1,type4,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV4,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type4), "-SES-ME"+base1.getName()+"-"+type4));
//		n.addBaseInventory(base1,type5,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV5,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type5), "-SES-ME"+base1.getName()+"-"+type5));
//		n.addBaseInventory(base1,type6,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV6,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type6), "-SES-ME"+base1.getName()+"-"+type6));
		
		
		
//		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV1,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1));
//		n.addBaseInventory(base1,type2,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV2,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type2), "-SES-ME"+base1.getName()+"-"+type2));
//		n.addBaseInventory(base1,type3,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV3,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type3), "-SES-ME"+base1.getName()+"-"+type3));
//		n.addBaseInventory(base1,type4,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV4,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type4), "-SES-ME"+base1.getName()+"-"+type4));
//		n.addBaseInventory(base1,type5,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV5,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type5), "-SES-ME"+base1.getName()+"-"+type5));
//		n.addBaseInventory(base1,type6,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
//				myNHPPTimeBtwEventRV6,10.0,0.25,1000.0,
//				new ForecasterModelElement(base1, new SimpleExponentialSmoothing(0.9, base1.getName()+"-"+type6), "-SES-ME"+base1.getName()+"-"+type6));
		
		
		n.addBaseInventory(base1,type1,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV1,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type1), "-SES-ME"+base1.getName()+"-"+type1), true);
		n.addBaseInventory(base1,type2,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV2,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type2), "-SES-ME"+base1.getName()+"-"+type2), true);
		n.addBaseInventory(base1,type3,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV3,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type3), "-SES-ME"+base1.getName()+"-"+type3), true);
		n.addBaseInventory(base1,type4,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV4,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type4), "-SES-ME"+base1.getName()+"-"+type4), true);
		n.addBaseInventory(base1,type5,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV5,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type5), "-SES-ME"+base1.getName()+"-"+type5), true);
		n.addBaseInventory(base1,type6,new Bernoulli(0.1),new Exponential(0.25),new Constant(1.0/12.0),
				myNHPPTimeBtwEventRV6,10.0,1.0,1000.0,
				new ForecastManager(base1, new SimpleExponentialSmoothing(0.15, base1.getName()+"-"+type6), "-SES-ME"+base1.getName()+"-"+type6), true);
		

		
		
		return(m);
	}
	public static Model buildModel(){
//		Model m = test1();
		
//		Model m = test2();
		
		Model m = test3();
		
		return(m);
	}
	
	public static void testExperiment(){
		
		Model m = buildModel();	
		
		// create the experiment to run the model
		Experiment e = new Experiment(m,"ME-Inv-VariMetric-Ntwrk-LG-Iter-Case10c");
		
		try{
        	CSVReport ecsv = new CSVExperimentReport("ME-Inv-VariMetric-Ntwrk-LG-Iter-Case10c");
        	m.addObserver(ecsv);
        }
        catch(IOException exception){
			System.out.println("I/O exception opening file Output.txt");
			exception.printStackTrace();
        }
		
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(5.0);
		//e.setLengthOfWarmUp(365.0);
		
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}
}
