/*
 * Created on Mar 29, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.afosr.forecasting;

import java.util.List;
import java.util.Map;
import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jslx.supplychain.inventorylayer.Demand;
import jslx.supplychain.inventorylayer.DemandArrivalListenerAbstract;
import jslx.supplychain.inventorylayer.Inventory;

/**
 * @author vvarghe
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 * 
 * This demand forecast arrival listener can be added to an inventory and it can 
 * respond appropriately as the demand and forecast arrives (as implemented in the concrete
 * implementation of this subclass)
 * 
 */
public class DemandAndForecastArrivalListenerDefault extends DemandArrivalListenerAbstract implements ForecastArrivalListenerIfc {
	
	/**
	 * It drives the forecasting technique that underlies the system
	 */
	protected ForecastManager myDemandForecaster;
	
	/** 
	 *  Assigned the current forecast estimate
	 */
	protected double myCurrentForecastEstimate;	
	
	/** 
	 *  Assigned the inventory model that is associated with it that needs to be optimized
	 */
	//private UpdatableIfc myUpdatable;
	
	public boolean FORECASTINITIALIZED = false;
	
	/**
	 * @param parent
	 * @param inv
	 */
	public DemandAndForecastArrivalListenerDefault(ModelElement parent, Inventory inv, UpdatableIfc updt) {
		this(parent, inv, updt, null);
	}
	
	/**
	 * @param parent
	 * @param inv
	 */
	public DemandAndForecastArrivalListenerDefault(ModelElement parent,  Inventory inv, UpdatableIfc updt, String name) {
		this(parent, inv, updt, name, 1);
	}

	/**
	 * @param parent
	 * @param inv
	 * @param name
	 */
	public DemandAndForecastArrivalListenerDefault(ModelElement parent, Inventory inv, UpdatableIfc updt, String name, double agglevel) {
		this(parent, inv, updt, name, agglevel, 1.0);
	}
	
	/**
	 * @param parent
	 * @param inv
	 * @param name
	 * @param agglevel
	 */
	public DemandAndForecastArrivalListenerDefault(ModelElement parent, Inventory inv, UpdatableIfc updt, String name, double agglevel, double initforecastestimate) {
		super(parent, inv, name, agglevel);
		//myUpdatable = updt;
		myCurrentForecastEstimate = initforecastestimate;
		
	}
	
	public void initialize(){
		//System.out.println("aggregation");
		scheduleEvent(myAggregationEventTime, 1, 1, "Demand Aggregation");		
	}
	
	
	/**
	 * This method adds the user specified forecaster as the demand forecaster.
	 * The demand forecaster listens to the demand arrival listener, ie. when
	 * the demand arrives and as it is aggregated the forecaster which listens
	 * (the one added) will be triggered and will be asked to forecast.
	 * 
	 * This method also adds this class as a listener to the demand forecaster.
	 * When a forecast is made, this class is triggered.
	 * 
	 * It has to be noted that this class is also a listener to the inventory
	 * class.
	 * 
	 */
	public void setDemandForecaster(ForecastManager demfcst){
		myDemandForecaster = demfcst;
		// TODO remove it if needed
		myDemandForecaster.addDemandArrivalListener(this);
		myDemandForecaster.addForecastArrivalListener(this);		
	}
	
	/** 
	 *  collects the aggregated demand at the end of the update period and then 
	 *  reschedule the next update
	 *  
	 */	
	protected void schedulePeriodicDemandProcessing(JSLEvent event){
		//System.out.println("aggregation");
		double dem = sumDemandCurrentBucket;
		if(myDemandForecaster != null){
			myDemandForecaster.updateForecast(this,dem);
		}
		sumDemandCurrentBucket = 0.0;
		scheduleEvent(myAggregationEventTime, 1, 1, "Demand Aggregation");
		//rescheduleEvent(event,myAggregationEventTime,"Demand Aggregation");
	}
	
	protected void handleEvent(JSLEvent event){
		schedulePeriodicDemandProcessing(event);
	}
	

	
	/** 
	 *  respond to a demand arrival by aggregating the demand wiuthin the current update period
	 *  
	 */	
	public void demandArrived(Inventory inv, Demand demand) {
		if(inv != myInventory){			
			throw new IllegalArgumentException ("DemandArrivalListener can be updated by its Inventory alone");
		}else{
			myCurrentDemand = demand.getOriginalAmountDemanded();
			sumDemandAcrossBuckets = sumDemandAcrossBuckets + myCurrentDemand;
			sumDemandCurrentBucket = sumDemandCurrentBucket + myCurrentDemand;
		}
	}	

	/** 
	 *  respond to a forecast arrival by triggering the optimization within the
	 *  inventory model that is optimizable
	 *  
	 */	
	public void forecastArrived(ForecastManager forecaster, double dem) {
		if(forecaster != myDemandForecaster){			
			throw new IllegalArgumentException ("ForecastArrivalListener can be updated by its forecaster alone");
		}else{
			FORECASTINITIALIZED = true;
			myCurrentForecastEstimate = myDemandForecaster.getDemand();			
		}		
	}

	
	/**
	 * @return myCurrentForecastEstimate.
	 */
	public double getCurrentForecastEstimate() {
		return myCurrentForecastEstimate;
	}

	/**
	 * @return Returns the myDemandForecaster.
	 */
	public ForecastManager getDemandForecaster() {
		return myDemandForecaster;
	}
	
}
