/*
 * Created on Mar 29, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.afosr.forecasting;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jslx.forecasting.IntermittentDemandStatistic;
import jslx.supplychain.inventorylayer.Demand;
import jslx.supplychain.inventorylayer.DemandArrivalListenerAbstract;
import jslx.supplychain.inventorylayer.Inventory;

/**
 * @author vvarghe
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 *
 * This demand forecast arrival listener can be added to an inventory and it can
 * respond appropriately as the demand and forecast arrives (as implemented in the concrete
 * implementation of this subclass)
 *
 */
public class DemandAndMultipleForecastArrivalListener extends DemandArrivalListenerAbstract implements ForecastArrivalListenerIfc {

    /**
     * It drives the forecasting technique that underlies the system
     */
    protected List<ForecastManager> myDemandForecasterList;
    /**
     *  Assigned the current forecast estimate
     */
    protected double myInitialForecastEstimate;
    protected Map<ForecastManager, Double> myCurrentForecastEstimateMap;
    protected Map<ForecastManager, Boolean> FORECASTINITIALIZEDMap;
    protected int myTimeIndexToTurnOnHistoryCollection = 3000;
    protected double[] myDemandHistoryToInitialize;
    protected int myTimeIndexToTurnOnForecast = 3100;
    public String myDemandScenario;

    protected IntermittentDemandStatistic myIntStat;

    /**
     * @param parent
     * @param inv
     * @param name
     * @param agglevel
     */
    public DemandAndMultipleForecastArrivalListener(ModelElement parent, Inventory inv,
            String name, double agglevel,
            double initforecastestimate,
            double f_start_index, double hist_start_index, String scenario) {

        super(parent, inv, name, agglevel);
        myTimeIndexToTurnOnHistoryCollection = (int) hist_start_index;
        myTimeIndexToTurnOnForecast = (int) f_start_index;
        myInitialForecastEstimate = initforecastestimate;
        myCurrentForecastEstimateMap = new HashMap<ForecastManager, Double>();
        FORECASTINITIALIZEDMap = new HashMap<ForecastManager, Boolean>();
        myDemandForecasterList = new ArrayList<ForecastManager>();
        myDemandScenario = scenario;
        myCurrentTime = 0;
        myDemandHistoryToInitialize = new double[myTimeIndexToTurnOnForecast - myTimeIndexToTurnOnHistoryCollection];

        myIntStat = new IntermittentDemandStatistic();

    }

    public void initialize() {
        myCurrentTime = 0;
        
        myIntStat.reset();
        
        //System.out.println("aggregation");
        scheduleEvent(myAggregationEventTime, 1, 1, "Demand Aggregation");
    }

    /**
     * This method adds the user specified forecaster as the demand forecaster.
     * The demand forecaster listens to the demand arrival listener, ie. when
     * the demand arrives and as it is aggregated the forecaster which listens
     * (the one added) will be triggered and will be asked to forecast.
     *
     * This method also adds this class as a listener to the demand forecaster.
     * When a forecast is made, this class is triggered.
     *
     * It has to be noted that this class is also a listener to the inventory
     * class.
     *
     */
    public void setDemandForecaster(ForecastManager demfcst) {
        myDemandForecasterList.add(demfcst);
        myCurrentForecastEstimateMap.put(demfcst, myInitialForecastEstimate);
        FORECASTINITIALIZEDMap.put(demfcst, false);
        // TODO remove it if needed
        demfcst.addDemandArrivalListener(this);
        demfcst.addForecastArrivalListener(this);
    }

    /**
     *  collects the aggregated demand at the end of the update period and then
     *  reschedule the next update
     *
     */
    protected void schedulePeriodicDemandProcessing(JSLEvent event) {
        myCurrentTime++;
        double dem = sumDemandCurrentBucket;
        myIntStat.collect(dem);
        if (!myDemandForecasterList.isEmpty()) {
//                if(myCurrentTime >= myTimeIndexToTurnOnHistoryCollection
//                            && myCurrentTime < myTimeIndexToTurnOnForecast){
//                    System.out.println(myCurrentTime +" (H): "+dem);
//                }
//                if(myCurrentTime == myTimeIndexToTurnOnForecast){
//                    System.out.println(myCurrentTime +" (I): "+dem);
//                }
//                if(myCurrentTime > myTimeIndexToTurnOnForecast){
//                    System.out.println(myCurrentTime +" (F): "+dem);
//                }
            for (ForecastManager fm : myDemandForecasterList) {

                if (myCurrentTime >= myTimeIndexToTurnOnHistoryCollection && myCurrentTime < myTimeIndexToTurnOnForecast) {
                    //System.out.println(myCurrentTime +" (H): "+dem);
                    myDemandHistoryToInitialize[(int) myCurrentTime - myTimeIndexToTurnOnHistoryCollection] = dem;
                }
                if (myCurrentTime == myTimeIndexToTurnOnForecast) {
                    //System.out.println(myCurrentTime +" (I): "+dem);
                    double[] forecasts = new double[1];
                    fm.initalize(myDemandHistoryToInitialize, forecasts, myCurrentTime);
                }
                if (myCurrentTime > myTimeIndexToTurnOnForecast) {
                    //System.out.println(myCurrentTime +" (F): "+dem);
                    fm.updateForecast(this, dem);
                }

            }
        }
        sumDemandCurrentBucket = 0.0;
        scheduleEvent(myAggregationEventTime, 1, 1, "Demand Aggregation");
    }

    protected void handleEvent(JSLEvent event) {
        schedulePeriodicDemandProcessing(event);
    }

    /**
     *  respond to a demand arrival by aggregating the demand wiuthin the current update period
     *
     */
    public void demandArrived(Inventory inv, Demand demand) {
        if (inv != myInventory) {
            throw new IllegalArgumentException("DemandArrivalListener can be updated by its Inventory alone");
        } else {
            myCurrentDemand = demand.getOriginalAmountDemanded();
            sumDemandAcrossBuckets = sumDemandAcrossBuckets + myCurrentDemand;
            sumDemandCurrentBucket = sumDemandCurrentBucket + myCurrentDemand;
        }
    }

    /**
     *  respond to a forecast arrival by triggering the optimization within the
     *  inventory model that is optimizable
     *
     */
    public void forecastArrived(ForecastManager forecaster, double dem) {
        /*if(!myDemandForecasterList.contains(forecaster)){
        throw new IllegalArgumentException ("ForecastArrivalListener can be updated by its forecaster alone");
        }else{
        myCurrentForecastEstimateMap.remove(forecaster);
        this.myCurrentForecastEstimateMap.put(forecaster,forecaster.getDemand());
        if(!this.FORECASTINITIALIZEDMap.get(forecaster)){
        FORECASTINITIALIZEDMap.remove(forecaster);
        this.FORECASTINITIALIZEDMap.put(forecaster,true);

        }}*/
    }

    public double[] getStatistics(){
        return myIntStat.getStatistics();
    }

    /**
     * @return myCurrentForecastEstimate.
     */
    public double getCurrentForecastEstimate() {
        return myInitialForecastEstimate;
    }

    /**
     * @return Returns the myDemandForecaster.
     */
    public List<ForecastManager> getDemandForecaster() {
        return myDemandForecasterList;
    }
}
