/*
 * Created on Mar 30, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.afosr.forecasting;


import jsl.modeling.ModelElement;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.metrics.ForecastMetric;
import jslx.supplychain.inventorylayer.DemandArrivalListenerAbstract;

/**
 * @author vvarghe
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 * 
 * It encapsulate a jsl model element that holds a forecaster
 * and drives this forecaster 
 * 
 */
public class ForecastManager extends ModelElement {

	/** 
	 *  Assigned by a forecaster 
	 *  
	 */	
	private ForecastTechnique myForecaster;
	
	/** 
	 *  Holds whether the forecast is initialized 
	 *  
	 */	
	public boolean FORECASTINITIALIZED = false;
	
	/** 
	 *  demand at time index t-1 (previous demand) 
	 *  
	 */	
	protected double myDemand_t_minus_1;
	
	/** 
	 *  demand at time index t (current demand) 
	 *  
	 */	
	protected double myDemand_t;
	
	/** 
	 *  forecast for time index t (previous forecast) 
	 *  
	 */	
	protected double myForecast_t;
	
	/** 
	 *  forecast for time index t+1 (current forecast) 
	 *  
	 */	
	protected double myForecast_t_plus_1;
	
	/** 
	 *  error at time index t 
	 *  
	 */	
	protected double myError;
	
	
	/** 
	 *  CurrentTimeIndex 
	 *  
	 */	
	protected int myCurrentTimeIndex;
	
	/** 
	 *  forecast arrival listener that listens to this class 
	 *  
	 */	
	public ForecastArrivalListenerIfc myForecastArrivalListener;
	
	/** 
	 *  demand arrival listener is listened by this class 
	 *  
	 */	
	public DemandArrivalListenerAbstract myDemandArrivalListener;
	
	public ForecastMetric myMSE;
	
	/**
	 * @param parent
	 * @param forecaster
	 * @param name
	 */
	public ForecastManager(ModelElement parent, ForecastTechnique forecaster, String name) {
		super(parent, name);
		myForecaster = forecaster;
		myMSE = ForecastMetric.createMeanSquareError();
		myForecaster.attachMetric(myMSE);
		myDemand_t_minus_1 = 0;
		myDemand_t=0;
		myForecast_t=0;
		myForecast_t_plus_1=0;
		myError = 0;
		myCurrentTimeIndex=0;
	}	
	
	
	public void addDemandArrivalListener(DemandArrivalListenerAbstract provider){
		myDemandArrivalListener = provider;
	}
	
	public void addForecastArrivalListener(ForecastArrivalListenerIfc listener){
		myForecastArrivalListener = listener;
	}
	
	/** 
	 *  triggers the update of forecast 
	 *  
	 */	
	public void updateForecast(DemandArrivalListenerAbstract provider, double demand) {
		if(provider != myDemandArrivalListener)
			throw new IllegalArgumentException("The update method is not invoked by the proper demand data provider");
		updateForecast(demand);
	}
        
        public void initalize(double [] d, double [] f, double index){
            
            myCurrentTimeIndex = (int)index;
            myForecaster.initializeHistory(d, f, myForecaster.estimateInitialForecastParameter(d));
        
        }
	
	/** 
	 *  triggers the update of forecast 
	 *  
	 */	
	public void updateForecast(double demand) {
		myCurrentTimeIndex++;
		myForecaster.collect(demand);
		FORECASTINITIALIZED = myForecaster.isInitialized();
		myForecastArrivalListener.forecastArrived(this, getForecast());
	}
	
	public double getDemand(){
		myDemand_t = myForecaster.getObservation();
		return myDemand_t; 
	}
	
	public double getLastDemand(){
		myDemand_t_minus_1 = myForecaster.getPrevObservation();
		return myDemand_t_minus_1;
	}
	
	public double getForecast(){
		myForecast_t_plus_1 = myForecaster.getNextPeriodForecast();
		return myForecast_t_plus_1; 
	}	
	public double getLastForecast(){
		myForecast_t = myForecaster.getCurrentPeriodForecast(); 
		return myForecast_t; 
	}	
	public double getError(){
		myError = myForecaster.getError();
		return myError; 
	}
	public int getCurrentTimeIndex() {
		return myForecaster.getTimeIndex();
	}
	public ForecastTechnique getForecastTechnique() {
		return myForecaster;
	}
	
	public double getMSE(){
		myMSE.computeMetricValue();
		
		double mse = myMSE.getMetricValue();
		
		return mse;
		
	}
	
}
