/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jslx.supplychain.afosr.generation;

import java.util.ArrayList;
import java.util.List;

import jsl.utilities.random.distributions.Binomial;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DEmpirical;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.TruncatedDistribution;
import jsl.utilities.random.distributions.Uniform;
import jsl.utilities.random.robj.DEmpiricalList;
import jsl.utilities.random.robj.DPopulation;

/**
 *
 * @author vvarghe
 */
public class ItemGenerator {
    
    public double [] myRatioOfCostLevels = new double [2];
    public double myHorizonPlan;
    public double [] myRatioOfPatternLevels = new double [3];
    public double myBudgetConstant;
    
    public DEmpiricalList<DistributionIfc> myDistCostTypesList;
    /**
     * index 0 has the distribution of repair delay of low cost (low rep delay)
     * index 1 has the distribution of repair delay of medium cost (medium rep delay)
     * index 2 has the distribution of repair delay of high cost (high rep delay)
     * 
     */
    public List<DistributionIfc> myDistRepairDelay;
    /**
     * index 0 has the distribution of repair prob of low cost (high rep prob)
     * index 1 has the distribution of repair prob of medium cost (medium rep prob)
     * index 2 has the distribution of repair prob of high cost (low rep prob)
     * 
     */
    public List<DistributionIfc> myDistRepairProb;
    /**
     * index 0 has the distribution of ship delay of low cost (low ship delay)
     * index 1 has the distribution of ship delay of medium cost (medium ship delay)
     * index 2 has the distribution of ship delay of high cost (high ship delay)
     * 
     */
    public List<DistributionIfc> myDistShipDelay;
    /**
     * index 0 has the distribution of avg demand rate of low cost (high avg demand rate)
     * index 1 has the distribution of avg demand rate of medium cost (medium avg demand rate)
     * index 2 has the distribution of avg demand rate of high cost (low avg demand rate)
     * 
     */
    public List<DistributionIfc> myDistAverageDemandRate;
    /**
     * index 0 has the distribution of prob stocking BItem of low cost (high prob stocking BItem)
     * index 1 has the distribution of prob stocking BItem of medium cost (medium prob stocking BItem)
     * index 2 has the distribution of prob stocking BItem of high cost (low prob stocking BItem)
     * 
     */
    public List<DistributionIfc> myDistProbStockingBItem;
    
    
    /**
     * index 0 has the distribution of type 1 (increase, constant and then decrease)
     * index 1 has the distribution of type 2 (constant)
     * index 2 has the distribution of type 3 (decrease)
     * index 2 has the distribution of type 4 (increase)
     * 
     */
    public DEmpirical myDistPatternTypes; 
    public DistributionIfc myDistPatternPoint;    
    public int myNumBases;
    
    public int myITemID = 0;
    public int myBaseID = 0;
    public double [] myBaseIDs;    
    public DistributionIfc myDistNumStockedBases;    
    public DPopulation myBasePopulation;
    
    public ItemGenerator(double [] cratio, double [] pratio, double tp, double budgetconstant){
        this(1, cratio, pratio, tp, budgetconstant);
    
    }
    
    public ItemGenerator(int bases, double [] cratio, double [] pratio, double tp, double budgetconstant){
        myNumBases = bases;
        myHorizonPlan = tp;
        myBudgetConstant = budgetconstant;
        setRatioOfCostLevels(cratio);
        setRatioOfPatternLevels(pratio);
        setDistNumStockedBases(0.5);   
    }
    
    public void setDistNumStockedBases(double prob){        
        if(myNumBases<=1){
            myDistNumStockedBases = new Constant(1.0);        
        }else{           
            // if num bases = 1, then there is still a possibility of binomial 
            // to generate value greater than 1 for num stocked bases and 
            // thus greater than num bases
            // if we shift the binomial by 1 then there is a possibility of
            // generating num stocked base greater than num base
            
            Binomial n = new Binomial(prob, myNumBases);
            myDistNumStockedBases = new TruncatedDistribution(n, 0.0, 
                Double.POSITIVE_INFINITY, 1.0, myNumBases);        
        }      
        // is this a good assumption 
        myBaseIDs = new double[myNumBases];
        myBaseID = 0;
        for(int i = 1; i <= myNumBases; i++){
            myBaseID++;
            myBaseIDs[i-1] = myBaseID;        
        }        
        myBasePopulation = new DPopulation(myBaseIDs);   
    }
    
    public void setRatioOfCostLevels(double [] ratio){
        myDistCostTypesList = new DEmpiricalList<DistributionIfc>();
        myDistRepairDelay = new ArrayList<DistributionIfc>();
        myDistRepairProb = new ArrayList<DistributionIfc>();
        myDistShipDelay = new ArrayList<DistributionIfc>();
        myDistAverageDemandRate = new ArrayList<DistributionIfc>();
        myDistProbStockingBItem = new ArrayList<DistributionIfc>();
        
        myDistCostTypesList.add(new Uniform(1.0,1000.0), ratio[0]);
        myDistCostTypesList.add(new Uniform(1000.0,10000.0), ratio[1]);
        myDistCostTypesList.addLast(new Uniform(10000.0,100000.0));
        
        myDistRepairDelay.add(new Uniform(1.0,5.0));
        myDistRepairDelay.add(new Uniform(5.0,20.0));
        myDistRepairDelay.add(new Uniform(20.0,200.0));
        
        myDistRepairProb.add(new Uniform(0.4,0.6));
        myDistRepairProb.add(new Uniform(0.1,0.4));
        myDistRepairProb.add(new Uniform(0.0,0.1));
        
        myDistShipDelay.add(new Uniform(1.0,5.0));
        myDistShipDelay.add(new Uniform(5.0,20.0));
        myDistShipDelay.add(new Uniform(20.0,200.0));
        
        myDistAverageDemandRate.add(new Uniform(100.0,500.0));
        myDistAverageDemandRate.add(new Uniform(10.0,100.0));
        myDistAverageDemandRate.add(new Uniform(0.5,10.0));
        
        myDistProbStockingBItem.add(new Uniform(0.4,0.6));
        myDistProbStockingBItem.add(new Uniform(0.1,0.4));
        myDistProbStockingBItem.add(new Uniform(0.0,0.1));
        
    }
    
    public void setRatioOfPatternLevels(double [] ratio){
        
    	myDistPatternTypes = new DEmpirical();        
        /*
         * Level 1 increases and then constant and then decreases
         * Level 2 constant
         * Level 3 decreases
         * Level 4 increases
         * 
         */
        // 1 we cannot add zero probability
        myDistPatternTypes.addProbabilityPoint(1.0, ratio[0]);
        // 2
        myDistPatternTypes.addProbabilityPoint(2.0, ratio[1]);
        // 3
        myDistPatternTypes.addProbabilityPoint(3.0, ratio[2]);
        // 4
        myDistPatternTypes.addLastProbabilityPoint(4.0);
        
        myDistPatternPoint = new Uniform();
    }
    
    public DItem generate(){
        myITemID ++;
        int id = myITemID;
        
        DistributionIfc dist = myDistCostTypesList.getRandomElement();
        int costlevel = myDistCostTypesList.indexOf(dist) +1;
        double cost = dist.getValue();
        double prob = myDistProbStockingBItem.get(costlevel-1).getValue();
        
        int patternlevel = (int) myDistPatternTypes.getValue();
        // i assume that each item has the same pattern level at each base
        
        DItem d_item = generateDItem(id, costlevel, cost, prob, myBudgetConstant);
        int numstockedbases = d_item.myNumStockedBases;
        
        double [] stockedbaseid = myBasePopulation.getSampleWithoutReplacement(numstockedbases);        
        for(int i = 0; i < stockedbaseid.length; i++){            
            BItem b_item = generateBItem(id, (int)stockedbaseid[i], d_item, 
                    patternlevel, cost, costlevel, myBudgetConstant);           
            d_item.addBItem(b_item);        
        }
        
        return d_item;
    
    }
    
    private DItem generateDItem(int id , int costlevel, double cost, double prob, double budgetconstant){ 
        setDistNumStockedBases(prob);
        double n = myDistNumStockedBases.getValue();
        int numstockedbases = (int)n;
        double repairdelay = myDistRepairDelay.get(costlevel-1).getValue();
        
        return new DItem(id, cost, costlevel, numstockedbases, repairdelay, budgetconstant);    
    }
    
    
    
    private BItem generateBItem(int item_id, int base_id, DItem d_item, 
            int patternlevel, double cost, int costlevel, double budgetconstant){
        double lambda = myDistAverageDemandRate.get(costlevel-1).getValue();
        double reparability = myDistRepairProb.get(costlevel-1).getValue();
        double repairdelay = myDistRepairDelay.get(costlevel-1).getValue();
        double shipdelay = myDistShipDelay.get(costlevel-1).getValue();
        
        double td;
        double ts;
        double tr;
        //end point of the deployment phase and start point of sustainment phase
        double p1;
        //end point of the sustainment phase and start point of the reconfiguration phase
        double p2;
        double a;
        double b;
        double c;
        double [] para = new double[2];
        
        switch(patternlevel){
            case 1:
            	a = 0.5 * myHorizonPlan;
		    	para [0] = 0.0;
		    	para [1] = a;
		    	myDistPatternPoint.setParameters(para);
		    	p1 = myDistPatternPoint.getValue();
		    	td = p1;
            	
            	c = 0.5 * myHorizonPlan;
		    	para [0] = myHorizonPlan - c;
		    	para [1] = myHorizonPlan;
		    	myDistPatternPoint.setParameters(para);
		    	p2 = myDistPatternPoint.getValue();
		    	tr = myHorizonPlan - p2;
		    	
		    	ts = myHorizonPlan - td - tr;
		    	
                break;
            case 2:
            	a = 0.02 * myHorizonPlan;
		    	para [0] = 0.0;
		    	para [1] = a;
		    	myDistPatternPoint.setParameters(para);
		    	p1 = myDistPatternPoint.getValue();
		    	td = p1;
            	
            	c = 0.02 * myHorizonPlan;
		    	para [0] = myHorizonPlan - c;
		    	para [1] = myHorizonPlan;
		    	myDistPatternPoint.setParameters(para);
		    	p2 = myDistPatternPoint.getValue();
		    	tr = myHorizonPlan - p2;
		    	
		    	ts = myHorizonPlan - td - tr;
		    	
                break;
            case 3:
            	a = 0.02 * myHorizonPlan;
		    	para [0] = 0.0;
		    	para [1] = a;
		    	myDistPatternPoint.setParameters(para);
		    	p1 = myDistPatternPoint.getValue();
		    	td = p1;
		    	
		    	b = 0.02 * (myHorizonPlan - td);
		    	para [0] = td;
		    	para [1] = td + b;
		    	myDistPatternPoint.setParameters(para);
            	p2 = myDistPatternPoint.getValue();
		    	ts = p2 - td;
		    	
		    	tr = myHorizonPlan - td - ts;
		    	
                break;
            case 4:
            	c = 0.02 * myHorizonPlan;
		    	para [0] = myHorizonPlan - c;
		    	para [1] = myHorizonPlan;
		    	myDistPatternPoint.setParameters(para);
		    	p2 = myDistPatternPoint.getValue();
		    	tr = myHorizonPlan - p2;
		    	
		    	b = 0.02 * p2;
		    	para [0] = p2 - b;
		    	para [1] = p2;
		    	myDistPatternPoint.setParameters(para);		    	
            	p1 = myDistPatternPoint.getValue();
		    	ts = p2 - p1;
		    	
		    	td = myHorizonPlan - ts - tr;
		    	
                break;
            default:
                throw new IllegalArgumentException("Illegal");                
        }
        
        return new BItem(item_id, base_id, d_item, 
            cost, costlevel, patternlevel, myHorizonPlan, 
            td, ts, lambda, repairdelay, 
            reparability, shipdelay, budgetconstant);
    
    }
    
    

}
