package jslx.supplychain.afosr.varimetricsimulator;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.DUniform;
import jslx.forecasting.ForecastTechnique;
import jslx.forecasting.MovingAverage;
import jslx.supplychain.afosr.forecasting.DemandArrivalListenerWithForecaster;
import jslx.supplychain.afosr.forecasting.ForecastArrivalListenerDefault;
import jslx.supplychain.afosr.forecasting.ForecastManager;
import jslx.supplychain.afosr.generation.BItem;
import jslx.supplychain.afosr.generation.DItem;
import jslx.supplychain.afosr.generation.NetworkConfiguration;
import jslx.supplychain.afosr.generation.NetworkGenerator;
import jslx.supplychain.inventorylayer.Inventory;

public class SimVMModelNCUpdatableModel extends SimVMModelNCModel {
	
	private ForecastTechnique myForecastTechnique;
	private double myUpdatePeriod;
	
	private Map <Inventory, DemandArrivalListenerWithForecaster> mapInventoryToDemandArrivalListener;
	private Map <Inventory, ForecastArrivalListenerDefault> mapInventoryToForecastArrivalListener;
		
	
	public SimVMModelNCUpdatableModel(ModelElement parent, NetworkConfiguration nc, ForecastTechnique ft, double period) {
		this(parent, nc, ft, period, null);
	}
	
	public SimVMModelNCUpdatableModel(ModelElement parent, NetworkConfiguration nc, ForecastTechnique ft, double period, String name) {
		super(parent, nc, name);
		myForecastTechnique = ft;
		myUpdatePeriod = period;
		mapInventoryToDemandArrivalListener = new HashMap <Inventory, DemandArrivalListenerWithForecaster>();
		mapInventoryToForecastArrivalListener = new HashMap <Inventory, ForecastArrivalListenerDefault>();
	}
	
	protected void addDemandAndForecastListeners(NetworkConfiguration nc){
		for(DItem di:nc.myItems){
			
			Inventory inv_depot = mapDItemToInventory.get(di);
			
			//TODO the depot doesn't require forecasting. Must be taken out later
			ForecastTechnique ft_depot = myForecastTechnique.copyInstance();
			ft_depot.setName("FT-"+inv_depot.getName());				
			ForecastManager fm_depot = new ForecastManager(this, ft_depot, "FM-"+inv_depot.getName());
			
	        DemandArrivalListenerWithForecaster dal_depot = new DemandArrivalListenerWithForecaster(this, inv_depot, fm_depot,
					"DAL-" + inv_depot.getName(), myUpdatePeriod, true);
	        System.out.println(dal_depot.getName());
	        
	        ForecastArrivalListenerDefault fal_depot = new ForecastArrivalListenerDefault(fm_depot, di.myLambda, 
					"FAL-" + inv_depot.getName());	
	        System.out.println(fal_depot.myName);

			mapInventoryToDemandArrivalListener.put(inv_depot,dal_depot);
			mapInventoryToForecastArrivalListener.put(inv_depot,fal_depot);
			
			for(BItem bi:di.myBItems){				
				Inventory inv_base = mapBItemToInventory.get(bi);
				
				ForecastTechnique ft_base = myForecastTechnique.copyInstance();
				ft_base.setName("FT-"+inv_base.getName());				
				ForecastManager fm_base = new ForecastManager(this, ft_base, "FM-"+inv_base.getName());
				
		        DemandArrivalListenerWithForecaster dal_base = new DemandArrivalListenerWithForecaster(this, inv_base, fm_base,
						"DAL-" + inv_base.getName(), myUpdatePeriod, true);
		        //System.out.println(dal_base.getName());
		        
		        ForecastArrivalListenerDefault fal_base = new ForecastArrivalListenerDefault(fm_base, bi.myLambdaMin, 
						"FAL-" + inv_base.getName());	
		        //System.out.println(fal_base.myName);

				mapInventoryToDemandArrivalListener.put(inv_base,dal_base);
				mapInventoryToForecastArrivalListener.put(inv_base,fal_base);				
			}			
		}
	}
	
	public DemandArrivalListenerWithForecaster getDemandArrivalListener(Inventory inv) {
		return mapInventoryToDemandArrivalListener.get(inv);
	}

	public ForecastArrivalListenerDefault getForecastArrivalListener(Inventory inv) {
		return mapInventoryToForecastArrivalListener.get(inv);
	}
	
	public static void main(String [] args){
        double [] cratio = {0.6, 0.1};
        double tp = 500;
        double [] pratio = {0.1, 0.1, 0.1};
        DUniform items = new DUniform(1, 5);
        DUniform bases = new DUniform(1, 10);
        double budgetconstant = 0.7;
        
        NetworkGenerator ng = new NetworkGenerator(items, bases, budgetconstant, 
                cratio, pratio, tp);
        
        NetworkConfiguration nc =  ng.getNetwork();
        
        System.out.println(nc);
        
		Model m = Model.createModel();
		
		MovingAverage ma = new MovingAverage(9);
		double period = 7;
		
		//Create the network
		SimVMModelNCUpdatableModel n = new SimVMModelNCUpdatableModel(m, nc, ma, period, "SimVMModelNCModel-T1");
		
		n.addDemandAndForecastListeners(nc);
        
		System.out.println(n.mySimVMModel.getModelStringFormat());
		
		// create the experiment to run the model
		Experiment e = new Experiment(m,"SimVMModelNCModel-T1");
		
		try{
        	CSVReport ecsv = new CSVExperimentReport("SimVMModelNCModel-T1");
        	m.addObserver(ecsv);
        }
        catch(IOException exception){
			System.out.println("I/O exception opening file Output.txt");
			exception.printStackTrace();
        }
        
		// set the parameters of the experiment
		e.setNumberOfReplications(1);
		e.setLengthOfReplication(3650.0);
		e.setLengthOfWarmUp(100.0);
		
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput(true);
		
		// tell the experiment to run
		e.runAll();
		
		
		
	}
}
