package jslx.supplychain.afosr.varimetricsimulator;

import java.util.ArrayList;
import java.util.List;

import jsl.modeling.Experiment;
import jsl.modeling.ExperimentProvider;
import jsl.utilities.random.distributions.DistributionIfc;
import jslx.forecasting.ForecastTechnique;
import jslx.supplychain.afosr.generation.NetworkConfiguration;
import jslx.supplychain.afosr.generation.NetworkGenerator;

public class UpdatableVMNetworkLGExperimentProvider extends ExperimentProvider {

	private List<NetworkConfiguration> myNetworks = new ArrayList<NetworkConfiguration>();
	private UpdatableVMNetworkLGOptim myUpdatableVMNetworkLGOptim;
	private List<ForecastTechnique> myForecastTechniques;
	private double[] myUpdatePeriods;
	private int myCurrentUpdatePeriodIndex;
	private int myCurrentUpdatePeriod;
	private int myCurrentForecastTechniqueIndex;
	private ForecastTechnique myCurrentForecastTechnique;
	private int myCurrentNetworkIndex;
	private int myCurrentNetwork;
	
	private int myCurrentExperiment;
	private int myMaxNumExperiments;
	
	
	public static List<NetworkConfiguration> getNetwork(int num, double [] cratio, double tp, double [] pratio, 
			DistributionIfc items, DistributionIfc bases, double budgetconstant){
		List<NetworkConfiguration> networks = new ArrayList<NetworkConfiguration>(); 
        
        NetworkGenerator ng = new NetworkGenerator(items, bases, budgetconstant, 
                cratio, pratio, tp);
        
        for(int i = 1; i <= num; i++){
            NetworkConfiguration nw = ng.getNetwork();
            networks.add(nw);
        }
        
        return networks;
		
	}
	
	public UpdatableVMNetworkLGExperimentProvider(List<NetworkConfiguration> networks, List<ForecastTechnique> fts, double[] updtprds) {
		this(networks, fts, updtprds, null);
	}

	public UpdatableVMNetworkLGExperimentProvider(List<NetworkConfiguration> networks, List<ForecastTechnique> fts, double[] updtprds, String name) {
		super();
		myExperiment = new Experiment(myModel);
		
		myNetworks =  networks;
		myForecastTechniques = fts;
		myUpdatePeriods = updtprds;
		
		myCurrentExperiment = 1;
		myMaxNumExperiments = myNetworks.size() * myForecastTechniques.size() * myUpdatePeriods.length;
		myCurrentUpdatePeriodIndex = 0;
		myCurrentForecastTechniqueIndex = 0;
		myCurrentNetworkIndex = 0;
	}

	@Override
	public boolean hasNext() {
		return myCurrentExperiment < myMaxNumExperiments;
	}

	@Override
	public Experiment next() {
		myCurrentExperiment++;
		
		setNextCase();
		
		
		NetworkConfiguration nw = getNextNetwork();
		ForecastTechnique ft = getNextForecastTechnique();
		double prd = getNextUpdatePeriod();
		
		System.out.println(nw);
		System.out.println(ft.getName());
		System.out.println(prd);
		
		myUpdatableVMNetworkLGOptim = new UpdatableVMNetworkLGOptim(myModel, prd, nw, ft, "Exp-"+nw.myName+"-"+ft.getName()+"-"+prd);
 
		myExperiment.setNumberOfReplications(5);
		myExperiment.setLengthOfReplication(nw.myItemGenerator.myHorizonPlan);
		myExperiment.turnOnExperimentReport();
		myExperiment.turnOnReplicationReport();	
		
		return myExperiment;
	}
	
	public void setNextCase(){
		if(myCurrentNetworkIndex < myNetworks.size()){			
			if(myCurrentForecastTechniqueIndex < myForecastTechniques.size()){				
				if(myCurrentUpdatePeriodIndex < myUpdatePeriods.length){
					myCurrentUpdatePeriodIndex++;					
				}else{
					myCurrentUpdatePeriodIndex = 0;
					myCurrentForecastTechniqueIndex++;
					setNextCase();				
				}				
			}else{
				myCurrentForecastTechniqueIndex = 0;
				myCurrentNetworkIndex++;
				setNextCase();						
			}				
		}
		
	}

	public NetworkConfiguration getNextNetwork() {
		return myNetworks.get(myCurrentNetworkIndex);
	}

	public ForecastTechnique getNextForecastTechnique() {
		return myForecastTechniques.get(myCurrentForecastTechniqueIndex);
	}

	public double getNextUpdatePeriod() {
		return myUpdatePeriods[myCurrentUpdatePeriodIndex];
	}

}
