package jslx.supplychain.afosr.varimetricsimulator;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.variable.AggregateCounter;
import jsl.modeling.elements.variable.AggregateTimeWeightedVariable;
import jsl.modeling.elements.variable.Counter;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.modeling.elements.variable.ResponseVariableAverageObserver;
import jsl.modeling.elements.variable.TimeWeighted;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.DUniform;
import jslx.forecasting.Croston;
import jslx.forecasting.ForecastTechnique;
import jslx.inventory.varimetric.VMLagrangeAlgorithm;
import jslx.inventory.varimetric.VMLagrangeEnumerationAlgorithm;
import jslx.inventory.varimetric.VariMetricModel;
import jslx.supplychain.afosr.forecasting.DemandArrivalListenerWithForecaster;
import jslx.supplychain.afosr.generation.BItem;
import jslx.supplychain.afosr.generation.DItem;
import jslx.supplychain.afosr.generation.NetworkConfiguration;
import jslx.supplychain.afosr.generation.NetworkGenerator;
import jslx.supplychain.inventorylayer.Inventory;
import jslx.supplychain.inventorylayer.InventoryRepairingHoldingPoint;

public class UpdatableVMNetworkLGOptim extends SchedulingElement {
	
	private double myUpdatePeriod;
	private SimVMModelNCModel mySimVMModelNCModel;
	private VariMetricModel myVMModel;
	private VariMetricModel myOriginalVMModel;
	private VMLagrangeAlgorithm myVMLagrangeAlgorithm;
	private NetworkConfiguration myNetworkConfiguration;
	private StringBuilder myModelOutputString;
	
	private InventoryRepairingHoldingPoint myDepot;
	
	
	private static double BUDGET = 200.0;
	private static double MIN_THETA = 0.1;
	private static double MAX_THETA = 1.0;
	private static double BUDGET_TOLERANCE = 0.1;
	private static int NUM_POINTS = 10000;
	private static int MAX_ITERATIONS = 10000;
	
	private ForecastTechnique myForecastTechnique;
	private boolean myUpdatable;
	
	private Counter numConvergedOptimization;
	private Counter numUnConvergedOptimization;
	
	
	private Map <Inventory,Counter> mapInventoryToNumBaseStockIncrement;
	private Map <Inventory,Counter> mapInventoryToNumBaseStockDecrement;
	private Map <Inventory,ResponseVariable> mapInventoryToBaseStockIncrement;
	private Map <Inventory,ResponseVariable> mapInventoryToBaseStockIncrementVariance;
	private Map <Inventory,ResponseVariable> mapInventoryToBaseStockDecrement;
	private Map <Inventory,ResponseVariable> mapInventoryToBaseStockDecrementVariance;
	private Map <Inventory, TimeWeighted> mapInventoryToSurplusOnHand;
	private Map <Inventory, TimeWeighted> mapInventoryToTimeSpentInSurplusOnHand;
	private Map <Inventory, ResponseVariable> mapInventoryToEndSurplusOnHand;
	private Map <Inventory, ResponseVariable> mapInventoryToEndProbabilityOfSurplusOnHand;
	
	private Map <InventoryRepairingHoldingPoint, AggregateCounter> mapLocationToTotalNumBaseStockIncrement;
	private Map <InventoryRepairingHoldingPoint, AggregateCounter> mapLocationToTotalNumBaseStockDecrement;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToTotalBaseStockIncrement;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToTotalBaseStockIncrementVariance;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToTotalBaseStockDecrement;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToTotalBaseStockDecrementVariance;
	private Map <InventoryRepairingHoldingPoint, AggregateTimeWeightedVariable> mapLocationToSurplusOnHandAtIHP;
	private Map <InventoryRepairingHoldingPoint, AggregateTimeWeightedVariable> mapLocationToTimeSpentInSurplusOnHandAtIHP;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToEndSurplusOnHandAtIHP;
	private Map <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver> mapLocationToEndProbabilityOfSurplusOnHandAtIHP;
	
	private AggregateCounter myTotalNumBaseStockIncrement;
	private AggregateCounter myTotalNumBaseStockDecrement;
	private ResponseVariableAverageObserver myTotalBaseStockIncrement;
	private ResponseVariableAverageObserver myTotalBaseStockIncrementVariance;
	private ResponseVariableAverageObserver myTotalBaseStockDecrement;
	private ResponseVariableAverageObserver myTotalBaseStockDecrementVariance;
	private AggregateTimeWeightedVariable mySurplusInventoryInTheNetwork;
	private AggregateTimeWeightedVariable myTimeSpentInSurplusOnHandInTheNetwork;
	private ResponseVariableAverageObserver myEndSurplusInventoryInTheNetwork;
	private ResponseVariableAverageObserver myEndProbabilityOfSurplusOnHandInTheNetwork;
	
	public UpdatableVMNetworkLGOptim(ModelElement parent, NetworkConfiguration nw, String name) {
		super(parent, name);

		myModelOutputString = new StringBuilder();
		
		//TODO
		/*myModelOutputString.append("MODEL OUTPUT STRING");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("MODEL OUTPUT STRING");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
		myUpdatePeriod = Double.POSITIVE_INFINITY;
		myUpdatable = false;
		/*myModelOutputString.append("The model is not updatable");
		myModelOutputString.append("\n");*/
		System.out.println("The model is not updatable");
		System.out.println("\n");
		
		myNetworkConfiguration = nw;
		
		//TODO
		/*myModelOutputString.append("Network Configuration");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append(nw.getDataString());
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("Network Configuration");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println(nw.getDataString());
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
		mySimVMModelNCModel = new SimVMModelNCModel(parent, nw, name);
		
		//TODO
		/*myModelOutputString.append("Simulation Model");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append(mySimVMModelNCModel.mySimVMModel.getModelStringFormat());
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("Simulation Model");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println(mySimVMModelNCModel.mySimVMModel.getModelStringFormat());
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
		myVMModel = VariMetricModel.getVarimetricModel(parent, nw);
		
		//TODO
		/*myModelOutputString.append("Varimetric Model");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append(myVMModel.toString());
		myModelOutputString.append("\n");
		myModelOutputString.append("Maps within the VMModel");
		myModelOutputString.append("\n");
		myModelOutputString.append("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("Varimetric Model");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println(myVMModel.toString());
		System.out.println("\n");
		System.out.println("Maps within the VMModel");
		System.out.println("\n");
		/*System.out.println("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		System.out.println("\n");
		System.out.println("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");*/
		
		myVMLagrangeAlgorithm = new VMLagrangeEnumerationAlgorithm(myVMModel, nw.myBudget, NUM_POINTS, MIN_THETA, MAX_THETA);
		//myVMLagrangeAlgorithm = new VMLagrangeIterativeAlgorithm(myVMModel, nw.myBudget, MAX_ITERATIONS, BUDGET_TOLERANCE, MIN_THETA, MAX_THETA);
		
		//TODO
		/*myModelOutputString.append("Varimetric Algorithm");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Budget: "+ nw.myBudget);
		myModelOutputString.append("\n");
		myModelOutputString.append("Number of Points: "+ NUM_POINTS);
		myModelOutputString.append("\n");
		myModelOutputString.append("Minimum theta: "+ MIN_THETA);
		myModelOutputString.append("\n");
		myModelOutputString.append("Maximum theta: "+ MAX_THETA);
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("Varimetric Algorithm");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Budget: "+ nw.myBudget);
		System.out.println("\n");
		System.out.println("Number of Points: "+ NUM_POINTS);
		System.out.println("\n");
		System.out.println("Minimum theta: "+ MIN_THETA);
		System.out.println("\n");
		System.out.println("Maximum theta: "+ MAX_THETA);
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
		constructVolatilityMeasures();
		myDepot = this.mySimVMModelNCModel.getDepot();
		addVolatilityMeasuresToDepot();
		addVolatilityMeasuresToBases();
		addVolatilityMeasuresToInventory();	
		
	}
	

	public UpdatableVMNetworkLGOptim(ModelElement parent, NetworkConfiguration nw, ForecastTechnique ft) {
		this(parent, nw, ft, "UpdatableVMNetwork");
	}

	public UpdatableVMNetworkLGOptim(ModelElement parent, NetworkConfiguration nw, ForecastTechnique ft, String name) {
		this(parent, 1.0, nw, ft, name);
	}

	public UpdatableVMNetworkLGOptim(ModelElement parent, double updtprd, NetworkConfiguration nw, ForecastTechnique ft, String name) {
		super(parent, name);
		
		myModelOutputString = new StringBuilder();	
		
		//TODO
		/*myModelOutputString.append("MODEL OUTPUT STRING");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/
		System.out.println("MODEL OUTPUT STRING");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
		myUpdatePeriod = updtprd;		
		
		myNetworkConfiguration = nw;
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Network Configuration");
		myModelOutputString.append("\n");
		myModelOutputString.append(nw.getDataString());
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Network Configuration");
		System.out.println("\n");
		System.out.println(nw.getDataString());
		System.out.println("\n");
		
		myForecastTechnique = ft;
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Forecast technique: "+myForecastTechnique.getName());
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Forecast technique: "+myForecastTechnique.getName());
		System.out.println("\n");
		
		myVMModel = VariMetricModel.getVarimetricModel(parent, myNetworkConfiguration);	
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Varimetric Model");
		myModelOutputString.append("\n");
		myModelOutputString.append(myVMModel.toString());
		myModelOutputString.append("\n");
		myModelOutputString.append("Maps within the VMModel");
		myModelOutputString.append("\n");
		myModelOutputString.append("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Varimetric Model");
		System.out.println("\n");
		System.out.println(myVMModel.toString());
		System.out.println("\n");
		/*System.out.println("Maps within the VMModel");
		System.out.println("\n");
		System.out.println("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		System.out.println("\n");
		System.out.println("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		System.out.println("\n");*/
		
		myVMLagrangeAlgorithm = new VMLagrangeEnumerationAlgorithm(myVMModel, nw.myBudget, NUM_POINTS, MIN_THETA, MAX_THETA);
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Varimetric Algorithm");
		myModelOutputString.append("\n");
		myModelOutputString.append("Budget: "+ nw.myBudget);
		myModelOutputString.append("\n");
		myModelOutputString.append("Number of Points: "+ NUM_POINTS);
		myModelOutputString.append("\n");
		myModelOutputString.append("Minimum theta: "+ MIN_THETA);
		myModelOutputString.append("\n");
		myModelOutputString.append("Maximum theta: "+ MAX_THETA);
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Varimetric Algorithm");
		System.out.println("\n");
		System.out.println("Budget: "+ nw.myBudget);
		System.out.println("\n");
		System.out.println("Number of Points: "+ NUM_POINTS);
		System.out.println("\n");
		System.out.println("Minimum theta: "+ MIN_THETA);
		System.out.println("\n");
		System.out.println("Maximum theta: "+ MAX_THETA);
		System.out.println("\n");
		
		if(new Double(updtprd).isInfinite()){
			myUpdatable = false;
			
			//TODO
			/*myModelOutputString.append("**********************************************************");
			myModelOutputString.append("\n");
			myModelOutputString.append("The model is not updatable");
			myModelOutputString.append("\n");*/
			System.out.println("**********************************************************");
			System.out.println("\n");
			System.out.println("The model is not updatable");
			System.out.println("\n");
			
			mySimVMModelNCModel = new SimVMModelNCModel(parent, myNetworkConfiguration, name + "-NC");
			
		}else{
			myUpdatable = true;
			
			//TODO
			/*myModelOutputString.append("**********************************************************");
			myModelOutputString.append("\n");
			myModelOutputString.append("The update period: "+myUpdatePeriod);
			myModelOutputString.append("\n");*/
			System.out.println("**********************************************************");
			System.out.println("\n");
			System.out.println("The update period: "+myUpdatePeriod);
			System.out.println("\n");
			
			mySimVMModelNCModel = new SimVMModelNCUpdatableModel(parent, myNetworkConfiguration, myForecastTechnique, myUpdatePeriod, name + "-UpdatableNC");
			((SimVMModelNCUpdatableModel)mySimVMModelNCModel).addDemandAndForecastListeners(myNetworkConfiguration);
		}
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Simulation Model");
		myModelOutputString.append("\n");
		myModelOutputString.append(mySimVMModelNCModel.mySimVMModel.getModelStringFormat());
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Simulation Model");
		System.out.println("\n");
		System.out.println(mySimVMModelNCModel.mySimVMModel.getModelStringFormat());
		System.out.println("\n");
		
		myDepot = this.mySimVMModelNCModel.getDepot();
		
		if(myUpdatable){
			numConvergedOptimization = new Counter(this, getName() + "numConvergedOptimization");
			numUnConvergedOptimization = new Counter(this, getName() + "numUnConvergedOptimization");
			constructVolatilityMeasures();
			addVolatilityMeasuresToDepot();
			addVolatilityMeasuresToBases();
			addVolatilityMeasuresToInventory();			
		}	
		
	}
	
	private void constructVolatilityMeasures(){
		mapInventoryToNumBaseStockIncrement = new HashMap <Inventory,Counter>();
		mapInventoryToNumBaseStockDecrement = new HashMap <Inventory,Counter>();
		mapInventoryToBaseStockIncrement = new HashMap <Inventory,ResponseVariable>();
		mapInventoryToBaseStockIncrementVariance = new HashMap <Inventory,ResponseVariable>();
		mapInventoryToBaseStockDecrement = new HashMap <Inventory,ResponseVariable>();
		mapInventoryToBaseStockDecrementVariance = new HashMap <Inventory,ResponseVariable>();
		mapInventoryToSurplusOnHand = new HashMap <Inventory,TimeWeighted>();
		mapInventoryToTimeSpentInSurplusOnHand = new HashMap <Inventory,TimeWeighted>();
		mapInventoryToEndSurplusOnHand = new HashMap <Inventory,ResponseVariable>();
		mapInventoryToEndProbabilityOfSurplusOnHand = new HashMap <Inventory,ResponseVariable>();
		
		mapLocationToTotalNumBaseStockIncrement =  new HashMap <InventoryRepairingHoldingPoint, AggregateCounter>();
		mapLocationToTotalNumBaseStockDecrement =  new HashMap <InventoryRepairingHoldingPoint, AggregateCounter>();
		mapLocationToTotalBaseStockIncrement =  new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		mapLocationToTotalBaseStockIncrementVariance =  new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		mapLocationToTotalBaseStockDecrement =  new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		mapLocationToTotalBaseStockDecrementVariance =  new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		mapLocationToSurplusOnHandAtIHP = new HashMap <InventoryRepairingHoldingPoint, AggregateTimeWeightedVariable>();
		mapLocationToTimeSpentInSurplusOnHandAtIHP = new HashMap <InventoryRepairingHoldingPoint, AggregateTimeWeightedVariable>();
		mapLocationToEndSurplusOnHandAtIHP = new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		mapLocationToEndProbabilityOfSurplusOnHandAtIHP = new HashMap <InventoryRepairingHoldingPoint, ResponseVariableAverageObserver>();
		
		myTotalNumBaseStockIncrement = new AggregateCounter(this, getName() + "-Network-AggregateNumBaseStockIncrement");
		myTotalNumBaseStockDecrement = new AggregateCounter(this, getName() + "-Network-AggregateNumBaseStockDecrement");
		myTotalBaseStockIncrement = new ResponseVariableAverageObserver(this, getName() + "-Network-AggregateBaseStockIncrement");
		myTotalBaseStockIncrementVariance = new ResponseVariableAverageObserver(this, getName() + "-Network-AggregateBaseStockIncrementVariance");
		myTotalBaseStockDecrement = new ResponseVariableAverageObserver(this, getName() + "-Network-AggregateBaseStockDecrement");
		myTotalBaseStockDecrementVariance = new ResponseVariableAverageObserver(this, getName() + "-Network-AggregateBaseStockDecrementVariance");
		mySurplusInventoryInTheNetwork = new AggregateTimeWeightedVariable(this, getName() + "-Network-SurplusOnHand");
		myTimeSpentInSurplusOnHandInTheNetwork = new AggregateTimeWeightedVariable(this, getName() + "-Network-TimeSpentInSurplusOnHand");
		myEndSurplusInventoryInTheNetwork = new ResponseVariableAverageObserver(this, getName() + "-Network-EndSurplusOnHand");
		myEndProbabilityOfSurplusOnHandInTheNetwork = new ResponseVariableAverageObserver(this, getName() + "-Network-EndProbabilityOfSurplusOnHand");
			
	}
	
	private void setVolatilityMeasuresAtLocationLevel(InventoryRepairingHoldingPoint irhp){
		AggregateCounter c1 = new AggregateCounter(this, getName() + "-"+ irhp.getName() +"-AggregateNumBaseStockIncrement");
		AggregateCounter c2 = new AggregateCounter(this, getName() + "-"+ irhp.getName() +"-AggregateNumBaseStockDecrement");
		ResponseVariableAverageObserver r1 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-AggregateBaseStockIncrement");
		ResponseVariableAverageObserver r2 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-AggregateBaseStockIncrementVariance");
		ResponseVariableAverageObserver r3 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-AggregateBaseStockDecrement");
		ResponseVariableAverageObserver r4 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-AggregateBaseStockDecrementVariance");
		AggregateTimeWeightedVariable t1 = new AggregateTimeWeightedVariable(this, getName() +  "-"+ irhp.getName() +"-SurplusOnHandAtIHP");
		AggregateTimeWeightedVariable t2 = new AggregateTimeWeightedVariable(this, getName() +  "-"+ irhp.getName() +"-TimeSpentInSurplusOnHandAtIHP");
		ResponseVariableAverageObserver t3 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-EndSurplusOnHandAtIHP");
		ResponseVariableAverageObserver t4 = new ResponseVariableAverageObserver(this, getName() +  "-"+ irhp.getName() +"-EndProbabilityOfSurplusOnhHandAtIHP");
				
		mapLocationToTotalNumBaseStockIncrement.put(irhp, c1);
		mapLocationToTotalNumBaseStockDecrement.put(irhp, c2);
		mapLocationToTotalBaseStockIncrement.put(irhp, r1);
		mapLocationToTotalBaseStockIncrementVariance.put(irhp, r2);
		mapLocationToTotalBaseStockDecrement.put(irhp, r3);
		mapLocationToTotalBaseStockDecrementVariance.put(irhp, r4);
		mapLocationToSurplusOnHandAtIHP.put(irhp, t1);
		mapLocationToTimeSpentInSurplusOnHandAtIHP.put(irhp, t2);
		mapLocationToEndSurplusOnHandAtIHP.put(irhp, t3);
		mapLocationToEndProbabilityOfSurplusOnHandAtIHP.put(irhp, t4);
		
	}

	private void addVolatilityMeasuresToDepot(){
		setVolatilityMeasuresAtLocationLevel(myDepot);
	}
	
	private void addVolatilityMeasuresToBases(){
		for(InventoryRepairingHoldingPoint base: this.mySimVMModelNCModel.getBases()){	
			setVolatilityMeasuresAtLocationLevel(base);			
		}		
	}
	
	private void addVolatilityMeasuresToInventory(){
		for(DItem di: myNetworkConfiguration.myItems){
			Inventory invdepot = mySimVMModelNCModel.getMapDItemToInventory().get(di);
			this.setVolatilityMeasuresAtInventoryLevel(invdepot, myDepot);
			for(BItem bi: di.myBItems){
				Inventory invbase = mySimVMModelNCModel.getMapBItemToInventory().get(bi);
				InventoryRepairingHoldingPoint base = mySimVMModelNCModel.getMapBItemToIRHP().get(bi);
				this.setVolatilityMeasuresAtInventoryLevel(invbase, base);
			}
		}		
	}
	
	private void setVolatilityMeasuresAtInventoryLevel(Inventory inv, InventoryRepairingHoldingPoint base){
		
		Counter c1 = new Counter(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-NumBaseStockIncrement");
		Counter c2 = new Counter(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-NumBaseStockDecrement");		
		ResponseVariable r1 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-BaseStockIncrement");		
		ResponseVariable r2 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-BaseStockIncrementVariance");
		ResponseVariable r3 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-BaseStockDecrement");
		ResponseVariable r4 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-BaseStockDecrementVariance");
		TimeWeighted t1 = new TimeWeighted(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-SurplusInventory");
		TimeWeighted t2 = new TimeWeighted(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-TimeSpentInSurplusInventory");
		ResponseVariable t3 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-EndSurplusInventory");
		ResponseVariable t4 = new ResponseVariable(this, getName()+"-"+inv.getItemType()+"-"+base.getName()+"-EndProbabilityOfSurplusInventory");
				
		mapInventoryToNumBaseStockIncrement.put(inv, c1);		
		mapInventoryToNumBaseStockDecrement.put(inv, c2);		
		mapInventoryToBaseStockIncrement.put(inv, r1);		
		mapInventoryToBaseStockIncrementVariance.put(inv, r2);		
		mapInventoryToBaseStockDecrement.put(inv, r3);		
		mapInventoryToBaseStockDecrementVariance.put(inv, r4);
		mapInventoryToSurplusOnHand.put(inv, t1);
		mapInventoryToTimeSpentInSurplusOnHand.put(inv, t2);
		mapInventoryToEndSurplusOnHand.put(inv, t3);
		mapInventoryToEndProbabilityOfSurplusOnHand.put(inv, t4);
		
		mapLocationToTotalNumBaseStockIncrement.get(base).subscribeTo(c1);
		mapLocationToTotalNumBaseStockDecrement.get(base).subscribeTo(c2);
		mapLocationToTotalBaseStockIncrement.get(base).subscribeTo(r1);
		mapLocationToTotalBaseStockIncrementVariance.get(base).subscribeTo(r2);
		mapLocationToTotalBaseStockDecrement.get(base).subscribeTo(r3);
		mapLocationToTotalBaseStockDecrementVariance.get(base).subscribeTo(r4);
		mapLocationToSurplusOnHandAtIHP.get(base).subscribeTo(t1);
		mapLocationToTimeSpentInSurplusOnHandAtIHP.get(base).subscribeTo(t2);
		mapLocationToEndSurplusOnHandAtIHP.get(base).subscribeTo(t3);
		mapLocationToEndProbabilityOfSurplusOnHandAtIHP.get(base).subscribeTo(t4);
		
		myTotalNumBaseStockIncrement.subscribeTo(c1);
		myTotalNumBaseStockDecrement.subscribeTo(c2);
		myTotalBaseStockIncrement.subscribeTo(r1);
		myTotalBaseStockIncrementVariance.subscribeTo(r2);
		myTotalBaseStockDecrement.subscribeTo(r3);
		myTotalBaseStockDecrementVariance.subscribeTo(r4);
		mySurplusInventoryInTheNetwork.subscribeTo(t1);
		myTimeSpentInSurplusOnHandInTheNetwork.subscribeTo(t2);
		myEndSurplusInventoryInTheNetwork.subscribeTo(t3);
		myEndProbabilityOfSurplusOnHandInTheNetwork.subscribeTo(t4);		
	}
    
	public void beforeExperiment(){  
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("BEFORE EXPERIMENT >> INITIAL OPTIMIZATION");
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("BEFORE EXPERIMENT >> INITIAL OPTIMIZATION");
		System.out.println("\n");
		System.out.println("\n");
		
        myVMLagrangeAlgorithm.optimize();
        
        //TODO
		/*myModelOutputString.append(".......... optimized");
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");
		myModelOutputString.append(myVMModel.toString());
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");
		myModelOutputString.append("Maps within the VMModel");
		myModelOutputString.append("\n");
		myModelOutputString.append("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");*/
		/*System.out.println(".......... optimized");
		System.out.println("\n");
		System.out.println("\n");
		System.out.println(myVMModel.toString());
		System.out.println("\n");
		System.out.println("\n");
		System.out.println("Maps within the VMModel");
		System.out.println("\n");
		System.out.println("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
		System.out.println("\n");
		System.out.println("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
		System.out.println("\n");
		System.out.println("\n");*/
				
        myOriginalVMModel = VariMetricModel.newInstance(myVMModel); 
        
        //TODO
		/*myModelOutputString.append(".......... the original VMModel is set");
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");
		myModelOutputString.append(myOriginalVMModel.toString());
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");
		myModelOutputString.append("Maps within the VMModel");
		myModelOutputString.append("\n");
		myModelOutputString.append("MapDItemToVMItem: "+myOriginalVMModel.getMapDItemToVMItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("MapBItemToVMItem: "+myOriginalVMModel.getMapBItemToVMBaseItem());
		myModelOutputString.append("\n");
		myModelOutputString.append("\n");*/ 
		System.out.println(".......... the original VMModel is set");
		System.out.println("\n");
		System.out.println("\n");
		System.out.println(myOriginalVMModel.toString());
		System.out.println("\n");
		System.out.println("\n");
		/*System.out.println("Maps within the VMModel");
		System.out.println("\n");
		System.out.println("MapDItemToVMItem: "+myOriginalVMModel.getMapDItemToVMItem());
		System.out.println("\n");
		System.out.println("MapBItemToVMItem: "+myOriginalVMModel.getMapBItemToVMBaseItem());
		System.out.println("\n");
		System.out.println("\n");*/
        
        initializePolicy();	  
		
	}
	
	public void beforeReplication(){
		if(myUpdatable){ 
			
			//TODO
			/*myModelOutputString.append("**********************************************************");
			myModelOutputString.append("\n");
			myModelOutputString.append("Replication "+this.getCurrentReplication());
			myModelOutputString.append("\n"); 
			myModelOutputString.append("\n"); 
			myModelOutputString.append("BEFORE REPLICATION >> RESET THE VMMODEL");
			myModelOutputString.append("\n");
			myModelOutputString.append("\n");*/
			System.out.println("**********************************************************");
			System.out.println("\n");
			System.out.println("Replication "+this.getCurrentReplication());
			System.out.println("\n"); 
			System.out.println("\n"); 
			System.out.println("BEFORE REPLICATION >> RESET THE VMMODEL");
			System.out.println("\n");
			System.out.println("\n");
			
			myVMModel = VariMetricModel.newInstance(myOriginalVMModel);	
			
			//TODO
			/*myModelOutputString.append(".......... reset the VMModel of the current replication");
			myModelOutputString.append("\n");
			myModelOutputString.append("\n");
			myModelOutputString.append(myVMModel.toString());
			myModelOutputString.append("\n");
			myModelOutputString.append("\n");
			myModelOutputString.append("Maps within the VMModel");
			myModelOutputString.append("\n");
			myModelOutputString.append("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
			myModelOutputString.append("\n");
			myModelOutputString.append("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
			myModelOutputString.append("\n");
			myModelOutputString.append("**********************************************************");
			myModelOutputString.append("\n");
			myModelOutputString.append("\n");	*/
			System.out.println(".......... reset the VMModel of the current replication");
			System.out.println("\n");
			System.out.println("\n");
			/*System.out.println(myVMModel.toString());
			System.out.println("\n");
			System.out.println("\n");
			System.out.println("Maps within the VMModel");
			System.out.println("\n");
			System.out.println("MapDItemToVMItem: "+myVMModel.getMapDItemToVMItem());
			System.out.println("\n");
			System.out.println("MapBItemToVMItem: "+myVMModel.getMapBItemToVMBaseItem());
			System.out.println("\n");
			System.out.println("**********************************************************");
			System.out.println("\n");
			System.out.println("\n");*/
			
		}		
	}
	
	public void initialize(){
		if(myUpdatable){
			scheduleEvent(myUpdatePeriod, 2, 2, "PolicyUpdatingInterval");
		}		
	}
	
	protected void handleEvent(JSLEvent event){
		schedulePolicyUpdating(event);
	}
	
	protected void schedulePolicyUpdating(JSLEvent event){
		
		//TODO
		/*myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("Time "+this.getTime());
		myModelOutputString.append("\n");		
    	myModelOutputString.append(" .......... retreiving the forecast estimate at the bases and resetting the demand rates at the VMItems");	
		myModelOutputString.append("\n");*/
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("Time "+this.getTime());
		System.out.println("\n");		
    	System.out.println(" .......... retreiving the forecast estimate at the bases and resetting the demand rates at the VMItems");	
		System.out.println("\n");
		
		for(DItem di: myNetworkConfiguration.myItems){
			Inventory invdepot = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapDItemToInventory().get(di);
			for(BItem bi: di.myBItems){
				Inventory invbase = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapBItemToInventory().get(bi);
				DemandArrivalListenerWithForecaster demlistener_base = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getDemandArrivalListener(invbase);
				double agglevel_base = demlistener_base.getAggregationLevel();
				double forecast_estimate_at_agg_level_base = 
					((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getForecastArrivalListener(invbase).getCurrentForecastEstimate();
				double demandrate_base = Math.max(0.0000001, forecast_estimate_at_agg_level_base/agglevel_base);
				
				//TODO
	    		/*myModelOutputString.append(".. base inventory "+invbase.getName()+" forecasts "+demandrate_base);
	    		myModelOutputString.append("\n");*/
	    		System.out.println(".. base inventory "+invbase.getName()+" forecasts "+demandrate_base);
	    		System.out.println("\n");
	    		
				myVMModel.getMapBItemToVMBaseItem().get(bi).setDemandRate(demandrate_base);
				
				//TODO
	    		/*myModelOutputString.append(".. demand rate of VMBaseItem-IID-"+myVMModel.getMapBItemToVMBaseItem().get(bi).myItemNumber
	    				+"-BID-"+myVMModel.getMapBItemToVMBaseItem().get(bi).getBaseNumber()
	    				+" set to "+demandrate_base);
	    		myModelOutputString.append("\n");*/
	    		System.out.println(".. demand rate of VMBaseItem-IID-"+myVMModel.getMapBItemToVMBaseItem().get(bi).myItemNumber
						+"-BID-"+myVMModel.getMapBItemToVMBaseItem().get(bi).getBaseNumber()
						+" set to "+demandrate_base);
				System.out.println("\n");
			}
		}
		
		//TODO
		/*myModelOutputString.append("\n");
		myModelOutputString.append("\n");
		myModelOutputString.append(" .......... optimizing after resetting the demand rates");
		myModelOutputString.append("\n");*/
		System.out.println("\n");
		System.out.println("\n");
		System.out.println(" .......... optimizing after resetting the demand rates");
		System.out.println("\n");
		
		myVMLagrangeAlgorithm.optimize();
		if(myVMLagrangeAlgorithm.isConvergedWithinSearchLimit()){
			numConvergedOptimization.increment();
		}else{
			numUnConvergedOptimization.increment();
		}
		updatePolicy();
		
		double t = myUpdatePeriod;
		scheduleEvent(t, 2, 2, "PolicyUpdatingInterval");		
	}
	
	protected void initializePolicy(){
		if(myUpdatable){
			
			//TODO
	    	/*myModelOutputString.append(" .......... setting the inital policies at depot and bases");	
    		myModelOutputString.append("\n");*/	
	    	System.out.println(" .......... setting the inital policies at depot and bases");	
    		System.out.println("\n");	
    		
			for(DItem di: myNetworkConfiguration.myItems){
				Inventory invdepot = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapDItemToInventory().get(di);
				int default_basestock_d = myVMModel.getMapDItemToVMItem().get(di).getStockLevel();
	            double [] param_d = {default_basestock_d-1,1};
	            invdepot.setInitialPolicyParameters(param_d);
	            invdepot.setInitialOnHand((int)param_d[1]);
	            
	            //TODO
	    		/*myModelOutputString.append(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" , "+param_d[1]+" , "+param_d[1]+"}");
	    		myModelOutputString.append("\n");*/
	    		System.out.println(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" , "+param_d[1]+" , "+param_d[1]+"}");
	    		System.out.println("\n");
	    		
				for(BItem bi: di.myBItems){
					Inventory invbase = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapBItemToInventory().get(bi);
					int default_basestock_b = myVMModel.getMapBItemToVMBaseItem().get(bi).getStockLevel();
		            double [] param_b = {default_basestock_b-1,1};
		            invbase.setInitialPolicyParameters(param_b);
		            invbase.setInitialOnHand((int)param_b[1]);
		            
		            //TODO
		    		/*myModelOutputString.append(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" , "+param_b[1]+" , "+param_b[1]+"}");
		    		myModelOutputString.append("\n");*/
		    		System.out.println(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" , "+param_b[1]+" , "+param_b[1]+"}");
		    		System.out.println("\n");
		    		
				}
			}			
		}else{
			
			//TODO
	    	/*myModelOutputString.append(" .......... setting the inital policies at depot and bases");	
    		myModelOutputString.append("\n");*/
	    	System.out.println(" .......... setting the inital policies at depot and bases");	
    		System.out.println("\n");
    		
			for(DItem di: myNetworkConfiguration.myItems){
				Inventory invdepot = mySimVMModelNCModel.getMapDItemToInventory().get(di);
				int default_basestock_d = myVMModel.getMapDItemToVMItem().get(di).getStockLevel();
	            double [] param_d = {default_basestock_d-1,1};
	            invdepot.setInitialPolicyParameters(param_d);
	            invdepot.setInitialOnHand((int)param_d[1]);
	            
	            //TODO
	    		/*myModelOutputString.append(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" , "+param_d[1]+" , "+param_d[1]+"}");
	    		myModelOutputString.append("\n");*/
	    		System.out.println(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" , "+param_d[1]+" , "+param_d[1]+"}");
	    		System.out.println("\n");
	    		
				for(BItem bi: di.myBItems){
					Inventory invbase = mySimVMModelNCModel.getMapBItemToInventory().get(bi);
					int default_basestock_b = myVMModel.getMapBItemToVMBaseItem().get(bi).getStockLevel();
		            double [] param_b = {default_basestock_b-1,1};
		            invbase.setInitialPolicyParameters(param_b);
		            invbase.setInitialOnHand((int)param_b[1]);
		            
		            //TODO
		    		/*myModelOutputString.append(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" , "+param_b[1]+" , "+param_b[1]+"}");
		    		myModelOutputString.append("\n");*/
		    		System.out.println(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" , "+param_b[1]+" , "+param_b[1]+"}");
		    		System.out.println("\n");
		    		
				}
			}			
		}
	}
	
	protected void updatePolicy(){
		
		//TODO
    	/*myModelOutputString.append(" .......... resetting the policies at depot and bases after optimization");	
		myModelOutputString.append("\n");*/
    	System.out.println(" .......... resetting the policies at depot and bases after optimization");	
		System.out.println("\n");
		
		for(DItem di: myNetworkConfiguration.myItems){
			Inventory invdepot = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapDItemToInventory().get(di);
			double original_d = invdepot.getInventoryPolicyParameters()[0] + 1;
			int default_basestock_d = myVMModel.getMapDItemToVMItem().get(di).getStockLevel();

            double change_d = original_d - default_basestock_d;			

            double [] param_d = {default_basestock_d - 1,1};
            invdepot.setInventoryPolicyParameters(param_d);
            
            //TODO
    		/*myModelOutputString.append(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" - "+param_d[1]+"}");
    		myModelOutputString.append("\n");*/
    		System.out.println(">> depot inventory "+invdepot.getName()+" set to {"+param_d[0]+" , "+param_d[1]+"}");
    		System.out.println("\n");

            int ip_d = invdepot.getInventoryPosition();

            if(change_d > 0){
                mapInventoryToNumBaseStockIncrement.get(invdepot).increment();
                mapInventoryToBaseStockIncrement.get(invdepot).setValue(change_d);					
                if(ip_d > default_basestock_d + 1){						
                    this.mapInventoryToSurplusOnHand.get(invdepot).setValue(ip_d - default_basestock_d + 1);
                    this.mapInventoryToTimeSpentInSurplusOnHand.get(invdepot).setValue(1.0);
                }else{
                    this.mapInventoryToSurplusOnHand.get(invdepot).setValue(0.0);
                    this.mapInventoryToTimeSpentInSurplusOnHand.get(invdepot).setValue(0.0);
                }
            }else if (change_d < 0){
                mapInventoryToNumBaseStockDecrement.get(invdepot).increment();
                mapInventoryToBaseStockDecrement.get(invdepot).setValue(Math.abs(change_d));
                this.mapInventoryToSurplusOnHand.get(invdepot).setValue(0.0);
                this.mapInventoryToTimeSpentInSurplusOnHand.get(invdepot).setValue(0.0);
            }
            
			for(BItem bi: di.myBItems){
				Inventory invbase = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapBItemToInventory().get(bi);
				double original_b = invbase.getInventoryPolicyParameters()[0] + 1;
				int default_basestock_b = myVMModel.getMapBItemToVMBaseItem().get(bi).getStockLevel();

	            double change_b = original_b - default_basestock_b;			

	            double [] param_b = {default_basestock_b - 1,1};
	            invbase.setInventoryPolicyParameters(param_b);
	            
	            //TODO
	    		/*myModelOutputString.append(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" - "+param_b[1]+"}");
	    		myModelOutputString.append("\n");*/
	    		System.out.println(">> base inventory "+invbase.getName()+" set to {"+param_b[0]+" , "+param_b[1]+"}");
	    		System.out.println("\n");
	            
	            
	            int ip_b = invbase.getInventoryPosition();

	            if(change_b > 0){
	                mapInventoryToNumBaseStockIncrement.get(invbase).increment();
	                mapInventoryToBaseStockIncrement.get(invbase).setValue(change_b);					
	                if(ip_b > default_basestock_b + 1){						
	                    this.mapInventoryToSurplusOnHand.get(invbase).setValue(ip_b - default_basestock_b + 1);
	                    this.mapInventoryToTimeSpentInSurplusOnHand.get(invbase).setValue(1.0);
	                }else{
	                    this.mapInventoryToSurplusOnHand.get(invbase).setValue(0.0);
	                    this.mapInventoryToTimeSpentInSurplusOnHand.get(invbase).setValue(0.0);
	                }
	            }else if (change_b < 0){
	                mapInventoryToNumBaseStockDecrement.get(invbase).increment();
	                mapInventoryToBaseStockDecrement.get(invbase).setValue(Math.abs(change_b));
	                this.mapInventoryToSurplusOnHand.get(invbase).setValue(0.0);
	                this.mapInventoryToTimeSpentInSurplusOnHand.get(invbase).setValue(0.0);
	            }
			}
		}
    }

    protected void replicationEnded() {	
    	if(myUpdatable){
    		for(DItem di: myNetworkConfiguration.myItems){
    			Inventory invdepot = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapDItemToInventory().get(di);
    			
    			double var_inc_d = mapInventoryToBaseStockIncrement.get(invdepot).getWithinReplicationStatistic().getVariance();
                mapInventoryToBaseStockIncrementVariance.get(invdepot).setValue(var_inc_d);			

                double last_surplus_on_hand_d = mapInventoryToSurplusOnHand.get(invdepot).getWithinReplicationStatistic().getLastValue();
                mapInventoryToEndSurplusOnHand.get(invdepot).setValue(last_surplus_on_hand_d);
                if(last_surplus_on_hand_d>0){
                    mapInventoryToEndProbabilityOfSurplusOnHand.get(invdepot).setValue(1.0);
                }else{
                    mapInventoryToEndProbabilityOfSurplusOnHand.get(invdepot).setValue(0.0);
                }
                double var_dec_d = mapInventoryToBaseStockDecrement.get(invdepot).getWithinReplicationStatistic().getVariance();
                mapInventoryToBaseStockDecrementVariance.get(invdepot).setValue(var_dec_d);
    			
    			
                for(BItem bi: di.myBItems){
                	Inventory invbase = ((SimVMModelNCUpdatableModel)mySimVMModelNCModel).getMapBItemToInventory().get(bi);
                    double var_inc_b = mapInventoryToBaseStockIncrement.get(invbase).getWithinReplicationStatistic().getVariance();
                    mapInventoryToBaseStockIncrementVariance.get(invbase).setValue(var_inc_b);			
                    double last_surplus_on_hand_b = mapInventoryToSurplusOnHand.get(invbase).getWithinReplicationStatistic().getLastValue();
                    mapInventoryToEndSurplusOnHand.get(invbase).setValue(last_surplus_on_hand_b);
                    if(last_surplus_on_hand_b>0){
                        mapInventoryToEndProbabilityOfSurplusOnHand.get(invbase).setValue(1.0);
                    }else{
                        mapInventoryToEndProbabilityOfSurplusOnHand.get(invbase).setValue(0.0);
                    }
                    double var_dec_b = mapInventoryToBaseStockDecrement.get(invbase).getWithinReplicationStatistic().getVariance();
                    mapInventoryToBaseStockDecrementVariance.get(invbase).setValue(var_dec_b);			}
    		}    		
    	}
    	
		//TODO
		/*myModelOutputString.append("REPLICATION ENDED");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");
		myModelOutputString.append("**********************************************************");
		myModelOutputString.append("\n");*/	
    	System.out.println("REPLICATION ENDED");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		System.out.println("**********************************************************");
		System.out.println("\n");
		
	}

	public VariMetricModel getVMModel() {
		return myVMModel;
	}

	public void setVMLagrangeAlgorithm(VMLagrangeAlgorithm myVMLagrangeAlgorithm) {
		this.myVMLagrangeAlgorithm = myVMLagrangeAlgorithm;
	}
	public String getModelStringFormat(){
		return this.myModelOutputString.toString();
	}
	
	public static Model getUpdatableNetworkModel(NetworkConfiguration nc, ForecastTechnique ft, double period, String name){

        Model m = Model.createModel(name);
		
		UpdatableVMNetworkLGOptim nw = new UpdatableVMNetworkLGOptim(m, period, nc, ft, "UpdatableSparingModel-"+name);

		try {
	        BufferedWriter out = new BufferedWriter(new FileWriter("ModelOutput-" + name + ".txt"));
	        //out.write(nw.getModelStringFormat());
	        out.write(nc.getDataString());
	        out.close();
	    } 
		catch (IOException e1){	
	    }

		return m;
		
	}
	
	
	public static void main(String[] args) {
		
		//double [] cratio = {0.8, 0.1};
		double [] cratio = {0.01, 0.01};
        double tp = 500;
        //double [] pratio = {0.1, 0.1, 0.1};
        double [] pratio = {0.1, 0.1, 0.4};
        DUniform items = new DUniform(3, 5);
        DUniform bases = new DUniform(3, 10);
        double budgetconstant = 1.0;
        
        Croston sb = new Croston(0.15);
        double period = 7;
				        
        NetworkGenerator ng = new NetworkGenerator(items, bases, budgetconstant, 
                cratio, pratio, tp);
                
		for(int i = 1; i <= 1; i++){
			
			NetworkConfiguration nc =  ng.getNetwork();
	        
	        System.out.println(nc);	        

	        Model m = UpdatableVMNetworkLGOptim.getUpdatableNetworkModel(nc, sb.copyInstance(), period, "Nwrk"+i);
			
			// create the experiment to run the model
			Experiment e = new Experiment(m,"Nwrk"+i);
			
			try{
	        	CSVReport ecsv = new CSVExperimentReport("Nwrk"+i);
	        	m.addObserver(ecsv);
	        }
	        catch(IOException exception){
				System.out.println("I/O exception opening file Output.txt");
				exception.printStackTrace();
	        }
	        
			// set the parameters of the experiment
			e.setNumberOfReplications(10);
			e.setLengthOfReplication(500.0);
			//e.setLengthOfWarmUp(100.0);
			
			e.turnOnExperimentReport();
			//e.turnOnReplicationReport();
			//e.turnOnExperimentConsoleOutput(true);
			
			// tell the experiment to run
			e.runAll();	
			
			
		}

	}

}
