package jslx.supplychain.inventorylayer;


import jsl.modeling.elements.variable.AggregateCounter;
import jsl.modeling.elements.variable.ResponseVariableAverageObserver;
import jsl.modeling.elements.variable.AggregateTimeWeightedVariable;

public interface AggregateInventoryIfc {

	/**
	 * @return Returns the Aggregate Avg Customer Wait Time ResponseVariable.
	 */
	public ResponseVariableAverageObserver getAggregateAvgCustomerWaitTimeResponseVariable();

	/**
	 * @return Returns the get Aggregate Avg First FillRate ResponseVariable.
	 */
	public ResponseVariableAverageObserver getAggregateAvgFirstFillRateResponseVariable();

	/**
	 * @return Returns the Aggregate Amount BackOrdered ResponseVariable.
	 */
	public AggregateTimeWeightedVariable getAggregateAmountBackOrderedResponseVariable();

	/**
	 * @return Returns the Aggregate Number BackOrdered ResponseVariable.
	 */
	public AggregateTimeWeightedVariable getAggregateNumberBackOrderedResponseVariable();

	/**
	 * @return Returns the Aggregate OnHand Inventory ResponseVariable.
	 */
	public AggregateTimeWeightedVariable getAggregateOnHandInventoryResponseVariable();

	/**
	 * @return Returns the Aggregate OnOrder ResponseVariable.
	 */
	public AggregateTimeWeightedVariable getAggregateAmountOnOrderResponseVariable();

	/** 
	 * 
	 * @return Returns a counter for the total number of replenishment orders made
	 */
	public AggregateCounter getAggregateNumberOfReplenishmentDemandsCounter();

	/**
	 * @return Returns the Aggregate StockOut Percentage ResponseVariable.
	 */
	public AggregateTimeWeightedVariable getAggregateStockOutPercentageResponseVariable();
	
	/**
	 * 
	 * @param aggregate The aggregate to attach to the customer waiting time
	 */
	public void subscribeToAggregateAvgCustomerWaitTime(ResponseVariableAverageObserver aggregate);
	
	/**
	 * 
	 * @param aggregate The aggregate to attach to the first fill rate
	 */
	public void subscribeToAggregateAvgFirstFillRate(ResponseVariableAverageObserver aggregate);
	
	/**
	 * @param aggregate The aggregate time weighted variable to attach
	 */
	public void subscribeToAggregateAmountBackOrdered(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to attach
	 */
	public void subscribeToAggregateNumberBackOrdered(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to attach
	 */
	public void subscribeToAggregateOnHandInventory(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to attach
	 */
	public void subscribeToAggregateAmountOnOrder(AggregateTimeWeightedVariable aggregate);

	/** Attaches an AggregateCounter to the aggregate replenishment demand counter
	 * 
	 * @param counter
	 */
	public void subscribeToAggregateReplenishmentDemandCounter(AggregateCounter counter);
	
	/**
	 * @param aggregate The aggregate time weighted variable to attach
	 */
	public void subscribeToAggregateStockOutIndicator(AggregateTimeWeightedVariable aggregate);
	
	/**
	 * 
	 * @param aggregate The aggregate to detach from the customer waiting time
	 */
	public void unsubscribeFromAggregateAvgCustomerWaitTime(ResponseVariableAverageObserver aggregate);
	
	/**
	 * 
	 * @param aggregate The aggregate to detach from the first fill rate
	 */
	public void unsubscribeFromAggregateAvgFirstFillRate(ResponseVariableAverageObserver aggregate);
	
	/**
	 * @param aggregate The aggregate time weighted variable to detach
	 */
	public void unsubscribeFromAggregateAmountBackOrdered(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to detach
	 */
	public void unsubscribeFromAggregateNumberBackOrdered(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to detach
	 */
	public void unsubscribeFromAggregateOnHandInventory(AggregateTimeWeightedVariable aggregate);

	/**
	 * @param aggregate The aggregate time weighted variable to detach
	 */
	public void unsubscribeFromAggregateAmountOnOrder(AggregateTimeWeightedVariable aggregate);

	/** Attaches an AggregateCounter to the aggregate replenishment demand counter
	 * 
	 * @param counter
	 */
	public void unsubscribeFromAggregateReplenishmentDemandCounter(AggregateCounter counter);
	
	/**
	 * @param aggregate The aggregate time weighted variable to detach
	 */
	public void unsubscribeFromAggregateStockOutIndicator(AggregateTimeWeightedVariable aggregate);
}