/*
 * Created on Sept 04, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.inventorylayer;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.utilities.random.distributions.Constant;


/**
 * @author vvarghe
 *
 * This is an abstract template for demand arrival listener and  
 * its concrete class serves to encapsulate the behavior of responding to 
 * demand arrival
 *  
 */
public abstract class DemandArrivalListenerAbstract extends SchedulingElement implements InventoryDemandArrivalListenerIfc {
	
	/** 
	 *  total aggregate demand
	 *  
	 */
	protected double sumDemandAcrossBuckets;
	/** 
	 *  aggregate demand within the current aggregation period
	 *  
	 */
	 protected double sumDemandCurrentBucket;
	 
	 /** 
		 *  current demand
		 *  
		 */
	 protected double myCurrentDemand;
	
	 /** 
		 *  current time
		 *  
		 */
	 public double myCurrentTime;
	
	 /** 
		 *  aggregation period
		 *  
		 */
	 protected double myAggregationLevel;
	 
	 /** 
		 *  aggregation period
		 *  
		 */
	 protected Constant myAggregationEventTime;
	
	 /** 
		 *  inventory to which the listener is attached
		 *  
		 */	 
	 protected Inventory myInventory;
	
	/**
	 * @param parent
	 * @param inv
	 */	
	 public DemandArrivalListenerAbstract(ModelElement parent, Inventory inv) {
		this(parent, inv, null, 1);
	}

	/**
	 * @param parent
	 * @param inv
	 * @param name
	 */
	public DemandArrivalListenerAbstract(ModelElement parent,  Inventory inv, String name) {
		this(parent, inv, name, 1);
	}
	
	
	
	/**
	 * @param parent
	 * @param inv
	 * @param name
	 * @param agglevel
	 */
	public DemandArrivalListenerAbstract(ModelElement parent, Inventory inv, String name, double agglevel) {
		super(parent, name);
		setInventory(inv);
		myAggregationLevel = agglevel;
		sumDemandAcrossBuckets = 0.0;
		sumDemandCurrentBucket = 0.0;
		myAggregationEventTime = new Constant((double)myAggregationLevel);
	}
	
	/** 
	 *  respond to a demand arrival by aggregating the demand wiuthin the current update period
	 *  
	 */	
	public abstract void demandArrived(Inventory inv, Demand demand);
	
	protected final void setInventory(Inventory inv){
		myInventory = inv;
		myInventory.setDemandArrivalListener(this);
	}
	
	/** 
	 *  what to do at the end of each aggregation period and reschedule the next aggregation period
	 *  
	 */	
	protected abstract void schedulePeriodicDemandProcessing(JSLEvent event);	
	
	public final double getAmountOnHand() {
		return myInventory.getAmountOnHand();
	}
	
	public final double getAmountOnOrder() {
		return myInventory.getAmountOnOrder();
	}
	
	public final double getAmountBackLogged() {
		return myInventory.getBackLogInfo().getAmountBackLogged();
	}
	
	public final double getAmountPending() {
		return myInventory.getAmountPending();
	}
	
	public final double getInventoryPosition() {
		return myInventory.getInventoryPosition();
	}
	
	public final double [] getInventoryPolicyParameter() {
		return myInventory.getInventoryPolicyParameters();
	}	
	
	public final double getAggregationLevel() {
		return myAggregationLevel;
	}
	
	protected void removedFromModel(){
		super.removedFromModel();
		myAggregationEventTime = null;
		myInventory = null;
	}
}
