/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jslx.supplychain.transportlayer.DemandCarrierIfc;

abstract public class DemandFillerAbstract extends SchedulingElement implements DemandFillerIfc {

	/** Indicates whether or not this filler is available
	 */
	private boolean myAvailabilityFlag;

	/** Indicates whether or not this object is available at the beginning of a replication
	 */
	private boolean myInitialAvailabilityFlag;

	/** The carrier that sends replenishments to this IHP's
	 *  customers
	 *  If supplied, will be used to ensure that a filled demand
	 *  is transported back to the customer.  If no carrier is supplied,
	 *  the demand is transported immediately (with no delay) to the customer.
	 */
	protected DemandCarrierIfc myExternalDemandCarrier;

	public DemandFillerAbstract(ModelElement parent){
		this(parent, true, null);
	}

	public DemandFillerAbstract(ModelElement parent, String name){
		this(parent, true, name);
	}
	
	public DemandFillerAbstract(ModelElement parent, boolean availability, String name) {
		super(parent, name);
		setInitialAvailability(availability);
		setAvailability(availability);
	}
	
	public final boolean isAvailable() {
		return myAvailabilityFlag;
	}

	public final boolean getInitialAvailability() {
		return myInitialAvailabilityFlag;
	}

	/**
	 * @param availabilityFlag The availabilityFlag to set.
	 */
	public final void setInitialAvailability(boolean availabilityFlag) {
		myInitialAvailabilityFlag = availabilityFlag;	
	}
	
	//  ===========================================
	//      PROTECTED METHODS
	//  ===========================================

	/** Used to initialize prior to a replication
	 * 
	 */
	protected void intialize(){
		setAvailability(getInitialAvailability());
	}
	
	/**
	 * @param availabilityFlag The availabilityFlag to set.
	 */
	protected final void setAvailability(boolean availabilityFlag) {
		myAvailabilityFlag = availabilityFlag;
	}

	/** Sets the carrier that this IHP will use when sending demands
	 *  to customers of its inventory.
	 *  
	 * @param carrier
	 */
	public final void setDemandCarrier(DemandCarrierIfc carrier) {
		myExternalDemandCarrier = carrier;
	}

	/** Gets the carrier for this IHP
	 * 
	 * @return
	 */
	public final DemandCarrierIfc getDemandCarrier() {
		return myExternalDemandCarrier;
	}

}
