/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class DemandFillerFinder implements DemandFillerFinderIfc {

	/** Holds the customer (sender) to supplier (filler) relationship
	 *  The sender is the key.  Thus, a customer may have 1 and only 1 supplier, but
	 *  a supplier can have many customers
	 * 
	 */
	private Map <DemandSenderIfc, DemandFillerIfc> myCustomerSupplierMap;	

	public DemandFillerFinder() {
		myCustomerSupplierMap = new HashMap <DemandSenderIfc, DemandFillerIfc>();
	}

	public boolean containsCustomer(DemandSenderIfc customer) {
		return myCustomerSupplierMap.containsKey(customer);
	}

	public boolean containsSupplier(DemandFillerIfc filler) {
		return myCustomerSupplierMap.containsValue(filler);
	}

	public boolean isEmpty() {
		return myCustomerSupplierMap.isEmpty();
	}

	public Set<DemandSenderIfc> customerSet() {
		return myCustomerSupplierMap.keySet();
	}

	public DemandFillerIfc put(DemandSenderIfc customer, DemandFillerIfc supplier) {
		if (customer == null)
			throw new IllegalArgumentException("The customer was null");
		
		return myCustomerSupplierMap.put(customer, supplier);
	}

	public int size() {
		return myCustomerSupplierMap.size();
	}

	public DemandFillerIfc findDemandFiller(Demand demand) {
		if (demand == null)
			throw new IllegalArgumentException("The demand was null");
		
		DemandSenderIfc sender = demand.getSender();
		
		DemandFillerIfc filler = myCustomerSupplierMap.get(sender);
		if (filler == null)
			throw new NoDemandFillerFoundException("No demand filler was found for demand: " + demand);
		return filler;
	}

}
