/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.modeling.Model;

/**
 *
 */
public class DemandMessage implements DemandMessageIfc {

	private DemandFillerIfc myFiller;
	
	private int myStatus;
	
	private int myRequestFillAmount;
	
	private InventoryIfc myInventory;
	
	private boolean myCanFillItemTypeFlag;
	
	private boolean myMayPartialFillItemTypeFlag;
		
	private boolean myMayBackLogDemandsFlag;
	
	private double myTimeStamp;
	
	public DemandMessage(DemandFillerIfc filler){
		this(filler, false, Demand.NO_STATUS, 0, null, false, false);
	}
	
	public DemandMessage(DemandFillerIfc filler, boolean canFillItemTypeFlag, int status, int amount){
		this(filler, canFillItemTypeFlag, status, amount, null, false, true);
	}
	
	public DemandMessage(DemandFillerIfc filler, boolean canFillItemTypeFlag, int status, int amount, InventoryIfc inventory) {
		this(filler, canFillItemTypeFlag, status, amount, inventory, false, true);
	}
	
	public DemandMessage(DemandFillerIfc filler, boolean canFillItemTypeFlag, int status, int requestFillAmount, InventoryIfc inventory, 
			boolean mayPartialFillItemTypeFlag, boolean mayBackLogDemandsFlag) {

		if (filler == null)
			throw new IllegalArgumentException("The supplied filler was null");
		
		if (requestFillAmount < 0)
			throw new IllegalArgumentException("The request fill amount was < 0");
		
		myTimeStamp = Model.getTime();
		myFiller = filler;
		myCanFillItemTypeFlag = canFillItemTypeFlag;
		myStatus = status;
		myRequestFillAmount = requestFillAmount;
		myInventory = inventory;
		myMayPartialFillItemTypeFlag = mayPartialFillItemTypeFlag;
		myMayBackLogDemandsFlag = mayBackLogDemandsFlag;
	}
	
	public final DemandFillerIfc getDemandFiller(){
		return(myFiller);
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#getRequestStatus()
	 */
	public final int getRequestStatus() {
		return myStatus;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#getRequestFillAmount()
	 */
	public final int getRequestFillAmount() {
		return myRequestFillAmount;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#getInventory()
	 */
	public final InventoryIfc getInventory() {
		return myInventory;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#canFillItemType()
	 */
	public final boolean canFillItemType() {
		return myCanFillItemTypeFlag;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#mayBackLogDemands()
	 */
	public final boolean mayBackLogDemands() {
		return myMayBackLogDemandsFlag;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#mayPartiallyFillDemands()
	 */
	public final boolean mayPartiallyFillDemands() {
		return myMayPartialFillItemTypeFlag;
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandMessageIfc#getTimeStamp()
	 */
	public final double getTimeStamp() {
		return myTimeStamp;
	}

}
