/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.modeling.IllegalStateException;

/**
 *
 */
public class DemandRejectionListener implements DemandListenerRejectedIfc {

	public void demandRejected(Demand demand){
		int status = demand.getStatus();
		if (status == Demand.FILLER_UNAVAILABLE)
			fillerUnavailable(demand);
		else if (status == Demand.ITEMTYPE_MISMATCH)
			itemTypeMisMatch(demand);
		else if (status == Demand.NBLD_SENT_TO_BLR_NIF)
			NBLDSentToBLRNoImmediateFill(demand);
		else if (status == Demand.BLD_SENT_TO_NBLR_NIF)
			BLDSentToNBLRNoImmediateFill(demand);
		else if (status == Demand.NBLD_SENT_TO_NBLR_NIF)
			NBLDSentToNBLRNoImmediateFill(demand);	
		else if (status == Demand.ORDER_REJECTED)
			orderRejected(demand);
	}

	protected void orderRejected(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: The order that it was placed on was rejected.");								
	}

	protected void NBLDSentToNBLRNoImmediateFill(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: No backlogging demand sent to no backlogging filler with no immediate fill.");								
	}

	protected void BLDSentToNBLRNoImmediateFill(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: backlogging demand sent to no backlogging filler with no immediate fill.");										
	}

	protected void NBLDSentToBLRNoImmediateFill(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: No backlogging demand sent to backlogging filler with no immediate fill.");										
	}

	protected void itemTypeMisMatch(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: Item type mismatch between sender and filler. " + demand);								
	}

	protected void fillerUnavailable(Demand demand) {
		throw new IllegalStateException("Replenishment demand was rejected: The filler was unavailable.");										
	}
	
}
