/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.utilities.IdentityIfc;

/** DemandSenderIfc's make demand requests on DemandFillIfc's
 * 
 *
 */
public interface DemandSenderIfc extends IdentityIfc {
	 
	/** Indicates whether or not the sender may send demand requests for the given type
	 * 
	 * @param type
	 * @return
	 */
	public boolean mightRequest(ItemType type);
	
	/** The supplied finder is responsible for finding an appropriate filler
	 *  so that the generator's demand requests can be fulfilled.
	 * 
	 * @param finder Must not be null
	 */
	public void setDemandFillerFinder(DemandFillerFinderIfc finder);
	
	/**  Allows access the generator's filler finder
	 * 
	 * @return
	 */
	public DemandFillerFinderIfc getDemandFillerFinder();
	
	/** Can be used to directly set the filler for this generator, rather than relying
	 *  on the filler finder
	 * 
	 * @param fillerFinder Must not be null
	 */
	public void setDemandFiller(DemandFillerIfc filler);

	/**  Allows access the generator's filler
	 * 
	 * @return
	 */
	public DemandFillerIfc getDemandFiller();

}
