/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.modeling.IllegalStateException;

public class DemandState {
	
	protected String myStateName = null;
	
	protected DemandState(String name){	
		myStateName = name;
	}
	
	protected void setFiller(Demand d, DemandFillerIfc filler){
		throw new IllegalStateException("Tried to set the filler of the demand from state " + myStateName);								
	}
	
	protected void setSender(Demand d, DemandSenderIfc sender){
		throw new IllegalStateException("Tried to set the sender of the demand from state " + myStateName);						
	}
	
	protected void setAllowPartialFillingFlag(Demand d, boolean allowPartialFillingFlag) {
		throw new IllegalStateException("Tried to set the allow partial filling flag of the demand from state " + myStateName);		
	}

	protected void setAllowCancellingFlag(Demand d, boolean flag){
		throw new IllegalStateException("Tried to set the allow cancel flag of the demand from state " + myStateName);		
	}

	protected void setAllowBackLoggingFlag(Demand d, boolean allowBackLoggingFlag) {
		throw new IllegalStateException("Tried to set the allow backloggin of the demand from state " + myStateName);		
	}

	protected void setStatus(Demand d, int status){
		throw new IllegalStateException("Tried to set the status of the demand from state " + myStateName);						
	}

	protected void prepare(Demand d){
		throw new IllegalStateException("Tried to reopen the demand from state " + myStateName);						
	}

	protected void negotiate(Demand d){
		throw new IllegalStateException("Tried to negotiate the demand from state " + myStateName);					
	}

	protected void sent(Demand d) {
		throw new IllegalStateException("Tried to send the demand from state " + myStateName);						
	}
	
	protected void receive(Demand d) {
		throw new IllegalStateException("Tried to send the demand from state " + myStateName);						
	}

	protected void reject(Demand d){
		throw new IllegalStateException("Tried to reject the demand from state " + myStateName);						
	}
	
	protected void process(Demand d) {
		throw new IllegalStateException("Tried to process the demand from state " + myStateName);						
	}
	
	protected void cancel(Demand d){
		throw new IllegalStateException("Tried to cancel the demand from state " + myStateName);						
	}	
	
	protected void fill(Demand d, int amtSupplied) {
		throw new IllegalStateException("Tried to fill demand from state " + myStateName);						
	}		

	protected void backlog(Demand d) {
		throw new IllegalStateException("Tried to backlog the demand from state " + myStateName);						
	}		

	protected void ship(Demand d){
		throw new IllegalStateException("Tried to ship the demand from state " + myStateName);						
	}

	protected void deliver(Demand d){
		throw new IllegalStateException("Tried to deliver the demand from state " + myStateName);						
	}
	
	public final String getName(){
		return(myStateName);
	}
	
	public String toString(){
		return(myStateName);
	}
}