/*
 * Created on Jan 30, 2007
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package jslx.supplychain.inventorylayer;

import java.util.HashMap;
import java.util.Map;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.variable.RandomVariable;
import jsl.utilities.random.distributions.DistributionIfc;

/**
 * @author vvarghe
 *
 * It encapsulates the features and behavious of an inventory point that can
 * stock and repair
 * 
 */
public class InventoryRepairingHoldingPoint extends InventoryHoldingPoint {

	/** A Map to hold the inventory
	 *  key = ItemType, object = RandomVariable
	 */	
	protected Map<ItemType, RandomVariable> myRepairProbability;
	
	/** 
	 * models the local repair station
	 * 
	 */	
	private LeadTimeDemandFiller myLocalRepairingPoint;
	
	/**
	 * @param parent
	 */
	public InventoryRepairingHoldingPoint(ModelElement parent) {
		this(parent, true, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public InventoryRepairingHoldingPoint(ModelElement parent, String name) {
		this(parent, true, name);
	}

	/**
	 * @param parent
	 * @param initialAvailability
	 * @param name
	 */
	public InventoryRepairingHoldingPoint(ModelElement parent,
			boolean initialAvailability, String name) {
		super(parent, initialAvailability, name);
		myRepairProbability = new HashMap<ItemType, RandomVariable>();
		myLocalRepairingPoint = new LeadTimeDemandFiller(this, name + "LocalRepairingPoint");
		setReplenishmentRequester(new IHPReplenishmentRequester());
	}

	/**
	 * @param type
	 * @param repairprob
	 * @param repairtime
	 * 
	 * sets the repair probability at the local repair station
	 * and the repair delay
	 * 
	 */
	public void setRepairStation(ItemType type, DistributionIfc repairprob, DistributionIfc repairtime){
		if (type == null)
			throw new IllegalArgumentException("The ItemType type must be non-null.");

		if (repairprob == null)
			throw new IllegalArgumentException("The repair prob distribution must be non-null.");
		
		if (repairtime == null)
			throw new IllegalArgumentException("The repair time distribution must be non-null.");
		
		if (myRepairProbability.containsKey(type)){ // item type has already been added
			// need only to set the distribution associated with the type
			RandomVariable rv = myRepairProbability.get(type);
			rv.setInitialRandomSource(repairprob);			
		} else {
			// new item type being entered
			// create the random variable
			RandomVariable rv = new RandomVariable(this, repairprob);
			myRepairProbability.put(type, rv);
		}
		
		myLocalRepairingPoint.addLeadTime(type,repairtime);	
		
	}
	
	
	
	/** Used by all the internal inventory within the holder to send
	 *  demand (replenishment) requests to the outside
	 *
	 */
	private final class IHPReplenishmentRequester implements ReplenishmentRequesterIfc {
		public void requestReplenishment(Inventory inventory, Demand demand) {
			ItemType type = demand.getItemType();
			RandomVariable probrv = myRepairProbability.get(type);
			double occur = probrv.getValue();			
			if(occur < 1.0){
				demand.setSender(InventoryRepairingHoldingPoint.this);		
				// if the filler is set, use it, otherwise try to find a filler using the filler finder
				DemandFillerIfc filler;
				
				filler = InventoryRepairingHoldingPoint.this.getDemandFiller();
				
				// set the filler on the demand so it uses the one suggested by the IHP
				demand.setFiller(filler);
				//set the demand listeners, have the internal inventory supply the listeners
				inventory.setDemandListeners(demand);
				// place the demand in the SENT state
				demand.sent();
				
				demand.getFiller().receive(demand);				
			}else{
				demand.setSender(InventoryRepairingHoldingPoint.this);		
				demand.setFiller(myLocalRepairingPoint);
				
				inventory.setDemandListeners(demand);
				// place the demand in the SENT state
				demand.sent();				
				demand.getFiller().receive(demand);				
			}						
		}			
	}
	
	public double getRepairProbabilityParameters(ItemType type) {
		return myRepairProbability.get(type).getParameters()[0];
	}

	public double getRepairingDelay(ItemType type) {
		return myLocalRepairingPoint.getLeadTime(type).getParameters()[0];
	}	
	
}
