/*
* Created on Mar 7, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import jsl.modeling.Model;

/** An OrderMessage consists of a list of DemandMessageIfc object, the intended OrderFillerIfc,
 *  whether or not all the item types in the order can be filled by the filler, and a time
 *  stamp for when the message was created. 
 *  
 *  NOTE: The DemandMessageIfc object may be null in the list (meaning no information was available and
 *  that the item type cannot be filled).
 *
 */
public class OrderMessage implements OrderMessageIfc {

	private List<DemandMessageIfc> myDemandMessages;
	
	private boolean myCanFillItemTypesFlag = true;
	
	private OrderFillerIfc myOrderFiller;

	private double myTimeStamp;

	/**
	 * @param orderFiller
	 * @param canFillItemTypesFlag
	 */
	public OrderMessage(OrderFillerIfc orderFiller) {
		myDemandMessages = new ArrayList<DemandMessageIfc>();
		myOrderFiller = orderFiller;
		myTimeStamp = Model.getTime();
	}
	
	/**
	 * @param arg0
	 * @return
	 * @see java.util.List#add(java.lang.Object)
	 */
	public boolean add(DemandMessageIfc message) {
		if (!message.canFillItemType() || (message == null))
			myCanFillItemTypesFlag = false;
		return myDemandMessages.add(message);
	}

	/**
	 * @param arg0
	 * @return
	 * @see java.util.List#contains(java.lang.Object)
	 */
	public boolean contains(Object arg0) {
		return myDemandMessages.contains(arg0);
	}

	/**
	 * @param arg0
	 * @return
	 * @see java.util.List#get(int)
	 */
	public DemandMessageIfc get(int arg0) {
		return myDemandMessages.get(arg0);
	}

	/**
	 * @return
	 * @see java.util.List#isEmpty()
	 */
	public boolean isEmpty() {
		return myDemandMessages.isEmpty();
	}

	/**
	 * @return
	 * @see java.util.List#iterator()
	 */
	public Iterator<DemandMessageIfc> iterator() {
		return myDemandMessages.iterator();
	}

	/**
	 * @return
	 * @see java.util.List#size()
	 */
	public int size() {
		return myDemandMessages.size();
	}

	/* (non-Javadoc)
	 * @see jsl.research.supplychain.facilitylayer.OrderMessageIfc#canFillItemTypes()
	 */
	public final boolean canFillItemTypes() {
		if (isEmpty())
			return false;
		else
			return myCanFillItemTypesFlag;
	}

	/* (non-Javadoc)
	 * @see jsl.research.supplychain.facilitylayer.OrderMessageIfc#getDemandMessages()
	 */
	public final List<DemandMessageIfc> getDemandMessages() {
		return myDemandMessages;
	}

	/* (non-Javadoc)
	 * @see jsl.research.supplychain.facilitylayer.OrderMessageIfc#getOrderFiller()
	 */
	public final OrderFillerIfc getOrderFiller() {
		return myOrderFiller;
	}

	/* (non-Javadoc)
	 * @see jsl.research.supplychain.facilitylayer.OrderMessageIfc#getTimeStamp()
	 */
	public final double getTimeStamp() {
		return myTimeStamp;
	}

}
