/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer;

import jsl.modeling.IllegalStateException;

class OrderState {
	
	protected String myStateName = null;

	protected OrderState(String name){
		myStateName = name;		
	}

	protected void setFiller(Order order, OrderFillerIfc filler){
		throw new IllegalStateException("Tried to set the filler on the order when in state: " + myStateName);
	}
	
	protected void setPartialShippingFlag(Order order, boolean flag){
		throw new IllegalStateException("Tried to allow partial shipping when in state: " + myStateName);			
	}

	protected void setAllowBackLoggingFlag(Order order, boolean flag){
		throw new IllegalStateException("Tried to allow backlogging when in state: " + myStateName);			
	}
	
	protected void setAllowCancellingFlag(Order order, boolean flag){
		throw new IllegalStateException("Tried to allow order cancelling when in state: " + myStateName);			
	}
	
	protected void setStatus(Order order, int status){
		throw new IllegalStateException("Tried set the order status when in state: " + myStateName);			
	}

	protected void addDemand(Order order, Demand demand){
		throw new IllegalStateException("Tried to add demand to the order when in state: " + myStateName);						
	}

	protected void removeDemand(Order order, Demand demand){
		throw new IllegalStateException("Tried to remove demand from the order when in state: " + myStateName);						
	}

	protected void sent(Order order){
		throw new IllegalStateException("Tried to place the order in the SENT state when in state: " + myStateName);						
	}
	
	protected void receive(Order order){
		throw new IllegalStateException("Tried to place the order in the RECEIVED state when in state: " + myStateName);						
	}

	protected void prepare(Order order){
		throw new IllegalStateException("Tried to place the order in the IN_PREPARATION state when in state: " + myStateName);						
	}

	protected void process(Order order){
		throw new IllegalStateException("Tried to place the order in the IN_PROCESS state when in state: " + myStateName);						
	}

	protected void negotiate(Order order){
		throw new IllegalStateException("Tried to place the order in the NEGOTIATING state when in state: " + myStateName);						
	}
	
	protected void reject(Order order){
		throw new IllegalStateException("Tried to place the order in the REJECTED state when in state: " + myStateName);						
	}
	
	protected void cancel(Order order){
		throw new IllegalStateException("Tried to place the order in the CANCELLED state when in state: " + myStateName);						
	}	
	
	protected void ship(Order order){
		throw new IllegalStateException("Tried to place the order in the SHIPPED state when in state: " + myStateName);						
	}
	
	protected void deliver(Order order){
		throw new IllegalStateException("Tried to place the order in the DELIVERED state when in state: " + myStateName);						
	}
	
	protected void backlog(Order order){
		throw new IllegalStateException("Tried to place the order in the BACKLOGGED state when in state: " + myStateName);						
	}

	protected void fill(Order order){
		throw new IllegalStateException("Tried to place the order in the FILLED state when in state: " + myStateName);						
	}

	protected void beginReceivingDemands(Order order){
		throw new IllegalStateException("Tried to begin receiving the demands when in state: " + myStateName);						
	}
	
	protected void beginProcessingDemands(Order order){
		throw new IllegalStateException("Tried to begin processing the demands when in state: " + myStateName);						
	}

	protected void setDemandStateToShipped(Order order){
		throw new IllegalStateException("Tried to set the demand's state to shipped on the order when in state: " + myStateName);						
	}

	protected void setDemandStateToDelivered(Order order){
		throw new IllegalStateException("Tried to set the demand's state to delivered on the order when in state: " + myStateName);						
	}

	public final String getName(){
		return(myStateName);
	}
	
	public String toString(){
		return(myStateName);
	}
}