/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer.testing;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.reporting.JSL;

/**
 */
public class CopyOfTestRQInventoryModel {

	public static void main(String[] args) {

		System.out.println("RQInventoryModel Test");
        
//		JSL.LOGGER.setLevel(Level.INFO);
   
//		testReplication();
		testExperiment();
        
		System.out.println("Done!");		
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		DistributionIfc tbd = new Exponential(1.0/14.0);
		DistributionIfc lt = new Constant(45.0/365.0);
		int rpt = 3;
		int qty = 4;
		int level = rpt + qty;
		RQInventoryModel rq = new RQInventoryModel(m, tbd, lt, rpt, qty, level);
		
//		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(30);
		e.setLengthOfReplication(15.0);
		e.setLengthOfWarmUp(5.0);
//		e.setNumberOfReplications(1);
//		e.setLengthOfReplication(10.0);

		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport("RQInventoryModel");
		e.turnOnExperimentConsoleOutput();
//		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}

}
