/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer.testing;

import jsl.modeling.ModelElement;

import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.supplychain.inventorylayer.*;
import jslx.supplychain.transportlayer.TimeBasedDemandCarrier;

public class MEModel extends ModelElement {

	public MEModel(ModelElement parent) {
		this(parent,null);
	}

	public MEModel(ModelElement parent, String name) {
		super(parent, name);
		
		
		//Define item types
		ItemType type1=new ItemType(this,"Type-1");
		ItemType type2=new ItemType(this,"Type-2");
		ItemType type3=new ItemType(this,"Type-3");
		ItemType type4=new ItemType(this,"Type-4");
		
		//Define external supplier
		LeadTimeDemandFiller myLeadTimeDemandFiller=new LeadTimeDemandFiller(this);
//		 Leadtime from factory to warehouse
		DistributionIfc LT_F_To_W=new Constant (1.0);
		
		//Add LeadTime from factory to warehouse
		myLeadTimeDemandFiller.addLeadTime( type1,LT_F_To_W);
		myLeadTimeDemandFiller.addLeadTime( type2,LT_F_To_W);
		myLeadTimeDemandFiller.addLeadTime( type3,LT_F_To_W);
		myLeadTimeDemandFiller.addLeadTime( type4,LT_F_To_W);
		
//		Create an object or DemandFillerFinderForDG which finds demand filler for demand generators
		DemandFillerFinderForDG demandFillerFinderForDG=new DemandFillerFinderForDG();
		
		//Define Warehouses
		
		InventoryHoldingPoint warehouse=new InventoryHoldingPoint (this, "warehouse");
		warehouse.setDemandFillerFinder(demandFillerFinderForDG);
		
		//		Define  and add shipper for warehouse
		TimeBasedDemandCarrier myWarehouseShipper=new TimeBasedDemandCarrier(this);
		warehouse.setDemandCarrier(myWarehouseShipper);
		demandFillerFinderForDG.addFillerForDG(warehouse,myLeadTimeDemandFiller);
		
		
		
		// Create inventory policy at warehouse levels
		
		InventoryPolicyAbstract InventoryPolicy_W1 = new InventoryPolicyReorderPointReorderQuantity(this, 4, 1);
		Inventory Inventory_w1 = new Inventory(this, type1, InventoryPolicy_W1, 20);
		BackLogQueue bpolicy_w1 = new BackLogQueue(Inventory_w1, " Item 1 in W");
		
		InventoryPolicyAbstract InventoryPolicy_W2 = new InventoryPolicyReorderPointReorderQuantity(this, 4, 1);
		Inventory Inventory_w2 = new Inventory(this, type2, InventoryPolicy_W2, 20);
		BackLogQueue bpolicy_w2 = new BackLogQueue(Inventory_w2, " Item 2 in W");
		
		InventoryPolicyAbstract InventoryPolicy_W3 = new InventoryPolicyReorderPointReorderQuantity(this, 4, 1);
		Inventory Inventory_w3 = new Inventory(this, type3, InventoryPolicy_W3, 20);
		BackLogQueue bpolicy_w3 = new BackLogQueue(Inventory_w3, " Item 3 in W");
		
		//Add the inventory to warehouse
		warehouse.addInventory( Inventory_w1);
		warehouse.addInventory( Inventory_w2);
		warehouse.addInventory( Inventory_w3);

		// Create the retailers which use the warehouse as its filler
		
		InventoryHoldingPoint retailer1=new InventoryHoldingPoint(this,"R1");
		retailer1.setDemandFillerFinder(demandFillerFinderForDG);
		
		InventoryHoldingPoint retailer2=new InventoryHoldingPoint(this,"R2");
		retailer2.setDemandFillerFinder(demandFillerFinderForDG);
		
		InventoryHoldingPoint retailer3=new InventoryHoldingPoint(this,"R3");
		retailer3.setDemandFillerFinder(demandFillerFinderForDG);
		
		InventoryHoldingPoint retailer4=new InventoryHoldingPoint(this,"R4");
		retailer4.setDemandFillerFinder(demandFillerFinderForDG);
				
		// set warehouse as filler for retailer
		demandFillerFinderForDG.addFillerForDG(retailer1,warehouse);
		demandFillerFinderForDG.addFillerForDG(retailer2,warehouse);
		demandFillerFinderForDG.addFillerForDG(retailer3,warehouse);
		demandFillerFinderForDG.addFillerForDG(retailer4,warehouse);
		
//		Set ship time
//		 The warehouse must use a shipper to ship to its customers
		DistributionIfc ltA1 = new Constant(1.0);
		myWarehouseShipper.setTransportTime(retailer1,ltA1);
		myWarehouseShipper.setTransportTime(retailer2,ltA1);
		myWarehouseShipper.setTransportTime(retailer3,ltA1);
		myWarehouseShipper.setTransportTime(retailer4,ltA1);

		//Create the inventory policy at each retailer

		InventoryPolicyAbstract InventoryPolicy_11 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_11 = new Inventory(this, type1, InventoryPolicy_11, 10);
		BackLogQueue bpolicy_11 = new BackLogQueue(Inventory_11, " Item 1 in R1");
		
		InventoryPolicyAbstract InventoryPolicy_12 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_12 = new Inventory(this, type2, InventoryPolicy_12, 10);
		BackLogQueue bpolicy_12 = new BackLogQueue(Inventory_12, " Item 2 in R1");
		
		InventoryPolicyAbstract InventoryPolicy_21 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_21 = new Inventory(this, type1, InventoryPolicy_21, 10);
		BackLogQueue bpolicy_21 = new BackLogQueue(Inventory_21, " Item 1 in R2");
		
		InventoryPolicyAbstract InventoryPolicy_22 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_22 = new Inventory(this, type2, InventoryPolicy_22, 10);
		BackLogQueue bpolicy_22 = new BackLogQueue(Inventory_22, " Item 2 in R2");
		
		InventoryPolicyAbstract InventoryPolicy_23 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_23 = new Inventory(this, type3, InventoryPolicy_23, 10);
		BackLogQueue bpolicy_23 = new BackLogQueue(Inventory_23, " Item 3 in R2");
		
		InventoryPolicyAbstract InventoryPolicy_31 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_31 = new Inventory(this, type1, InventoryPolicy_31, 10);
		BackLogQueue bpolicy_31 = new BackLogQueue(Inventory_31, " Item 1 in R3");
		
		InventoryPolicyAbstract InventoryPolicy_32 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_32 = new Inventory(this, type2, InventoryPolicy_32, 10);
		BackLogQueue bpolicy_32 = new BackLogQueue(Inventory_32, " Item 2 in R3");

		InventoryPolicyAbstract InventoryPolicy_41 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_41 = new Inventory(this, type1, InventoryPolicy_41, 10);
		BackLogQueue bpolicy_41 = new BackLogQueue(Inventory_41, " Item 1 in R4");
		
		InventoryPolicyAbstract InventoryPolicy_42 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_42 = new Inventory(this, type2, InventoryPolicy_42, 10);
		BackLogQueue bpolicy_42 = new BackLogQueue(Inventory_42, " Item 2 in R4");
		
		InventoryPolicyAbstract InventoryPolicy_43 = new InventoryPolicyReorderPointReorderQuantity(this, 2, 1);
		Inventory Inventory_43 = new Inventory(this, type3, InventoryPolicy_43, 10);
		BackLogQueue bpolicy_43 = new BackLogQueue(Inventory_43, " Item 3 in R4");
		
		retailer1.addInventory( Inventory_11);
		retailer1.addInventory( Inventory_12);
		retailer2.addInventory( Inventory_21);
		retailer2.addInventory( Inventory_22);
		retailer2.addInventory( Inventory_23);
		retailer3.addInventory( Inventory_31);
		retailer3.addInventory( Inventory_32);
		retailer4.addInventory( Inventory_41);
		retailer4.addInventory( Inventory_42);
		retailer4.addInventory( Inventory_43);
		
		//		 Here we create the distibutions of the daily demand
		// of each item at each retailer location
		// Define the demand generator
						
		DistributionIfc dA11 = new Exponential(1.0);				
		DemandGenerator dgA11 = new DemandGenerator(this,type1, dA11, dA11);
		dgA11.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG(dgA11,retailer1);
		
		
		DistributionIfc dA12 = new Exponential(1.0);				
		DemandGenerator dgA12 = new DemandGenerator(this,type2, dA12, dA12);
		dgA12.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG(dgA12,retailer1);
		
		DistributionIfc dA21 = new Exponential(1.0);				
		DemandGenerator dgA21 = new DemandGenerator(this,type1, dA21, dA21);
		dgA21.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG(dgA21,retailer2);	
		
		DistributionIfc dA22 = new Exponential(1.0);				
		DemandGenerator dgA22 = new DemandGenerator(this,type2, dA22, dA22);
		dgA22.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG( dgA22,retailer2);
		
		DistributionIfc dA23 = new Exponential(1.0);				
		DemandGenerator dgA23 = new DemandGenerator(this,type3, dA23, dA23);
		dgA23.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG( dgA23,retailer2);

		DistributionIfc dA31 = new Exponential(1.0);				
		DemandGenerator dgA31 = new DemandGenerator(this,type1, dA31, dA31);
		dgA31.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG( dgA31,retailer3);
		
		DistributionIfc dA32 = new Exponential(1.0);				
		DemandGenerator dgA32 = new DemandGenerator(this,type2, dA32, dA32);
		dgA32.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG (dgA32,retailer3);	

		DistributionIfc dA41 = new Exponential(1.0);				
		DemandGenerator dgA41 = new DemandGenerator(this,type1, dA41, dA41);
		dgA41.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG (dgA41,retailer4);
		
		DistributionIfc dA42 = new Exponential(1.0);				
		DemandGenerator dgA42 = new DemandGenerator(this,type2, dA42, dA42);
		dgA42.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG (dgA42,retailer4);	
		
		DistributionIfc dA43 = new Exponential(1.0);				
		DemandGenerator dgA43 = new DemandGenerator(this,type3, dA43, dA43);
		dgA43.setDemandFillerFinder(demandFillerFinderForDG);
		demandFillerFinderForDG.addFillerForDG (dgA43,retailer4);

		
	}

}
