/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer.testing;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jslx.supplychain.inventorylayer.*;

/**
 */
public class RQInventoryModel extends ModelElement implements DemandFillerFinderIfc {

	private ItemType myItemType;
	private DemandGenerator myDemandGenerator;
	private Inventory myInventory;
    private InventoryPolicyAbstract myInventoryPolicy;
    private LeadTimeDemandFiller myLeadTimeDemandFiller;
    
	/**
	 * @param parent
	 */
	public RQInventoryModel(ModelElement parent, DistributionIfc timebtwDemand, 
						    DistributionIfc leadTime, int reorderPt, int reorderQty, int initialLevel) {
		this(parent, timebtwDemand, leadTime, reorderPt, reorderQty, initialLevel, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public RQInventoryModel(ModelElement parent, DistributionIfc timebtwDemand, DistributionIfc leadTime, 
	                        int reorderPt, int reorderQty, int initialLevel, String name) {
		super(parent, name);

		// define the type
		myItemType = new ItemType(this, "Type-A");
		
		// define the filler
		myLeadTimeDemandFiller = new LeadTimeDemandFiller(this);
		myLeadTimeDemandFiller.addLeadTime(myItemType,leadTime);
		
		myInventoryPolicy = new InventoryPolicyReorderPointReorderQuantity(this, reorderPt, reorderQty);
		myInventory = new Inventory(this, myItemType, myInventoryPolicy, initialLevel);
	    myInventory.setDemandFillerFinder(this);
		BackLogQueue bpolicy = new BackLogQueue(myInventory);
		
		// define the demand generator
		myDemandGenerator = new DemandGenerator(this, myItemType, timebtwDemand, timebtwDemand);
		myDemandGenerator.setDemandFillerFinder(this);
	}

	/* (non-Javadoc)
	 * @see jsl.supplychain.inventorylayer.DemandFillerFinderIfc#findDemandFiller(jsl.supplychain.inventorylayer.DemandSenderIfc, jsl.supplychain.inventorylayer.ItemType, int)
	 */
	public DemandFillerIfc findDemandFiller(Demand demand) {
		DemandSenderIfc sender = demand.getSender();
		if (sender == myDemandGenerator)
			return(myInventory);
		else
			return(myLeadTimeDemandFiller);

	}

}
