/*
* Created on Feb 1, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.inventorylayer.testing;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jslx.supplychain.inventorylayer.ItemType;
import jslx.supplychain.inventorylayer.LeadTimeOrderFiller;
import jslx.supplychain.inventorylayer.OrderGenerator;
import jslx.supplychain.inventorylayer.RandomOrderCreator;

/**
 *
 */
public class TestLeadTimeOrderFiller {

	public static void main(String[] args) {
		
		Model m = Model.createModel("Test LeadTimeOrderFiller");
		
		//Define item types
		ItemType type1=new ItemType(m,"Type-1");
		ItemType type2=new ItemType(m,"Type-2");
		ItemType type3=new ItemType(m,"Type-3");
		ItemType type4=new ItemType(m,"Type-4");
		
		DistributionIfc ltd = new Constant(2.0);
		LeadTimeOrderFiller ltof = new LeadTimeOrderFiller(m, "LT Order filler");
		ltof.addLeadTime(type1, ltd);
		ltof.addLeadTime(type2, ltd);
		ltof.addLeadTime(type3, ltd);
		ltof.addLeadTime(type4, ltd);

		RandomOrderCreator roc = new RandomOrderCreator(m, "OrderCreator");
		roc.addItemTypeDistribution(type1);
		roc.addItemTypeDistribution(type2);
		roc.addItemTypeDistribution(type3);
		roc.addItemTypeDistribution(type4);

		DistributionIfc tbo = Constant.ZERO;
		OrderGenerator og = new OrderGenerator(m,"OrderGenerator");
		og.setTimeBetweenEventsForReplications(tbo, 1);
		og.setOrderCreator(roc);
		og.setOrderFiller(ltof);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(1);
		e.setLengthOfReplication(200.0);
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();

	}
}
