/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.proliferationYC;

import jsl.modeling.ModelElement;
import jsl.utilities.random.RandomIfc;
import jslx.supplychain.inventorylayer.ItemType;

/** An item type that can be put in a category
 * 
 *
 */
public class CItemType extends ItemType {

	protected ItemCategoryAbstract myCategory;
	
	/**
	 * @param parent
	 */
	public CItemType(ModelElement parent) {
		this(parent, null, 1.0, 1.0, null);
	}
	
	/**
	 * @param parent
	 * @param name
	 */
	public CItemType(ModelElement parent, String name) {
		this(parent, name, 1.0, 1.0, null);
	}

	/**
	 * 
	 * @param parent
	 * @param weight
	 * @param cube
	 */
	public CItemType(ModelElement parent, double weight, double cube) {
		this(parent, null, weight, cube, null);
	}
	
	/**
	 * 
	 * @param parent
	 * @param name
	 * @param weight
	 * @param cube
	 */
	public CItemType(ModelElement parent, String name, double weight, double cube) {
		this(parent, name, weight, cube, null);
	}

	/**
	 * 
	 * @param parent
	 * @param name
	 * @param weight
	 * @param cube
	 */
	public CItemType(ModelElement parent, String name, double weight, double cube, RandomIfc leadTime) {
		super(parent, name);
		setWeight(weight);
		setCube(cube);
		setLeadTime(leadTime);
	}
	
	protected void setCategory(ItemCategoryAbstract category){
		myCategory = category;
	}
	
	public ItemCategoryAbstract getCategory(){
		return myCategory;
	}
	
	protected void removedFromModel(){
		super.removedFromModel();
		myCategory = null;
	}

}
