/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.proliferationYC;

import jsl.observers.textfile.*;
import java.io.IOException;
import java.util.List;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.elements.variable.Counter;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.observers.variable.CounterObserver;
import jsl.observers.variable.StatisticalObserver;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 * 
 */
public class CSVExperimentReport_YC extends CSVReport {

	/**
	 * @param name
	 * @throws IOException
	 */
	public CSVExperimentReport_YC(String name) throws IOException {
		this(null, name);
	}

	/**
	 * @param directory
	 * @param name
	 * @throws IOException
	 */
	public CSVExperimentReport_YC(String directory, String name)
			throws IOException {
		super(directory, name);
	}

	protected void writeHeader() {
		if (myHeaderFlag == true)
			return;
		myHeaderFlag = true;

		StringBuilder sb = new StringBuilder();
		//sb.append("\"ResponseID\",");
		//sb.append("ExpNum,");
		//sb.append("ExpName,");
		sb.append("Name,");
		//sb.append("Count,");
		sb.append("Average,");
		//sb.append("Standard Deviation,");
		//sb.append("Standard Error,");
		sb.append("Half-width,");     
		//sb.append("Confidence Coefficient,");  
		//sb.append("Minimum,");
		//sb.append("Maximum,");
		//sb.append("Sum,");
		//sb.append("Variance,");
		//sb.append("Weighted Average,");
		//sb.append("Weighted Sum,");
		//sb.append("Sum of Weights,");
		//sb.append("Weighted Sum of Squares,");
		//sb.append("Deviation Sum of Squares,");
		//sb.append("Last value collected,");
		//sb.append("Last weighted collected,");
		//sb.append("Kurtosis,");
		//sb.append("Skewness,");
		//sb.append("Lag 1 Covariance,");
		//sb.append("Lag 1 Correlation,");
		//sb.append("Von Neumann Lag 1 Test Statistic,");
		//sb.append("Number of missing observations");
		myWriter.println(sb.toString());
	}
	
	protected void afterExperiment(ModelElement m, Object arg) {

    	Model model = m.getModel();
        Replication r = model.getReplication();
		Experiment e = r.getExperiment();
        
        List<ResponseVariable> rvs = model.getResponseVariables();
        List<Counter> counters = model.getCounters();
        
        for(ResponseVariable rv: rvs){     
        	if (rv.getDefaultReportingOption()){
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
                if(stat.getName().contains("Aggregate Fill Rate"))
    				writeRow(stat, e, (long) rv.getId());        		
        	}
        }
        for(ResponseVariable rv: rvs){     
        	if (rv.getDefaultReportingOption()){
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
                if(stat.getName().contains("Total On Hand"))
    				writeRow(stat, e, (long) rv.getId());        		
        	}
        }
        for(ResponseVariable rv: rvs){     
        	if (rv.getDefaultReportingOption()){
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
                if(stat.getName().contains("AmtBackordered"))
    				writeRow(stat, e, (long) rv.getId());        		
        	}
        }
        for(ResponseVariable rv: rvs){     
        	if (rv.getDefaultReportingOption()){
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
                if(stat.getName().contains("Aggregate Number of Stockouts"))
    				writeRow(stat, e, (long) rv.getId());        		
        	}
        }
        for(ResponseVariable rv: rvs){     
        	if (rv.getDefaultReportingOption()){
                StatisticalObserver so = rv.getStatisticalObserver();
                StatisticAccessorIfc stat = so.getAcrossReplicationStatistic();
                if(stat.getName().contains("Aggregate#ReplenishmentRequests"))
    				writeRow(stat, e, (long) rv.getId());        		
        	}
        }
        
        for(Counter c: counters){     
        	if (c.getDefaultReportingOption()){
                CounterObserver co = c.getCounterObserver();
                StatisticAccessorIfc stat = co.getAcrossReplicationStatistic();
                if(stat.getName().contains(" #Shipments"))
    				writeRow(stat, e, (long) c.getId());        		
        	}
        }
        
//		List<Counter> counters = model.getCounters();
//		
//		for(Counter c: counters){
//			if (c.getDefaultReportingOption()){
//				CounterObserver co = c.getCounterObserver();
//				StatisticAccessorIfc stat = co.getAcrossReplicationStatistic();
////				if (stat.getCount() > 1.0) // record only if more than one replication
//					writeRow(stat, e, (long) c.getId()); 
//				stat = co.getAcrossRepTimedUpdateStatistic();
//				if (stat != null)
//					writeRow(stat, e, (long) c.getId()); 				
//			}
//		}
	}
	
	public void writeRow(StatisticAccessorIfc stat, Experiment e, long id) {
	    
		String row;
	    
		if (e == null)
			row = makeRepRow(stat, id);
		else
			row = makeExpRow(stat, e, id);  
		
		myWriter.println(row);
	    
	}
	
	public String makeExpRow(StatisticAccessorIfc stat, Experiment e, long id) {
		StringBuilder sb = new StringBuilder();
		//sb.append(id);
		//sb.append(",");
		//sb.append(e.getId());
		//sb.append(",\"");
		//sb.append(e.getName());
		//sb.append("\",\"");
		sb.append(stat.getName());
		sb.append(",");
		//sb.append(stat.getCount());
		//sb.append(",");
		sb.append(stat.getAverage());
		sb.append(",");
		//sb.append(stat.getStandardDeviation());
		//sb.append(",");
		//sb.append(stat.getStandardError());
		//sb.append(",");
		sb.append(stat.getHalfWidth());
		//sb.append(",");
		//sb.append(stat.getConfidenceCoefficient());
		//sb.append(",");
		//sb.append(stat.getMin());
		//sb.append(",");
		//sb.append(stat.getMax());
		//sb.append(",");
		//sb.append(stat.getSum());
//		sb.append(",");
//		sb.append(stat.getVariance());
//		sb.append(",");
//		sb.append(stat.getWeightedAverage());
//		sb.append(",");
//		sb.append(stat.getWeightedSum());
//		sb.append(",");
//		sb.append(stat.getSumOfWeights());
//		sb.append(",");
//		sb.append(stat.getWeightedSumOfSquares());
//		sb.append(",");
//		sb.append(stat.getDeviationSumOfSquares());
//		sb.append(",");
//		sb.append(stat.getLastValue());
//		sb.append(",");
//		sb.append(stat.getLastWeight());
//		sb.append(",");
//		sb.append(stat.getKurtosis());
//		sb.append(",");
//		sb.append(stat.getSkewness());
//		sb.append(",");
//		sb.append(stat.getLag1Covariance());
//		sb.append(",");
//		sb.append(stat.getLag1Correlation());
//		sb.append(",");
//		sb.append(stat.getVonNeumannLag1TestStatistic());
//		sb.append(",");
//		sb.append(stat.getNumberMissing());
		return(sb.toString());		
	}

	public String makeRepRow(StatisticAccessorIfc stat, long id) {
		StringBuilder sb = new StringBuilder();
//		sb.append(id);
//		sb.append(",\"");
		sb.append(stat.getName());
		sb.append(",");
//		sb.append(stat.getCount());
//		sb.append(",");
		sb.append(stat.getAverage());
		sb.append(",");
//		sb.append(stat.getStandardDeviation());
//		sb.append(",");
//		sb.append(stat.getStandardError());
//		sb.append(",");
		sb.append(stat.getHalfWidth());
//		sb.append(",");
//		sb.append(stat.getConfidenceCoefficient());
//		sb.append(",");
//		sb.append(stat.getMin());
//		sb.append(",");
//		sb.append(stat.getMax());
//		sb.append(",");
//		sb.append(stat.getSum());
//		sb.append(",");
//		sb.append(stat.getVariance());
//		sb.append(",");
//		sb.append(stat.getWeightedAverage());
//		sb.append(",");
//		sb.append(stat.getWeightedSum());
//		sb.append(",");
//		sb.append(stat.getSumOfWeights());
//		sb.append(",");
//		sb.append(stat.getWeightedSumOfSquares());
//		sb.append(",");
//		sb.append(stat.getDeviationSumOfSquares());
//		sb.append(",");
//		sb.append(stat.getLastValue());
//		sb.append(",");
//		sb.append(stat.getLastWeight());
//		sb.append(",");
//		sb.append(stat.getKurtosis());
//		sb.append(",");
//		sb.append(stat.getSkewness());
//		sb.append(",");
//		sb.append(stat.getLag1Covariance());
//		sb.append(",");
//		sb.append(stat.getLag1Correlation());
//		sb.append(",");
//		sb.append(stat.getVonNeumannLag1TestStatistic());
//		sb.append(",");
//		sb.append(stat.getNumberMissing());
		return(sb.toString());			
	}
}
