/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.proliferationYC;

import java.io.IOException;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReplicationReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Triangular;
import jslx.supplychain.inventorylayer.*;
import jslx.supplychain.multiechelon.*;
import jslx.supplychain.transportlayer.DemandLoadBuilder;
/**
 * @author rossetti
 *
 */
public class ProliferationTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Starting experiment.");
		Model m = buildModel();
		
		try{
        	CSVReport rcsv = new CSVReplicationReport("Replication");
        	CSVReport bcsv = new CSVExperimentReport("Experiment");
        	m.addObserver(rcsv);
        	m.addObserver(bcsv);
        } catch(IOException e){
			System.out.println("I/O exception opening file Output.txt");
			e.printStackTrace();
        }
        
		runExperiment(m);
		System.out.println("Done!");
	}
	
	public static Model buildModel(){
		
		Model m = Model.createModel();
		
		OneWarehouseNRetailersWithLoadBuildingCarrier network = new OneWarehouseNRetailersWithLoadBuildingCarrier(m, "network");
		
/**     // to add a constant transport time consider this
 *      ItemType Type0Brand0 = network.addItemType("Type0Brand0", new ShiftedDistribution(new Triangular(7,11,15), 8));
 */		
		
		ItemType Type0Brand0 = network.addItemType("Type0Brand0", new Triangular(7,11,15));
		ItemType Type1Brand0 = network.addItemType("Type1Brand0", new Triangular(7,11,15));
		ItemType Type2Brand0 = network.addItemType("Type2Brand0", new Triangular(7,11,15));
		ItemType Type3Brand0 = network.addItemType("Type3Brand0", new Triangular(7,11,15));
		ItemType Type4Brand0 = network.addItemType("Type4Brand0", new Triangular(7,11,15));
		ItemType Type5Brand0 = network.addItemType("Type5Brand0", new Triangular(7,11,15));
		
		InventoryHoldingPoint SSU_AR = network.addRetailer("SSU_AR", new Constant(3.3));
		InventoryHoldingPoint SSU_MO = network.addRetailer("SSU_MO", new Constant(3.4));
		InventoryHoldingPoint SSU_OK = network.addRetailer("SSU_OK", new Constant(4.5));
		InventoryHoldingPoint SSU_KS = network.addRetailer("SSU_KS", new Constant(2.0));

		network.setDefaultLoadFormingOption(SSU_AR, DemandLoadBuilder.LoadFormingOption.CUBE);
		network.setDefaultLoadFormingOption(SSU_MO, DemandLoadBuilder.LoadFormingOption.CUBE);
		network.setDefaultLoadFormingOption(SSU_OK, DemandLoadBuilder.LoadFormingOption.CUBE);
		network.setDefaultLoadFormingOption(SSU_KS, DemandLoadBuilder.LoadFormingOption.CUBE);
		
		network.setCubeFormingLimits(SSU_AR, 50, 200);
		network.setCubeFormingLimits(SSU_MO, 50, 200);
		network.setCubeFormingLimits(SSU_OK, 50, 200);
		network.setCubeFormingLimits(SSU_KS, 50, 200);
		
		network.attachDemandGeneratorToRetailer(SSU_AR, Type0Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_AR, Type1Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_AR, Type2Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_AR, Type3Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_AR, Type4Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_AR, Type5Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type0Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type1Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type2Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type3Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type4Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_MO, Type5Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type0Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type1Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type2Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type3Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type4Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_OK, Type5Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type0Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type1Brand0, new Exponential(10.0));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type2Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type3Brand0, new Exponential(1.0));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type4Brand0, new Exponential(0.1));
		network.attachDemandGeneratorToRetailer(SSU_KS, Type5Brand0, new Exponential(0.1));
		
		SSU_AR.addReorderPointReorderQuantityInventory(Type0Brand0,2,4,6,"Inventory at SSU_AR for Type0Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type0Brand0,2,4,6,"Inventory at SSU_MO for Type0Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type0Brand0,2,4,6,"Inventory at SSU_OK for Type0Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type0Brand0,2,4,6,"Inventory at SSU_KS for Type0Brand0");
		SSU_AR.addReorderPointReorderQuantityInventory(Type1Brand0,2,4,6,"Inventory at SSU_AR for Type1Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type1Brand0,2,4,6,"Inventory at SSU_MO for Type1Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type1Brand0,2,4,6,"Inventory at SSU_OK for Type1Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type1Brand0,2,4,6,"Inventory at SSU_KS for Type1Brand0");
		SSU_AR.addReorderPointReorderQuantityInventory(Type2Brand0,10,20,30,"Inventory at SSU_AR for Type2Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type2Brand0,10,20,30,"Inventory at SSU_MO for Type2Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type2Brand0,10,20,30,"Inventory at SSU_OK for Type2Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type2Brand0,10,20,30,"Inventory at SSU_KS for Type2Brand0");
		SSU_AR.addReorderPointReorderQuantityInventory(Type3Brand0,10,20,30,"Inventory at SSU_AR for Type3Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type3Brand0,10,20,30,"Inventory at SSU_MO for Type3Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type3Brand0,10,20,30,"Inventory at SSU_OK for Type3Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type3Brand0,10,20,30,"Inventory at SSU_KS for Type3Brand0");
		SSU_AR.addReorderPointReorderQuantityInventory(Type4Brand0,30,60,90,"Inventory at SSU_AR for Type4Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type4Brand0,30,60,90,"Inventory at SSU_MO for Type4Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type4Brand0,30,60,90,"Inventory at SSU_OK for Type4Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type4Brand0,30,60,90,"Inventory at SSU_KS for Type4Brand0");
		SSU_AR.addReorderPointReorderQuantityInventory(Type5Brand0,30,60,90,"Inventory at SSU_AR for Type5Brand0");
		SSU_MO.addReorderPointReorderQuantityInventory(Type5Brand0,30,60,90,"Inventory at SSU_MO for Type5Brand0");
		SSU_OK.addReorderPointReorderQuantityInventory(Type5Brand0,30,60,90,"Inventory at SSU_OK for Type5Brand0");
		SSU_KS.addReorderPointReorderQuantityInventory(Type5Brand0,30,60,90,"Inventory at SSU_KS for Type5Brand0");
		
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type0Brand0,8,16,24,"Inventory at CSC for Type0Brand0");
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type1Brand0,8,16,24,"Inventory at CSC for Type1Brand0");
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type2Brand0,40,30,70,"Inventory at CSC for Type2Brand0");
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type3Brand0,40,30,70,"Inventory at CSC for Type3Brand0");
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type4Brand0,120,180,360,"Inventory at CSC for Type4Brand0");
		network.getWarehouse().addReorderPointReorderQuantityInventory(Type5Brand0,120,180,360,"Inventory at CSC for Type5Brand0");
		return m;
	}

	public static void runExperiment(Model m){
		// create the experiment to run the model
		Experiment e = new Experiment(m);

		// set the parameters of the experiment
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(3650.0);
		e.setLengthOfWarmUp(100.0);

		e.turnOnReplicationConsoleOutput();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}
}
