package jslx.supplychain.proliferationYC;

import java.util.ArrayList;
import java.util.List;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.EventGenerator;
import jsl.utilities.random.RandomIfc;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.reporting.JSL;

public abstract class ProliferatorAbstract extends EventGenerator {
	
	protected List<ProliferationListenerIfc> myProliferationListeners;

	public ProliferatorAbstract(ModelElement parent) {
		this(parent, Constant.ZERO, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}

	public ProliferatorAbstract(ModelElement parent, String name) {
		this(parent, Constant.ZERO, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, name);
	}
	
	public ProliferatorAbstract(ModelElement parent, RandomIfc timeUntilFirst) {
		this(parent, timeUntilFirst, Constant.POSITIVE_INFINITY, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}
	
	public ProliferatorAbstract(ModelElement parent,RandomIfc timeUntilFirst, RandomIfc timeUntilNext) {
		this(parent, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, null);
	}
		
	public ProliferatorAbstract(ModelElement parent, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, String name) {
		this(parent, timeUntilFirst, timeUntilNext, Long.MAX_VALUE, Double.POSITIVE_INFINITY, name);
	}

	public ProliferatorAbstract(ModelElement parent, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, long maxNum) {
		this(parent, timeUntilFirst, timeUntilNext, maxNum, Double.POSITIVE_INFINITY, null);
	}

	public ProliferatorAbstract(ModelElement parent, RandomIfc timeUntilFirst,
			RandomIfc timeUntilNext, long maxNum, double timeUntilLast, String name) {
		super(parent, null, timeUntilFirst, timeUntilNext, maxNum, timeUntilLast, name);
		
	}

	/** Sub-classes should override this method to provide behavior when
	 *  a proliferation event occurs
	 * 
	 * @param event
	 */
	abstract protected void proliferationAction(JSLEvent event);

	/* (non-Javadoc)
	 * @see jsl.modeling.elements.EventGenerator#generate(jsl.modeling.JSLEvent)
	 */
	@Override
	protected void generate(JSLEvent event) {
//		JSL.out.println("*******************************************************");
//		JSL.out.println(getTime() + "> In " + getName() + " generate()");
		proliferationAction(event);
//		JSL.out.println("*******************************************************");
	}
	
	/**
	 * @param listener
	 * @return
	 */
	public boolean attachProliferationListener(ProliferationListenerIfc listener) {
		if (listener == null)
			return false;
		
		if (myProliferationListeners == null)
			myProliferationListeners  = new ArrayList<ProliferationListenerIfc>();

		return myProliferationListeners.add(listener);
	}

	/**
	 * @param listener
	 * @return
	 */
	public boolean containsProliferationListener(ProliferationListenerIfc listener) {
		if (listener == null)
			return false;
		return myProliferationListeners.contains(listener);
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean detachProliferationListener(ProliferationListenerIfc listener) {
		if (listener == null)
			return false;

		return myProliferationListeners.remove(listener);
	}

	/** Notifies the listeners with the supplied message
	 * 
	 * @param message
	 */
	protected void notifyProliferationListeners(ProliferationMessage message){
//		JSL.out.println(getName() + " notifying proliferation listeners");
//		JSL.out.println(message);
		
		for(ProliferationListenerIfc listener: myProliferationListeners){
			listener.handleProliferation(this, message);
		}
	}


}