/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.proliferationYC;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jslx.supplychain.inventorylayer.*;
import jslx.supplychain.multiechelon.*;

/**
 * @author rossetti
 *
 */
public class Test {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Starting experiment.");
		Model m = buildModel();
		
        
		runExperiment(m);
                System.out.println("Done!");
	}
	
	public static Model buildModel(){
		
		Model m = Model.createModel();
		
		// create the network structure
		OneWarehouseNRetailers n = new OneWarehouseNRetailers(m, "network");
		InventoryHoldingPoint w = n.getWarehouse();
		InventoryHoldingPoint r1 = n.addRetailer("R1");
		InventoryHoldingPoint r2 = n.addRetailer("R2");
		
		DefaultItemCategory c1 = new DefaultItemCategory(m, "c1");
		ItemType c1t1 = c1.addInitialItemType();
		ItemType c1t2 = c1.addInitialItemType();
		ItemType c1t3 = c1.addInitialItemType();

		DefaultItemCategory c2 = new DefaultItemCategory(m, "c2");
		ItemType c2t1 = c2.addInitialItemType();
		ItemType c2t2 = c2.addInitialItemType();
		ItemType c2t3 = c2.addInitialItemType();

		// needs to be a category proliferator for each category at a location
		CategoryLocationProliferator pc1r1 = new CategoryLocationProliferator(m);
		pc1r1.setTimeBetweenEventsForReplications(Constant.ONE);
		pc1r1.setMaximumNumberOfEventsForReplications(2);
		pc1r1.setCategoryLocation(c1, r1);
		
		CategoryLocationProliferator pc2r1 = new CategoryLocationProliferator(m);
		pc2r1.setTimeBetweenEventsForReplications(Constant.TWO);
		pc2r1.setMaximumNumberOfEventsForReplications(2);
		pc2r1.setCategoryLocation(c2, r1);
		
		CategoryLocationProliferator pc1r2 = new CategoryLocationProliferator(m);
		pc1r2.setTimeBetweenEventsForReplications(new Constant(3.1));
		pc1r2.setMaximumNumberOfEventsForReplications(2);		
		pc1r2.setCategoryLocation(c1, r2);
		
		CategoryLocationProliferator pc2r2 = new CategoryLocationProliferator(m);
		pc2r2.setTimeBetweenEventsForReplications(new Constant(5.1));
		pc2r2.setMaximumNumberOfEventsForReplications(2);
		pc2r2.setCategoryLocation(c2, r2);

		n.addItemType(c1t1, c1t1.getLeadTime());
		n.addItemType(c1t2, c1t2.getLeadTime());
		n.addItemType(c1t3, c1t3.getLeadTime());

		n.addItemType(c2t1, c2t1.getLeadTime());
		n.addItemType(c2t2, c2t2.getLeadTime());
		n.addItemType(c2t3, c2t3.getLeadTime());

		w.addReorderPointOrderUpToLevelInventory(c1t1, 1, 1, 10);
		w.addReorderPointOrderUpToLevelInventory(c1t2, 1, 1, 10);
		w.addReorderPointOrderUpToLevelInventory(c1t3, 1, 1, 10);
		
		w.addReorderPointOrderUpToLevelInventory(c2t1, 1, 1, 10);
		w.addReorderPointOrderUpToLevelInventory(c2t2, 1, 1, 10);
		w.addReorderPointOrderUpToLevelInventory(c2t3, 1, 1, 10);
		
		// note this essentially indicates which categories are at each location
		// by indicating the item types at each location, but the network
		// is not aware since directly added to ihp
		r1.addReorderPointOrderUpToLevelInventory(c1t1, 1, 1, 10);
		r1.addReorderPointOrderUpToLevelInventory(c1t2, 1, 1, 10);
		r1.addReorderPointOrderUpToLevelInventory(c2t3, 1, 1, 10);//note

		r2.addReorderPointOrderUpToLevelInventory(c2t1, 1, 1, 10);
		r2.addReorderPointOrderUpToLevelInventory(c2t2, 1, 1, 10);
		r2.addReorderPointOrderUpToLevelInventory(c1t3, 1, 1, 10);//note


		
		return m;
	}

	public static void runExperiment(Model m){
		// create the experiment to run the model
		Experiment e = new Experiment(m);

		// set the parameters of the experiment
		// set the parameters of the experiment
		e.setNumberOfReplications(2);
		e.setLengthOfReplication(50.0);
//		e.setLengthOfWarmUp(100.0);

		e.turnOnReplicationConsoleOutput();
		e.turnOnExperimentReport();
//		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}
}
