/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.proliferationYC;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.observers.textfile.*;

import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Triangular;
import jsl.utilities.reporting.JSL;
import jslx.supplychain.inventorylayer.*;

/**
 * @author rossetti
 *
 */
public class TestProliferationNetworkYC1 {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("Building the model");
		Model m = buildModel();
//		try{
        	//CSVReport rcsv = new CSVReplicationReport("Rep-TestProliferationNetworkYC1");
   //     	CSVReport bcsv = new CSVExperimentReport_YC("Experiment-TestProliferationNetworkYC1");
 //       	CSVReport bcsvf = new CSVExperimentReport("ExperimentFull-TestProliferationNetworkYC1");
        	//m.addObserver(rcsv);
 //       	m.addObserver(bcsv);
 //       	m.addObserver(bcsvf);
 //       } catch(IOException e){
	//		JSL.out.println("I/O exception opening file Output.txt");
//			e.printStackTrace();
   //     }
		runExperiment(m);
                System.out.println("Done!");
	}
	
	public static Model buildModel(){
		
		Model m = Model.createModel();
		
		//STEP 1: Create the network structure
        //-------------------------------------
		JSL.out.println("Step 1: Creating the network structure");
		Proliferation1WNRLoadBuildingRQNetwork n = new Proliferation1WNRLoadBuildingRQNetwork(m,"1W4R");
		
		DistributionIfc transportTimeToWarehouse = new Constant(1);
		n.setTransportTimeToWarehouse(transportTimeToWarehouse);
		
		DistributionIfc transportTimeToRetailers = new Constant(1);
		InventoryHoldingPoint r1 = n.addRetailer("R1", transportTimeToRetailers);
		InventoryHoldingPoint r2 = n.addRetailer("R2", transportTimeToRetailers);
		InventoryHoldingPoint r3 = n.addRetailer("R3", transportTimeToRetailers);
		InventoryHoldingPoint r4 = n.addRetailer("R4", transportTimeToRetailers);
		
		//Step 2: Set up the load building options
        //-------------------------------------
		JSL.out.println("Step 2: Setting up the load building options");
		//TODO use whatever load forming criteria you want here
		int countLimit = 1;
		n.setCountLimit(r1, countLimit);
		n.setCountLimit(r2, countLimit);
		n.setCountLimit(r3, countLimit);
		n.setCountLimit(r4, countLimit);
		
		//Step 3: Create the basic categories (only makes them)
        //-------------------------------------
		JSL.out.println("Step 3: Creating the basic categories");
		List<AdoptableItemCategory> categories = new ArrayList<AdoptableItemCategory>();
		
		AdoptableItemCategory c1 = new AdoptableItemCategory(m, "C1");
		c1.setCubeCDFInitialRandomSource(new Triangular(1,3,6));
		c1.setLeadTime(new Triangular(7,11,15));
		c1.setMeanTimeBetweenAdoption(20.0);
		categories.add(c1);
		
		//TODO create other categories here

		//Step 4: Register the categories on the network
        //-------------------------------------
		JSL.out.println("Step 4: Registering the categories on the network");
		for(AdoptableItemCategory c: categories)
			n.registerItemCategory(c);

		//Step 5: Setup the categories with initial item types
        //-------------------------------------
		JSL.out.println("Step 5: Setting up the categories with initial item types");
		// this only creates and adds the item type to the category
		// it does not notify category listeners
		CItemType i1c1 = c1.addInitialItemType();
		

		//TODO create other initial item types in each category here

		//Step 6: Setup the initial item types in the network
        //-------------------------------------
		// set lead time for item type for external supplier in network, based on category
		JSL.out.println("Step 6: Setting up the initial item types in the network");
		n.addInitialItemTypesFromCategories(categories);
		
		//Step 7: Setup the initial item types at the warehouse
        //-------------------------------------
		JSL.out.println("Step 7: Setting up the initial item types at the warehouse");

		int r = 40;
		int q = 80;
		int initialLevel = 120;
		n.addItemTypeToWarehouse(i1c1, r, q, initialLevel);

		//TODO setup other item types at warehouse here
		
		//Step 8: Setup the initial item types at each retailer and add initial demand generators
        //-------------------------------------
		JSL.out.println("Step 8: Setting up the initial item types at each retailer and adding initial demand generators");
		r = 10;
		q = 20;
		initialLevel = 30;
		double mtbd = 1.0;
		n.addInitialItemTypeToRetailer(i1c1, r1, r, q, initialLevel, mtbd);
		
		//TODO setup other item types at each retailer here


		//Step 9: Setup the category/location proliferation
        //-------------------------------------
		JSL.out.println("Step 9: Setting up the category/location proliferation");
		double mtbProliferations = 20.0;
		n.setUpRetailerCategoryProliferation(c1, r1, mtbProliferations, "p_c1r1");
		
		//TODO set up other category/location proliferation here

		// at this point the initial network should be ready
		// proliferation should be started after the warm up event
		
		categories.clear();
		m.setElementAddedAfterReplicationStartedWarningOption(false);
		return m;
	}

	public static void runExperiment(Model m){
		// create the experiment to run the model
		Experiment e = new Experiment(m);

		// set the parameters of the experiment
		// set the parameters of the experiment
		e.setNumberOfReplications(2);
		e.setLengthOfWarmUp(5.0);
		e.setLengthOfReplication(500.0);

		e.turnOnReplicationConsoleOutput();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		JSL.out.println("Running the experiment .....");
		e.runAll();
	}
}
