/*
* Created on Sep 6, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.supplychain.transportlayer;

import jsl.modeling.ActionListenerIfc;
import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.ActionSchedule;

/**
 * @author rossetti
 *
 */
abstract public class ScheduledLoadBuilderAbstract extends DemandLoadBuilder {

	/**
	 * Load forming schedule
	 */
	protected ActionSchedule myLoadBuildingSchedule;
	
	protected LoadBuildingAction myLoadBuildingAction = new LoadBuildingAction();
	
	/**
	 * @param parent
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent) {
		this(parent, 0.0, Double.POSITIVE_INFINITY, true, true, true, null);
	}

	/**
	 * @param parent
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent, String name) {
		this(parent, 0.0, Double.POSITIVE_INFINITY, true, true, true, name);
	}

	/**
	 * 
	 * @param parent
	 * @param scheduleLength The total time available for the schedule
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent,  double scheduleLength) {
		this(parent, 0.0, scheduleLength, true, true, true, null);
	}
	
	/**
	 * 
	 * @param parent
	 * @param startTime The time that the schedule should start
	 * @param scheduleLength The total time available for the schedule
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent, double startTime, double scheduleLength) {
		this(parent, startTime, scheduleLength, true, true, true, null);
	}
	
	/**
	 * 
	 * @param parent
	 * @param scheduleLength The total time available for the schedule
	 * @param name
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent, double scheduleLength, String name) {
		this(parent, 0.0, scheduleLength, true, true, true, name);
	}
	
	/**
	 * 
	 * @param parent
	 * @param startTime The time that the schedule should start
	 * @param scheduleLength The total time available for the schedule
	 * @param name
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent, double startTime, double scheduleLength, String name) {
		this(parent, startTime, scheduleLength, true, true, true, name);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public ScheduledLoadBuilderAbstract(ModelElement parent, double startTime, double scheduleLength, boolean repeatSchedule, boolean repeatActions, boolean startAutomatically, String name) {
		super(parent, name);
		myLoadBuildingSchedule = new ActionSchedule(this, startTime, scheduleLength, repeatSchedule, repeatActions, startAutomatically, getName() + "Schedule");
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getAutomaticStartFlag()
	 */
	public boolean getAutomaticStartFlag() {
		return myLoadBuildingSchedule.getAutomaticStartFlag();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getCycleStartTime()
	 */
	public double getCycleStartTime() {
		return myLoadBuildingSchedule.getCycleStartTime();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getDurationRemainingOnSchedule()
	 */
	public double getDurationRemainingOnSchedule() {
		return myLoadBuildingSchedule.getDurationRemainingOnSchedule();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getDurationTotal()
	 */
	public double getDurationTotal() {
		return myLoadBuildingSchedule.getDurationTotal();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getElapsedCycleTime()
	 */
	public double getElapsedCycleTime() {
		return myLoadBuildingSchedule.getElapsedCycleTime();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getInitialStartTime()
	 */
	public double getInitialStartTime() {
		return myLoadBuildingSchedule.getInitialStartTime();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getNumberOfActions()
	 */
	public int getNumberOfActions() {
		return myLoadBuildingSchedule.getNumberOfActions();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getRemainingCycleTime()
	 */
	public double getRemainingCycleTime() {
		return myLoadBuildingSchedule.getRemainingCycleTime();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getScheduleLength()
	 */
	public double getScheduleLength() {
		return myLoadBuildingSchedule.getScheduleLength();
	}

	/**
	 * @return
	 * @see jsl.modeling.elements.ActionSchedule#getScheduleRepeatFlag()
	 */
	public boolean getScheduleRepeatFlag() {
		return myLoadBuildingSchedule.getScheduleRepeatFlag();
	}

	/**
	 * 
	 * @see jsl.modeling.elements.ActionSchedule#scheduleStart()
	 */
	public void scheduleStart() {
		myLoadBuildingSchedule.scheduleStart();
	}

	/**
	 * @param flag
	 * @see jsl.modeling.elements.ActionSchedule#setAutomaticStartFlag(boolean)
	 */
	public void setAutomaticStartFlag(boolean flag) {
		myLoadBuildingSchedule.setAutomaticStartFlag(flag);
	}

	/**
	 * @param flag
	 * @see jsl.modeling.elements.ActionSchedule#setScheduleRepeatFlag(boolean)
	 */
	public void setScheduleRepeatFlag(boolean flag) {
		myLoadBuildingSchedule.setScheduleRepeatFlag(flag);
	}

	/**
	 * @param duration
	 */
	public void addLoadBuildingAction(double duration) {
		myLoadBuildingSchedule.addAction(duration, myLoadBuildingAction);
	}
	
	abstract void loadBuildingAction();
	
	private class LoadBuildingAction implements ActionListenerIfc {
		public void action(JSLEvent evt) {
			loadBuildingAction();
		}
	}

}
