package jslx.supplychain.transportlayer.testing;
import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Triangular;

import java.io.IOException;
import java.util.logging.Level;
import jsl.utilities.reporting.JSL;
import jsl.modeling.*;
import jsl.observers.textfile.*;
import jslx.supplychain.inventorylayer.DemandGenerator;
import jslx.supplychain.inventorylayer.InventoryHoldingPoint;
import jslx.supplychain.inventorylayer.ItemType;
import jslx.supplychain.inventorylayer.LeadTimeDemandFiller;
import jslx.supplychain.transportlayer.DemandLoadBuilder;
import jslx.supplychain.transportlayer.TimeBasedDemandCarrier;
import jslx.supplychain.transportlayer.TimeBasedLoadCarrier;
public class Mercy4 {
	
	public static void main(String[] args) {
		System.out.println("Mercy Health System Supply Chain Model 4");
		testExperiment();
		System.out.println("Done!");	
	}
	public static Model buildModel(){

		Model m = Model.createModel();
		LeadTimeDemandFiller supplier = new LeadTimeDemandFiller(m, "supplier");
		InventoryHoldingPoint CSC = new InventoryHoldingPoint(m, "CSC");
		CSC.setDemandFiller(supplier);
		InventoryHoldingPoint SSU_AR = new InventoryHoldingPoint(m, "SSU_AR");
		InventoryHoldingPoint SSU_MO = new InventoryHoldingPoint(m, "SSU_MO");
		SSU_AR.setDemandFiller(CSC);
		SSU_MO.setDemandFiller(CSC);
		TimeBasedLoadCarrier carrier = new TimeBasedLoadCarrier(m, "Carrier");
		TimeBasedDemandCarrier external_carrier = new TimeBasedDemandCarrier(m, "external_carrier");
		DemandLoadBuilder lb2AR = carrier.assignLoadBuilder(SSU_AR, "LoadBuilder CSC to SSU_AR");
		DemandLoadBuilder lb2MO = carrier.assignLoadBuilder(SSU_MO, "LoadBuilder CSC to SSU_MO");
		lb2AR.setCountLimit(10);
		lb2MO.setCountLimit(10);
//		lb2AR.setCountLimit(3);
//		lb2MO.setCountLimit(3);

		carrier.setTransportTime(SSU_AR, new Constant(3.3/24));
		carrier.setTransportTime(SSU_MO, new Constant(3.4/24));
		external_carrier.setTransportTime(CSC, new Exponential(8.0/24));
		carrier.setReactToLoadBuildersFlag(true);
		supplier.setDemandCarrier(external_carrier);
		CSC.setDemandCarrier(carrier);
		
		// Everything about ItemType2; 2222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222
		ItemType type2=new ItemType(m,"Item 2", 0.2, 0.037);
		
		supplier.addLeadTime(type2, new Triangular(7.6,8.8,10));
		CSC.addReorderPointReorderQuantityInventory(type2, 40, 30, 70, "type2_CSC_Inventory");
		SSU_AR.addReorderPointReorderQuantityInventory(type2,10,20,30,"Inventory at SSU_AR for type2");
		SSU_MO.addReorderPointReorderQuantityInventory(type2,10,20,30,"Inventory at SSU_MO for type2");
		DistributionIfc d2_AR = new Exponential(1.0);				
		DemandGenerator dg2_AR = new DemandGenerator(m, type2, d2_AR, d2_AR, "Item2AR DemandGenerator");
		DistributionIfc d2_MO = new Exponential(1.0);				
		DemandGenerator dg2_MO = new DemandGenerator(m, type2, d2_MO, d2_MO, "Item2MO DemandGenerator");
		dg2_AR.setDemandFiller(SSU_AR);
		dg2_MO.setDemandFiller(SSU_MO);
		
		// Everything about ItemType3; 333333333333333333333333333333333333333333333333333333333333333333333333333333333333333
//		ItemType type3 = new ItemType(m,"Item 3");
//		type3.setWeight(0.2);
//		type3.setCube(0.037);
//		supplier.addLeadTime(type3, new Triangular(7.6,8.8,10));
//		CSC.addReorderPointReorderQuantityInventory(type3, 30, 40, 70, "type3_CSC_Inventory");
//		SSU_AR.addReorderPointReorderQuantityInventory(type3,10,20,30,"Inventory at SSU_AR for type3");
//		SSU_MO.addReorderPointReorderQuantityInventory(type3,10,20,30,"Inventory at SSU_MO for type3");
//		DistributionIfc d3_AR = new Exponential(1.0);				
//		DemandGenerator dg3_AR = new DemandGenerator(m, type3, d3_AR, d3_AR, "Item3AR DemandGenerator");
//		DistributionIfc d3_MO = new Exponential(1.0);				
//		DemandGenerator dg3_MO = new DemandGenerator(m, type3, d3_MO, d3_MO, "Item3MO DemandGenerator");
//		dg3_AR.setDemandFiller(SSU_AR);
//		dg3_MO.setDemandFiller(SSU_MO);
		
		
		return m;
	}
	
	public static void testExperiment(){
		Model m = buildModel();
	//	System.out.println(m.getModelHierarchyAsString());
		
	//	m.setAutoRemoveMarkedElementsOption(true);
		
	//	m.setElementAddedAfterReplicationStartedWarningOption(false);
		/*
		try{
        	CSVReport rcsv = new CSVReplicationReport("Mercy4Replication");
        	CSVReport bcsv = new CSVExperimentReport("Mercy4Experiment");
        	m.addObserver(rcsv);
        	m.addObserver(bcsv);
        }
        catch(IOException e){
			System.out.println("I/O exception opening file Output.txt");
			e.printStackTrace();
        }
        */
	//	m.turnOnLogReport();
		
		Experiment e = new Experiment(m);
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(1000);
		e.setLengthOfWarmUp(0);
		e.turnOnExperimentConsoleOutput();
		e.turnOffBatchReport();
		e.turnOffDefaultTraceReport();
		e.turnOffExperimentReport();
		e.runAll();
	}
}
