package jslx.supplychain.transportlayer.testing;

/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/


import java.io.IOException;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.observers.textfile.CSVExperimentReport;
import jsl.observers.textfile.CSVReport;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Triangular;
import jslx.supplychain.MultiEchelonIHPNetwork;
import jslx.supplychain.inventorylayer.InventoryHoldingPoint;
import jslx.supplychain.inventorylayer.ItemType;
import jslx.supplychain.transportlayer.TimeBasedDemandCarrier;

public class testIHPNetwork {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("IHPNetwork Test for Mercy4");
       
		testExperiment();
        
		System.out.println("Done!");	
	}

	public static Model buildModel(){
		Model m = Model.createModel();
		
		//Create the network
		MultiEchelonIHPNetwork n = new MultiEchelonIHPNetwork(m, "ME-Inventory-Network");
		
		//Define item types
		DistributionIfc d = new Triangular(7.6,8.8,10);
		ItemType type1 = n.addItemType("Type-1", d);
				
		//Define CSC
		InventoryHoldingPoint CSC = n.addInventoryHoldingPoint("CSC");
		
		//addInventory to CSC
		CSC.addReorderPointReorderQuantityInventory(type1,40,30,70);
		
		// Create the retailers which use the CSC as its filler
		InventoryHoldingPoint SSU_AR = n.addInventoryHoldingPoint("SSU_AR");
		InventoryHoldingPoint SSU_MO = n.addInventoryHoldingPoint("SSU_MO");
		InventoryHoldingPoint SSU_OK = n.addInventoryHoldingPoint("SSU_OK");
		InventoryHoldingPoint SSU_KS = n.addInventoryHoldingPoint("SSU_KS");
		
		////////////////////Optional Section Starts//////////////////////////////////////////////////
		// Option1: Configure the demand carrier from CSC to SSUs
		TimeBasedDemandCarrier carrier = new TimeBasedDemandCarrier(m, "Carrier");
		carrier.setTransportTime(SSU_AR, new Constant(3.3/24));
		carrier.setTransportTime(SSU_MO, new Constant(3.4/24));
		carrier.setTransportTime(SSU_OK, new Constant(4.5/24));
		carrier.setTransportTime(SSU_KS, new Constant(2.0/24));
		CSC.setDemandCarrier(carrier);
		
		// Option2: Configure the load carrier from CSC to SSUs
//		TimeBasedLoadCarrier carrier = new TimeBasedLoadCarrier(m, "Carrier");
//		DemandLoadBuilder lb2AR = carrier.assignLoadBuilder(SSU_AR, "LoadBuilder CSC to SSU_AR");
//		DemandLoadBuilder lb2MO = carrier.assignLoadBuilder(SSU_MO, "LoadBuilder CSC to SSU_MO");
//		DemandLoadBuilder lb2OK = carrier.assignLoadBuilder(SSU_OK, "LoadBuilder CSC to SSU_OK");
//		DemandLoadBuilder lb2KS = carrier.assignLoadBuilder(SSU_KS, "LoadBuilder CSC to SSU_KS");
//		lb2AR.setCountLimit(10);
//		lb2MO.setCountLimit(10);
//		lb2OK.setCountLimit(10);
//		lb2KS.setCountLimit(10);
//		carrier.setTransportTime(SSU_AR, new Constant(3.3/24));
//		carrier.setTransportTime(SSU_MO, new Constant(3.4/24));
//		carrier.setTransportTime(SSU_OK, new Constant(4.5/24));
//		carrier.setTransportTime(SSU_KS, new Constant(2.0/24));
//		CSC.setDemandCarrier(carrier);
		////////////////////Optional Section Ends//////////////////////////////////////////////////////
		
		//Create the inventory at each retailer
		SSU_AR.addReorderPointReorderQuantityInventory(type1,10,20,30);
		
		SSU_MO.addReorderPointReorderQuantityInventory(type1,10,20,30);
		
		SSU_OK.addReorderPointReorderQuantityInventory(type1,10,20,30);

		SSU_KS.addReorderPointReorderQuantityInventory(type1,10,20,30);

		
		// create the structure of the network
		// attach the CSC to the external supplier
		n.attachIHPToExternalSupplier(CSC);
		
		// attach the retailers to the CSC
		n.attachIHPToSupplier(CSC, SSU_AR);
		n.attachIHPToSupplier(CSC, SSU_MO);
		n.attachIHPToSupplier(CSC, SSU_OK);
		n.attachIHPToSupplier(CSC, SSU_KS);
		
		// create the demand generators for the retailers						
		DistributionIfc dA = new Exponential(1.0);				
		
		n.attachDemandGeneratorToIHP(SSU_AR, type1, dA);

		n.attachDemandGeneratorToIHP(SSU_MO, type1, dA);

		n.attachDemandGeneratorToIHP(SSU_OK, type1, dA);

		n.attachDemandGeneratorToIHP(SSU_KS, type1, dA);

		return(m);
	}
	
	public static void testExperiment(){
		
		Model m = buildModel();	
		
	//	m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m, "ME-Inventory-Network Test");
		
		try{
        	CSVReport ecsv = new CSVExperimentReport("ME-Inventory-Network Test");
        	m.addObserver(ecsv);
        }
        catch(IOException exception){
			System.out.println("I/O exception opening file Output.txt");
			exception.printStackTrace();
        }
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(3650.0);
		e.setLengthOfWarmUp(100.0);
		
		// turn on the desired reporting
	//	e.turnOnBatchReport(); 
		e.turnOnExperimentReport("ME-Inventory-Network");
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput(true);
		
		// tell the experiment to run
		e.runAll();
		
	}
	
	public static void testReplication(){
		
		Model m = buildModel();	
		
		// create the experiment to run the model
		Replication r = new Replication(m);
		
		r.initialize();
		r.runNext();
		
	}
}
