/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer;

import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;

public abstract class AbstractDispatcher extends SchedulingElement {
	
	/** Represents the Transportation Network of the dispatcher 
	 */
	private TransportationNetwork myNetwork;
	
	/** An instance of LoadSelectionRuleIfc 
	 */
	protected DispatchLoadSelectionRuleIfc myLoadSelectionRule;
	
	public AbstractDispatcher(ModelElement parent, TransportationNetwork network) {
		this(parent, network, null);
	}
	
	public AbstractDispatcher(ModelElement parent, TransportationNetwork network, String name) {
		super(parent, name);
		setNetwork(network);
	}
	
	/** Allows the dispatcher to receive an process a load
	 * 
	 * @param load
	 */
	abstract protected void receive(Load load);
	
	/** Used to indicate that the dispatch task has been completed
	 * 
	 * @param task
	 */
	abstract protected void dispatchCompleted(DispatchTask task);
	
	/**
	 * @return Returns the loadSelectionRule.
	 */
	protected final DispatchLoadSelectionRuleIfc getLoadSelectionRule() {
		return myLoadSelectionRule;
	}

	/**
	 * @param loadSelectionRule The loadSelectionRule to set.
	 */
	protected final void setLoadSelectionRule(DispatchLoadSelectionRuleIfc loadSelectionRule) {
		myLoadSelectionRule = loadSelectionRule;
	}

	/**
	 * @return Returns the network.
	 */
	protected final TransportationNetwork getNetwork() {
		return myNetwork;
	}

	/**
	 * @param network The network to set.
	 */
	private final void setNetwork(TransportationNetwork network) {
		if (network == null)
			throw new IllegalArgumentException("The network must not be null");
		
		myNetwork = network;
		myNetwork.setDispatcher(this);
		setSpatialModel(myNetwork.getSpatialModel());
	}

}
