/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.transportlayer;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.spatial.CoordinateIfc;
import jsl.modeling.elements.spatial.SpatialElementIfc;
import jsl.modeling.elements.spatial.SpatialModel;
import jsl.modeling.elements.spatial.SpatialModelElement;

/**
 *
 */
abstract public class AbstractLocation extends SpatialModelElement implements TransportLocationIfc {

	/**
	 * @param parent
	 */
	public AbstractLocation(ModelElement parent) {
		this(parent, null, null, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public AbstractLocation(ModelElement parent, String name) {
		this(parent, name, null, null);
	}

	/**
	 * @param parent
	 * @param position
	 */
	public AbstractLocation(ModelElement parent, CoordinateIfc position) {
		this(parent, null, null, position);
	}

	/**
	 * @param parent
	 * @param position
	 */
	public AbstractLocation(ModelElement parent, SpatialElementIfc position) {
		this(parent, null, null, position.getCurrentPosition());
	}

	/**
	 * @param parent
	 * @param name
	 * @param spatialModel
	 */
	public AbstractLocation(ModelElement parent, String name, SpatialModel spatialModel) {
		this(parent, name, spatialModel, null);
	}

	/**
	 * @param parent
	 * @param spatialModel
	 */
	public AbstractLocation(ModelElement parent, SpatialModel spatialModel) {
		this(parent, null, spatialModel, null);
	}

	/**
	 * @param parent
	 * @param spatialModel
	 * @param element
	 */
	public AbstractLocation(ModelElement parent, SpatialModel spatialModel, SpatialElementIfc element) {
		this(parent, null, spatialModel, element.getCurrentPosition());
	}

	/**
	 * @param parent
	 * @param name
	 * @param spatialModel
	 * @param coordinate
	 */
	public AbstractLocation(ModelElement parent, String name, SpatialModel spatialModel, CoordinateIfc coordinate) {
		super(parent, name, spatialModel, coordinate);
	}

}
