/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer;

import jsl.modeling.ModelElement;
import jsl.modeling.elements.spatial.CoordinateIfc;
import jsl.modeling.elements.spatial.MobileResource;

/** Driver is the self moving resource in the network. It extends the MobileResource of the spatial package.
 *  This class gets initialized by setting the initial location of the driver.
 *  
 *
 */
public class Driver extends MobileResource  {
	    
    /** Represents the current Task of the driver 
     */
	protected DispatchTask myCurrentTask;    

	/** Creates the driver with the same spatial model as that of its parent.
	 *  The initial location of the driver is passed as co ordinates 
	 * 
	 * @param parent
	 * @param name
	 * @param coordinate
	 */
	public Driver(ModelElement parent, String name, CoordinateIfc coordinate) {
		super(parent,1, name, null, coordinate);
	}
	
	protected void performTask(DispatchTask task){
		// set the current task
		myCurrentTask = task;
		if (task.isTransport())
			performTransport();
		else if (task.isDeadHeadToLoad())
			performDeadHeadToLoad();
		else if (task.isBobTailToLoad())
			performBobTailToLoad();
		else if (task.isBobTailNoLoad())
			performBobTailNoLoad();
		else if (task.isDeadHeadNoLoad())
			performDeadHeadNoLoad();
	}
	
	protected void performBobTailToLoad(){
		//TODO
		moveEmpty(myCurrentTask.getLoadOrigin());
	}
	
	protected void performDeadHeadToLoad(){
		//TODO
		moveEmpty(myCurrentTask.getLoadOrigin());
	}

	protected void performBobTailNoLoad(){
		//TODO
		moveEmpty(myCurrentTask.getLoadOrigin());
	}
	
	protected void performDeadHeadNoLoad(){
		//TODO
		moveEmpty(myCurrentTask.getLoadOrigin());
	}

	protected void performTransport(){
		//TODO
		transport(myCurrentTask.getLoadDestination());
	}
	
	/** Driver notifies the dispatcher about the completion of the task MoveEmpty
	 */
	protected void emptyMoveCompleted() {
		//TODO need to distinguish between bobtail and deadhead
		// what about loading and unloading time
		// what about processing at the location
		myCurrentTask.taskCompleted();
	}
	
	/** Driver notifies the dispatcher about the completion of the task Transport 
	 */
	protected void transportCompleted() {
		//TODO????
		// what about loading and unloading time
		// what about processing at the location
		myCurrentTask.taskCompleted();
	}
	
}


 