/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer;

import jsl.modeling.elements.queue.QObject;

/** A Load represents a request to move material from one
 *  one location (origin) to another location (destination).
 *  A load is sent by a specific sender to be completed.
 *
 */
public class Load extends QObject {
	
	/** The transportation network that will serve this load
	 */
	private TransportationNetwork myNetwork;
	
	/** Represents the object that sent the load
	 */
	private LoadSenderIfc mySender;
	
	/** Represents origin of the load 
	 */
	private TransportLocationIfc myOrigin;
	
	/** Represents destination of the load 
	 */
	private TransportLocationIfc myDestination;
	
	private double myDueDate;

	/** Creates a load with no due date
	 * 
	 * @param network
	 * @param sender
	 * @param origin
	 * @param destination
	 */
	protected Load(TransportationNetwork network, LoadSenderIfc sender, TransportLocationIfc origin,
			TransportLocationIfc destination){
	}
	
	/** Creates a load
	 * 
	 * @param network
	 * @param sender
	 * @param origin
	 * @param destination
	 * @param dueDate
	 */
	protected Load(TransportationNetwork network, LoadSenderIfc sender, TransportLocationIfc origin,
			TransportLocationIfc destination, double dueDate){
		setNetwork(network);
		setDestination(destination);
		setOrigin(origin);
		setSender(sender);
		setDueDate(dueDate);
	}
    
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getSender()
	 */
	public final LoadSenderIfc getSender() {
		return mySender;
	}
	/**
	 * @param mySender The sender to set.
	 */
	protected final void setSender(LoadSenderIfc sender) {
		if (sender == null)
			throw new IllegalArgumentException("The LoadSender must not be null");
		mySender = sender;
	}
	
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getDestination()
	 */
	public final TransportLocationIfc getDestination() {
		return myDestination;
	}

	/** Sets the Destination of the load
	 * @param destination
	 */
	protected final void setDestination(TransportLocationIfc destination) {
		if (destination == null)
			throw new IllegalArgumentException("The Location destination must not be null");
		
		myDestination = destination;
	}

	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getOrigin()
	 */
	public final TransportLocationIfc getOrigin() {
		return myOrigin;
	}

	/** Sets the destination of the load
	 * @param origin
	*/
	protected void setOrigin(TransportLocationIfc origin) {
		if (origin == null)
			throw new IllegalArgumentException("The Location origin must not be null");
		myOrigin = origin;
	}

	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getNetwork()
	 */
	public final TransportationNetwork getNetwork() {
		return myNetwork;
	}

	/**
	 * @param myNetwork The myNetwork to set.
	 */
	protected final void setNetwork(TransportationNetwork network) {
		myNetwork = network;
	}

	/**
	 * @return Returns the dueDate.
	 */
	public final double getDueDate() {
		return myDueDate;
	}

	/**
	 * @param dueDate The dueDate to set.
	 */
	protected final void setDueDate(double dueDate) {
		//TODO How to set the properly
		myDueDate = dueDate;
	}

}



