/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer;

import java.util.Observable;
import java.util.Observer;

/** Lane class is for creating lane given origin and destination. The given origin and destination must be 
 *  different otherwise an exception will be thrown 
 * 
 *
 */
public class SMLane extends Lane implements Observer {
	
	/** Creates Lane with the given origin and destination as parameters
	 * 
	 * @param origin
	 * @param destination
	 */
	public SMLane(Network network, TransportLocationIfc origin, TransportLocationIfc destination) {
		super(network, origin, destination);
		destination.attachPositionObserver(this);
		origin.attachPositionObserver(this);
		setDistance(origin.distanceTo(destination));		
	}

	/** A lane is attached as a position observer on the Locations
	 *  serving as an origin or a destitation.  If their positions
	 *  change then the lane distance needs to be recalculated and 
	 *  the network notified via updateLane()
	 * 
	 * @param arg0 The SpatialElementIfc that is changing its position
	 * @param arg1
	 */
	public void update(Observable spatialModelElement, Object arg1) {
		setDistance(getOrigin().distanceTo(getDestination()));
		getNetwork().updateLane(this);
	}
}
