/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package jslx.transportlayer;

import jsl.modeling.elements.spatial.CoordinateIfc;
import jsl.modeling.elements.spatial.SpatialElement;
import jsl.modeling.elements.spatial.SpatialModel;

/**
 *
 */
public class SMLocation extends SpatialElement implements TransportLocationIfc {

	/**
	 * @param spatialModel
	 */
	public SMLocation(SpatialModel spatialModel) {
		this(spatialModel, spatialModel.getDefaultCoordinate(), null);
	}

	/**
	 * @param spatialModel
	 * @param coordinate
	 */
	public SMLocation(SpatialModel spatialModel, CoordinateIfc coordinate) {
		this(spatialModel, coordinate, null);
	}

	/**
	 * @param spatialModel
	 * @param name
	 */
	public SMLocation(SpatialModel spatialModel, String name) {
		this(spatialModel, spatialModel.getDefaultCoordinate(), name);
	}

	/**
	 * @param spatialModel
	 * @param x
	 * @param y
	 */
	public SMLocation(SpatialModel spatialModel, double x, double y) {
		this(spatialModel, spatialModel.getCoordinate(x, y), null);
	}

	/**
	 * @param spatialModel
	 * @param x
	 * @param y
	 * @param name
	 */
	public SMLocation(SpatialModel spatialModel, double x, double y, String name) {
		this(spatialModel, spatialModel.getCoordinate(x, y), name);
	}

	/**
	 * @param spatialModel
	 * @param x
	 * @param y
	 * @param z
	 */
	public SMLocation(SpatialModel spatialModel, double x, double y, double z) {
		this(spatialModel, spatialModel.getCoordinate(x, y, z), null);
	}

	/**
	 * @param spatialModel
	 * @param x
	 * @param y
	 * @param z
	 * @param name
	 */
	public SMLocation(SpatialModel spatialModel, double x, double y, double z, String name) {
		this(spatialModel, spatialModel.getCoordinate(x, y, z), name);
	}

	/**
	 * @param spatialModel
	 * @param coordinate
	 * @param name
	 */
	public SMLocation(SpatialModel spatialModel, CoordinateIfc coordinate, String name) {
		super(spatialModel, coordinate, name);
	}

	/* (non-Javadoc)
	 * @see transportlayer.TransportTaskReceiverIfc#receiveTransportTask(transportlayer.TransportTask)
	 */
	public void receiveTransportTask(DispatchTask task) {
		
	}

}
