/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer.graph;

public class AdjMatrixGraph {
	
	private int myVcnt;
	private int myEcnt;
	private boolean myDirectedFlag;
	
	private boolean[][] myMatrix;
	
	public AdjMatrixGraph(int numVertices){
		this(numVertices, true);
	}
	
	public AdjMatrixGraph(int numVertices, boolean directed){
		myVcnt = numVertices;
		myEcnt = 0;
		myDirectedFlag = directed;
		myMatrix = new boolean[myVcnt][myVcnt];
	}
	
	public int getNumVertices(){
		return(myVcnt);
	}
	
	public int getNumEdges(){
		return(myEcnt);
	}
	
	public boolean isDirected(){
		return(myDirectedFlag);
	}
	
	public void insertEdge(int v, int w){
		if (myMatrix[v][w] == false)
			myEcnt++;
		myMatrix[v][w]= true;
		if (!myDirectedFlag)
			myMatrix[w][v] = true;
	}
	
	public void removeEdge(int v, int w){
		if (myMatrix[v][w] == true)
			myEcnt--;
		myMatrix[v][w] = false;
		if (!myDirectedFlag)
			myMatrix[w][v] = false;
	}
	
	public boolean edge(int v, int w){
		return myMatrix[v][w];
	}
	
	public boolean isAllConnected(){
		
		boolean b = true;
		
		for(int i = 0; i < myMatrix.length; i++){
			for(int j = 0; j < myMatrix[i].length; j++){
				if (i == j)
					continue;
				else {
					b = b && myMatrix[i][j];
					if (b == false) // no need to check further
						return(b);
				}
				
			}
		}
		
		return(b);
		
	}
	
	public String toString(){
		StringBuffer s = new StringBuffer();		
		for(int i = 0; i < myMatrix.length; i++){
			s.append(i + " : ");
			for(int j = 0; j < myMatrix[i].length; j++){
				if (myMatrix[i][j])
					s.append(j + " ");
			}
			s.append("\n");
		}
		s.append("\n");
		for(int i = 0; i < myMatrix.length; i++){
			for(int j = 0; j < myMatrix[i].length; j++){
				s.append("(" + i + "," + j + ") = " + myMatrix[i][j] + "\t");
			}
			s.append("\n");
		}

		s.append("\n");
		
		return(s.toString());
	}

}
