/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.transportlayer.test;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.elements.spatial.RectangularGridSpatialModel2D;
import jsl.modeling.elements.spatial.SpatialModel;
import jsl.modeling.elements.spatial.Vector3D;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.transportlayer.Lane;
import jslx.transportlayer.LaneIfc;
import jslx.transportlayer.LoadGenerator;
import jslx.transportlayer.SMLocation;
import jslx.transportlayer.TransportationNetwork;

public class TestLoadGenerator {

	
		public static void main(String[] args){
	
		
		System.out.println("Simple Transportation Model");
				
		test();
		
		System.out.println("Done!");
	}
	
	public static void test(){
		
		System.out.println("Testing Load Generator Class");
				
		Model m = Model.createModel();
		System.out.println("Model Created");
		
		DistributionIfc tl = new Exponential(1.0);
		
		SpatialModel smodel = new RectangularGridSpatialModel2D();
		
		TransportationNetwork t=new TransportationNetwork(m, smodel);
		
		System.out.println("Here is the Transportation Network");		
		
		SMLocation l1 = new SMLocation(smodel, new Vector3D(0.0,0.0,0.0),"Kansas City");
		SMLocation l2 = new SMLocation(smodel, new Vector3D(1.0,2.0,3.0),"Chicago");
		SMLocation l3 = new SMLocation(smodel, new Vector3D(5.0,4.0,0.0),"New York");
		SMLocation l4 = new SMLocation(smodel, new Vector3D(3.0,7.0,0.0),"Boston");
		SMLocation l5 = new SMLocation(smodel, new Vector3D(4.0,9.0,0.0),"Atlanta");
		SMLocation l6 = new SMLocation(smodel, new Vector3D(3.0,4.0,0.0),"Los Angeles");
		
		LaneIfc lane12= new Lane(t,l1,l2,1.0);
		LaneIfc lane13= new Lane(t,l1,l3,2.0);
		LaneIfc lane14= new Lane(t,l1,l4,3.0);
		LaneIfc lane15= new Lane(t,l1,l5,4.0);
		LaneIfc lane16= new Lane(t,l1,l6,5.0);
		
		LaneIfc lane21= new Lane(t,l2,l1,1.0);
		LaneIfc lane23= new Lane(t,l2,l3,2.0);
		LaneIfc lane24= new Lane(t,l2,l4,3.0);
		LaneIfc lane25= new Lane(t,l2,l5,4.0);
		LaneIfc lane26= new Lane(t,l2,l6,5.0);
		
		LaneIfc lane31= new Lane(t,l3,l1,1.0);
		LaneIfc lane32= new Lane(t,l3,l2,2.0);
		LaneIfc lane34= new Lane(t,l3,l4,3.0);
		LaneIfc lane35= new Lane(t,l3,l5,4.0);
		LaneIfc lane36= new Lane(t,l3,l6,5.0);
		
		LaneIfc lane41= new Lane(t,l4,l1,1.0);
		LaneIfc lane42= new Lane(t,l4,l2,2.0);
		LaneIfc lane43= new Lane(t,l4,l3,3.0);
		LaneIfc lane45= new Lane(t,l4,l5,4.0);
		LaneIfc lane46= new Lane(t,l4,l6,5.0);
		
		LaneIfc lane51= new Lane(t,l5,l1,1.0);
		LaneIfc lane52= new Lane(t,l5,l2,2.0);
		LaneIfc lane53= new Lane(t,l5,l3,3.0);
		LaneIfc lane54= new Lane(t,l5,l4,4.0);
		LaneIfc lane56= new Lane(t,l5,l6,5.0);
		
		LaneIfc lane61= new Lane(t,l6,l1,1.0);
		LaneIfc lane62= new Lane(t,l6,l2,2.0);
		LaneIfc lane63= new Lane(t,l6,l3,3.0);
		LaneIfc lane64= new Lane(t,l6,l4,4.0);
		LaneIfc lane65= new Lane(t,l6,l5,5.0);
		
		t.addLane(lane12);
		t.addLane(lane13);
		t.addLane(lane14);
		t.addLane(lane15);
		t.addLane(lane16);
		
		t.addLane(lane21);
		t.addLane(lane23);
		t.addLane(lane24);
		t.addLane(lane25);
		t.addLane(lane26);
		
		t.addLane(lane31);
		t.addLane(lane32);
		t.addLane(lane34);
		t.addLane(lane35);
		t.addLane(lane36);
		
		t.addLane(lane41);
		t.addLane(lane42);
		t.addLane(lane43);
		t.addLane(lane45);
		t.addLane(lane46);
		
		t.addLane(lane51);
		t.addLane(lane52);
		t.addLane(lane53);
		t.addLane(lane54);
		t.addLane(lane56);
		
		t.addLane(lane61);
		t.addLane(lane62);
		t.addLane(lane63);
		t.addLane(lane64);
		t.addLane(lane65);
		
		System.out.println(t);
		
//		Create the Load generator
		LoadGenerator lg = new LoadGenerator(m,t,tl,tl); 
		
		System.out.println("Here is the load generator");
		
		lg.setOriginProbability(l1, 0.2);
		lg.setOriginProbability(l2, 0.2);
		lg.setOriginProbability(l3, 0.1);
		lg.setOriginProbability(l4, 0.1);
		lg.setOriginProbability(l5, 0.1);
		lg.setLastOriginProbability(l6);
		
		System.out.println("origin probability set");
		
		lg.setDestinationProbability(lane12,0.2);
		lg.setDestinationProbability(lane13,0.2);
		lg.setDestinationProbability(lane14,0.2);
		lg.setDestinationProbability(lane15,0.1);
		lg.setLastDestinationProbability(lane16);
		
		lg.setDestinationProbability(lane21,0.2);
		lg.setDestinationProbability(lane23,0.1);
		lg.setDestinationProbability(lane24,0.2);
		lg.setDestinationProbability(lane25,0.1);
		lg.setLastDestinationProbability(lane26);
		
		lg.setDestinationProbability(lane31,0.2);
		lg.setDestinationProbability(lane32,0.1);
		lg.setDestinationProbability(lane34,0.2);
		lg.setDestinationProbability(lane35,0.1);
		lg.setLastDestinationProbability(lane36);
		
		lg.setDestinationProbability(lane41,0.2);
		lg.setDestinationProbability(lane42,0.1);
		lg.setDestinationProbability(lane43,0.2);
		lg.setDestinationProbability(lane45,0.1);
		lg.setLastDestinationProbability(lane46);
		
		lg.setDestinationProbability(lane51,0.1);
		lg.setDestinationProbability(lane52,0.1);
		lg.setDestinationProbability(lane53,0.1);
		lg.setDestinationProbability(lane54,0.1);
		lg.setLastDestinationProbability(lane56);
		
		lg.setDestinationProbability(lane61,0.2);
		lg.setDestinationProbability(lane62,0.1);
		lg.setDestinationProbability(lane63,0.2);
		lg.setDestinationProbability(lane64,0.1);
		lg.setLastDestinationProbability(lane65);
		
		System.out.println("destination probability set");
		
		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(1);
		e.setLengthOfReplication(10);
		e.setLengthOfWarmUp(0.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
	//	e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput();
		
		// tell the experiment to run
		e.runAll();
		
	}
	
}
