package jslx.transportlayerFinal;

import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;

/** AbstractDispatcher receives loads from the TransportationNetwork. It can dispatch load
 *  based on the supplied load selection rules.
 *   
 * @author snangia
 *
 */
public abstract class AbstractDispatcher extends SchedulingElement {
	
	/** Represents the Transportation Network of the dispatcher 
	 */
	private TransportationNetwork myNetwork;
	
	/** An instance of LoadSelectionRuleIfc 
	 */
	protected DispatchLoadSelectionRuleIfc myLoadSelectionRule;

	//  ===========================================
	//  CONSTRUCTORS
	//  ===========================================
	
	/**
	 * 
	 * @param parent
	 * @param network
	 */
	public AbstractDispatcher(ModelElement parent, TransportationNetwork network) {
		this(parent, network, null);
	}
	
	/**
	 * 
	 * @param parent
	 * @param network
	 * @param name
	 */
	public AbstractDispatcher(ModelElement parent, TransportationNetwork network, String name) {
		super(parent, name);
		setNetwork(network);
	}

	//  ===========================================
	//  ABSTRACT METHODS
	//  ===========================================
	
	/** Allows the dispatcher to receive a load
	 * 
	 * @param load
	 */
	abstract protected void receive(Load load);
	
	/** Used to indicate that the dispatch task has been completed
	 * 
	 * @param task
	 */
	abstract protected void dispatchCompleted(DispatchTask task);
	
	//  ===========================================
	//  PROTECTED METHODS
	//  ===========================================
	
	/**
	 * @return Returns the loadSelectionRule.
	 */
	protected final DispatchLoadSelectionRuleIfc getLoadSelectionRule() {
		return myLoadSelectionRule;
	}

	/**
	 * @param loadSelectionRule The loadSelectionRule to set.
	 */
	protected final void setLoadSelectionRule(DispatchLoadSelectionRuleIfc loadSelectionRule) {
		myLoadSelectionRule = loadSelectionRule;
	}

	/**
	 * @return Returns the network.
	 */
	protected final TransportationNetwork getNetwork() {
		return myNetwork;
	}

	//  ===========================================
	//  PRIVATE METHODS
	//  ===========================================
	
	/** Sets the network for the dispatcher and sets the spatial model 
	 *  for the dispatcher same as that of its network.
	 * @param network The network to set.
	 */
	private final void setNetwork(TransportationNetwork network) {
		if (network == null)
			throw new IllegalArgumentException("The network must not be null");
		
		myNetwork = network;
		myNetwork.setDispatcher(this);
		setSpatialModel(myNetwork.getSpatialModel());
	}

}
