package jslx.transportlayerFinal;

import java.util.List;
import jsl.utilities.reporting.JSL;

/**
 * DriverSelectionRule class implements DriverSelectionRuleIfc.It implements selectNextDriver() method. 
 * It provides two selection rules for driver. First selection rule is based on selecting the first idle driver
 * from its idle list. The second is the select the driver based on the minimum time taken to complete the 
 * task 
 * 
 * @author snangia
 *
 */
public class DriverSelectionRule implements DriverSelectionRuleIfc{
	
	/** Constant to represent that the driver selection is to select the first driver in the idle 
	 *  list 
	 */
	public static final int SELECT_FIRST = JSL.getNextEnumConstant();
	
	/** Constant to represent that the driver selection is to select the driver which takes minimum time to complete the task 
	 *  
	 */
	public static final int SELECT_MINIMUM_DELIVERY_TIME = JSL.getNextEnumConstant();
	
	/** Represents the type(SELECT_FIRST,SELECT_MINIMUM_DELIVERY_TIME)
	 * 
	 */
	protected int mySelectionRuleType;
	
	public DriverSelectionRule() {
		
	}
	
	/**
	 *  sets the selection rule  to type which which selects the first idle driver from the idle driver list
	 */
	public final void setTypeToSelectFirst(){
	//	System.out.println("Driver Selection Rule is set as to Remove First from the Idle List at the same Location");
		mySelectionRuleType = SELECT_FIRST;
	}
	
	/**
	 * sets the selection rule  to type which which selects the on the basis on minimum task completion time from the idle driver list
	 */
	 public final void setTypeToSelectWithMinDeliverTime(){
	//	System.out.println("Driver Selection Rule is set as to Select One which takes Minimum Delivery Time");
		mySelectionRuleType = SELECT_MINIMUM_DELIVERY_TIME;
	}
	
	/**
	 * 
	 * @return true if it is SELECT_FIRST rule
	 */
	 public final boolean isSelectFirst(){
		return(mySelectionRuleType == SELECT_FIRST);	
	}
	
	/**
	 * 
	 * @return true if it is SELECT_MINIMUM_DELIVERY_TIME rule
	 */
	 public final boolean isSelectWithMinDeliveryTime(){
		return(mySelectionRuleType == SELECT_MINIMUM_DELIVERY_TIME);	
	}
	
	/**
	 * 
	 * @return the driver selection rule set
	 */
	 public final int getSelectionRuleType(){
		return(mySelectionRuleType);
	}

	/**
	 *  Selects the next driver based on the selection rule set either SELECT_FIRST or SELECT_MINIMUM_DELIVERY_TIME
	 */
	 public Driver selectNextDriver(AbstractDispatcherFTL dispatcher,TransportLocationIfc location,DispatchTask task,List<Driver> drivers){
		
		Driver driver=null;
		
		if(mySelectionRuleType==SELECT_FIRST){			
			if(drivers.isEmpty())
				driver =null;
			else {
				driver = drivers.get(0);
			}
		}
		
		if(mySelectionRuleType==SELECT_MINIMUM_DELIVERY_TIME){
			
			if(drivers.isEmpty())
				return (driver);
			else {	
				double tmin=(drivers.get(0).getExpectedTotalTaskTime(task,location));
				driver=drivers.get(0);
				for(Driver d : drivers){
					double t=d.getExpectedTotalTaskTime(task,location);
					if(t<=tmin){	
						tmin=t;
						driver=d;
						}
					}
				}
			}
			
	//	System.out.println("Driver "+driver.getId()+" at "+task.getOrigin().getName()+" is selected "+" as it takes minimum delivery task time of: "+driver.getExpectedTotalTaskTime(task,location));
			return(driver);
		}
}