package jslx.transportlayerFinal;

import jsl.modeling.elements.queue.QObject;

/** A Load represents a request to move material from one
 *  location (origin) to another location (destination).
 *  A load is sent by a specific sender to be completed.A Load
 *  a QObject.
 *
 */
public class Load extends QObject {
	
	//  =================================
	//  Attributes of a Load
	//  =================================
	
	/** The Transportation network that will serve this load
	 */
	private TransportationNetwork myNetwork;
	
	/** Represents the object that sent the load
	 */
	private LoadSenderIfc mySender;
	
	/** Represents origin of the load 
	 */
	private TransportLocationIfc myOrigin;
	
	/** Represents destination of the load 
	 */
	private TransportLocationIfc myDestination;
	
	//TODO havent implemented it anywhere
	/** Represents due date on the load. 
	 * 	Due date is the date by which the client expects the load to be delivered 
	 */
	private double myDueDate;
	
	//  ===========================================
	//  CONSTRUCTORS
	//  ===========================================
	
	/** Creates a load with no due date
	 * 
	 * @param network
	 * @param sender
	 * @param origin
	 * @param destination
	 */
	protected Load(TransportationNetwork network, LoadSenderIfc sender, TransportLocationIfc origin,
			TransportLocationIfc destination){
	}
	
	/** Creates a load with due date set on the delivery of the load
	 * 
	 * @param network
	 * @param sender
	 * @param origin
	 * @param destination
	 * @param dueDate
	 */
	protected Load(TransportationNetwork network, LoadSenderIfc sender, TransportLocationIfc origin,
			TransportLocationIfc destination, double dueDate){
		
		setNetwork(network);
		setDestination(destination);
		setOrigin(origin);
		setSender(sender);
		setDueDate(dueDate);
	}
	
	//  ===========================================
	//  PROTECTED METHODS
	//  ===========================================
    
	/**
	 * @param mySender The sender to set.
	 */
	protected final void setSender(LoadSenderIfc sender) {
		if (sender == null)
			throw new IllegalArgumentException("The LoadSender must not be null");
		mySender = sender;
	}
	
	/** Sets the Destination of the load
	 * @param destination
	 */
	protected final void setDestination(TransportLocationIfc destination) {
		if (destination == null)
			throw new IllegalArgumentException("The Location destination must not be null");
		myDestination = destination;
	}

	/** Sets the destination of the load
	 * @param origin
	*/
	protected void setOrigin(TransportLocationIfc origin) {
		if (origin == null)
			throw new IllegalArgumentException("The Location origin must not be null");
		myOrigin = origin;
	}
	
	/** Sets the Network of the load
	 * @param myNetwork The myNetwork to set.
	 */
	protected final void setNetwork(TransportationNetwork network) {
		myNetwork = network;
	}

	//	TODO havent used this method
	/** Sets the Due Date for the Load 
	 * @param dueDate The dueDate to set. 
	 */
	protected final void setDueDate(double dueDate) {
		
		myDueDate = dueDate;
	}
	
	//  ===========================================
	//  PUBLIC METHODS
	//  ===========================================
    
	
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getOrigin()
	 */
	public final TransportLocationIfc getOrigin() {
		return myOrigin;
	}
	
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getDestination()
	 */
	public final TransportLocationIfc getDestination() {
		return myDestination;
	}
	
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getSender()
	 */
	public final LoadSenderIfc getSender() {
		return mySender;
	}
	
	/* (non-Javadoc)
	 * @see transportlayer.LoadIfc#getNetwork()
	 */
	public final TransportationNetwork getNetwork() {
		return myNetwork;
	}

	/** Returns the Due Date of the Load
	 * @return returns the dueDate
	 */
	public final double getDueDate() {
		return myDueDate;
	}
}



