package jslx.transportlayerFinal;

import java.util.Observable;
import java.util.Observer;

/** Lane class is for creating lane given origin and destination. The given origin and destination must be 
 *  different otherwise an exception will be thrown 
 * 
 * @author snangia
 *
 */
public class SMLane extends Lane implements Observer {
	
	/** Creates Lane with the given origin and destination as parameters
	 * 
	 * @param origin
	 * @param destination
	 */
	public SMLane(Network network, TransportLocationIfc origin, TransportLocationIfc destination) {
		super(network, origin, destination);
		destination.attachPositionObserver(this);
		origin.attachPositionObserver(this);
		setDistance(origin.distanceTo(destination));		
	}

	/** A lane is attached as a position observer on the Locations
	 *  serving as an origin or a destitation.  If their positions
	 *  change then the lane distance needs to be recalculated and 
	 *  the network notified via updateLane()
	 * 
	 * @param arg0 The SpatialElementIfc that is changing its position
	 * @param arg1
	 */
	public void update(Observable spatialModelElement, Object arg1) {
		setDistance(getOrigin().distanceTo(getDestination()));
		getNetwork().updateLane(this);
	}
}
