package jslx.utilities.random.sp;

import jsl.utilities.random.GetValueIfc;
import jsl.utilities.random.ParametersIfc;
import jsl.utilities.random.distributions.Bernoulli;
import jsl.utilities.random.rng.RandomStreamIfc;

public class TwoSMarkovChainGeneration implements ParametersIfc, GetValueIfc, RandomStreamIfc{
    private int myPreviousEvent;
    private int FIRSTEVENT;
    private double myProbSuccessAfterSuccess;
    private double myProbSuccessAfterFailure;
    private double myProbSuccess;
    private double myProbFailure;
    private Bernoulli myBn1;
    private Bernoulli myBn0;	    
    
    public TwoSMarkovChainGeneration(){
    	this(1,0.5,0.5);
    }
    
    public TwoSMarkovChainGeneration(int firstEvent,double probSuccessAfterSuccess,double probSuccessAfterFailure){
        myBn1 = new Bernoulli(probSuccessAfterSuccess);
        myBn0 = new Bernoulli(probSuccessAfterFailure);
        FIRSTEVENT = firstEvent;
        myPreviousEvent = FIRSTEVENT;
        double [] prob = {probSuccessAfterSuccess, probSuccessAfterFailure}; 
        setParameters(prob);
    }
    public double getValue(){
        if(myPreviousEvent ==1){
        	myPreviousEvent = (int)myBn1.getValue();
            return myPreviousEvent;
        }else{
        	myPreviousEvent = (int)myBn0.getValue();
            return myPreviousEvent;
        }
    }
	public double getProbFailure() {
		return myProbFailure;
	}
	public double getProbSuccess() {
		return myProbSuccess;
	}
	public double getProbSuccessAfterFailure() {
		return myProbSuccessAfterFailure;
	}
	public double getProbSuccessAfterSuccess() {
		return myProbSuccessAfterSuccess;
	}
	public double[] getParameters() {
        double [] prob = {myProbSuccessAfterSuccess, myProbSuccessAfterFailure};
		return prob;
	}
	public void setParameters(double[] parameters) {
        myProbSuccessAfterSuccess=parameters[0];
        myProbSuccessAfterFailure=parameters[1];
        myProbFailure = 1-(myProbSuccessAfterFailure/(1-myProbSuccessAfterSuccess+myProbSuccessAfterFailure));
        myProbSuccess = 1-myProbFailure;
	}
	public void resetNextSubstream() {
		myBn1.resetNextSubstream();
		myBn0.resetNextSubstream();		
	}
	public void resetStartStream() {
		myBn1.resetStartStream();
		myBn0.resetStartStream();	
	}
	public void resetStartSubstream() {
		myBn1.resetStartSubstream();
		myBn0.resetStartSubstream();	
	}
	public void setAntithetic(boolean flag) {
		myBn1.setAntithetic(flag);
		myBn0.setAntithetic(flag);	
	}	    
}
