/*
* Created on Feb 21, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package jslx.vrnetwork;

import jsl.modeling.elements.spatial.SpatialElementIfc;

/**
 * @author rossetti
 *
 */
public class VRTask implements VRTaskIfc {

	protected VRLoadIfc myDropOffLoad;
	protected VRLoadIfc myPickUpLoad;
	protected SpatialElementIfc myLocation;
	protected LoadHandlerIfc myLoadHandler;

	/**
	 * @param location
	 */
	public VRTask(SpatialElementIfc location) {
		this(location, null, null, null);
	}

	/**
	 * @param location
	 * @param dropOffLoad
	 * @param loadHandler
	 */
	public VRTask(SpatialElementIfc location, LoadHandlerIfc loadHandler, VRLoadIfc dropOffLoad) {
		this(location, null, dropOffLoad, loadHandler);
	}
	
	/**
	 * @param location
	 * @param pickUpLoad
	 * @param loadHandler
	 */
	public VRTask(SpatialElementIfc location, VRLoadIfc pickUpLoad, LoadHandlerIfc loadHandler) {
		this(location, pickUpLoad, null, loadHandler);
	}

	/**
	 * @param location
	 * @param pickUpLoad
	 * @param dropOffLoad
	 * @param loadHandler
	 */
	public VRTask(SpatialElementIfc location, VRLoadIfc pickUpLoad, VRLoadIfc dropOffLoad) {
		this(location, pickUpLoad, dropOffLoad, null);
	}
	
	/**
	 * @param location
	 * @param pickUpLoad
	 * @param dropOffLoad
	 * @param loadHandler
	 */
	public VRTask(SpatialElementIfc location, VRLoadIfc pickUpLoad, VRLoadIfc dropOffLoad, LoadHandlerIfc loadHandler) {
		if (location == null)
			throw new IllegalArgumentException("The location of the task must not be null");
		myLocation = location;
		myDropOffLoad = dropOffLoad;
		myPickUpLoad = pickUpLoad;
		myLoadHandler = loadHandler;
	}

	/* (non-Javadoc)
	 * @see jsl.research.vrnetwork.VRTaskIfc#getDropOffLoad()
	 */
	public final VRLoadIfc getDropOffLoad() {
		return myDropOffLoad;
	}

	/* (non-Javadoc)
	 * @see jsl.research.vrnetwork.VRTaskIfc#getLoadHandler()
	 */
	public final LoadHandlerIfc getLoadHandler() {
		return myLoadHandler;
	}

	/* (non-Javadoc)
	 * @see jsl.research.vrnetwork.VRTaskIfc#getLocation()
	 */
	public final SpatialElementIfc getLocation() {
		return myLocation;
	}

	/* (non-Javadoc)
	 * @see jsl.research.vrnetwork.VRTaskIfc#getPickUpLoad()
	 */
	public final VRLoadIfc getPickUpLoad() {
		return myPickUpLoad;
	}

}
