/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.misc;

import java.util.LinkedList;
import java.util.List;

public class JSLObservable<T> implements ObservableIfc<T> {

	private List<ObserverIfc<T>> myObservers;
	
	/**
	 * 
	 */
	public JSLObservable() {
		super();
		myObservers = new LinkedList<ObserverIfc<T>>();
	}

	/** Attaches the observer to this observable.  The order of attachment
	 *  is perserved in the notification.
	 *  
	 * @param observer
	 */
	public final void attachObserver(ObserverIfc<T> observer) {
		if(observer == null)
			throw new IllegalArgumentException("The supplied observer was null.");
		
		if(myObservers.contains(observer))
			throw new IllegalArgumentException("The supplied observer is already attached to this observable.");
		
		myObservers.add(observer);
	
	}

	public final void removeObserver(ObserverIfc<T> observer) {
		if(observer == null)
			throw new IllegalArgumentException("The supplied observer was null.");
		myObservers.remove(observer);
	}

	public final void removeAllObservers() {
		myObservers.clear();
	}
	
	public final void notifyObservers(){
		notifyObservers(null);
	}
	
	public final void notifyObservers(T obj){
		for(ObserverIfc<T> o: myObservers)
			o.update(this, obj);
	}

}
