/*
* Created on Oct 6, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.modeling;

import jsl.modeling.JSLEvent;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.variable.Variable;

/**
 * @author rossetti
 *
 */
public class AddModelElementTest extends SchedulingElement {

	/**
	 * @param parent
	 */
	public AddModelElementTest(ModelElement parent) {
		super(parent, null);
		// create some model elements
		System.out.println("Constructing");
		Variable v1 = new Variable(this, "v1");
		// v12 is a child of v1
		Variable v12 = new Variable(v1, "v12");
		// v12 has two children v121, v122
		Variable v121 = new Variable(v12, "v121");
		Variable v122 = new Variable(v12, "v122");
	}
	
	protected void initialize() {
		System.out.println("initializing replication " + getModel().getReplication().getReplicationNumber());
		scheduleEvent(10.0);
		System.out.println("Initial Model element hierarchy");
		System.out.println(getModel().getModelHierarchyAsString());
	}
	
	protected void afterReplication(){
		System.out.println("Model element hierarchy afterReplication()");
		System.out.println(getModel().getModelHierarchyAsString());		
	}
	
	protected void beforeReplication(){
		System.out.println("before replication");
	}
	
	protected void handleEvent(JSLEvent e){
		System.out.println("Adding to model hierarchy");
		
		Variable v2 = new Variable(this, "v2");
		Variable v21 = new Variable(v2, "v21");
		Variable v22 = new Variable(v2, "v22");
		Variable v221 = new Variable(v22, "v221");
		
		System.out.println("Updated Model element hierarchy");
		
		v2.setRemoveFromModelPriorToRepFlag(true);
		System.out.println(getModel().getModelHierarchyAsString());			
	}

}
