/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.modeling;

import java.util.logging.*;
import java.util.*;
import jsl.modeling.*;
import jsl.utilities.reporting.JSL;

/**
 */
public class IterativeProcessTest {

	/**
	 * 
	 */
	public IterativeProcessTest() {
		super();
	}

	public static void main(String[] args) {
		JSL.LOGGER.setLevel(Level.FINE);
//		testReplication();
		testExperiment();
//		testSimulation();
		System.out.println("Done!");
	}
	
	public static void testReplication(){
		Model m = Model.createModel();
		m.setTimedUpdateInterval(1.0);
		m.turnOnLogReport();
		Replication r = new Replication(m);
		r.setLengthOfReplication(10.0);
		r.setLengthOfWarmUp(5.0);
		r.turnOnLogReport();
		r.turnOnDefaultTraceReport();
//		r.runAll();
		r.initialize();
		r.runNext();
		r.runNext();
		r.end();
		
		r.initialize();
		r.runNext();
		r.runNext();
		r.runNext();
		r.end();
		
		r.runAll();
		
	}
	
	public static void testExperiment(){
		Model m = Model.createModel();
		m.setTimedUpdateInterval(1.0);
		Experiment e = new Experiment(m);
		e.setNumberOfReplications(3);
		e.setLengthOfReplication(10.0);
		e.setLengthOfWarmUp(5.0);
		e.turnOnLogReport();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput();
		System.out.println("1st run all");
		e.runAll();
		System.out.println("2nd run all");		
		e.runAll();
	}

	public static void testSimulation(){
		Model m = Model.createModel();
		m.setTimedUpdateInterval(1.0);
		
		Experiment e1 = new Experiment(m, "Exp1");
		e1.setNumberOfReplications(3);
		e1.setLengthOfReplication(10.0);
		e1.setLengthOfWarmUp(5.0);
		//e1.turnOnLogReport();

		Experiment e2 = new Experiment(m, "Exp2");
		e2.setNumberOfReplications(2);
		e2.setLengthOfReplication(15.0);
		e2.setLengthOfWarmUp(10.0);
		//e2.turnOnLogReport();

		Experiment e3 = new Experiment(m, "Exp3");
		e3.setNumberOfReplications(1);
		e3.setLengthOfReplication(5.0);
		e3.setLengthOfWarmUp(2.0);
		//e3.turnOnLogReport();

		ArrayList<Experiment> list = new ArrayList<Experiment>();
		list.add(e1);
		list.add(e2);
		list.add(e3);
		
		ExperimentRunner s = new ExperimentRunner("Sim1", list.iterator());
		s.turnOnLogReport();
		s.runAll();
	}
}
