/*
* Created on Jan 14, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.modeling;

import jsl.modeling.ActionListenerIfc;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.SchedulingElement;

/**
 *
 */
public class SchedulingEventExamples extends SchedulingElement {

	private ActionListenerIfc myEventAction;
	
	public SchedulingEventExamples(ModelElement parent) {
		this(parent, null);
	}

	public SchedulingEventExamples(ModelElement parent, String name) {
		super(parent, name);
		myEventAction = new EventAction();
	}

	protected void initialize(){
		// schedule a type 1 event for time 10.0
		scheduleEvent(10.0, 1);
		// schedule an event that uses the EventAction listener for time 20.0
		scheduleEvent(myEventAction, 20.0);
	}
	
	protected void handleEvent(JSLEvent event){
		// get the type of event
		int type = event.getType();
		// call the appropriate event routine based on the type
		if (type == 1){
			// could call other methods
			System.out.println("Type 1 event occurred at: " + getTime());
			// schedule a type 2 event for time t + 5.0
			scheduleEvent(5.0, 2);			
		}
		if (type == 2){
			System.out.println("Type 2 event occurred at: " + getTime());			
		}
	}
	
	private class EventAction implements ActionListenerIfc {
		public void action(JSLEvent evt) {
			System.out.println("EventAction listener event occurred at: " + getTime());
			// schedule a type 2 event for time t + 15.0
			scheduleEvent(15.0, 2);			
		}
	}
	
	public static void main(String[] args) {
		Model m = Model.createModel("Event scheduling example");
		// create an instance of SchedulingEventExamples, and make sure it uses the model
		new SchedulingEventExamples(m);
        // create a replication to run the model
		Replication r = new Replication(m);
		// set the running parameters of the replication
		r.setLengthOfReplication(100.0);		
		// tell the replication to run
		r.runAll();
	}
}
