/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.modeling;

import java.util.Iterator;
import java.util.logging.Level;

import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.elements.variable.Variable;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestModelElements {

	public static void main(String[] args) {
		// turns on the level of logging
		// watch out FINEST is very verbose, e.g. every event
		JSL.LOGGER.setLevel(Level.FINEST);
		
		System.out.println("Starting");
		// create the containing model
		Model m = Model.createModel();

		// create some model elements
		
		Variable v1 = new Variable(m, "v1");
		// v12 is a child of v1
		Variable v12 = new Variable(v1, "v12");
		// v12 has two children v121, v122
		Variable v121 = new Variable(v12, "v121");
		Variable v122 = new Variable(v12, "v122");
		System.out.println("v12's parent = " + v12.getParentModelElement());
		System.out.println("v121's parent = " + v121.getParentModelElement());
		System.out.println("v122's parent = " + v122.getParentModelElement());
		
		Variable v2 = new Variable(m, "v2");
		// tell v12 to use v2 as a parent
		System.out.println("Changing v12 parent to v2");
		v12.changeParentModelElement(v2);
		System.out.println("v12's parent = " + v12.getParentModelElement());
		System.out.println("v121's parent = " + v121.getParentModelElement());
		System.out.println("v122's parent = " + v122.getParentModelElement());
		
		Iterator<ModelElement> i = m.getModelElementIterator();
		System.out.println("All Model Elements in m");
		while(i.hasNext()){
			System.out.println(i.next());
		}
		
		System.out.println(m.getModelHierarchyAsString());
		
		System.out.println("Make m2");
		Model m2 = Model.createModel();
		// tell v12 to use m2 as a parent
		System.out.println("Changing v12 parent to m2");
		v12.changeParentModelElement(m2);
		System.out.println("v12's parent = " + v12.getParentModelElement());
		System.out.println("v121's parent = " + v121.getParentModelElement());
		System.out.println("v122's parent = " + v122.getParentModelElement());

		i = m2.getModelElementIterator();
		System.out.println("All Model Elements in m2");
		while(i.hasNext()){
			System.out.println(i.next());
		}
		
		System.out.println(m2.getModelHierarchyAsString());
		
		System.out.println("Done");	
		
	}
}
