/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.modeling;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.modeling.elements.EventGenerator;
import jsl.modeling.elements.operation.*;
import jsl.modeling.elements.operation.Process;
import jsl.modeling.elements.resource.ResourceProvider;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Uniform;

public class TestOperation {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		System.out.println("Started");
		runReplication();
		System.out.println("Done");
	}
	
	public static Model buildModel(){
		
		Model m = Model.createModel();
		
		OperationList list = new OperationList(m);
		
		// make task creator
		
		TaskGenerationListener tgl = new TaskGenerationListener(list);
		
		DistributionIfc tba = new Exponential(1.0);
		
//		DistributionIfc tba = new Constant(1.0);
		
		EventGenerator creator = new EventGenerator(m, tgl, tba, tba);

	
		// make server 1
		DistributionIfc server1PT = new Exponential(0.7);
		
		ResourceProvider server1 = new ResourceProvider(m, "Server1");
		
		server1.addResource();

		Process server1Process = new Process(m, server1, server1PT);

		list.addOperation(server1Process);

		
		// make travel time
		
		DistributionIfc travelTimeD = new Uniform(0,2);
		
		Activity travelTime = new Activity(m, travelTimeD);
		
		list.addOperation(travelTime);
		
		// make server 2
		
		DistributionIfc server2PT = new Exponential(0.9);
		
		ResourceProvider server2 = new ResourceProvider(m, "Server2");
		
		server2.addResource();

		Process server2Process = new Process(m, server2, server2PT);

		list.addOperation(server2Process);
		
		return(m);
	}

	
	public static void runReplication(){
		// create the containing model
		Model m = buildModel();
		
		m.turnOnLogReport();
        
        // create a replication to run the model
		Replication r = new Replication(m);
		
		// set the running parameters of the replication
		r.setLengthOfReplication(500.0);
//     	r.setLengthOfWarmUp(100000.0);
//		r.setMaximumExecutionTime(1000);
//		r.setLengthOfWarmUp(1000.0);
		
		// turn on the desired reporting
		r.turnOnBatchReport();
//		r.turnOnDefaultTraceReport();

		// tell the replication to run
		r.runAll();
//		r.initialize();
//		r.runNext();
//		r.runNext();
 
	}
	
	public static void runExperiment(){
		// create the containing model
		Model m = buildModel();
		
		m.turnOnLogReport();

		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(20000.0);
		e.setLengthOfWarmUp(5000.0);
		
		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput();
		e.turnOnReplicationConsoleOutput();
		
		// tell the experiment to run
		e.runAll();
	}
}
