/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.modeling;

import java.util.logging.Level;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.variable.Variable;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestRemoveModelElements {

	public static void main(String[] args) {
//		JSL.LOGGER.setLevel(Level.FINEST);

		test1();

//		test2();
		
	}
	
	protected static void test1(){
		System.out.println("Starting");
		// create the containing model
		Model m = Model.createModel();

		// create some model elements
		
		Variable v1 = new Variable(m, "v1");
		// v12 is a child of v1
		Variable v12 = new Variable(v1, "v12");
		// v12 has two children v121, v122
		Variable v121 = new Variable(v12, "v121");
		Variable v122 = new Variable(v12, "v122");
		
		Variable v2 = new Variable(m, "v2");
		Variable v21 = new Variable(v2, "v21");
		Variable v22 = new Variable(v2, "v22");
		Variable v221 = new Variable(v22, "v221");
		
		System.out.println("Model element hierarchy");

		System.out.println(m.getModelHierarchyAsString());
		
		v2.removeFromModel();

		System.out.println("Model element hierarchy");

		System.out.println(m.getModelHierarchyAsString());

		System.out.println("Done");			
	}
	
	protected static void test2(){
		System.out.println("Starting");
		// create the containing model
		Model m = Model.createModel();

		// create some model elements
		AddModelElementTest t = new AddModelElementTest(m);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(5);
		e.setLengthOfReplication(200.0);
		e.runAll();

		System.out.println("Done");			
	}
		
}
