/*
* Created on Aug 5, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.modeling;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.elements.ActionSchedule;
import jsl.modeling.ActionListenerIfc;

/**
 * @author rossetti
 *
 */
public class TestSchedule {

	public static void main(String[] args) {
		System.out.println("Test Schedule");
            
		// create the containing model
		Model m = Model.createModel();

		ActionSchedule s = new ActionSchedule(m, 40.0);
		
		Action a1 = new Action("action 1");
		Action a2 = new Action("action 2");
		
		s.addAction(20.0, a1);
		s.addAction(15.0, a2);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(2);
		e.setLengthOfReplication(100.0);
		
		// turn on the desired reporting
		e.turnOnExperimentReport();
		
		// tell the experiment to run
		e.runAll();

		System.out.println("Done!");
        
	}

	static protected class Action implements ActionListenerIfc {

		public String myString;
		
		public Action(String s){
			myString = s;
		}
		
		public void action(JSLEvent evt) {
			System.out.println(evt.getTime() + "> " + myString);
		}
		
	}
}
