/*
* Created on Jan 14, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.modeling;

import jsl.modeling.ActionListenerIfc;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.SchedulingElement;

/**
 *
 */
public class TestTNOWEvents extends SchedulingElement {

	private ActionListenerIfc myEventAction;
	
	public TestTNOWEvents(ModelElement parent) {
		this(parent, null);
	}

	public TestTNOWEvents(ModelElement parent, String name) {
		super(parent, name);
		myEventAction = new EventAction();
	}

	protected void initialize(){
		// schedule an event that uses the EventAction listener for time 0.0
/*
		scheduleEvent(myEventAction, 0.0, "e1", 1);
		scheduleEvent(myEventAction, 0.0, "e2", 2);
		scheduleEvent(myEventAction, 0.0, "e3", 3);
		scheduleEvent(myEventAction, 0.0, "e4", 4);
*/
	
        scheduleEvent(myEventAction, 0.0, "e1");
		scheduleEvent(myEventAction, 0.0, "e2");
		scheduleEvent(myEventAction, 0.0, "e3");
		scheduleEvent(myEventAction, 0.0, "e4");

	}
		
	private class EventAction implements ActionListenerIfc {
		public void action(JSLEvent evt) {
			System.out.println(getTime()+"> " + evt.getName() + " id = " + evt.getId() + " priority = " + evt.getPriority());
		}
	}
	
	public static void main(String[] args) {
		Model m = Model.createModel("Event scheduling example");
		// create an instance of SchedulingEventExamples, and make sure it uses the model
		new TestTNOWEvents(m);
        // create a replication to run the model
		Replication r = new Replication(m);
		// set the running parameters of the replication
		r.setLengthOfReplication(100.0);		
		// tell the replication to run
		r.runAll();
	}
}
