/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.modeling;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Exponential;
import jsl.modeling.elements.*;

/**
 */
public class TestWorkCenter {

	public static void main(String[] args) {

		System.out.println("Test Work Center");
        
//		JSL.LOGGER.setLevel(Level.FINER);
   
    	testReplication();
//		testExperiment();
        
		System.out.println("Done!");	
	}
	
	private static Model makeModel(){
		// create the containing model
		Model m = Model.createModel();
		
		Exponential a = new Exponential(1.0);
		Exponential s = new Exponential(0.5);
				
		EntityGenerator eg = new EntityGenerator(m, a, a, "Entity Creator");
		
		WorkCenter wc = new WorkCenter(m, "Work center", 1);
		wc.setServiceDistributionInitialRandomSource(s);
		
		eg.setDefaultEntityReceiver(wc);
		
		return(m);
	}
	
	public static void testReplication(){

		Model m = makeModel();
		
		m.turnOnLogReport();
        
        // create a replication to run the model
		Replication r = new Replication(m);

		// set the running parameters of the replication
		r.setLengthOfReplication(11000.0);
		r.setLengthOfWarmUp(1000.0);
		
		// turn on the desired reporting
		r.turnOnBatchReport();
		
		// tell the replication to run
		r.runAll();
 
	}
	
	public static void testExperiment(){

		Model m = makeModel();

		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(10);
		e.setLengthOfReplication(20000.0);
		e.setLengthOfWarmUp(5000.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}

}
