package testing.parameters;

import java.util.Map.Entry;
import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DUniform;
import jsl.utilities.random.distributions.DistributionIfc;
import jslx.forecasting.IterativeDataSource;

public class DataRQInventorySimParameterized extends RQInventorySimParameterized {
        
        public DataRQInventorySimParameterized(ModelElement parent,double leadTime,double[] data) {
		this(parent,new Constant(leadTime),1,1,1,1,data);
	}
        
        public DataRQInventorySimParameterized(ModelElement parent,double leadTime,IterativeDataSource ds) {
		this(parent,new Constant(leadTime),1,1,1,1,ds);
	}
        
        public DataRQInventorySimParameterized(ModelElement parent,DistributionIfc leadTime,
			int reorderPt, int reorderQty, int initialLevel,int periodLength,double[] data) {
		super(parent,new Constant(1), leadTime, reorderPt, reorderQty, initialLevel);
		//System.out.println(this.myReorderPt+"\t"+this.myReorderQty);
		myDemandGenerator=new DataDemandGenerator(parent,myItemType,periodLength,data,"Gen");
		myDemandGenerator.setDemandFillerFinder(this);
	}
        
     	public DataRQInventorySimParameterized(ModelElement parent,DistributionIfc leadTime,
			int reorderPt, int reorderQty, int initialLevel,double periodLength,IterativeDataSource ds) {
		super(parent,new Constant(1), leadTime, reorderPt, reorderQty, initialLevel);
		//System.out.println(this.myReorderPt+"\t"+this.myReorderQty);
		myDemandGenerator=new DataDemandGenerator(parent,myItemType,periodLength,ds,"Gen");
		myDemandGenerator.setDemandFillerFinder(this);
	}
	
        public double getDemandPerPeriod(){
            double average=((DataDemandGenerator)myDemandGenerator).getAverage();
            //System.out.println("periodLength "+myPeriodLength+"\t"+average);
            return myPeriodLength/average;
    }
        
	public static void main(String[] args) {
		System.out.println("Running...");
		// create the containing model
		Model m = Model.createModel();
		// create the model element and attach it to the main model
		DistributionIfc lt = new Constant(2);
		int rpt = 2;
		int qty = 1;
		int level = rpt+qty;
		int periodLength=1;
		int numObs=1000;
		double[] demandData=new double[numObs];
		DUniform du=new DUniform(0,1);
		DUniform du2=new DUniform(1,2);
		for(int i=0;i<numObs;i++){
			if(du.getValue()==1.0)
				demandData[i]=du2.getValue();
			else
				demandData[i]=0;
		}
		DataRQInventorySimParameterized rq = new DataRQInventorySimParameterized(m,lt, rpt, qty, level,periodLength,demandData);
		//System.out.println(rq.myReorderPt+"\t"+rq.myReorderQty);
		StringBuilder sb = new StringBuilder();
		m.getInitializationOrderAsString(sb);
		Experiment e = new Experiment(m);
		e.setNumberOfReplications(1);
		e.turnOnExperimentConsoleOutput();
		// tell the experiment to run
		e.runAll();
		for(Entry<String,Double> entry:rq.getResponses().getAllResponses().entrySet())
			System.out.println(entry.getKey()+"\t"+entry.getValue());
	}
}
