package testing.parameters;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import jsl.modeling.Model;
import testing.parameters.ExperimentableIfc;
import testing.parameters.Responses;
import testing.parameters.optimize.AcceptanceCriterionIfc;
import testing.parameters.optimize.Constraint;
import testing.parameters.optimize.FunctionOfModelResponsesIfc;
import testing.parameters.optimize.GreaterOrEqual;
import testing.parameters.optimize.LessOrEqual;
import testing.parameters.optimize.WeightedFunctionOfModelResponses;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Poisson;

public class EnumerationOptimizer {
	
	private TreeMap<String,double[]> values;
	private double[] bestSolution;
	private double bestObjective;
	private ExperimentableIfc model;
	private int minMax;
	private FunctionOfModelResponsesIfc objectiveFunction;
	private List<Constraint> constraints;
	private Responses bestResponses;
	
	public EnumerationOptimizer(Map<String,double[]> values,ExperimentableIfc model){
		this.values=new TreeMap<String,double[]>();
		for(Entry<String,double[]> e:values.entrySet())
			this.values.put(e.getKey(),e.getValue());
		this.model=model;
		minMax=1;
		constraints=new ArrayList<Constraint>();
                System.out.println("created");
	}
	
	public void setMaximize(){
		minMax=-1;
	}
	
	public void setMinimize(){
		minMax=1;
	}
	
	private boolean checkConstraints(Responses r){
		for(Constraint c:constraints){
			if(!c.feasible(r))
				return false;
		}
		return true;
	}
	
	public void optimize(){
		double[][] combinations=getAllCombinations();
		bestObjective=minMax*Double.MAX_VALUE;
		bestSolution=combinations[0];
		bestResponses=model.getResponses();
		//System.out.println("BEST: "+Arrays.toString(getBestSolution()));
                System.out.println("comb= "+Arrays.deepToString(combinations));
		for(double[] combination:combinations){
                        //System.out.println("comb= "+Arrays.toString(combination));
			Parameters p=model.getParameters();
			int i=0;
			for(String paramName:values.keySet())
				p.setDouble(paramName,combination[i++]);
			model.setParameters(p);
			model.runModel();
			Responses r=model.getResponses();
			double objectiveValue=objectiveFunction.getCombinedResponse(r);
                        System.out.println("comb= "+Arrays.toString(combination)+"\tobj="+objectiveValue);
			//System.out.println("responses="+r);
                        if(minMax*objectiveValue<minMax*bestObjective){
				if(checkConstraints(r)){
					//System.out.println("change");
					bestObjective=objectiveValue;
					bestSolution=combination;
					bestResponses=r;
				}
			}
		}
	}
	
	public double getBestObjectiveValue(){
		return bestObjective;
	}
	
	public double[] getBestSolution(){
		return bestSolution;
	}
	
	private double[][] getAllCombinations(){
		double[][] arrays=new double[values.size()][];
		int q=0;
		for(double[] array:values.values())
			arrays[q++]=array;
        int idx = 1;
        for (int i = 0; i < arrays.length; idx *= arrays[i].length, i++);
        double[][] cartProd=new double[idx][];
        int k=0;
        while (idx-- > 0) {
            int j = 1;
            int i=0;
            cartProd[k]=new double[arrays.length];
            for (double[] a : arrays) {
                //System.out.print(a[(idx / j) % a.length] +"\t");
                cartProd[k][i]=a[(idx / j) % a.length]; 
                j *= a.length;
                i++;
            }
            k++;
        }
        return cartProd;
	}
	
	public void setWeightedObjective(Map<String,Double> functionTerms){
		WeightedFunctionOfModelResponses fn=new WeightedFunctionOfModelResponses(model);
		for(Entry<String,Double> e:functionTerms.entrySet())
			fn.addWeightedReponse(e.getKey(),e.getValue());
		this.objectiveFunction=fn;
	}
	
	public void setSingleObjective(String responseName){
		WeightedFunctionOfModelResponses fn=new WeightedFunctionOfModelResponses(model);
		fn.addWeightedReponse(responseName,1.0);
		this.objectiveFunction=fn;
	}
	
	public void addWeightedSumConstraint(Map<String,Double> functionTerms,AcceptanceCriterionIfc acceptance,double rhs){
		WeightedFunctionOfModelResponses fn=new WeightedFunctionOfModelResponses(model);
		for(Entry<String,Double> e:functionTerms.entrySet())
			fn.addWeightedReponse(e.getKey(),e.getValue());
		this.constraints.add(new Constraint(fn,acceptance,rhs));
	}
	
	public void addVariableConstraint(String responseName,AcceptanceCriterionIfc acceptance,double rhs){
		WeightedFunctionOfModelResponses fn=new WeightedFunctionOfModelResponses(model);
		fn.addWeightedReponse(responseName,1);
		this.constraints.add(new Constraint(fn,acceptance,rhs));
	}
	
        public Responses getBestResponses(){
            return bestResponses;
        }
        
	public static void main(String[] args) {
		Model m = Model.createModel();
		//Poisson d = new Poisson(1.726);  // mean demand during replenishment       
                //RQInventorySimParameterized m = new RQInventorySimParameterized(3,4,d);
		DistributionIfc tbd = new Exponential(1.0/3.6);
		//DistributionIfc tbd = new Exponential(1.0/100.0); // a lot of back ordering
		DistributionIfc lt = new Constant(0.5);
		int rpt = 2;
		int qty = 1;
		int level = rpt+qty;
		RQInventorySimParameterized rq = new RQInventorySimParameterized(m, tbd, lt, rpt, qty, level);
		rq.setUnitCost(150.0);
		rq.setHoldingCostRatePerPeriod(1.0);
		rq.setCostPerOrder(65.138);
		rq.setBackOrderCostPerUnitPerPeriod(116.5);
		rq.setCostPerStockout(40.0);
		//rq.setLeadTime(30.0); // 45.0 days
		//rq.setPeriodLength(365.0); // 365 days per year
        Map<String,double[]> values=new TreeMap<String,double[]>();
        values.put("Reorder Point", new double[]{1,2,3,4,5});
        values.put("Reorder Quantity", new double[]{1,2,3,4,5});
        Map<String, Double> objectiveTerms=new HashMap<String,Double>();
        objectiveTerms.put("Backorder model total cost per period", 1.0);
        objectiveTerms.put("Stockout cost per period", 1.0);
        EnumerationOptimizer e=new EnumerationOptimizer(values,rq);
        e.setWeightedObjective(objectiveTerms);
        e.addVariableConstraint("Fill rate", new GreaterOrEqual(),.95);
        e.setMinimize();
        e.optimize();
        //System.out.println(e.getBestObjectiveValue());
        System.out.println(Arrays.toString(e.getBestSolution()));
        System.out.println(e.getBestObjectiveValue());
        System.out.println(e.bestResponses);
	}
}
