/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.parameters;

import jsl.utilities.IdentityIfc;


/** Implementors of this interface should be able to return an instance of
 *  the Parameters class and should be able to take in an instance of Parameters
 *  and use it correctly to set the internal state of the implementation.
 *
 *
 */
public interface ParameterizedIfc extends IdentityIfc {

	/** Returns a valid instance of Parameters that can be used with
	 *  this ParameterizedIfc
	 * 
	 * @return
	 */
	public Parameters getParameters();
	
	/** Takes in a valid instance of Parameters for this class
	 *  If parameters is null or if it was not created by this
	 *  class this method should throw an IllegalArgumentException
	 * 
	 * @param parameters
	 */
	public void setParameters(Parameters parameters);
}
