/*
 * RQInventoryModel.java
 *
 * Created on May 28, 2003, 1:20 PM
 */

package testing.parameters;

import jsl.utilities.*;
import jsl.utilities.random.distributions.LossFunctionDistributionIfc;
import jsl.utilities.random.distributions.Poisson;
import jslx.inventory.RQInventoryModel;

/**
 *
 * @author  rossetti
 */
public class RQInventoryModelParameterized extends RQInventoryModel 
        implements ExperimentableIfc{
    
    
	/** Creates a new instance of RQInventoryModel */
    public RQInventoryModelParameterized(double reorderPt, double reorderQty, LossFunctionDistributionIfc distribution) {
        super(reorderPt,reorderQty,distribution);
    }
    
    public Parameters getParameters() {
        return new RQParameters(this);
    }

    public void setParameters(Parameters p) {
        myReorderPt=p.getDouble("Reorder Point");
        myReorderQty=p.getDouble("Reorder Quantity");
        myLeadTime = p.getDouble("Lead Time");
        myUnitCost = p.getDouble("Unit Cost");
        myCostPerOrder = p.getDouble("Cost Per Order");    
        myHoldingCostRatePerPeriod = p.getDouble("Holding Cost Rate Per Period");
        myBackOrderCostPerUnitPerPeriod = p.getDouble("Back Order Cost Per Unit Per Period");
        myCostPerStockout = p.getDouble("Cost Per Stockout");
        myPeriodLength = p.getDouble("Period Length");
        myDesiredFillRate=p.getDouble("Desired Fill Rate");
    }
    
    public String getName() {
        return "RQInventoryModelParameterized";
    }

    public long getId() {
        return 1;
    }
    
    public class RQParameters extends Parameters{
        public RQParameters(ParameterizedIfc p){
            super(p);
            this.myParameters.put("Reorder Point",Double.toString(RQInventoryModelParameterized.this.myReorderPt));
            this.myParameters.put("Reorder Quantity",Double.toString(RQInventoryModelParameterized.this.myReorderQty));
            this.myParameters.put("Lead Time",Double.toString(RQInventoryModelParameterized.this.myLeadTime));
            this.myParameters.put("Unit Cost",Double.toString(RQInventoryModelParameterized.this.myUnitCost));
            this.myParameters.put("Cost Per Order",Double.toString(RQInventoryModelParameterized.this.myCostPerOrder));
            this.myParameters.put("Holding Cost Rate Per Period",Double.toString(RQInventoryModelParameterized.this.myHoldingCostRatePerPeriod));
            this.myParameters.put("Back Order Cost Per Unit Per Period",Double.toString(RQInventoryModelParameterized.this.myBackOrderCostPerUnitPerPeriod));
            this.myParameters.put("Cost Per Stockout",Double.toString(RQInventoryModelParameterized.this.myCostPerStockout));
            this.myParameters.put("Period Length",Double.toString(RQInventoryModelParameterized.this.myPeriodLength));
            this.myParameters.put("Desired Fill Rate",Double.toString(RQInventoryModelParameterized.this.myDesiredFillRate));
        }


    }

    public Responses getResponses() {
        Responses r=new Responses();
        r.setResponse("Reorder point", myReorderPt);
        r.setResponse("Reorder Quantity", myReorderQty);
        r.setResponse("Fill rate", getFillRate());
        r.setResponse("Probability of Stockout", getProbStockout());
        r.setResponse("Expected backorders", getExpectedBackOrders());
        r.setResponse("Expected customer waittime", getExpectedCustomerWaitTime());
        r.setResponse("Expected on hand inventory", getExpectedOnHandInventory());
        r.setResponse("Order Frequency", getOrderFrequency());
        r.setResponse("Order cost per period", getOrderCostPerPeriod());
        r.setResponse("Holding cost per period", getHoldingCostPerPeriod());
        r.setResponse("Backorder cost per period", getBackOrderCostPerPeriod());
        r.setResponse("Stockout cost per period", getStockoutCostPerPeriod());
        r.setResponse("Backorder model total cost per period", getBackOrderModelTotalCostPerPeriod());
        r.setResponse("Stockout model total cost per period", getStockoutModelTotalCostPerPeriod());
        r.setResponse("Approximate EOQ", getApproxEOQ());
        r.setResponse("Approximate optimal reorder point via backorder model", getApproxOptimalReorderPtViaBackOrderModel());     
        r.setResponse("Approximate optimal reorder point via stockout model", getApproxOptimalReorderPtViaStockoutModel());
        return r;
    }
    
    public static void main( String[] args ){
        System.out.println("Started!");
        
        Poisson d = new Poisson(1.726);  // mean demand during replenishment       
       
        RQInventoryModelParameterized m = new RQInventoryModelParameterized(3,4,d);
  //      m.setUnitCost(150.0);
 //       m.setHoldingCostRatePerPeriod(1.0);
  //      m.setCostPerOrder(65.138);
 //       m.setBackOrderCostPerUnitPerPeriod(116.5);
  //      m.setCostPerStockout(40.0);
   //     m.setLeadTime(30.0); // 45.0 days
   //     m.setPeriodLength(365.0); // 365 days per year
        
        System.out.println(m);
        
        System.out.println("Done!");
    }

	public void runModel() {
		//do nothing
	}
  
}
