package testing.parameters;


import java.util.HashMap;
import java.util.Map;
import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import testing.parameters.ExperimentableIfc;
import testing.parameters.GetResponsesIfc;
import testing.parameters.Responses;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jslx.supplychain.inventorylayer.BackLogQueue;
import jslx.supplychain.inventorylayer.Demand;
import jslx.supplychain.inventorylayer.DemandFillerFinderIfc;
import jslx.supplychain.inventorylayer.DemandFillerIfc;
import jslx.supplychain.inventorylayer.DemandGenerator;
import jslx.supplychain.inventorylayer.DemandSenderIfc;
import jslx.supplychain.inventorylayer.Inventory;
import jslx.supplychain.inventorylayer.InventoryPolicyAbstract;
import jslx.supplychain.inventorylayer.InventoryPolicyReorderPointReorderQuantity;
import jslx.supplychain.inventorylayer.ItemType;
import jslx.supplychain.inventorylayer.LeadTimeDemandFiller;

public class RQInventorySimParameterized extends ModelElement implements DemandFillerFinderIfc,ExperimentableIfc{
	protected ItemType myItemType;
	protected DemandGenerator myDemandGenerator;
	protected Inventory myInventory;
	protected InventoryPolicyAbstract myInventoryPolicy;
	protected LeadTimeDemandFiller myLeadTimeDemandFiller;
        protected Experiment e;
	
    protected double myUnitCost=1.0;
    
    protected double myCostPerOrder=1.0;
    
    protected double myHoldingCostRatePerPeriod=1.0;
        
    protected double myBackOrderCostPerUnitPerPeriod=1.0;
    
    protected double myCostPerStockout=1.0;
    
    protected double myLeadTime=1.0;
    protected double myPeriodLength=1.0;
    protected double myDemandMean=1.0;
    protected double myDesiredFillRate=1.0;
    private Responses inventoryResponses;
    
	public RQInventorySimParameterized(ModelElement parent,
			DistributionIfc timebtwDemand, DistributionIfc leadTime,
			int reorderPt, int reorderQty, int initialLevel) {
		this(parent, timebtwDemand, leadTime, reorderPt, reorderQty, initialLevel,null);
		//System.out.println(this.myReorderPt+"\t"+this.myReorderQty);
	}
	
	public RQInventorySimParameterized(ModelElement parent, DistributionIfc timebtwDemand, DistributionIfc leadTime, 
            int reorderPt, int reorderQty, int initialLevel, String name) {
		//super(parent,timebtwDemand,leadTime,reorderPt,reorderQty,initialLevel,name);
		super(parent, name);

		// define the type
		myItemType = new ItemType(this, "Type-A");
		
		// define the filler
		myLeadTimeDemandFiller = new LeadTimeDemandFillerGetResponses(this);
		myLeadTimeDemandFiller.addLeadTime(myItemType,leadTime);
		
		myInventoryPolicy = new InventoryPolicyReorderPointReorderQuantity(this, reorderPt, reorderQty);
		myInventory = new InventoryGetResponses(this, myItemType, myInventoryPolicy, initialLevel);
	    myInventory.setDemandFillerFinder(this);
		BackLogQueue bpolicy = new BackLogQueue(myInventory);
		
		// define the demand generator
		myDemandGenerator = new DemandGenerator(this, myItemType, timebtwDemand, timebtwDemand);
		myDemandGenerator.setDemandFillerFinder(this);
		myDemandMean=timebtwDemand.getMean();
		myUnitCost = 1.0;
        myCostPerOrder = 1.0;
        myHoldingCostRatePerPeriod = 1.0;
        myBackOrderCostPerUnitPerPeriod = 1.0;
        myCostPerStockout = 1.0;
        myLeadTime = 1.0;
        myPeriodLength = 1.0;
	}
	
	public Parameters getParameters() {
        return new RQParameters(this);
    }
    
    public void setWarmUpPeriod(double warmUp){
        //e.
    }
    
    public void setParameters(Parameters p) {
    	double[] initPolicyParams=new double[]{p.getDouble("Reorder Point"),p.getDouble("Reorder Quantity")};
        myInventoryPolicy.setInitialPolicyParameters(initPolicyParams);
        //myLeadTime = p.getDouble("Lead Time");
        myUnitCost = p.getDouble("Unit Cost");
        myCostPerOrder = p.getDouble("Cost Per Order");    
        myHoldingCostRatePerPeriod = p.getDouble("Holding Cost Rate Per Period");
        myBackOrderCostPerUnitPerPeriod = p.getDouble("Back Order Cost Per Unit Per Period");
        myCostPerStockout = p.getDouble("Cost Per Stockout");
        myPeriodLength = p.getDouble("Period Length");
        //myDesiredFillRate=p.getDouble("Desired Fill Rate");
    }
	
	class RQParameters extends Parameters{
        public RQParameters(ParameterizedIfc p){
            super(p);
            double[] initPolicyParams=myInventoryPolicy.getInitialPolicyParameters();
            this.myParameters.put("Reorder Point",Double.toString(initPolicyParams[0]));
            this.myParameters.put("Reorder Quantity",Double.toString(initPolicyParams[1]));
            this.myParameters.put("Unit Cost",Double.toString(RQInventorySimParameterized.this.myUnitCost));
            this.myParameters.put("Cost Per Order",Double.toString(RQInventorySimParameterized.this.myCostPerOrder));
            this.myParameters.put("Holding Cost Rate Per Period",Double.toString(RQInventorySimParameterized.this.myHoldingCostRatePerPeriod));
            this.myParameters.put("Back Order Cost Per Unit Per Period",Double.toString(RQInventorySimParameterized.this.myBackOrderCostPerUnitPerPeriod));
            this.myParameters.put("Cost Per Stockout",Double.toString(RQInventorySimParameterized.this.myCostPerStockout));
            this.myParameters.put("Period Length",Double.toString(RQInventorySimParameterized.this.myPeriodLength));
            //this.myParameters.put("Desired Fill Rate",Double.toString(RQInventorySimParameterized.this.myDesiredFillRate));
        }
    }
    
    public static Map<String,Double> getDefaultParametersMap(){
        Map<String,Double> params=new HashMap<String,Double>();
        params.put("Reorder Point",1.0);
        params.put("Reorder Quantity",1.0);
        params.put("Unit Cost",1.0);
        params.put("Cost Per Order",1.0);
        params.put("Holding Cost Rate Per Period",1.0);
        params.put("Back Order Cost Per Unit Per Period",1.0);
        params.put("Cost Per Stockout",1.0);
        params.put("Period Length",1.0);
        //params.put("Desired Fill Rate",1.0);
        return params;
    }
    
    public Responses getResponses() {
    	inventoryResponses=((GetResponsesIfc)myInventory).getResponses();
        Responses r=new Responses();
        double[] initPolicyParams=myInventoryPolicy.getInitialPolicyParameters();
        r.setResponse("Reorder point", initPolicyParams[0]);
        r.setResponse("Reorder Quantity", initPolicyParams[1]);
        r.setResponse("Fill rate", getFillRate());
        r.setResponse("Probability of Stockout", getProbStockout());
        r.setResponse("Expected backorders", getExpectedBackOrders());
        r.setResponse("Expected customer waittime", getExpectedCustomerWaitTime());
        r.setResponse("Expected on hand inventory", getExpectedOnHandInventory());
        r.setResponse("Order Frequency", getOrderFrequency());
        r.setResponse("Order cost per period", getOrderCostPerPeriod());
        r.setResponse("Holding cost per period", getHoldingCostPerPeriod());
        r.setResponse("Backorder cost per period", getBackOrderCostPerPeriod());
        r.setResponse("Stockout cost per period", getStockoutCostPerPeriod());
        r.setResponse("Backorder model total cost per period", getBackOrderModelTotalCostPerPeriod());
        r.setResponse("Stockout model total cost per period", getStockoutModelTotalCostPerPeriod());
        return r;
    }
    
    public double getDemandPerPeriod(){
        return myPeriodLength/myDemandMean;
    }
    
    public double getOrderFrequency(){
        return(getDemandPerPeriod()/myInventoryPolicy.getInitialPolicyParameters()[1]);
    }
    
    public double getFillRate() {
        return inventoryResponses.getResponse("Fill rate");
    }
    
    public double getProbStockout(){
        return(1.0-getFillRate());
    }
    
    public double getExpectedStockoutsPerPeriod(){
        double d = getDemandPerPeriod();
        return(d*getProbStockout());
    }
    
    public double getExpectedBackOrders() {
    	return inventoryResponses.getResponse("Average on order");
    }
    
    public double getExpectedOnHandInventory() {
    	return inventoryResponses.getResponse("Average on hand");
    }
    
    public double getExpectedCustomerWaitTime(){
        double d = getDemandPerPeriod();
        double b = getExpectedBackOrders(); // expected number of back orders
        //System.out.println(b+"\t"+d);
        return(b/d);
    }
    
    //costs
    public void setCostPerOrder(double costPerOrder){
        myCostPerOrder = costPerOrder;
    }
    
    public double getCostPerOrder(){
        return(myCostPerOrder);
    }
    
    public void setUnitCost(double unitCost){
        myUnitCost = unitCost;
    }
    
    public double getUnitCost(){
        return(myUnitCost);
    }
        
    public void setHoldingCostRatePerPeriod(double holdingCostRatePerPeriod){
        myHoldingCostRatePerPeriod = holdingCostRatePerPeriod;
    }
    
    public double getHoldingCostRatePerPeriod(){
        return(myHoldingCostRatePerPeriod);
    }
    
    public double getHoldingCostPerUnitPerPeriod(){
        return(myUnitCost*myHoldingCostRatePerPeriod);
    }
    
    public void setBackOrderCostPerUnitPerPeriod(double backOrderCostPerUnitPerPeriod){
        myBackOrderCostPerUnitPerPeriod = backOrderCostPerUnitPerPeriod;
    }
    
    public double getBackOrderCostPerUnitPerPeriod(){
        return(myBackOrderCostPerUnitPerPeriod);
    }
    
    public void setCostPerStockout(double costPerStockout){
        myCostPerStockout = costPerStockout;
    }
    
    public double getCostPerStockout(){
        return(myCostPerStockout);
    }
    
    public double getOrderCostPerPeriod(){
        return(myCostPerOrder*getOrderFrequency());
    }
    
    public double getHoldingCostPerPeriod(){
        return(getHoldingCostPerUnitPerPeriod()*getExpectedOnHandInventory());
    }
    
    public double getBackOrderCostPerPeriod(){
        return(myBackOrderCostPerUnitPerPeriod*getExpectedBackOrders());
    }
    
    public double getStockoutCostPerPeriod(){
        return(myCostPerStockout*getExpectedStockoutsPerPeriod());
    }
    
    public double getBackOrderModelTotalCostPerPeriod(){
        return(getOrderCostPerPeriod() + getHoldingCostPerPeriod() + getBackOrderCostPerPeriod());
    }
    
    public double getStockoutModelTotalCostPerPeriod(){
        return(getOrderCostPerPeriod() + getHoldingCostPerPeriod() + getStockoutCostPerPeriod());
    }
	
	public void runModel() {
		Experiment e = new Experiment(this.getModel());
		e.setNumberOfReplications(30);
		e.setLengthOfReplication(50000.0);
		e.setLengthOfWarmUp(10000.0);
		e.runAll();
	}

	public DemandFillerIfc findDemandFiller(Demand demand) {
		DemandSenderIfc sender = demand.getSender();
		if (sender == myDemandGenerator)
			return(myInventory);
		else
			return(myLeadTimeDemandFiller);
	}
	
	public static void main(String[] args){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		DistributionIfc tbd = new Exponential(1.0/3.6);
		//DistributionIfc tbd = new Exponential(1.0/100.0); // a lot of back ordering
		DistributionIfc lt = new Constant(0.5);
		int rpt = 2;
		int qty = 1;
		int level = rpt+qty;
		RQInventorySimParameterized rq = new RQInventorySimParameterized(m, tbd, lt, rpt, qty, level);
		
//		m.turnOnLogReport();
		//System.out.println(m.getModelHierarchyAsString());
		//System.out.println("-----");
		StringBuilder sb = new StringBuilder();
		m.getInitializationOrderAsString(sb);
		//System.out.println(sb);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(2);
		e.setLengthOfReplication(1000.0);
		e.setLengthOfWarmUp(100.0);
//		e.setNumberOfReplications(1);
//		e.setLengthOfReplication(10.0);

		e.turnOnExperimentConsoleOutput();
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		//e.turnOnExperimentReport("RQInventoryModel");
//		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
		//for(String name:rq.getParameters().keySet())
		//	System.out.println(name+"\t"+rq.getParameters().getDouble(name));
		//System.out.println("****");
		//for(Entry<String,Double> entry:rq.getResponses().getAllResponses().entrySet())
		//	System.out.println(entry.getKey()+"\t"+entry.getValue());
	}
}
