/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.random;

import jsl.utilities.random.*;
import jslx.utilities.random.ar.AR1Normal;
import jsl.utilities.random.rng.AR1CorrelatedRng;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.rng.RngIfc;
import jsl.utilities.statistic.Statistic;

/**
 *
 */
public class TestCorrelatedDistribution {

	/**
	 * 
	 */
	public TestCorrelatedDistribution() {
		super();
	}

	public static void main(String[] args) {
		
		Statistic uStat = new Statistic();
		Statistic xStat = new Statistic();
		
		// create a correlated random number generator
		RngIfc rng = new AR1CorrelatedRng(0.8);
		// generate some correlated uniforms
		for (int i = 1; i <= 1000; i++){
			double u = rng.randU01();
			uStat.collect(u);
			System.out.println("u(" + i + ")= " + "\t" +  u);
		}
		
		// create a distribution that uses a correlated rng
		DistributionIfc d1 = new Exponential(2.0, rng);
		System.out.println();
		System.out.println("Generate some correlated exponentials");
		for (int i = 0; i < 100; i++) {
			double x = d1.getValue();
			xStat.collect(x);
			System.out.println("x(" + i + ")= " + "\t" +  x);
		}
		System.out.println();
		System.out.println(uStat);
		System.out.println(xStat);
		
		Statistic aStat = new Statistic();
		// generate an AR(1) process
		AR1Normal arn = new AR1Normal(2, 2, 0.6);
		for (int i = 1; i <= 1000; i++){
			double a = arn.getValue();
			aStat.collect(a);
			System.out.println("a(" + i + ")= " + "\t" +  a);
		}
		
		System.out.println("Done");
	}
}
