/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.random;

import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Distribution;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Gamma;
import jsl.utilities.random.distributions.Normal;
import jsl.utilities.random.distributions.Triangular;
import jsl.utilities.statistic.Statistic;

/**
 *
 */
public class TestDistribution {

	/**
	 * 
	 */
	public TestDistribution() {
		super();
	}

	public static void main(String[] args) {

		DistributionIfc n = new Normal(2.0, 4.0);

		System.out.println("normal mean = " + n.getMean());
		System.out.println("normal variance = " + n.getVariance());
		System.out.println("pdf(2) = " + n.pdf(2.0));

		System.out.println("cdf(3) = " + n.cdf(3.0));

		System.out.println("Normal.stdNormalCDF = " + Normal.stdNormalCDF(1.0));
		System.out.println("Normal.stdNormalCDF = " + Normal.stdNormalCDF(2.0));
		System.out.println("Normal.stdNormalCDF = " + Normal.stdNormalCDF(3.0));

		System.out.println("Normal.stdNormalInvCDF(.95) = " + Normal.stdNormalInvCDF(.95));
		System.out.println("Normal.stdNormalInvCDF(.5) = " + Normal.stdNormalInvCDF(.5));
		System.out.println("Normal.stdNormalInvCDF(.05) = " + Normal.stdNormalInvCDF(.05));

		for (int i = 1; i <= 5; i++)
			System.out.println("x(" + i + ")= " + n.getValue());

		DistributionIfc d1 = new Exponential();
		DistributionIfc d2 = new Exponential();
		System.out.println("Two different exponentials with different random number generators");
		System.out.println("Note that they produce different random numbers");
		for (int i = 0; i < 5; i++) {
			System.out.println("d1 = " + d1.getValue() + "\t d2 = " + d2.getValue());
		}

		System.out.println();

		d1.resetStartStream();
		d2.resetStartStream();
		System.out.println("Resetting to the starting stream simply");
		System.out.println("causes them to repeat the above.");

		for (int i = 0; i < 5; i++) {
			System.out.println("d1 = " + d1.getValue() + "\t d2 = " + d2.getValue());
		}

		System.out.println();

		Distribution t1 = new Exponential();

		Distribution t2 = new Exponential();

		System.out.println("Two different exponentials with different random number generators.");
		System.out.println("Now set them so they use the same generator.");

		t2.setRandomNumberGenerator(t1.getRandomNumberGenerator());

		for (int i = 0; i < 5; i++) {
			System.out.println("t1 = " + t1.getValue());
		}

		System.out.println();
		System.out.println("don't reset start stream, should be different");
		System.out.println("since t2 continues using the same stream");
		for (int i = 0; i < 5; i++) {
			System.out.println("t2 = " + t2.getValue());
		}

		t2.resetStartStream();
		System.out.println();
		System.out.println("reset start stream, should be same");
		System.out.println("since t2 starts back where t1 started");
		for (int i = 0; i < 5; i++) {
			System.out.println("t2 = " + t2.getValue());
		}

		DistributionIfc ad = new Gamma(0.0266494848851034, 1.34756065089927);

		System.out.println("Generating some gammas...");
		for (int i = 1; i <= 10; i++) {
			double rv1 = ad.getValue();
			double rv2 = ad.getValue();
			System.out.println(rv1 + "\t" + rv2);
			//			JSL.out(rv1 + "\t" + rv2);
		}
		
		DistributionIfc sd = new Triangular(0.540712193770341, 0.540712193770341, 2.35679921660711);
		System.out.println("Computing statistics on a Triagular RV");
		System.out.println(new Statistic(sd.getSample(1000)));

		System.out.println("Done");
	}
}
