/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
*
* Copyright (c) Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E.
*	Department of Industrial Engineering
*	University of Arkansas
*	4207 Bell Engineering Center
*	Fayetteville, AR 72701
*	Phone: (479) 575-6756
*	Email: rossetti@uark.edu
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution);
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA, or see www.fsf.org
*
 */

package testing.random;

import jsl.utilities.random.distributions.Beta;
import jsl.utilities.random.distributions.NegativeBinomial;
import jsl.utilities.random.rng.RngIfc;
import jsl.utilities.random.rng.RngStream;

/**
 *
 * @author rossetti
 */
public class TestNegativeBinomial {


    public static void main(String args[]) {
        double x = 0.9815723452128324;
        double a = 52.68053626449045;
        double b = 2.0;
        double bi = Beta.betaIncompleteFunction(x, a, b);
        System.out.println("bi = " + bi);
        double bf = Beta.betaFunction(a, b);
        System.out.println("bf = " + bf);

        //		NegativeBinomial n2 = new NegativeBinomial(0.6, 4);

        double mu = 0.989003756085243;
        double s2 = 1.0075709252697;
        double p = mu / s2;
        double r = (p * mu) / (1.0 - p);

        //	r = 318536.0; // very large r causes problems
        //	p = 0.9975;

        NegativeBinomial n2 = new NegativeBinomial(p, r);
        System.out.println("n2 = " + n2);
        System.out.println("r = " + r);
        System.out.println("p = " + p);


        for (int i = 0; i <= 10; i++) {
            System.out.println("pdf(" + i + ")= " + n2.pmf(i) + " \t cdf(" + i + ")= " + n2.cdf(i) +
                    " \t ccdf(" + i + ")= " + n2.complementaryCDF(i) + " \t L1(" + i + ")= " + n2.firstOrderLossFunction(i) + " \t L2(" + i + ")= " + n2.secondOrderLossFunction(i));
        }

        System.out.println("random");
        for (int i = 1; i <= 20; i++) {
            System.out.println("nextRandom(" + i + ")= " + n2.getValue());
        }

        System.out.println("Static testing");
        for (int i = 0; i <= 10; i++) {
            System.out.println("pdf(" + i + ")= " + NegativeBinomial.negBinomialPMF(i, r, p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.println("pdf(" + i + ")= " + NegativeBinomial.recursivePMF(i, r, p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.println("cdf(" + i + ")= " + NegativeBinomial.negBinomialCDF(i, r, p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.println("cdf(" + i + ")= " + NegativeBinomial.recursiveCDF(i, r, p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.println("L1(" + i + ")= " + NegativeBinomial.negBinomialLF1(i, r, p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.println("L2(" + i + ")= " + NegativeBinomial.negBinomialLF2(i, r, p));
        }
        System.out.println();

//		for (double x=0.0;x<1.0;x=x+0.01){
//			System.out.println(" static invCDF(" + x + ")= " + NegativeBinomial.negBinomialInvCDF(x, 4, 0.6));
//			System.out.println("        invCDF(" + x + ")= " + n2.invCDF(x));
//		}
//		System.out.println(" static invCDF(" + 1.0 + ")= " + NegativeBinomial.negBinomialInvCDF(1.0, 4, 0.6));
//		System.out.println("        invCDF(" + 1.0 + ")= " + n2.invCDF(1.0));

        NegativeBinomial f = new NegativeBinomial(0.428977334, 22.0);
        System.out.println("f " + f);

        
        r = 37;
        p = 0.002502;
        NegativeBinomial n3 = new NegativeBinomial(p, r);
        System.out.println("n2 = " + n2);
        System.out.println("r = " + r);
        System.out.println("p = " + p);


        for (int i = 0; i <= 10; i++) {
        System.out.println("pdf(" + i + ")= " + n3.pmf(i) + " \t cdf(" + i + ")= " + n3.cdf(i) +
        " \t ccdf(" + i + ")= " + n3.complementaryCDF(i) + " \t L1(" + i + ")= " +
        n3.firstOrderLossFunction(i) + " \t L2(" + i + ")= " + n3.secondOrderLossFunction(i));
        }
        

        RngIfc rng = new RngStream();
        NegativeBinomial n4 = new NegativeBinomial(0.3, 15, rng);
        System.out.println("NB " + n4);
        for (int i = 1; i <= 20; i++) {
            double u = rng.randU01();
            System.out.println("invCDF(" + u + ") = " + n4.invCDF(u));
        }
        System.out.println();

        /*
        rng.resetStartStream();
        NegativeBinomial n3 = new NegativeBinomial(0.3, 15, rng);
        System.out.println("NB " + n3);
        for (int i = 1; i <= 20; i++) {
            System.out.println("nextRandom(" + i + ")= " + n3.getValue());
        }
*/

        RngIfc rng2 = new RngStream();
        NegativeBinomial n5 = new NegativeBinomial(0.3, 1, rng2);
        System.out.println("NB " + n5);
        for (int i = 1; i <= 20; i++) {
            double u = rng2.randU01();
            System.out.println("invCDF(" + u + ") = " + n5.invCDF(u));
        }

        /*
        rng2.resetStartStream();
        NegativeBinomial n6 = new NegativeBinomial(0.3, 1, rng2);
        System.out.println("NB " + n6);
        for (int i = 1; i <= 20; i++) {
            System.out.println("nextRandom(" + i + ")= " + n6.getValue());
        }

*/
        System.out.println("done");

    }
}
