/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.random;

import jsl.utilities.random.distributions.Poisson;
import jsl.utilities.random.rng.RngStream;

public class TestPoisson {

    public static void main(String args[]) {

        test2();
       //test1();
    }

    public static void test2() {
        double lambda = 0.5;
        Poisson f = new Poisson(lambda);

        System.out.println("i \t  pdf(i) \t cdf");
        for (int i = 0; i <= 10; i++) {
            System.out.println(i + "\t" + f.pdf(i) + "\t" + f.cdf(i));
        }

        System.out.println("mode = " + f.getMode());
        System.out.println();
        System.out.println("invCDF(0.2) = " + f.invCDF(0.2));
        //System.out.println("approx invCDF(0.2) = " + invCDFViaNormalApprox(0.2, lambda));
        System.out.println("invCDF(0.983) = " + f.invCDF(0.983));
        //System.out.println("approx invCDF(0.983) = " + invCDFViaNormalApprox(0.983, lambda));

        System.out.println("invCDF(" + f.cdf(0) + ") = " + f.invCDF(f.cdf(0)));
        System.out.println("invCDF(" + f.cdf(1) + ") = " + f.invCDF(f.cdf(1)));
        System.out.println("invCDF(" + f.cdf(2) + ") = " + f.invCDF(f.cdf(2)));

        System.out.println();
        RngStream r = new RngStream();
        for (int i = 1; i <= 10; i++) {
            double p = r.randU01();
            System.out.println("invCDF(" + p + ") = " + f.invCDF(p));
        }
        System.out.println();

        for (int i = 0; i <= 10; i++) {
            System.out.print("pdf(" + i + ")= " + Poisson.recursivePMF(i, lambda));
            System.out.println("\t cdf(" + i + ")= " + Poisson.recursiveCDF(i, lambda));
        }

        System.out.println();
        for (int i = 0; i <= 10; i++) {
            System.out.print("pdf(" + i + ")= " + Poisson.poissonPMF(i, lambda, false));
            System.out.println("\t cdf(" + i + ")= " + Poisson.poissonCDF(i, lambda, false));
        }
        System.out.println();

    }

    public static void test1() {
        double lambda = 10146.23285;
        Poisson f = new Poisson(lambda);

        System.out.println("i \t  pdf(i) \t cdf");
        for (int i = 10000; i <= 10010; i++) {
            System.out.println(i + "\t" + f.pdf(i) + "\t" + f.cdf(i));
        }

        System.out.println();

        System.out.println("mode = " + f.getMode());
        System.out.println("invCDF(0.2) = " + f.invCDF(0.2));
        System.out.println("invCDF(0.983) = " + f.invCDF(0.983));
        System.out.println("invCDF(" + f.cdf(0) + ") = " + f.invCDF(f.cdf(0)));

         System.out.println();

        RngStream r = new RngStream();
        for (int i = 1; i <= 10; i++) {
            double p = r.randU01();
            System.out.println("invCDF(" + p + ") = " + f.invCDF(p));
        }

        System.out.println();

        for (int i = 10000; i <= 10010; i++) {
            System.out.print("pdf(" + i + ")= " + Poisson.recursivePMF(i, lambda));
            System.out.println("\t cdf(" + i + ")= " + Poisson.recursiveCDF(i, lambda));
        }

        System.out.println();
        for (int i = 10000; i <= 10010; i++) {
            System.out.print("pdf(" + i + ")= " + Poisson.poissonPMF(i, lambda, false));
            System.out.println("\t cdf(" + i + ")= " + Poisson.poissonCDF(i, lambda, false));
        }
        System.out.println();

    }

}
