/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.spatial;

import java.io.IOException;
import java.util.logging.Level;

import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.modeling.Replication;
import jsl.modeling.elements.spatial.RandomMover;
import jsl.modeling.elements.spatial.RectangularGridSpatialModel2D;
import jsl.utilities.reporting.JSL;
import jsl.observers.Mover2DObserver;
import jsl.observers.animation.AnimationTextFileGenerator;
import jsl.observers.animation.MoveableElementAnimationObserver;


/**
 *
 */
public class TestMovement {

	public static void main(String[] args) {
		// turns on the level of logging
		// watch out FINEST is very verbose, e.g. every event
		JSL.LOGGER.setLevel(Level.FINE);
		
		System.out.println("Starting");
		
		testReplication();
		
//		testExperiment();
		

		System.out.println("Done");	
		
	}
	
	public static void testReplication(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		// create some model elements
		
		RectangularGridSpatialModel2D grid = new RectangularGridSpatialModel2D();
//		RectangularGridSpatialModel2D grid = new RectangularGridSpatialModel2D(5.0, 5.0, 2, 2);
				
//		System.out.println(grid);
		
//		Mover2DObserver mseo = new Mover2DObserver();
		
		RandomMover rm = new RandomMover(m, grid);
		
//		rm.addObserver(mseo);
		
		AnimationTextFileGenerator atfg = null;
		try{
			atfg = new AnimationTextFileGenerator("animation");
		}
		catch(IOException e){
			System.out.println("I/O exception opening animation file");
			System.exit(1);
		}
		
		MoveableElementAnimationObserver ao = new MoveableElementAnimationObserver(atfg);
		
		rm.addObserver(ao);
		
//		m.turnOnLogReport();
        
        // create a replication to run the model
		Replication r = new Replication(m);

//		r.setMaximumExecutionTime(100000);
		// set the running parameters of the replication
		r.setLengthOfReplication(20);
//		r.setMaximumExecutionTime(1000);
//		r.setLengthOfWarmUp(100.0);
//		r.turnOnDefaultTraceReport();
		// turn on the desired reporting
//		r.turnOffBatchReport();
		r.turnOnBatchReport();
//		ReplicationShowTimeTask rTask = new ReplicationShowTimeTask(r);
//		r.turnOnTimer(10000, rTask);
		
		// tell the replication to run
		r.runAll();
 
	}
	
	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		// create some model elements
		
		RectangularGridSpatialModel2D grid = new RectangularGridSpatialModel2D();
		
		System.out.println(grid);
		
		RandomMover rm = new RandomMover(m, grid);
		
		//rm.addObserver(mseo);
		
		m.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(5);
		e.setLengthOfReplication(100.0);
//		e.setLengthOfWarmUp(5000.0);
		
		// turn on the desired reporting
//		e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		
		// tell the experiment to run
		e.runAll();
	}

}
