/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.temp;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import examples.queueing.*;

/**
 */
public class GGcExperimentProvider extends ExperimentProvider {

	private GGcWSModel myGGc;
	private DistributionIfc myAD;
	private DistributionIfc mySD;
	private double[] myAP;
	private double[] mySP;
	private int myNumServers;
	private int myNumExperiments;
	private int myMaxNumExperiments;
	
	/**
	 * @param model
	 */
	public GGcExperimentProvider() {
		super();
		
		// create the distributions 
//		myAD = new Gamma();
		myAD = new Exponential();
		mySD = new Exponential();
		myAP = new double[2];
		mySP = new double[2];

		// create a model element and attach it to the main model
		myGGc = new GGcWSModel(myModel);
		
		myExperiment = new Experiment(myModel);
		myMaxNumExperiments = 2;
	}

	/* (non-Javadoc)
	 * @see java.util.Iterator#hasNext()
	 */
	public boolean hasNext() {
		return(myNumExperiments < myMaxNumExperiments);
	}

	/* (non-Javadoc)
	 * @see java.util.Iterator#next()
	 */
	public Experiment next() {
		myNumExperiments++;
		getNextCase();
		setNextCase();
		setNextExperiment();
		outputNextCase();
		return myExperiment;
	}
	
	private void getNextCase(){
		/* specify the case
		 * could read in these
		 * variables
		*/
//		myAP[0] = 8.5349;
//		myAP[1] = 0.03261;
		myAP[0] = 1.0;
		myAD.setParameters(myAP);
//		mySP[0] = 12.1014;
		mySP[0] = 0.5;
		mySD.setParameters(mySP);
//		myNumServers = 49;
		myNumServers = 1;
	}
	
	private void setNextCase(){
		/* setup the GGc
		 */
		myGGc.setArrivalDistributionInitialRandomSource(myAD);
		myGGc.setServiceDistributionInitialRandomSource(mySD);
		myGGc.setNumberOfServers(myNumServers);
//		myGGc.setWSHalfWidthPrecision(0.1);

	}
	
	private void setNextExperiment(){
		/* can set experiment or model parameters here
		 * these can be set where the experiment is
		 * created or here, perhaps depending on the 
		 * specifics of the design
		 */
//		myExperiment = new Experiment(myModel); 
		myExperiment.setNumberOfReplications(1);
		myExperiment.setLengthOfReplication(150.0);
//		myExperiment.setMaximumExecutionTimePerReplication(200);
		myExperiment.setLengthOfWarmUp(50.0);
//		myExperiment.setDefaultBatchInterval(100.0);
//		myExperiment.setResetStartStreamOption(false);
//		if (myNumExperiments == 2)
//			myExperiment.turnOnDefaultTraceReport();
			
		// turn on the desired reporting
		myExperiment.turnOnExperimentReport();
		myExperiment.turnOnReplicationReport();
//		ExperimentShowTimeTask rTask = new ExperimentShowTimeTask(myExperiment);
//		myExperiment.turnOnTimer(1000, rTask);
		
//		myModel.turnOnLogReport();
		
	}
	
	private void outputNextCase(){

		/* can write out some info
		 * could write a method for this
		 */
		double mtba = myAD.getMean();
		double mst = mySD.getMean();
		double c = myGGc.getNumberOfServers();
		double rho = mst/(mtba*c);
		System.out.println("MTBA " + mtba);
		System.out.println("MST " + mst);
		System.out.println("Num Servers " + c);
		System.out.println("Rho " + rho);
	
	}

}
