/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.temp;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Gamma;
import examples.queueing.*;

/**
 */
public class GGcModelDesign implements DesignIfc{
	
	/* another way to achieve this is to have
	 * GGcModel implement the DesignIfc, then it
	 * could supply its own design setup method
	 * with direct access to these parameters
	 */
	private SimpleQRegenerativeMethod myGGc;
	private DistributionIfc myAD;
	private DistributionIfc mySD;
	private int myNumServers;
	
	/**
	 * 
	 */
	public GGcModelDesign(SimpleQRegenerativeMethod ggc) {
		super();
		myGGc = ggc;
	}

	/* (non-Javadoc)
	 * @see jsl.modeling.DesignIfc#setUp(jsl.modeling.Experiment, jsl.modeling.Model)
	 */
	public void setUp(Experiment experiment, Model model) {
		
		/* specify the case
		 * could write a method to read in these
		 * variables
		*/
		myAD = new Gamma(8.5349, 0.03261);
		mySD = new Exponential(12.1014);
		myNumServers = 49;
		
		/* setup the GGc
		 * could write a method to do this
		 */
		myGGc.setArrivalDistributionInitialRandomSource(myAD);
		myGGc.setServiceDistributionInitialRandomSource(mySD);
		myGGc.setNumberOfServers(myNumServers);
		
		/* can set experiment or model parameters here
		 * these can be set where the experiment is
		 * created or here, perhaps depending on the 
		 * specifics of the design
		 */
		experiment.setNumberOfReplications(1);
		experiment.setLengthOfReplication(1000.0);
		experiment.setLengthOfWarmUp(50.0);
 
		/* can write out some info
		 * could write a method for this
		 */
		double mtba = myAD.getMean();
		double mst = mySD.getMean();
		double c = myGGc.getNumberOfServers();
		double rho = mst/(mtba*c);
		System.out.println("MTBA " + mtba);
		System.out.println("MST " + mst);
		System.out.println("Num Servers " + c);
		System.out.println("Rho " + rho);
		
	}
}
