/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.temp;


import jsl.modeling.Experiment;
import jsl.modeling.Model;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Lognormal;
import jslx.scinventory.DemandGenerator;
import jslx.scinventory.InventoryHoldingPoint;
import jslx.scinventory.ItemType;
import jslx.scinventory.LeadTimeReplenishmentProvider;
import jslx.scinventory.NoDelayShipper;
import jslx.scinventory.TimeBasedCustomerShipper;
import jslx.scinventory.TimeBasedItemTypeShipper;

public class Test {

	/**
	 * @param args
	 */
	
	public static void main(String[] args) {
		System.out.println("Yisha MultipleProductMultipleLocation Test");
        
		//Project project = new Project("AFORS", "MultipleProductMultipleLocation");
		//JSL.LOGGER.setLevel(Level.INFO);
   
		testExperiment();
        
		System.out.println("Done!");		
	}
	
private static Model buildModel(){
		
		// Create the containing model
		Model m = Model.createModel();		
		
		////////////////////////////////////////////////////////////////////////////////
		// Define the types
		// Here we create the various item type
		// An item refers to a product type and the ware-house which supplies it????????
						
		ItemType type1 = new ItemType(m, "Type-1");
		
		// Define the top of the network, can supply anything, in any amount, with a delay
		LeadTimeReplenishmentProvider supplyFactory = new LeadTimeReplenishmentProvider(m);
		// Here we supply the data for factory which is also a time based item type shipper
		DistributionIfc pt1 = new Constant(5.0);
		supplyFactory.addLeadTime(type1,pt1);

		// The warehouse must use a shipper to ship to its customers
		DistributionIfc ltA1 = new Constant(2.0);
		TimeBasedItemTypeShipper warehouseShipper = new TimeBasedItemTypeShipper(m);
		warehouseShipper.addLeadTime(type1,ltA1);

		// The warehouse uses the supplyfactory as a provider and the warehouseshipper as the shipper
		InventoryHoldingPoint warehouse= new InventoryHoldingPoint(m, supplyFactory, warehouseShipper);

		/////////////////////////////////////////////////////////////////////////////////
		// Create the inventory policy at each IHP (warehouse) for each items, assumes backlogging allowed, 
		// no partial filling the r, Q parameters are fixed and back logging allowed
						
		int wrptA1 = 10;
		int wqtyA1 = 20;
		int wlevelA1 =20;
		warehouse.addRQInventoryItem(type1,wlevelA1,wrptA1,wqtyA1);

		// The retailer uses a NoDelayShipper to ship to its customers
		NoDelayShipper retailerShipper = new NoDelayShipper(m);
		
		// Create the retailer which uses the warehouse as its provider and a no delay shipper
		InventoryHoldingPoint retailerA = new InventoryHoldingPoint(m, warehouse, retailerShipper);

		////////////////////////////////////////////////////////////////////////////////
		// Create the inventory policy at each IHP (retailer) for each items, assumes backlogging allowed, 
		// no partial filling the r, Q parameters are fixed and back logging allowed
		
		int retailerrptA1 = 5;
		int retailerqtyA1 = 10;
		int retailerlevelA1 = 10;
		retailerA.addRQInventoryItem(type1,retailerlevelA1,retailerrptA1,retailerqtyA1);

		//////////////////////////////////////////////////////////////////////////////
		// Here we create the distibutions of the daily demand
		// of each item at each retailer location
						
		DistributionIfc dA1 = new Constant(1.0);				
		
		//////////////////////////////////////////////////////////////////////////
		// Define the demand generator
		
		DemandGenerator dgA1 = new DemandGenerator(m, dA1, dA1);		
		dgA1.setDemandRequirements(type1, true, false, retailerA);	
		return(m);		
	}
	
	public static void testExperiment(){
		Model n = buildModel();
		
		n.turnOnLogReport();
		
		// create the experiment to run the model
		Experiment e = new Experiment(n);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(5);
		// 360 days
		e.setLengthOfReplication(365.0);
		e.setLengthOfWarmUp(36.0);
		
		// turn on the desired reporting
		// e.turnOnBatchReport(); 
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.turnOnReplicationConsoleOutput();
//		e.turnOnTimer(10000);
		// tell the experiment to run
		e.runAll();		
	}


}
