/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.temp;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.modeling.utilities.ExperimentShowTimeTask;
import jsl.utilities.misc.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Gamma;
import jsl.utilities.random.distributions.Triangular;

import jsl.utilities.reporting.JSL;
import examples.queueing.*;

/**
 */
public class TestGGCLQModel {
	
	/**
	 * 
	 */
	public TestGGCLQModel() {
		super();
	}

	public static void main(String[] args) {
		System.out.println("GGcLQModel Test");
        
//		JSL.LOGGER.setLevel(Level.FINE);
   
		testCase2();
        
		System.out.println("Done!");
        
	}
	
	
	public static void testCase2(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		GGcLQModel ggc = new GGcLQModel(m);

		m.turnOnLogReport();
		
		// set up the case
		DistributionIfc ad = new Gamma(0.0266494848851034, 1.34756065089927);
		DistributionIfc sd = new Triangular(0.540712193770341, 0.540712193770341, 2.35679921660711);
		ggc.setArrivalDistributionInitialRandomSource(ad);
		ggc.setServiceDistributionInitialRandomSource(sd);
		ggc.setNumberOfServers(54);
		
		// print out some details
		double mtba = ad.getMean();
		double mst = sd.getMean();
		double c = ggc.getNumberOfServers();
		double rho = mst/(mtba*c);
		System.out.println("MTBA " + mtba);
		System.out.println("MST " + mst);
		System.out.println("Num Servers " + c);
		System.out.println("Rho " + rho);
		
		// create the experiment to run the model
		Experiment e = new Experiment(m);
		
		// set the parameters of the experiment
		e.setNumberOfReplications(1);
		e.setLengthOfReplication(1000000.0);
		e.setLengthOfWarmUp(500000.0);

//		e.setLengthOfReplication(1000.0);
//		e.setLengthOfWarmUp(500.0);
		
		// turn on the desired reporting
		e.turnOnBatchReport();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		ExperimentShowTimeTask rTask = new ExperimentShowTimeTask(e);
		e.turnOnTimer(10000, rTask);
		
		// tell the experiment to run
		e.runAll();
	}	
}
