/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.temp;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;
import jsl.utilities.random.distributions.Gamma;

import jsl.utilities.reporting.JSL;
import examples.queueing.*;

/**
 */
public class TestGGCWSModel3 {
	
	/**
	 * 
	 */
	public TestGGCWSModel3() {
		super();
	}

	public static void main(String[] args) {
		System.out.println("GGcWSModel Test");
        
		JSL.LOGGER.setLevel(Level.FINER);
   
		testExperiment();
//        testReplication();
        
		System.out.println("Done!");
        
	}
	
	public static void testReplication(){
		
		// create the distributions 
		DistributionIfc ad = new Gamma();
		DistributionIfc sd = new Exponential();
		double[] ap = new double[2];
		double[] sp = new double[2];
		
		// create the containing model
		Model m = Model.createModel();
			
		// create the model element and attach it to the main model
		SimpleQRegenerativeMethod ggc = new SimpleQRegenerativeMethod(m);

		// set the parameters of the experiment
		ap[0] = 8.5349;
		ap[1] = 0.03261;
		ad.setParameters(ap);
		sp[0] = 12.1014;
		sd.setParameters(sp);
		ggc.setArrivalDistributionInitialRandomSource(ad);
		ggc.setServiceDistributionInitialRandomSource(sd);
		ggc.setNumberOfServers(49);
		
		// output the case
		double mtba = ad.getMean();
		double mst = sd.getMean();
		double c = ggc.getNumberOfServers();
		double rho = mst/(mtba*c);
		System.out.println("MTBA " + mtba);
		System.out.println("MST " + mst);
		System.out.println("Num Servers " + c);
		System.out.println("Rho " + rho);

		Replication r1 = new Replication(m);
		r1.setName("Rep1");
		r1.setLengthOfReplication(120.0);
		r1.setLengthOfWarmUp(20.0);
		r1.setDefaultBatchInterval(2.0);
		r1.turnOnBatchReport();
		r1.turnOnDefaultTraceReport();
		
		r1.runAll();
		// second experiment
		// set the parameters of the experiment
		ap[0] = 8.5349;
		ap[1] = 0.03261;
		ad.setParameters(ap);
		sp[0] = 12.1014;
		sd.setParameters(sp);
		ggc.setArrivalDistributionInitialRandomSource(ad);
		ggc.setServiceDistributionInitialRandomSource(sd);
		ggc.setNumberOfServers(49);
		
		// output the case
		double mtba2 = ad.getMean();
		double mst2 = sd.getMean();
		double c2 = ggc.getNumberOfServers();
		double rho2 = mst2/(mtba2*c2);
		System.out.println("MTBA " + mtba2);
		System.out.println("MST " + mst2);
		System.out.println("Num Servers " + c2);
		System.out.println("Rho " + rho2);

		Replication r2 = new Replication(m);
		r2.setName("Rep2");
		
		r2.setLengthOfReplication(120.0);
		r2.setLengthOfWarmUp(20.0);
		r2.setDefaultBatchInterval(2.0);
		r2.turnOnBatchReport();
		r2.turnOnDefaultTraceReport();
//		r2.setResetStartStreamOption(false);
		r2.runAll();		
	}
	
	public static void testExperiment(){
		
		// create the distributions 
		DistributionIfc ad = new Gamma();
		DistributionIfc sd = new Exponential();
		double[] ap = new double[2];
		double[] sp = new double[2];
		
		// create the containing model
		Model m = Model.createModel();
			
		// create the model element and attach it to the main model
		SimpleQRegenerativeMethod ggc = new SimpleQRegenerativeMethod(m);

		// set the parameters of the experiment
		ap[0] = 8.5349;
		ap[1] = 0.03261;
		ad.setParameters(ap);
		sp[0] = 12.1014;
		sd.setParameters(sp);
		ggc.setArrivalDistributionInitialRandomSource(ad);
		ggc.setServiceDistributionInitialRandomSource(sd);
		ggc.setNumberOfServers(49);
		
		// output the case
		double mtba = ad.getMean();
		double mst = sd.getMean();
		double c = ggc.getNumberOfServers();
		double rho = mst/(mtba*c);
		System.out.println("MTBA " + mtba);
		System.out.println("MST " + mst);
		System.out.println("Num Servers " + c);
		System.out.println("Rho " + rho);

		Experiment e1 = new Experiment(m);
		e1.setName("Exp1");
		e1.setNumberOfReplications(1);
		e1.setLengthOfReplication(120.0);
		e1.setLengthOfWarmUp(20.0);
		e1.setDefaultBatchInterval(2.0);
		e1.turnOnExperimentReport();
		e1.turnOnReplicationReport();
//		e1.turnOnDefaultTraceReport();
		
		e1.runAll();
		
		// second experiment
		// set the parameters of the experiment
		ap[0] = 8.5349;
		ap[1] = 0.03261;
		ad.setParameters(ap);
		sp[0] = 12.1014;
		sd.setParameters(sp);
		ggc.setArrivalDistributionInitialRandomSource(ad);
		ggc.setServiceDistributionInitialRandomSource(sd);
		ggc.setNumberOfServers(49);
		
		// output the case
		double mtba2 = ad.getMean();
		double mst2 = sd.getMean();
		double c2 = ggc.getNumberOfServers();
		double rho2 = mst2/(mtba2*c2);
		System.out.println("MTBA " + mtba2);
		System.out.println("MST " + mst2);
		System.out.println("Num Servers " + c2);
		System.out.println("Rho " + rho2);

		Experiment e2 = new Experiment(m);
		e2.setName("Exp2");
		e2.setNumberOfReplications(1);
		e2.setLengthOfReplication(120.0);
		e2.setLengthOfWarmUp(20.0);
		e2.setDefaultBatchInterval(2.0);
		e2.turnOnExperimentReport();
		e2.turnOnReplicationReport();
//		e2.turnOnDefaultTraceReport();
//		e2.setResetStartStreamOption(false);

		e2.runAll();		

	}

}
