/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.temp;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.modeling.utilities.ExperimentShowTimeTask;
import examples.queueing.*;
import jsl.utilities.misc.*;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestGGcModelDesign {

	/**
	 * 
	 */
	public TestGGcModelDesign() {
		super();
	}

	public static void main(String[] args) {
		System.out.println("Test");
        
		JSL.LOGGER.setLevel(Level.OFF);
		
		testExperiment();
		
		System.out.println("Done!");
	}
	
	private static void testExperiment(){
		// create the containing model
		Model m =Model.createModel();
		
		// create a model element and attach it to the main model
		SimpleQRegenerativeMethod ggc = new SimpleQRegenerativeMethod(m);
		
		// create a design to setup the model
		GGcModelDesign d = new GGcModelDesign(ggc);

		// create the experiment to run the model
		Experiment e = new Experiment(m);

		// attach the design to the experiment
		e.setDesign(d);		

		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		ExperimentShowTimeTask rTask = new ExperimentShowTimeTask(e);
		e.turnOnTimer(1000, rTask);
		
		// tell the experiment to run
		e.runAll();		
	}
}
