/*
* Created on Mar 22, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.variable;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.variable.RandomVariable;
import jsl.modeling.elements.variable.ResponseVariable;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;

/**
 * @author rossetti
 *
 */
public class TestAfterReplication extends SchedulingElement {

	private RandomVariable myRV;
	private DistributionIfc d;

    private ResponseVariable myX;
    private double value = 5.0;
	/**
	 * @param parent
	 */
	public TestAfterReplication(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public TestAfterReplication(ModelElement parent, String name) {
		super(parent, name);
		d = new Exponential();
		myRV = new RandomVariable(this, d);		// TODO Auto-generated constructor stub
		myX = new ResponseVariable(this, "myX");
//		myX.setAfterReplicationOption(false);
	}

	protected void initialize(){
		scheduleEvent(10.0);
	}
	
	protected void handleEvent(JSLEvent e){
	//	value = myRV.getValue();
//		System.out.println("after replication: setting myX to : " + value);
//		myX.setValue(value);
//		myRV.getValue();
	}

	protected void replicationEnded(){
		value = myRV.getValue();
		System.out.println("after replication: setting myX to : " + value);
		myX.setValue(value);
	}
	/*
	protected void afterReplication(){
		value = myRV.getValue();
		System.out.println("after replication: setting myX to : " + value);
		myX.setValue(value);
	}
*/	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestAfterReplication tv = new TestAfterReplication(m);
		        
		m.turnOnLogReport("TestAfterReplication");
		
		// create a replication to run the model
		Experiment e = new Experiment(m);

		// set the running parameters of the experiment
		e.setNumberOfReplications(5);
		e.setLengthOfReplication(50.0);
//		e.setLengthOfWarmUp(10.0);

		// turn on the desired reporting
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
				
		// tell the replication to run
		e.runAll();
	}

}
