/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.variable;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestAggregateCounter extends SchedulingElement {

	private double myCount;
	private double myCount1;
	private RandomVariable myTBA1;
	private double myCount2;
	private RandomVariable myTBA2;
	private Counter myCounter1;
	private Counter myCounter2;
	private Counter myTCounter;
	private ArrivalListener myArrivalListener = new ArrivalListener();
    private AggregateCounter myAC;
    private AggregateCounter myAC2;
    
	/**
	 * @param parent
	 */
	public TestAggregateCounter(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public TestAggregateCounter(ModelElement parent, String name) {
		super(parent, name);
		//DistributionIfc a = new Exponential(1.0);
		DistributionIfc a2 = new Constant(2.0);
		myTBA2 = new RandomVariable(this, a2, "tba2");
		myCounter2 = new Counter(this, "C2");
		
		DistributionIfc a1 = new Constant(1.0);
		myTBA1 = new RandomVariable(this, a1, "tba1");
		myCounter1 = new Counter(this, "C1");

		myTCounter = new Counter(this, "TC");

		myAC = new AggregateCounter(this, "c1c2");
		myAC.subscribeTo(myCounter2);
		myAC.subscribeTo(myCounter1);
		
		myAC2 = new AggregateCounter(this,"(c1c2)andc1");
		myAC2.subscribeTo(myCounter1);
		myAC2.subscribeTo(myAC);
		
		//myCountArrivals.setTimedUpdateInterval(10.0);
	}

	public void initialize(){
		myCount = 0.0;
		myCount1 = 0.0;
		myCount2 = 0.0;
		JSL.out.println("c1 \t c2 \t AC \t AC2");
		scheduleEvent(myTBA1,"arrival1");
		scheduleEvent(myArrivalListener, myTBA2, "arrival2");
	}
	
	protected void handleEvent(JSLEvent e){
		myCounter1.increment();
		myTCounter.increment();
		myCount1++;
		myCount++;
		System.out.println("*******");
        System.out.println("time: " + getTime() + " counter1 = " + myCount1);
        System.out.println("time: " + getTime() + " c1 = " + myCounter1.getValue());
        JSL.out.println(myCounter1.getValue() + "\t" + myCounter2.getValue() + "\t" + myAC.getValue() + "\t" + myAC2.getValue());
		rescheduleEvent(e, myTBA1);// after the down time elapse, state goes to up
	}
	
	class ArrivalListener implements ActionListenerIfc {
		public void action(JSLEvent event) {

			// there has been an arrival
			myCounter2.increment();
			myTCounter.increment();
			myCount2++;
			myCount++;
			System.out.println("#########");
            System.out.println("time: " + getTime() + " counter2 = " + myCount2);
            System.out.println("time: " + getTime() + " c2 = " + myCounter2.getValue());

			rescheduleEvent(event, myTBA2);// after the down time elapse, state goes to up

		}
	}

	public static void main(String[] args) {
		System.out.println("TestVariables test");
        
//		JSL.LOGGER.setLevel(Level.FINE);
   
//		testReplication();
        testExperiment();
        
		System.out.println("Done!");
	}

	public static void testReplication(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestAggregateCounter tv = new TestAggregateCounter(m);
		        
		// create a replication to run the model
		Replication r = new Replication(m);

		// set the running parameters of the replication
		r.setLengthOfReplication(50.0);
		r.setLengthOfWarmUp(10.0);
		
		// turn on the desired reporting
		r.turnOnBatchReport();
				
		// tell the replication to run
		r.runAll();
 
	}

	public static void testExperiment(){
		
		JSL.out.OUTPUT_ON = true;
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestAggregateCounter tv = new TestAggregateCounter(m);
		        
		// create a replication to run the model
		Experiment e = new Experiment(m);

		// set the running parameters of the experiment
		e.setNumberOfReplications(3);
		e.setLengthOfReplication(50.0);
		e.setLengthOfWarmUp(10.0);

		// turn on the desired reporting
//		e.turnOnBatchReport();
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
		e.turnOnReplicationReport();
				
		// tell the replication to run
		e.runAll();
 
	}
	

}
