/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.variable;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.modeling.elements.variable.AggregateTimeWeightedVariable;
import jsl.modeling.elements.variable.TimeWeighted;
import jsl.utilities.random.*;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestAggregation2 extends SchedulingElement {

	
	private TimeWeighted myTW1;
	private TimeWeighted myTW2;
	private TimeWeighted myTW3;
	
	private AggregateTimeWeightedVariable myATW1;
	private AggregateTimeWeightedVariable myATW2;
	private AggregateTimeWeightedVariable myATW3;
	
	/**
	 * @param parent
	 */
	public TestAggregation2(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public TestAggregation2(ModelElement parent, String name) {
		super(parent, name);
		
	    myTW1 = new TimeWeighted(this, 1.0, "tw1");
	    myTW2 = new TimeWeighted(this, 0.0, "tw2");
	    myTW3 = new TimeWeighted(this, 3.0, "tw3");

	    myATW1 = new AggregateTimeWeightedVariable(this, "atw1");
	    myATW1.subscribeTo(myTW1);
	    myATW1.subscribeTo(myTW2);
	    myATW1.subscribeTo(myTW3);
	    
	    myATW2 = new AggregateTimeWeightedVariable(this, "atw2");
	    myATW2.subscribeTo(myTW1);
	    myATW2.subscribeTo(myTW2);
	    myATW2.subscribeTo(myTW3);
	    
	    myATW3 = new AggregateTimeWeightedVariable(this, "atw3");
	    myATW3.subscribeTo(myTW1);
	    myATW3.subscribeTo(myTW2);
	    myATW3.subscribeTo(myTW3);
	}
	
	protected void w1Change(int x){
		myTW1.setValue(x);    
	}
	
	protected void w2Change(int x){
		myTW2.setValue(x);
	}

	protected void w3Change(int x){
		myTW3.setValue(x);
	}

	public void initialize(){
		System.out.println("time \t tw1 \t tw2 \t tw3 \t atw1 \t atw2 \t atw3");
		System.out.println(getTime() + " \t " + myTW1.getValue() + " \t " + myTW2.getValue() + " \t " + myTW3.getValue() + " \t " + myATW1.getValue() + " \t" + myATW2.getValue() + " \t" + myATW3.getValue());
		scheduleEvent(1, 2, "tw2");
		scheduleEvent(2, 3, "tw1");
		scheduleEvent(3, 1, "tw2");
		scheduleEvent(4, 2, "tw1");
		scheduleEvent(4, 2, "tw3");
		scheduleEvent(6, 3, "tw2");
		scheduleEvent(6, 1, "tw3");
		scheduleEvent(7, 3, "tw1");
		scheduleEvent(7, 2, "tw2");
		scheduleEvent(7, 0, "tw3");
		scheduleEvent(7, 1, "add");
		scheduleEvent(9, 1, "remove");
	}
	
	protected void handleEvent(JSLEvent e){
		int newvalue = e.getType();
		String type = (String)e.getMessage();  
		if (type == "tw1"){
			w1Change(newvalue);
		} else if (type == "tw2"){
			w2Change(newvalue);		
		} else if (type == "tw3"){
			w3Change(newvalue);		
		} else if(type == "add"){
			TimeWeighted myTW4 = new TimeWeighted(this, 1.0, "tw4");
			myATW2.subscribeTo(myTW4);
			TimeWeighted temp = (TimeWeighted)this.getModel().getModelElement("tw4");
			temp.setValue(newvalue);
			System.out.println("tw4 is added");
		} else if(type == "tw4"){
			TimeWeighted temp = (TimeWeighted)this.getModel().getModelElement("tw4");
			temp.setValue(newvalue);
		} else if(type == "remove"){
			myTW1.removeFromModel();
		}
	    System.out.println(getTime() + " \t " + myTW1.getValue() + " \t " + myTW2.getValue() + " \t " + myTW3.getValue() + " \t " + myATW1.getValue()+ " \t" + myATW2.getValue()+ " \t" + myATW3.getValue());
 	}
	
	public static void main(String[] args) {
		System.out.println("TestVariables test");
        testExperiment();
		System.out.println("Done!");
	}

	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestAggregation2 tv = new TestAggregation2(m);
		Experiment e = new Experiment(m);

		e.setNumberOfReplications(1);
		e.setLengthOfReplication(10.0);
		
		e.turnOnExperimentReport();
		e.turnOnExperimentConsoleOutput();
		e.turnOnReplicationReport();
		
		e.runAll();
 
	}
	

}
