/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.variable;

import java.util.logging.Level;

import jsl.modeling.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.Constant;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.reporting.JSL;

/**
 */
public class TestCounter1 extends SchedulingElement {

	private double myCount;
	private RandomVariable myTBA;
	private Counter myCountArrivals;
	//private TimeWeighted myCountA;
	private ArrivalListener myArrivalListener = new ArrivalListener();

	/**
	 * @param parent
	 */
	public TestCounter1(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public TestCounter1(ModelElement parent, String name) {
		super(parent, name);
		//DistributionIfc a = new Exponential(1.0);
		DistributionIfc a = new Constant(1.0);
		myTBA = new RandomVariable(this, a, "tba");
		myCountArrivals = new Counter(this, "count arrivals");
		//myCountA = new TimeWeighted(this, "count arrivals TW shadow");
		//myCountA.setBatchingOption(false);
		myCountArrivals.setTimedUpdateInterval(10.0);
	}

	public void initialize(){
		myCount = 0.0;

		scheduleEvent(myArrivalListener, myTBA.getValue(), "arrival");
	}
	
	class ArrivalListener implements ActionListenerIfc {
		public void action(JSLEvent event) {

			// there has been an arrival
			myCountArrivals.increment();
			//myCountA.increment();
			myCount++;
            System.out.println("time: " + getTime() + " num arrivals = " + myCount);

			rescheduleEvent(event, myTBA.getValue());// after the down time elapse, state goes to up

		}
	}

	public static void main(String[] args) {
		System.out.println("TestVariables test");
        
//		JSL.LOGGER.setLevel(Level.FINE);
   
//		testReplication();
        testExperiment();
        
		System.out.println("Done!");
	}

	public static void testReplication(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestCounter1 tv = new TestCounter1(m);
		        
		// create a replication to run the model
		Replication r = new Replication(m);

		// set the running parameters of the replication
		r.setLengthOfReplication(50.0);
		r.setLengthOfWarmUp(10.0);
		
		// turn on the desired reporting
		r.turnOnBatchReport();
				
		// tell the replication to run
		r.runAll();
 
	}

	public static void testExperiment(){
		// create the containing model
		Model m = Model.createModel();
		
		// create the model element and attach it to the main model
		TestCounter1 tv = new TestCounter1(m);
		        
		// create a replication to run the model
		Experiment e = new Experiment(m);

		// set the running parameters of the experiment
		e.setNumberOfReplications(5);
		e.setLengthOfReplication(50.0);
		e.setLengthOfWarmUp(10.0);

		// turn on the desired reporting
		e.turnOnBatchReport();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
				
		// tell the replication to run
		e.runAll();
 
	}
	

}
