/*
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/

package testing.variable;

import java.util.logging.*;
import java.util.*;
import jsl.modeling.*;
import jsl.modeling.elements.variable.*;
import jsl.utilities.reporting.JSL;
import jsl.utilities.random.*;
import jsl.utilities.random.distributions.DistributionIfc;
import jsl.utilities.random.distributions.Exponential;

/**
 */
public class TestRV extends ModelElement {

	public RandomVariable myRV;
	public DistributionIfc d;
	
	/**
	 * 
	 */
	public TestRV(ModelElement parent) {
		super(parent);
		d = new Exponential();
//		d = new Uniform();
		myRV = new RandomVariable(this, d);
		new Variable(this, Double.POSITIVE_INFINITY);
		
	}
	
	protected void montecarlo(){
		for (int i = 0; i < 10; i++) {
			JSL.out.println("x = " + myRV.getValue());			
		}
		JSL.out.println();
	}
	
	public static void main(String[] args) {
		JSL.LOGGER.setLevel(Level.FINE);
		//testReplication();
		testExperiment();
//		testSimulation();
		System.out.println("Done!");
	}
	
	public static void testReplication(){
		Model m = Model.createModel();
		new TestRV(m);

		m.turnOnLogReport();
		Replication r = new Replication(m);
		r.setMonteCarloOption(true);
//		r.clearEndCondition();
//		r.setLengthOfReplication(10.0);
//		r.setLengthOfWarmUp(5.0);
		r.runAll();
	}
	
	public static void testExperiment(){
		Model m = Model.createModel();;
		new TestRV(m);

		m.turnOnLogReport();
		Experiment e = new Experiment(m);
		e.setMonteCarloOption(true);
		e.setNumberOfReplications(3);
		e.setLengthOfReplication(10.0);
//		e.setLengthOfWarmUp(5.0);
		e.turnOnLogReport();
		e.turnOnExperimentReport();
		e.turnOnReplicationReport();
		e.runAll();
		
		e.turnOnExperimentReport("Test");
		e.runAll();
	}

	public static void testSimulation(){
		Model m = Model.createModel();;
		new TestRV(m);

		m.turnOnLogReport();
		
		Experiment e1 = new Experiment(m, "Exp1");
		e1.setNumberOfReplications(2);
		e1.setLengthOfReplication(10.0);
		e1.setLengthOfWarmUp(5.0);
		e1.setMonteCarloOption(true);

		Experiment e2 = new Experiment(m, "Exp2");
		e2.setNumberOfReplications(2);
		e2.setLengthOfReplication(10.0);
		e2.setLengthOfWarmUp(5.0);
		e2.setMonteCarloOption(true);

		Experiment e3 = new Experiment(m, "Exp3");
		e3.setNumberOfReplications(2);
		e3.setLengthOfReplication(10.0);
		e3.setLengthOfWarmUp(5.0);
		e3.setMonteCarloOption(true);

		ArrayList<Experiment> list = new ArrayList<Experiment>();
		list.add(e1);
		list.add(e2);
		list.add(e3);
		
		ExperimentRunner s = new ExperimentRunner("Sim1", list.iterator());
		s.turnOnLogReport();
		s.runAll();
	}
}
