/*
* Created on Apr 19, 2007
* Copyright (c) 2007, Manuel D. Rossetti (rossetti@uark.edu)
*
* Contact:
*	Manuel D. Rossetti, Ph.D., P.E. 
*	Department of Industrial Engineering 
*	University of Arkansas 
*	4207 Bell Engineering Center 
*	Fayetteville, AR 72701 
*	Phone: (479) 575-6756 
*	Email: rossetti@uark.edu 
*	Web: www.uark.edu/~rossetti
*
* This file is part of the JSL (a Java Simulation Library). The JSL is a framework
* of Java classes that permit the easy development and execution of discrete event
* simulation programs.
*
* The JSL is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The JSL is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with the JSL (see file COPYING in the distribution); 
* if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, 
* Boston, MA  02110-1301  USA, or see www.fsf.org
* 
*/
package testing.variable;

import jsl.modeling.Experiment;
import jsl.modeling.JSLEvent;
import jsl.modeling.Model;
import jsl.modeling.ModelElement;
import jsl.modeling.Replication;
import jsl.modeling.SchedulingElement;
import jsl.modeling.elements.variable.TimeWeighted;
import jsl.utilities.statistic.StatisticAccessorIfc;

/**
 * @author rossetti
 *
 */
public class TestTimeWeighted extends SchedulingElement {

	TimeWeighted myX;
	
	/**
	 * @param parent
	 */
	public TestTimeWeighted(ModelElement parent) {
		this(parent, null);
	}

	/**
	 * @param parent
	 * @param name
	 */
	public TestTimeWeighted(ModelElement parent, String name) {
		super(parent, name);
		myX = new TimeWeighted(this, 0.0);
	}

	protected void initialize(){
		scheduleEvent(50.0);
	}
	
	protected void handleEvent(JSLEvent e){
	//	myX.setValue(myX.getValue());
		System.out.println(getTime() +">");
		System.out.println(myX.getWithinReplicationStatistic());

	}
	
	protected void replicationEnded(){
		System.out.println("replicationEnded()");
		System.out.println(myX.getWithinReplicationStatistic());
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

	testExperiment();
			
//		testReplication();
	}

	public static void testExperiment(){
		Model m = Model.createModel();
		new TestTimeWeighted(m);
		
		// create a replication to run the model
		Experiment e = new Experiment(m);

		// set the running parameters of the experiment
		e.setNumberOfReplications(2);
		e.setLengthOfReplication(50.0);

		// turn on the desired reporting
//		e.turnOnExperimentReport();
//		e.turnOnReplicationReport();
		e.turnOnExperimentConsoleOutput();
				
		// tell the replication to run
		e.runAll();			
	}
	
	public static void testReplication(){
		Model m = Model.createModel();
		new TestTimeWeighted(m);
		        
		// create a replication to run the model
		Replication r = new Replication(m);

		// set the running parameters of the replication
		r.setLengthOfReplication(50.0);

		// turn on the desired reporting
		r.turnOnBatchReport();
				
		// tell the replication to run
		r.runAll();
		
		System.out.println("Done!");
 
	}
}
